Rice.require 'TableConverter'
Rice.require 'ExtensionLoader'
Rice.require 'commandString'
Rice.require 'Native'
Rice.require 'UTF8'

-----------------------------------------------------------------------------
-- Local Function : is Member
-----------------------------------------------------------------------------
local function isMember( value, tbl )
	for i = 1 , #tbl do
		if(tbl[i] == value) then
			return true
		end
	end
	return false
end

-----------------------------------------------------------------------------
-- Local Function : is Empty
-----------------------------------------------------------------------------
local function isEmpty( value )
	return ( not value or '' == value )
end

-----------------------------------------------------------------------------
-- Local Function : get Command Charcode
-----------------------------------------------------------------------------
local function GetCommandCharcode( tbl )
	if ('on' == tbl.utf8mode) then
		return 'utf8'
	elseif( 'on' == tbl.gb18030support	and
		'8' == tbl.jh_charset			) then
		return 'utf8'
	elseif ('10' == tbl.jh_charset or '11' == tbl.jh_charset or '33' == tbl.jh_charset) and ('on' == tbl.output_utf8_rtl_language) then
		return 'utf8'
	else
		return 'osdependent'
	end
end


-----------------------------------------------------------------------------
-- Conversion Table : setting_table -> job_command_table
-----------------------------------------------------------------------------
local convert_table = {

	loginusername = {
		item = function( value, tbl )
			local luname = get_trimmed_command_string( value, GetCommandCharcode(tbl), 217 )
			if( 'on' == tbl.userauthentication and not isEmpty( luname ) ) then
				return { 'AUTHENTICATIONUSERNAME', 'AUTHENTICATIONUSERNAMECHARSET', 'AUTHENTICATIONPASSWORDCHARSET' }
			else
				return { 'AUTHENTICATIONUSERNAME', 'AUTHENTICATIONUSERNAMECHARSET'}
			end
		end,
		values = function(value, tbl)
			local luname = get_trimmed_command_string( value, GetCommandCharcode(tbl), 217 )
			if( 'on' == tbl.userauthentication and not isEmpty( luname ) ) then
				if ( 'utf8' == GetCommandCharcode(tbl) ) then
					return { '"' .. luname .. '"', '33', '33'}
				elseif( isMember( tbl.jh_charset, { '1', '2', '3', '4', '5', '6', '8', '9', '11', '12', '13' } )) then
					return { '"' .. luname .. '"', tbl.jh_charset, tbl.jh_charset }
				else
					return { '"' .. luname .. '"', '2', '2' }
				end
			else
				if ( 'utf8' == GetCommandCharcode(tbl) ) then
					return { '""', '33' }
				elseif( isMember( tbl.jh_charset, { '1', '2', '3', '4', '5', '6', '8', '9', '11', '12', '13' } )) then
					return { '""', tbl.jh_charset }
				else
					return { '""', '2' }
				end
			end
		end
	},

	jh_loginpassword3 = {
		item = { 'AUTHENTICATIONPASSWORD', 'AUTHENTICATIONENCRYPTSETTING', 'AUTHENTICATIONENCRYPTMETHOD' },
		values = function(value, tbl)
			if( 'on' == tbl.userauthentication ) then
				local luname = get_trimmed_command_string( tbl.loginusername, GetCommandCharcode(tbl), 217 )
				if( isEmpty( luname ) ) then
					return nil
				else
					if( '5' == tbl.jobpasswordgeneration ) then
						return { '"' .. value .. '"', '2', '10' }
					else
						if( 'on' == tbl.withencryption ) then
							return { '"' .. value .. '"', '2', '0' }
						else
							return { '"' .. value .. '"', '1', '0' }
						end
					end
				end
			else
				return nil
			end
		end
	},

	jh_loginpassword4 = {
		item = { 'AUTHENTICATIONPASSWORD', 'AUTHENTICATIONENCRYPTSETTING', 'AUTHENTICATIONENCRYPTMETHOD' },
		values = function(value, tbl)
			if( 'on' == tbl.userauthentication ) then
				local luname = get_trimmed_command_string( tbl.loginusername, GetCommandCharcode(tbl), 217 )
				if( isEmpty( luname ) ) then
					return nil
				else
					if( '5' == tbl.jobpasswordgeneration ) then
						return { '"' .. value .. '"', '2', '10' }
					else
						if( 'on' == tbl.withencryption ) then
							return { '"' .. value .. '"', '2', '0' }
						else
							return { '"' .. value .. '"', '1', '0' }
						end
					end
				end
			else
				return nil
			end
		end
	},

	jh_loginpassword5 = {
		item = { 'AUTHENTICATIONPASSWORD', 'AUTHENTICATIONENCRYPTSETTING', 'AUTHENTICATIONENCRYPTMETHOD' },
		values = function(value, tbl)
			if( 'on' == tbl.userauthentication ) then
				local luname = get_trimmed_command_string( tbl.loginusername, GetCommandCharcode(tbl), 217 )
				if( isEmpty( luname ) ) then
					return nil
				else
					if( '5' == tbl.jobpasswordgeneration ) then
						return { '"' .. value .. '"', '2', '10' }
					else
						if( 'on' == tbl.withencryption ) then
							return { '"' .. value .. '"', '2', '0' }
						else
							return { '"' .. value .. '"', '1', '0' }
						end
					end
				end
			else
				return nil
			end
		end
	},

	windowslogonusername = {
		item = 'HOSTLOGINNAME',
		values = function(value, tbl)
			return '"' .. get_trimmed_command_string( tbl.windowslogonusername, GetCommandCharcode(tbl), 20 ) .. '"'
		end
	},

	jh_charset = {
		item =  { 'HOSTCHARSET2' },
		values = function(value, tbl)
			if( 'utf8' == GetCommandCharcode(tbl)) then
				return { '33' }
			elseif( isMember( value, { '1', '2', '3', '4', '5', '6', '8', '9', '11', '12', '13' } )) then
				return { value }
			else
				return { '2' }
			end
		end
	},

	letterheadmode = {
		item = 'LETTERHEADSETTING',
		values = function( value, tbl )
			if( 'off' == tbl.letterheadmode )then
					return 'OFF'
			elseif( 'auto' == tbl.letterheadmode )then
				return 'AUTO'
			elseif( 'always' == tbl.letterheadmode )then
				return 'ALWAYS'
			else
				return nil
			end
		end
	},
	
	duplexmode = {
		item = 'DUPLEXMODE',
		values = function( value, tbl )
			if( 'on' == value ) then
				return 'ON'
			elseif( 'off' == value )then
				if( 'off' == tbl.letterheadmode ) then
					return nil
				else
					return 'OFF'
				end
			else
				return nil
			end
		end
	},

	physicalpagecount = {
		item = 'PRINTFACESNUM',
		values = function( value, tbl )
			if( ( tbl.booklet and 'booklet' == tbl.booklet ) and
				( tbl.layout and '1pagepersheet' == tbl.layout ) and
				( ( tbl.pagelayout_distribution and 'printer' ~= tbl.pagelayout_distribution ) or
				  ( nil == tbl.pagelayout_distribution ) ) )then
				if(tbl.booklet_equalmagnification and 'on' == tbl.booklet_equalmagnification)	then
					if(nil == tbl.reduceenlarge or 'off' == tbl.reduceenlarge) then
						return nil
					else
						return value	
					end
				else
					return value
				end
			else
				return nil
			end
		end
	},

}

-----------------------------------------------------------------------------
-- ELocal Function : Generate JOB ADDITIONAL INFO 
-----------------------------------------------------------------------------
function generate_job_additional_info_user_domain_name(settings_table)
	if( nil ~= settings_table.userdomainname ) then
		return '@PJL SET JOBADDITIONALINFO = ' .. '"UserDomain=' .. get_trimmed_command_string( settings_table.userdomainname, GetCommandCharcode(settings_table), 211 ) .. '"\r\n'
	else
		return nil
	end
end

function generate_job_additional_info_process_name(settings_table)
	if( nil ~= settings_table.jh_process_name ) then
		return '@PJL SET JOBADDITIONALINFO = ' .. '"ProcessName=' .. get_trimmed_command_string( settings_table.jh_process_name, GetCommandCharcode(settings_table), 210 ) .. '"\r\n'
	else
		return nil
	end
end

-----------------------------------------------------------------------------
-- Export Function : Generate Command
-----------------------------------------------------------------------------
function generate_command(settings_table)

	local converter = TableConverter(convert_table)
	local job_commands = converter:run(settings_table)

	local commands = {}

	table.insert( commands, string.char(0x1b) .. '%-12345X')
	table.insert( commands, '@PJL JOB\r\n' )
	table.insert( commands, generate_job_additional_info_user_domain_name(settings_table) )
	table.insert( commands, generate_job_additional_info_process_name(settings_table) )

	for item, value in pairs(job_commands) do
		table.insert(commands, string.format('@PJL SET %s = %s\r\n', item, value))
	end

	table.insert( commands, '@PJL ENTER LANGUAGE = POSTSCRIPT\r\n' )

	return table.concat(commands)
end

return true
