Rice.require 'Native'
Rice.require 'Rice.Window'

local string = string
local type   = type
local Native = Native
local Rice   = Rice

module 'FileDialog'

-- Definition of Flags
OFN_READONLY                 = 0x00000001
OFN_OVERWRITEPROMPT          = 0x00000002
OFN_HIDEREADONLY             = 0x00000004
OFN_NOCHANGEDIR              = 0x00000008
OFN_SHOWHELP                 = 0x00000010
--OFN_ENABLEHOOK               = 0x00000020
--OFN_ENABLETEMPLATE           = 0x00000040
--OFN_ENABLETEMPLATEHANDLE     = 0x00000080
OFN_NOVALIDATE               = 0x00000100
--OFN_ALLOWMULTISELECT         = 0x00000200
--OFN_EXTENSIONDIFFERENT       = 0x00000400
OFN_PATHMUSTEXIST            = 0x00000800
OFN_FILEMUSTEXIST            = 0x00001000
OFN_CREATEPROMPT             = 0x00002000
OFN_SHAREAWARE               = 0x00004000
OFN_NOREADONLYRETURN         = 0x00008000
OFN_NOTESTFILECREATE         = 0x00010000
OFN_NONETWORKBUTTON          = 0x00020000
OFN_NOLONGNAMES              = 0x00040000 
OFN_EXPLORER                 = 0x00080000 
OFN_NODEREFERENCELINKS       = 0x00100000
OFN_LONGNAMES                = 0x00200000
--OFN_ENABLEINCLUDENOTIFY      = 0x00400000
OFN_ENABLESIZING             = 0x00800000
OFN_DONTADDTORECENT          = 0x02000000
OFN_FORCESHOWHIDDEN          = 0x10000000

-- Definition of OPENFILENAME Structure
local char_size = Native.sizeof('byte')
local OPENFILENAME = Native.Struct()
  :dword('lStructSize')
  :ptr('hwndOwner')
  :ptr('hInstance')
  :ptr('lpstrFilter', char_size)
  :ptr('lpstrCustomFilter', char_size)
  :dword('nMaxCustFilter')
  :dword('nFilterIndex')
  :ptr('lpstrFile', char_size)
  :dword('nMaxFile')
  :ptr('lpstrFileTitle', char_size)
  :dword('nMaxFileTitle')
  :ptr('lpstrInitialDir', char_size)
  :ptr('lpstrTitle', char_size)
  :dword('Flags')
  :word('nFileOffset')
  :word('nFileExtension')
  :ptr('lpstrDefExt', char_size)
  :ptr('lCustData')
  :ptr('lpfnHook')
  :ptr('lpTemplateName', char_size)
  :pack()

-- Load Comdlg32 DLL
local comdlg32 = Native.Dll('comdlg32.dll')  

-- Call GetOpenFileName or GetSaveFileName API
local function get_file_name(api_name, ext, showing, flags, initial_dir, initial_filename)
  
  local extension = (function()
    local pos = ext:find('[^%.]')
    return ext:sub(pos)
  end)()
  
  local filename = initial_filename and initial_filename or '*.' .. extension
  
  local dir = (function()
    local dir_type = type(initial_dir)
    if(dir_type == 'string') then
      return initial_dir
    else
      return (function()
        local shell32 = Native.Dll('shell32.dll')
        local buf = Native.Memory(512)
        shell32:api('SHGetSpecialFolderPath')
        :with_ptr(Native.null_ptr())
        :with_ptr(buf:ptr())
        :with_int(((dir_type == 'number') and initial_dir or 0))
        :with_int(0)
        :call_int()
        return buf:as_string()
      end)()
    end
  end)()
  
  local max_filename_size = 512
  local filename_buf = Native.Memory(max_filename_size):copy_string(filename)
  local filter = string.format('%s\0*.%s\0\0', showing, extension)
  local hwnd = Rice.Window and Rice.Window.get() or Native.null_ptr()
  
  local ofn = OPENFILENAME:instance()
    :set_dword('lStructSize', OPENFILENAME:size())
    :set_ptr('hwndOwner', hwnd)
    :set_string('lpstrFilter', filter)
    :set_dword('nFilterIndex', 1)
    :set_ptr('lpstrFile', filename_buf:ptr())
    :set_dword('nMaxFile', max_filename_size)
    :set_string('lpstrInitialDir', dir)
    :set_string('lpstrDefExt', extension)
    :set_dword('Flags', flags)

  return (comdlg32:api(api_name):with_ptr(ofn:memory():ptr()):call_int() ~= 0) and ofn:get_string('lpstrFile') or nil

end

----------------------------------------------------------
-- Export Function : Open File Dialog
----------------------------------------------------------
function open(ext, showing, flags, initial_dir, initial_filename)
  return get_file_name('GetOpenFileName', ext, showing, flags, initial_dir, initial_filename)
end

----------------------------------------------------------
-- Export Function : Save File Dialog
----------------------------------------------------------
function save(ext, showing, flags, initial_dir, initial_filename)
  return get_file_name('GetSaveFileName', ext, showing, flags, initial_dir, initial_filename)
end

return true
