Rice.require 'Rice.CapsDB'
Rice.require 'class'

class 'Plate'
function Plate:__init(params, occupied)
  self._height = params.height
  self._feature = params.feature
  self._layout = nil
  self._occupied = occupied
end

function Plate:__call(modules)
  if self._layout then
    error('plate setting is already done', 2)
  end

  if type(modules) ~= 'table' then
    error('argument is must be table' ,2)
  end

  local function valid_module(mod)
    if type(mod) == 'table' and type(mod.params) == 'table' then
      local feature = self._feature and self._feature or (mod.type:match '^.+::Feature$' and mod.name or nil)
      if feature then
        if not Rice.CapsDB.get(feature) then return false end
        local featuredetail = Rice.CapsDB.get_detail(feature)
        if featuredetail and not featuredetail.visible then return false end
      end
    end
    if type(mod.modules) ~= 'table' then return true end
    for i, child in ipairs(mod.modules) do
      if not valid_module(child) then return false end
    end
    return true
  end
  
  self._layout = modules
  for i, m in ipairs(self._layout) do
    if not valid_module(m) then
      self._layout = true
      break
    end
  end
  
  return self
end

function Plate:shift(x, y)
  if type(self._layout) ~= 'table' then return self end

  local function shift_coordinate(mod)
    if type(mod) ~= 'table' then return end
    
    if type(mod.params) == 'table' then
      if mod.params.x then mod.params.x = mod.params.x + x end
      if mod.params.y then mod.params.y = mod.params.y + y end

      for i, key in ipairs({'label', 'icon', 'balloon'}) do
        local params = mod.params[key]
        if type(params) == 'table' then
          if params.x then params.x = params.x + x end
          if params.y then params.y = params.y + y end
        end
      end
      local members = mod.params['members']
      if type(members) == 'table' then
        for key, val in pairs(members) do
          if type(val) == 'table' then
            if val.x then val.x = val.x + x end
            if val.y then val.y = val.y + y end
          end
        end
      end
    end
    
    if type(mod.modules) ~= 'table' then return end
    for i, child in ipairs(mod.modules) do
      shift_coordinate(child)
    end
    
  end
  
  for i, m in ipairs(self._layout) do
    shift_coordinate(m)
  end

  return self
end

function Plate:layout()
  if type(self._layout) == 'boolean' then
    return nil
  else
    return self._layout
  end
end

function Plate:occupied()
  return self._occupied
end

function Plate:height()
  return self._height
end

class('Plate.Ghost', Plate)
function Plate.Ghost:__init(params, occupied)
  self.super:__init(params, occupied)
end

function Plate.Ghost:layout()
  return self.super.layout(self) or {} 
end

class('Plate.Delimiter', Plate)
function Plate.Delimiter:__init(params)
  self.super:__init(params, 2)
end

function Plate.Delimiter:layout(next_getter, index)
  if next_getter then
    local i, p = next_getter(index + 1)
    return (p and not Plate.Delimiter:is_class_of(p)) and self.super.layout(self) or nil
  else
    return self.super.layout(self)
  end
end

-------------------------------------------

function Plate.Full(params)
  return Plate(params, 2)
end

Plate.Half = {}
setmetatable(Plate.Half, {
  __call = function(self, params)
    return Plate(params, 1)
  end}
)

function Plate.Half.Ghost(params)
  return Plate.Ghost(params, 1)
end

NonDisplay = {
  Component = function(components)
    return Plate({height = 0}, 0)(components)
  end
}

-------------------------------------------

class 'GuiPlateLayout'
function GuiPlateLayout:__init(params)
  self._width  = params.width
  self._margin_left = params.margin_left or 0
  self._margin_top = params.margin_top or 0
end

function GuiPlateLayout:__call(plates)

  if type(plates) ~= 'table' then
    error('argument is must be table of Plate objects' ,2)
  end
  
  local guis = {}
  local function add_guis(plate, x, height)
    for i, gui in ipairs(plate:shift(x + self._margin_left, height):layout() or {}) do
      guis[#guis + 1] = gui
    end      
  end

  local function add_component(plate)
    if plate and plate:occupied() == 0 then
      add_guis(plate, 0, 0)
      return true
    end
    return false
  end
  
  local next_plate = nil
  next_plate = (function(plates)
    return function(index)
      local plate = plates[index]
      while plate and (not plate:layout(next_plate, index) or add_component(plate)) do
        index = index + 1
        plate = plates[index]
      end
      index = index + 1
      return index, plate
    end
  end)(plates)

  local make_row = (function()
    local height = self._margin_top
    return function(index, plate)
      local occupied = plate:occupied()
      if occupied == 2 then
        add_guis(plate, 0, height)
        height = height + plate:height()
        return next_plate(index)
      elseif occupied == 1 then
        local plate2 = nil
        index, plate2 = next_plate(index)
        if plate2 == nil or plate2:occupied() == 1 then
          plate2 = plate2 or Plate.Half {height = 0}
          add_guis(plate, 0, height)
          add_guis(plate2, self._width/2, height)
          height = height + math.max(plate:height(), plate2:height())
          return next_plate(index)
        else
          add_guis(plate, 0, height)
          height = height + plate:height()
          return index, plate2
        end
      else
        error('invalid plate.occupied value' ,2)
      end
    end
  end)()

  local index, plate = next_plate(1)

  while plate and Plate.Delimiter:is_class_of(plate) do
    index, plate = next_plate(index)
  end
  
  while plate do
    index, plate = make_row(index, plate)
  end

  return guis
end
  
return true
