local OrderedHash_proto = {}

function OrderedHash(...)
  local obj = {}
  obj.hash = {}
  obj.order = {}
  setmetatable(obj, OrderedHash_proto)

  local arg = {...}
  for i, v in ipairs(arg) do
    if type(v) ~= 'table' or #v ~= 2 then
      error('OrderedHash : Initializer must be sequence of {key, value}.')
    end
    obj[v[1]] = v[2]
  end
  
  return obj
end

function OrderedHash_proto:__index(key)
  local t = type(key)
  if t == 'string' then
    return rawget(rawget(self, 'hash'), key)
  elseif t == 'number' then
    local order = rawget(self, 'order')
    local hash = rawget(self, 'hash')
    local str_key = rawget(order, key)
    if not str_key then return nil end
    return {key = str_key, value = rawget(hash, str_key)}
  end
end

function OrderedHash_proto:__newindex(key, value)
  if type(key) ~= 'string' then
    error('OrderedHash:__newindex : Key must be string value.')
  end

  local order = rawget(self, 'order')
  local hash = rawget(self, 'hash')
  if not rawget(hash, key) then
    rawset(order, #order + 1, key)
  end
  rawset(hash, key, value)
end

function OrderedHash_each(obj)
  return coroutine.wrap( function()
    local order = rawget(obj, 'order')
    local hash = rawget(obj, 'hash')
    local size = #order
    for i = 1, size do
      local key = rawget(order, i)
      coroutine.yield(i, key, rawget(hash, key))
    end
  end)
end

function OrderedHash_len(obj)
  return #rawget(obj, 'order')
end


