class 'TableConverter'

function TableConverter:__init(rules)
  self.rules = rules
end

function TableConverter:as_table(obj, value, tbl)
  local obj_type = type(obj)
  if(obj_type == 'table') then
    if(obj[value] ~= nil) then
      return self:as_table(obj[value], value, tbl)
    else
      return (table.maxn(obj) ~= 0) and obj or {}
    end
  elseif(obj_type == 'function') then
    return self:as_table(obj(value, tbl), value, tbl)
  else
    return {obj}
  end
end

function TableConverter:convert_item(item, value, tbl)

  local target = self.rules[item]

  if(type(target) ~= 'table') then
    return function() return nil end
  end

  local converted_items = self:as_table(target.item, value, tbl)
  local converted_values = self:as_table(target.values, value, tbl)

  if((not next(converted_items, nil) or not next(converted_values, nil)) and
    (type(target.default) == 'table' and table.maxn(target.default) == 2)) then
      converted_items = self:as_table(target.default[1], value, tbl)
      converted_values = self:as_table(target.default[2], value, tbl)
  end
  
  local item_index, value_index

  return function()
    local cur_item, cur_value
    item_index, cur_item = next(converted_items, item_index)
    value_index, cur_value = next(converted_values, value_index)
    if((item_index ~= nil) and (value_index ~= nil)) then
      return cur_item, cur_value
    else
      return nil
    end
  end
  
end

function TableConverter:run(before)
  local after = {}
  for k, v in pairs(before) do
    for after_key, after_value in self:convert_item(k, v, before) do
      after[after_key] = after_value
    end
  end
  return after
end

return true
