Rice.require 'exstring'

local function valid_class_name(class_name)
  local names = class_name:split '%.'
  for i, name in ipairs(names) do
    if not name:match '^[%u%l]%w*$' then return false end
  end
  return true
end

function class(class_name, super)
  if type(class_name) ~= 'string' then
    error('class name must be string', 2)
  end

  if not valid_class_name(class_name) then
    error('"' .. class_name .. '" is invalid class name', 2)
  end

  local cls = _G
  local defined = false
  for name in class_name:gmatch '(%w+)%.?' do
    if type(rawget(cls, name)) == 'nil' then
      rawset(cls, name, {})
      defined = false
    else
      defined = true
    end
    cls = rawget(cls, name)
  end

  if defined then
    error(class_name .. ' is already defined', 2)
  end
  
  if cls ~= _G then
    cls.__index = cls
    
    cls.__tostring = function(self)
      return '<class ' .. class_name .. '>' 
    end

    cls.is_class_of = function(self, obj)
      local obj_class = getmetatable(obj)
      while obj_class do
        if obj_class == self then return true end
        obj_class = obj_class.super
      end
      return false    end

    local constructor = {}
    
    function constructor:__call(...)
      if type(cls.__init) ~= 'function' then
        error('__init() is not defined in class ' .. class_name, 2)
      end

      local obj = {}
      setmetatable(obj, cls)
      cls.__init(obj, ...)
      return obj      
    end

    if super then
      cls.super = super
      cls.__add = super.__add
      cls.__sub = super.__sub
      cls.__mul = super.__mul
      cls.__div = super.__div
      cls.__mod = super.__mod
      cls.__pow = super.__pow
      cls.__unm = super.__unm
      cls.__concat = super.__concat
      cls.__len = super.__len
      cls.__eq = super.__eq
      cls.__lt = super.__lt
      cls.__le = super.__le
      cls.__call = super.__call      
      constructor.__index = super
    end
    
    setmetatable(cls, constructor)

    return cls
    
  end
end

return true
