Rice.require 'UTF8'
Rice.require 'Native'

local ansi_code_page	= Native.Dll('kernel32.dll'):api('GetACP'):call_int()

local function is_lead_byte( bytecode )
	local kernel32			= Native.Dll('kernel32.dll')

	return (kernel32:api('IsDBCSLeadByteEx')
	:with_int( ansi_code_page )
	:with_byte( bytecode )
	:call_int() ~= 0 )
end

local function get_trimmed_string( str, max_length )
	
	if (not str) or (not max_length) then
		return ''
	end

	local buf_size = math.min( string.len(str), max_length )

	local index = 1
	while index <= buf_size do
		if is_lead_byte( string.byte( str, index ) ) then
			if index == buf_size then
				buf_size = buf_size - 1
				do break end
			end
			index = index + 2
		else
			if index == buf_size  then
				do break end
			end
			index = index + 1
		end
	end

	return string.sub( str, 1, buf_size )
end

function trim_utf8_string( command_str, max_length )	
	local index = 1
	local buf_size = 0;

	local i = 0
	while index <= max_length do

		local temp = string.byte( command_str , index )
		if temp == 0 then
		-- eof
			return ''
		elseif temp < 128 then
			-- ascii
			index = index + 1
		elseif temp < 192 then
			--invalid byte
			return ''
		elseif temp < 224 then
			-- 2bytes
			index = index + 2
		elseif temp < 240 then
			-- 3bytes(japanese multibyte)
			index = index + 3
		elseif temp < 248 then
			-- 4bytes
			index = index + 4
		elseif temp < 252 then
			-- 5bytes
			index = index + 5
		elseif temp < 254 then
			-- 6bytes
			index = index + 6
		else
			-- unknown
			return ''
		end
		
		if( index <= max_length + 1 ) then
			buf_size = index - 1
		end
		
	end

	return string.sub( command_str, 1, buf_size )
end


local function get_utf8_trimmed_string( command_str, max_length )
	
	if (not command_str) or (not max_length) then
		return ''
	end
	
	if( string.len( command_str ) > max_length ) then
		command_str = trim_utf8_string( command_str, max_length )
	end
	
	return command_str

end


function get_trimmed_command_string( str, charcode, max_length  )

	if (not str) or (not max_length) then
		return ''
	end
	
	local command_str
	local trimmed_command_str
	
	if( 'utf8' ~= charcode ) then
		trimmed_command_str = get_trimmed_string( UTF8.decode( str ), max_length )
	else
		trimmed_command_str = get_utf8_trimmed_string( str, max_length )
	end
	
	return  trimmed_command_str
	
end

return true
