function table:find(value)
  if(type(self) ~= 'table') then
    error("bad argument #1 to 'cat' (table expected, got " .. type(self) .. ")", 2)
  end

  for k, v in pairs(self) do
    if v == value then return k end
  end

  return nil
end


function table:cat(other)

  if(type(self) ~= 'table') then
    error("bad argument #1 to 'cat' (table expected, got " .. type(self) .. ")", 2)
  end
  
  if(type(other) ~= 'table') then
    return self
  end

  local index = nil
  local value = nil

  while(true) do
    index, value =  next(other, index)
    if(not index) then break end

    if(type(index) == 'number') then
      table.insert(self, value)
    elseif(type(value) == 'table' and self[index] and type(self[index]) == 'table' ) then
      table.cat(self[index], value)
    else
      self[index] = value
    end
  end

  return self
end


function table:map(func)
  
  if(type(self) ~= 'table') then
    error("bad argument #1 to 'map' (table expected, got " .. type(self) .. ")", 2)
  end

  if(type(func) ~= 'function') then
    error("bad argument #2 to 'map' (function expected, got " .. type(func) .. ")", 2)
  end

  local ret = {}
  for k, v in pairs(self) do
    ret[k] = func(k, v)
  end

  return ret
end


function table:for_each(func)
  
  if(type(self) ~= 'table') then
    error("bad argument #1 to 'map' (table expected, got " .. type(self) .. ")", 2)
  end

  if(type(func) ~= 'function') then
    error("bad argument #2 to 'map' (function expected, got " .. type(func) .. ")", 2)
  end

  for i, v in ipairs(self) do
    func(i, v)
  end
  
end

return true
