Rice.require 'Rice.CapsDB'
Rice.require 'Rice.PaperDB'
Rice.require 'Rice.Encrypt'
Rice.require 'Native'
Rice.require 'ExtensionLoader'
Rice.require 'commandString'
Rice.require 'CDBTableItem'

-----------------------------------------------------------------------------
-- Local Function : is_custom_paper_size
-----------------------------------------------------------------------------
local function is_custom_paper_size( papersize )
	return tostring( papersize ) and ( nil ~= string.find( tostring( papersize ), 'custompapersize' ) ) or false
end
-----------------------------------------------------------------------------
-- Local Function : is_member
-----------------------------------------------------------------------------
local function is_member( value, tbl )
	for i = 1 , #tbl do
		if( tbl[i] == value ) then
			return true
		end
	end
	return false
end

-----------------------------------------------------------------------------
-- Local Function : get_pointvalue_from_inch
-----------------------------------------------------------------------------
local function get_pointvalue_from_inch( inchvalue )
	return (nil == inchvalue or "" == inchvalue ) and "" or tonumber( string.format( "%d", inchvalue * 72 / 100 + (function() return tonumber(inchvalue) < 0 and -0.5 or 0.5 end)()) )
end

-----------------------------------------------------------------------------
-- Local Function : get_pointvalue_decimal_from_mm
-----------------------------------------------------------------------------
local function get_pointvalue_decimal_from_mm( mmvalue )
	return (nil == mmvalue or "" == mmvalue ) and "" or tonumber( string.format( "%.1f", mmvalue * 72 / 254 ) )
end

-----------------------------------------------------------------------------
-- Local Function : add_feature_tags
-----------------------------------------------------------------------------
local function add_feature_tags( feature, option, command )

	if(	nil == command or	"" == command	or
		nil == feature or	"" == feature	or
		nil == option or	"" == option	) then
		return ""
	else
		return(
				"mark {\r\n" ..
				"%%BeginFeature: " ..
				feature .. " " .. option .. "\r\n" ..
				command .. "\r\n" ..
				"%%EndFeature\r\n" ..
				"} stopped cleartomark\r\n\r\n"
			  )
	end

end

-----------------------------------------------------------------------------
-- Local Function : get_custompapersize_from_inch
-----------------------------------------------------------------------------
local function get_custompapersize_from_inch( width_inch, height_inch, size_type )

	if(	nil == width_inch	or "" == width_inch		or
		nil == height_inch	or "" == height_inch	) then
		return {['width']=595, ['height']=842}	-- Default : A4
	end

	local offset = ("paper" == size_type) and 0 or 12

	local width = get_pointvalue_from_inch( width_inch ) - offset * 2
	local height = get_pointvalue_from_inch( height_inch ) - offset * 2

	return {['width']=width, ['height']=height}

end

-----------------------------------------------------------------------------
-- Local Function : get_custompapersize_decimal_from_mm
-----------------------------------------------------------------------------
local function get_custompapersize_decimal_from_mm( width_mm, height_mm, size_type )

	if(	nil == width_mm		or "" == width_mm		or
		nil == height_mm	or "" == height_mm		) then
		return {['width']=595, ['height']=842}	-- Default : A4
	end

	local offset = ("paper" == size_type) and 0 or 12.0

	local width = get_pointvalue_decimal_from_mm( width_mm ) - offset * 2.0
	local height = get_pointvalue_decimal_from_mm( height_mm ) - offset * 2.0

	return {['width']=width, ['height']=height}

end

-----------------------------------------------------------------------------
-- Local Function : get_papersize
-----------------------------------------------------------------------------
local function get_papersize( info_map, name, size_type, synchrocut, synchrocut_printsize )

	local result_tbl = {}

	local a4_tbl =
	{
		['mm']		= { ['width'] = 2100,	['height'] = 2970,	['offset_x'] = 42,	['offset_y'] = 42,	['page_width'] = 2016,	['page_height'] = 2886	},
		['inch']	= { ['width'] = 827,	['height'] = 1169,	['offset_x'] = 17,	['offset_y'] = 17,	['page_width'] = 793,	['page_height'] = 1136	},
		['point']	= { ['width'] = 595,	['height'] = 842,	['offset_x'] = 12,	['offset_y'] = 12,	['page_width'] = 571,	['page_height'] = 818	}
	}

	local function setSynchrocutPrintsize( paperinfo, unit, synchrocut, synchrocut_printsize )
		if( nil ~= synchrocut and nil ~= synchrocut_printsize ) then
			if( "on" == synchrocut and "sameasoriginalsize" ~= synchrocut_printsize ) then
				local paperinfo_synchrocut = Rice.PaperDB.get('standard', synchrocut_printsize, unit)
				if( nil ~= paperinfo_synchrocut ) then
					paperinfo.width = paperinfo_synchrocut.width
				end
			end
		end

		return
	end

	if( is_custom_paper_size( name ) ) then
		if(	nil ~= info_map									and
			"generic" == info_map["geneinneritem_driver"]	) then
			local paperinfo_inch = Rice.PaperDB.get('standard', name, 'inch')
			paperinfo_inch = ( nil ~= paperinfo_inch ) and paperinfo_inch or a4_tbl['inch']
			setSynchrocutPrintsize( paperinfo_inch, 'inch', synchrocut, synchrocut_printsize )
			result_tbl = get_custompapersize_from_inch( paperinfo_inch.width, paperinfo_inch.height, size_type )
		else
			local paperinfo_mm = Rice.PaperDB.get('standard', name, 'mm')
			paperinfo_mm = ( nil ~= paperinfo_mm ) and paperinfo_mm or a4_tbl['mm']
			setSynchrocutPrintsize( paperinfo_mm, 'mm', synchrocut, synchrocut_printsize )
			result_tbl = get_custompapersize_decimal_from_mm( paperinfo_mm.width, paperinfo_mm.height, size_type )
		end
	else
		local paperinfo_pt = Rice.PaperDB.get('standard', name, 'point')
		paperinfo_pt = ( nil ~= paperinfo_pt ) and paperinfo_pt or a4_tbl['point']
		result_tbl['width']		= ('region' == size_type ) and paperinfo_pt.page_width or paperinfo_pt.width
		result_tbl['height']	= ('region' == size_type ) and paperinfo_pt.page_height or paperinfo_pt.height
	end

	return result_tbl

end

-----------------------------------------------------------------------------
-- Local Function : get_printsize
-----------------------------------------------------------------------------
local function get_printsize( info_map, documentsize, printsize, synchrocut, synchrocut_printsize )

	if(	nil == documentsize		or "" == documentsize	or
		nil == printsize		or "" == printsize		) then
		return ""
	end
	local tbl = get_papersize( info_map, printsize, "paper", synchrocut, synchrocut_printsize )

	if( is_custom_paper_size( printsize ) ) then

		local cdb = Rice.CapsDB
		
		if( "on" == cdb.get("custompaper_swap")) then
			local swaped_tbl = {}
			swaped_tbl['width'] = tbl['height']
			swaped_tbl['height'] = tbl['width']
			return swaped_tbl
		else
			return tbl
		end
	else
		return tbl
	end

end

-----------------------------------------------------------------------------
-- Local Function : get_inputtray
-----------------------------------------------------------------------------
local function get_inputtray( tray, lctmediaposition, traymediaposition_type )

	local tbl =
	{
		["autotrayselect"] 		= "null",
		["bypasstray"] 			= 0,
		["tray1"]				= 1,
		["tray1roll"]			= 1,
		["tray2"] 				= 2,
		["tray2roll"]			= 2,
		["tray3"] 				= 3,
		["tray3lct"]			= 3,
		["tray3roll"]			= 3,
		["tray4"] 				= 4,
		["tray4roll"]			= 4,
		["tray5"]				= 5,
		["tray6"]				= 6,
		["tray7"]				= (	nil ~= traymediaposition_type	and 
									nil ~= tonumber( traymediaposition_type )and 
									2 <= tonumber( traymediaposition_type )) and 7 or 0,
		["tray8"]				= 8,
		["tray9"]				= 9,
		["tray10"]				= 10,
		["tray11"]				= 11,
		["traya"]				= 0,
		["onesheetbypass"]		= 10,
		["largecapacitytray"]	= (	nil ~= lctmediaposition	and 
									"" ~= lctmediaposition	and
									nil ~= tonumber( lctmediaposition )	) and lctmediaposition or 5,
		["bindinginterposer1"]	= 89,
		["bindinginterposer2"]	= 88,
		["largecapacityinterposer1"]	= 99,
		["largecapacityinterposer2"]	= 98,
		["interposeruppertray"]	= 99,
		["interposerlowertray"]	= 98
	}

	return tbl[tray]

end

-----------------------------------------------------------------------------
-- Local Function : get_mediatype
-----------------------------------------------------------------------------
local function get_mediatype( mediatype )

	local tbl =
	{
		["plainandrecycled"] 		=	"Auto",
		["plain"] 					=	"Plain",
		["plain1"] 					=	"Plain",
		["plain2"] 					=	"Plain2",
		["recycled"] 				=	"Recycled",
		["inkjetplain"] 			=	"InkjetPlain",
		["special"] 				=	"Special",
		["special1"]				=	"Special",
		["special2"]				=	"Special2",
		["special3"]				=	"Special3",
		["special4"]				=	"Special4",
		["special5"]				=	"Special5",
		["thick"] 					= 	"Thick",
		["middlethick"] 			= 	"Middlethick",
		["thick1"] 					= 	"Thick",
		["thick2"] 					= 	"Thick2",
		["thick3"] 					= 	"Thick3",
		["thick4"] 					= 	"Thick4",
		["thin"] 					=	"Thin",
		["transparency"] 			= 	"Transparency",
		["translucent"] 			= 	"Translucent",
		["color"] 					= 	"Color",
		["color1"] 					= 	"Color1",
		["color2"] 					= 	"Color2",
		["letterhead"] 				= 	"Letterhead",
		["preprinted"] 				= 	"Preprinted",
		["plainduplexbackside"]		= 	"PlainDuplexBackSide",
		["thickduplexbackside"]		= 	"ThickDuplexBackSide",
		["thick1duplexbackside"]	= 	"ThickDuplexBackSide",
		["thick2duplexbackside"]	= 	"ThickDuplexBackSide2",
		["thick3duplexbackside"]	= 	"ThickDuplexBackSide3",
		["thick4duplexbackside"]	= 	"ThickDuplexBackSide4",
		["prepunched"]				= 	"Prepunched",
		["labels"] 					= 	"Labels",
		["bond"]					= 	"Bond",
		["cardstock"] 				= 	"Cardstock",
		["tabstock"] 				= 	"Index",
		["glossytabstock"] 			=  	"GlossyIndex",
		["mattabstock"] 			=  	"MatIndex",
		["moreglossytabstock"] 		=  	"MoreGlossyIndex",
		["yellow"] 					= 	"Yellow",
		["green"] 					= 	"Green",
		["blue"] 					= 	"Blue",
		["purple"] 					= 	"Purple",
		["ivory"] 					= 	"Ivory",
		["orange"] 					= 	"Orange",
		["pink"] 					= 	"Pink",
		["red"] 					= 	"Red",
		["gray"] 					= 	"Gray",
		["glossy"] 					=  	"Glossy",
		["semiglossy"] 				=  	"SemiGlossy",
		["glossycoated"]			=	"GlossyCoated",
		["thickglossy"]				=	"ThickGlossy",
		["thickglossycoated"]		=	"ThickGlossyCoated",
		["matcoated"]				=	"MatCoated",
		["matfilm"] 				=	"MatFilm",
		["thickmatcoated"]			=	"ThickMatCoated",
		["waterproof"]				=	"WaterProof",
		["hgplain"] 				= 	"HGPlain",
		["envelope"] 				=  	"Envelope",
		["glossyenvelope"] 			=  	"GlossyEnvelope",
		["matenvelope"] 			=  	"MatEnvelope",
		["moreglossyenvelope"] 		=  	"MoreGlossyEnvelope",
		["coated"] 					= 	"Coated",
		["cadcoated"] 				=	"CadCoated",
		["postcard"]				=	"JPost",
		["inkjetpostcard"]			= 	"InkjetJPost",
		["postcardduplexbackside"]	= 	"JPostDuplexBackSide",
		["film"]					= 	"Film",
		["black"]					= 	"Black",
		["clearfile"]				= 	"ClearFile",
		["magnet"]					= 	"Magnet",
		["metallicpearl"]			= 	"MetallicPearl",
		["synthetic"]				= 	"Synthetic",
		["texture"]					= 	"Texture",
		["ncr"]						= 	"Carbonless",
		["none"]					= 	"null"
	}

	return tbl[mediatype] and tbl[mediatype] or "Auto"

end

-----------------------------------------------------------------------------
-- Local Function : get_resolution_info
-----------------------------------------------------------------------------
local function get_resolution_info( resolution, info )

	local resolution_tbl =
	{
		["300"]			= { ["resolutionvalue"] = 300,	["bitparpixelvalue"] = 1 },
		["600"]			= { ["resolutionvalue"] = 600,	["bitparpixelvalue"] = 1 },
		["1200x600"]	= { ["resolutionvalue"] = 600,	["bitparpixelvalue"] = 2 },
		["600x1200"]	= { ["resolutionvalue"] = 600,	["bitparpixelvalue"] = 2 },
		["2400x600"]	= { ["resolutionvalue"] = 600,	["bitparpixelvalue"] = 2 },
		["9600x600"]	= { ["resolutionvalue"] = 600,	["bitparpixelvalue"] = 4 },
		["38400x600"]	= { ["resolutionvalue"] = 600,	["bitparpixelvalue"] = 8 },
		["1200"]		= { ["resolutionvalue"] = 1200,	["bitparpixelvalue"] = 1 },
		["4800x1200"]	= { ["resolutionvalue"] = 1200,	["bitparpixelvalue"] = 2 }
	}

	return resolution and ( ( resolution_tbl[resolution] and resolution_tbl[resolution] or resolution_tbl["600"] )[info] ) or resolution_tbl["600"][info]

end

-----------------------------------------------------------------------------
-- Local Function : is_layout_nup
-----------------------------------------------------------------------------
local function is_layout_nup( layout )

	local result = false
	if( is_member( layout, {"2pagespersheet", "4pagespersheet", "6pagespersheet", "9pagespersheet", "16pagespersheet"} ) ) then

		result = true
	end

	return  result

end

-----------------------------------------------------------------------------
-- Local Function : is_reduceenlarge
-----------------------------------------------------------------------------
local function is_reduceenlarge( reduceenlarge )

	local result = false
	if( is_member( reduceenlarge, {"fittoprintsize", "zoom", "directionalzoom"} ) ) then
		result = true
	end

	return  result
	
end

-----------------------------------------------------------------------------
-- Local Function : is_layout_booklet
-----------------------------------------------------------------------------
local function is_layout_booklet( layout, booklet )

	local result = false
	if( "booklet" == booklet 	and
		is_layout_nup( layout )	) then
		result = true
	end

	return  result

end

-----------------------------------------------------------------------------
-- Local Function : is_equalmagnification_booklet
-----------------------------------------------------------------------------
local function is_equalmagnification_booklet( layout, booklet, reduceenlarge, booklet_equalmagnification )

	local result = false
	if( false	 	== is_layout_nup( layout )			and
		"booklet" 	== booklet							and
		false 		== is_reduceenlarge(reduceenlarge)	and
		"on" 		== booklet_equalmagnification				) then
		result = true
	end
	
	return result
	
end


-----------------------------------------------------------------------------
-- Local Function : is_printer_pagelayout
-----------------------------------------------------------------------------
local function is_printer_pagelayout( layout, booklet, pagelayout_distribution, booklet_equalmagnification, reduceenlarge )

	local result = false
	if( "printer" == pagelayout_distribution	and
 		(	is_layout_nup( layout )	or 
 			"booklet" == booklet	)			) then
		result = true
	end
	
	if( true == is_equalmagnification_booklet( layout, booklet, reduceenlarge, booklet_equalmagnification)	) then
		result = true
	end
	
	return result
	
end

-----------------------------------------------------------------------------
-- Local Function : is_driver_pagelayout
-----------------------------------------------------------------------------
local function is_driver_pagelayout( layout, booklet, pagelayout_distribution, booklet_equalmagnification, reduceenlarge )

	local result = false
	if( "printer" ~= pagelayout_distribution			and
		(	is_layout_nup( layout )	or
			"booklet" == booklet	)					and
		false == is_layout_booklet( layout, booklet )	and
		false == is_equalmagnification_booklet( layout, booklet, reduceenlarge, booklet_equalmagnification)	) then

		result = true
	end
	
	return result
	
end

-----------------------------------------------------------------------------
-- Local Function : is_frontcover_with_booklet
-----------------------------------------------------------------------------
local function is_frontcover_with_booklet( frontcover, staple, centerstaple_swapcover )

	local result = false
	if( ("on" == frontcover)			and
		("2atcenter" == staple)		and
		(is_member(centerstaple_swapcover, {"1","2","3"}))) then
		result = true
	end
	
	return result
	
end

-----------------------------------------------------------------------------
-- Local Function : output_watermarkpattern
-----------------------------------------------------------------------------
local function output_watermarkpattern(   wmk_print
										, pwmk_textstyle
										, pwmk_opaque
										, wmk_color
										, wmk_density )

	if( "on" ~= wmk_print ) then
		return ""
	end

	local rgb_value = {
		['red']			= {1,0,0},
		['green']		= {0,1,0},
		['blue']		= {0,0,1},
		['yellow']		= {1,1,0},
		['purple']		= {1,0,1},
		['lightblue']	= {0,1,1},
		['black']		= {0,0,0}
	}

	if not rgb_value[wmk_color] then
		wmk_color = 'black'
	end

	local density = 50

	if( ( nil  ~= tonumber( wmk_density ) )	 and
		( 'string' == type( wmk_density ) )	) 	then
		density = (( tonumber( wmk_density ) > 100 ) or ( tonumber( wmk_density ) < 0 )) and 50 or tonumber( wmk_density )
	end

	local color = {}
	for i = 1 , 3 do
		table.insert(color, string.format( '%0.2f', ( rgb_value[wmk_color][i] + (1- rgb_value[wmk_color][i]) / 100 * ( 100 - density ) ) ^ 1.8 ) )
	end

	if( "on" == pwmk_opaque or "outline" == pwmk_textstyle ) then
		return "/WmkCmd\r\n{" .. color[1] .." " .. color[2] .. " " .. color[3] .. " setrgbcolor}\r\ndef\r\n"
	else
		return "/WmkCmd\r\n<<\r\n/PatternType 1\r\n/PaintType 1\r\n/TilingType 3\r\n/BBox [ 0 0 1 1 ]\r\n/XStep 1 /YStep 1\r\n/PaintProc {\r\nbegin\r\n" .. color[1] .." " .. color[2] .. " " .. color[3] .. " setrgbcolor\r\n8 8\r\ntrue\r\n[ 8 0 0 -8 0 8 ]\r\n{<cc33cc33 cc33cc33>}\r\nimagemask\r\nend } >>\r\n[ 1 0 0 -1 0 1 ]\r\nmakepattern\r\ndef\r\n"
	end

end

-----------------------------------------------------------------------------
-- Local Function : output_header_footer_pattern
-----------------------------------------------------------------------------
local function output_header_footer_pattern( hf_print )

	if( "on" == hf_print ) then
		return "/HFCmd\r\n{0 0 0 setrgbcolor}\r\ndef\r\n"
	else
		return ""
	end

end

-----------------------------------------------------------------------------
-- Local Function : get_localeid
-----------------------------------------------------------------------------
local function get_localeid()
	
	local kernel32	= Native.Dll('kernel32.dll')
	
	return kernel32:api('GetUserDefaultUILanguage'):call_dword()
end
-----------------------------------------------------------------------------
-- Local Function : get_charsetvalue
-----------------------------------------------------------------------------
local function get_charsetvalue( gb18030, utf8mode, output_utf8_rtl_language )

	local locale_id = get_localeid()
	
	local locale_tbl =
	{
		[0x0411] = 1,
		[0x0415] = 3,
		[0x040E] = 3,
		[0x0405] = 3,
		[0x0408] = 4,
		[0x0419] = 5,
		[0x0819] = 5,
		[0x041F] = 6,
		[0x0804] = 8,
		[0x1004] = 8,
		[0x0C04] = 9,
		[0x0404] = 9,
		[0x1404] = 9,
		[0x040D] = 10,
		[0x0401] = 11,
		[0x0801] = 11,
		[0x0c01] = 11,
		[0x1001] = 11,
		[0x1401] = 11,
		[0x1801] = 11,
		[0x1c01] = 11,
		[0x2001] = 11,
		[0x2401] = 11,
		[0x2801] = 11,
		[0x2c01] = 11,
		[0x3001] = 11,
		[0x3401] = 11,
		[0x3801] = 11,
		[0x3c01] = 11,
		[0x4001] = 11,
		[0x041E] = 12,
		[0x0412] = 13,
		[0x0429] = 33
	}
	
	if ( ("on" == utf8mode) or (locale_tbl[locale_id] == 10 and output_utf8_rtl_language == "on") or (locale_tbl[locale_id] == 11 and output_utf8_rtl_language == "on") or (locale_tbl[locale_id] == 33 and output_utf8_rtl_language == "on")) then
		return 33
	else
	return locale_tbl[locale_id] and ( (locale_tbl[locale_id] == 8 and gb18030 == "on") and 33 or locale_tbl[locale_id] ) or 2 
	end
end

-----------------------------------------------------------------------------
-- Local Function : get_physicalpaper_orientation
-----------------------------------------------------------------------------
local function get_physicalpaper_orientation( orientation, layout )
	if( '2pagespersheet' == layout or '6pagespersheet' == layout ) then
		return ( 'landscape' == orientation ) and 'portrait' or 'landscape'
	else
		return ( 'landscape' ~= orientation ) and 'portrait' or 'landscape'
	end
end

-----------------------------------------------------------------------------
-- Local Function : get_foldvalue
-----------------------------------------------------------------------------
local function get_foldvalue( foldstamp )

	return ( "off" == foldstamp or "on" == foldstamp ) and 1 or 0
end

-----------------------------------------------------------------------------
-- Local Function : get_faceupvalue
-----------------------------------------------------------------------------
local function get_faceupvalue( duplex, multisheetfold )

	return ( ( "off" == duplex or nil == duplex ) or ( "off" == multisheetfold or nil == multisheetfold ) ) and "true" or "false"
end

-----------------------------------------------------------------------------
-- Local Function : get_stampvalue
-----------------------------------------------------------------------------
local function get_stampvalue( foldstamp )

	return ( "on" == foldstamp ) and "true" or "false"
end

-----------------------------------------------------------------------------
-- Local Function : get_foldbackvalue
-----------------------------------------------------------------------------
local function get_foldbackvalue( edgefold )

	return ( "on" == edgefold ) and "true" or "false"
end

-----------------------------------------------------------------------------
-- Local Function : get_fold_info
-----------------------------------------------------------------------------
local function get_fold_info( fold, info, duplex, multisheetfold, foldstamp )

	local fold_info_tbl =
	{
		["off"]								= { ["foldvalue"] = get_foldvalue( foldstamp ),	["foldtype"] = nil,	["faceup"] = nil										},
		["rightzfold"]						= { ["foldvalue"] = 1,							["foldtype"] = nil,	["faceup"] = "false"									},
		["leftzfold"]						= { ["foldvalue"] = 1,							["foldtype"] = nil,	["faceup"] = "false"									},
		["bottomzfold"]						= { ["foldvalue"] = 1,							["foldtype"] = nil,	["faceup"] = "false"									},
		["insidehalffold"]					= { ["foldvalue"] = 0,							["foldtype"] = 10,	["faceup"] = "false"									},
		["outsidehalffold"]					= { ["foldvalue"] = 0,							["foldtype"] = 10,	["faceup"] = get_faceupvalue( duplex, multisheetfold )	},
		["rightortopinsidespiralfold"]		= { ["foldvalue"] = 0,							["foldtype"] = 11,	["faceup"] = "false"									},
		["leftorbottominsidespiralfold"]	= { ["foldvalue"] = 0,							["foldtype"] = 11,	["faceup"] = "false"									},
		["rightortopoutsidespiralfold"]		= { ["foldvalue"] = 0,							["foldtype"] = 11,	["faceup"] = get_faceupvalue( duplex, multisheetfold )	},
		["leftorbottomoutsidespiralfold"]	= { ["foldvalue"] = 0,							["foldtype"] = 11,	["faceup"] = get_faceupvalue( duplex, multisheetfold )	},
		["outsidezigzagfold"]				= { ["foldvalue"] = 0,							["foldtype"] = 12,	["faceup"] = "false"									},
		["insidezigzagfold"]				= { ["foldvalue"] = 0,							["foldtype"] = 12,	["faceup"] = "false"									},
		["rightortopinsidedoublefold"]		= { ["foldvalue"] = 0,							["foldtype"] = 13,	["faceup"] = "false"									},
		["leftorbottominsidedoublefold"]	= { ["foldvalue"] = 0,							["foldtype"] = 13,	["faceup"] = "false"									},
		["rightortopoutsidedoublefold"]		= { ["foldvalue"] = 0,							["foldtype"] = 13,	["faceup"] = "true"										},
		["leftorbottomoutsidedoublefold"]	= { ["foldvalue"] = 0,							["foldtype"] = 13,	["faceup"] = "true"										},
		["insidedoublegatefold"]			= { ["foldvalue"] = 0,							["foldtype"] = 14,	["faceup"] = "false"									},
		["outsidedoublegatefold"]			= { ["foldvalue"] = 0,							["foldtype"] = 14,	["faceup"] = "true"										},
		["insidefold"]						= { ["foldvalue"] = 1,							["foldtype"] = nil,	["faceup"] = "false"									},
		["outsidefold"]						= { ["foldvalue"] = 1,							["foldtype"] = nil,	["faceup"] = "false"									},
		["superimposedfoldinside"]			= { ["foldvalue"] = 1,							["foldtype"] = nil,	["faceup"] = "false"									},
		["superimposedfoldoutside"]			= { ["foldvalue"] = 1,							["foldtype"] = nil,	["faceup"] = "false"									},
		["standardfold"]					= { ["foldvalue"] = 1,							["foldtype"] = nil,	["faceup"] = nil										},
		["narrowstandardfold"]				= { ["foldvalue"] = 1,							["foldtype"] = nil,	["faceup"] = nil										},
		["marginfold"]						= { ["foldvalue"] = 1,							["foldtype"] = nil,	["faceup"] = nil										},
		["fanfold140mm"]					= { ["foldvalue"] = 1,							["foldtype"] = nil,	["faceup"] = nil										},
		["fanfold170mm"]					= { ["foldvalue"] = 1,							["foldtype"] = nil,	["faceup"] = nil										},
		["fanfold210mm"]					= { ["foldvalue"] = 1,							["foldtype"] = nil,	["faceup"] = nil										},
		["fanfold297mm"]					= { ["foldvalue"] = 1,							["foldtype"] = nil,	["faceup"] = nil										},
		["fanfold8.5inch"]					= { ["foldvalue"] = 1,							["foldtype"] = nil,	["faceup"] = nil										},
		["fanfold9inch"]					= { ["foldvalue"] = 1,							["foldtype"] = nil,	["faceup"] = nil										},
		["fanfold11inch"]					= { ["foldvalue"] = 1,							["foldtype"] = nil,	["faceup"] = nil										},
		["fanfold12inch"]					= { ["foldvalue"] = 1,							["foldtype"] = nil,	["faceup"] = nil										},
		["fanfoldwithmargin"]				= { ["foldvalue"] = 1,							["foldtype"] = nil,	["faceup"] = nil										},
		["specialfanfold1"]					= { ["foldvalue"] = 1,							["foldtype"] = nil,	["faceup"] = nil										},
		["specialfanfold2"]					= { ["foldvalue"] = 1,							["foldtype"] = nil,	["faceup"] = nil										},
		["specialfold1"]					= { ["foldvalue"] = 1,							["foldtype"] = nil,	["faceup"] = nil										},
		["specialfold2"]					= { ["foldvalue"] = 1,							["foldtype"] = nil,	["faceup"] = nil										},
		["programfold1"]					= { ["foldvalue"] = 1,							["foldtype"] = nil,	["faceup"] = nil										},
		["programfold2"]					= { ["foldvalue"] = 1,							["foldtype"] = nil,	["faceup"] = nil										},
		["programfold3"]					= { ["foldvalue"] = 1,							["foldtype"] = nil,	["faceup"] = nil										},
		["programfold4"]					= { ["foldvalue"] = 1,							["foldtype"] = nil,	["faceup"] = nil										}
	}

	return fold and ( (fold_info_tbl[fold] and fold_info_tbl[fold] or fold_info_tbl["off"])[info] ) or fold_info_tbl["off"][info]

end

-----------------------------------------------------------------------------
-- Local Function : get_fold_details
-----------------------------------------------------------------------------
local function get_fold_details( fold, foldstamp, edgefold )

	local fold_detail_tbl =
	{
		["off"]						= { ["type"] = 10,	["foldtype"] = 9,	["imagerotation"] = nil,	["stack"] = nil,		["unit"] = nil,	["foldwidth"] = nil,	["preset"] = nil,	["foldback"] = nil,								["stamp"] = get_stampvalue( foldstamp )	},
		["rightzfold"]				= { ["type"] = 7,	["foldtype"] = 0,	["imagerotation"] = 1,		["stack"] = nil,		["unit"] = nil,	["foldwidth"] = nil,	["preset"] = nil,	["foldback"] = nil,								["stamp"] = nil							},
		["leftzfold"]				= { ["type"] = 7,	["foldtype"] = 0,	["imagerotation"] = 3,		["stack"] = nil,		["unit"] = nil,	["foldwidth"] = nil,	["preset"] = nil,	["foldback"] = nil,								["stamp"] = nil							},
		["bottomzfold"]				= { ["type"] = 7,	["foldtype"] = 0,	["imagerotation"] = 2,		["stack"] = nil,		["unit"] = nil,	["foldwidth"] = nil,	["preset"] = nil,	["foldback"] = nil,								["stamp"] = nil							},
		["insidefold"]				= { ["type"] = 11,	["foldtype"] = 100,	["imagerotation"] = nil,	["stack"] = "false",	["unit"] = nil,	["foldwidth"] = nil,	["preset"] = nil,	["foldback"] = nil,								["stamp"] = nil							},
		["outsidefold"]				= { ["type"] = 11,	["foldtype"] = 101,	["imagerotation"] = nil,	["stack"] = "false",	["unit"] = nil,	["foldwidth"] = nil,	["preset"] = nil,	["foldback"] = nil,								["stamp"] = nil							},
		["superimposedfoldinside"]	= { ["type"] = 11,	["foldtype"] = 100,	["imagerotation"] = nil,	["stack"] = "true",		["unit"] = nil,	["foldwidth"] = nil,	["preset"] = nil,	["foldback"] = nil,								["stamp"] = nil							},
		["superimposedfoldoutside"]	= { ["type"] = 11,	["foldtype"] = 101,	["imagerotation"] = nil,	["stack"] = "true",		["unit"] = nil,	["foldwidth"] = nil,	["preset"] = nil,	["foldback"] = nil,								["stamp"] = nil							},
		["standardfold"]			= { ["type"] = 10,	["foldtype"] = 3,	["imagerotation"] = nil,	["stack"] = nil,		["unit"] = nil,	["foldwidth"] = nil,	["preset"] = nil,	["foldback"] = nil,								["stamp"] = get_stampvalue( foldstamp )	},
		["narrowstandardfold"]		= { ["type"] = 10,	["foldtype"] = 4,	["imagerotation"] = nil,	["stack"] = nil,		["unit"] = nil,	["foldwidth"] = nil,	["preset"] = nil,	["foldback"] = nil,								["stamp"] = get_stampvalue( foldstamp )	},
		["marginfold"]				= { ["type"] = 10,	["foldtype"] = 5,	["imagerotation"] = nil,	["stack"] = nil,		["unit"] = nil,	["foldwidth"] = nil,	["preset"] = nil,	["foldback"] = get_foldbackvalue( edgefold ),	["stamp"] = get_stampvalue( foldstamp )	},
		["fanfold140mm"]			= { ["type"] = 10,	["foldtype"] = 1,	["imagerotation"] = nil,	["stack"] = nil,		["unit"] = 2,	["foldwidth"] = 140.0,	["preset"] = nil,	["foldback"] = nil,								["stamp"] = get_stampvalue( foldstamp )	},
		["fanfold170mm"]			= { ["type"] = 10,	["foldtype"] = 1,	["imagerotation"] = nil,	["stack"] = nil,		["unit"] = 2,	["foldwidth"] = 170.0,	["preset"] = nil,	["foldback"] = nil,								["stamp"] = get_stampvalue( foldstamp )	},
		["fanfold210mm"]			= { ["type"] = 10,	["foldtype"] = 1,	["imagerotation"] = nil,	["stack"] = nil,		["unit"] = 2,	["foldwidth"] = 210.0,	["preset"] = nil,	["foldback"] = nil,								["stamp"] = get_stampvalue( foldstamp )	},
		["fanfold297mm"]			= { ["type"] = 10,	["foldtype"] = 1,	["imagerotation"] = nil,	["stack"] = nil,		["unit"] = 2,	["foldwidth"] = 297.0,	["preset"] = nil,	["foldback"] = nil,								["stamp"] = get_stampvalue( foldstamp )	},
		["fanfold8.5inch"]			= { ["type"] = 10,	["foldtype"] = 1,	["imagerotation"] = nil,	["stack"] = nil,		["unit"] = 1,	["foldwidth"] = 8.5,	["preset"] = nil,	["foldback"] = nil,								["stamp"] = get_stampvalue( foldstamp )	},
		["fanfold9inch"]			= { ["type"] = 10,	["foldtype"] = 1,	["imagerotation"] = nil,	["stack"] = nil,		["unit"] = 1,	["foldwidth"] = 9.0,	["preset"] = nil,	["foldback"] = nil,								["stamp"] = get_stampvalue( foldstamp )	},
		["fanfold11inch"]			= { ["type"] = 10,	["foldtype"] = 1,	["imagerotation"] = nil,	["stack"] = nil,		["unit"] = 1,	["foldwidth"] = 11.0,	["preset"] = nil,	["foldback"] = nil,								["stamp"] = get_stampvalue( foldstamp )	},
		["fanfold12inch"]			= { ["type"] = 10,	["foldtype"] = 1,	["imagerotation"] = nil,	["stack"] = nil,		["unit"] = 1,	["foldwidth"] = 12.0,	["preset"] = nil,	["foldback"] = nil,								["stamp"] = get_stampvalue( foldstamp )	},
		["fanfoldwithmargin"]		= { ["type"] = 10,	["foldtype"] = 2,	["imagerotation"] = nil,	["stack"] = nil,		["unit"] = nil,	["foldwidth"] = nil,	["preset"] = nil,	["foldback"] = "false",							["stamp"] = get_stampvalue( foldstamp )	},
		["specialfanfold1"]			= { ["type"] = 10,	["foldtype"] = 7,	["imagerotation"] = nil,	["stack"] = nil,		["unit"] = nil,	["foldwidth"] = nil,	["preset"] = 101,	["foldback"] = "false",							["stamp"] = get_stampvalue( foldstamp )	},
		["specialfanfold2"]			= { ["type"] = 10,	["foldtype"] = 7,	["imagerotation"] = nil,	["stack"] = nil,		["unit"] = nil,	["foldwidth"] = nil,	["preset"] = 102,	["foldback"] = "false",							["stamp"] = get_stampvalue( foldstamp )	},
		["specialfold1"]			= { ["type"] = 10,	["foldtype"] = 7,	["imagerotation"] = nil,	["stack"] = nil,		["unit"] = nil,	["foldwidth"] = nil,	["preset"] = 1,		["foldback"] = get_foldbackvalue( edgefold ),	["stamp"] = get_stampvalue( foldstamp )	},
		["specialfold2"]			= { ["type"] = 10,	["foldtype"] = 7,	["imagerotation"] = nil,	["stack"] = nil,		["unit"] = nil,	["foldwidth"] = nil,	["preset"] = 2,		["foldback"] = get_foldbackvalue( edgefold ),	["stamp"] = get_stampvalue( foldstamp )	},
		["programfold1"]			= { ["type"] = 9,	["foldtype"] = 8,	["imagerotation"] = nil,	["stack"] = nil,		["unit"] = nil,	["foldwidth"] = nil,	["preset"] = 1,		["foldback"] = get_foldbackvalue( edgefold ),	["stamp"] = get_stampvalue( foldstamp )	},
		["programfold2"]			= { ["type"] = 9,	["foldtype"] = 8,	["imagerotation"] = nil,	["stack"] = nil,		["unit"] = nil,	["foldwidth"] = nil,	["preset"] = 2,		["foldback"] = get_foldbackvalue( edgefold ),	["stamp"] = get_stampvalue( foldstamp )	},
		["programfold3"]			= { ["type"] = 9,	["foldtype"] = 8,	["imagerotation"] = nil,	["stack"] = nil,		["unit"] = nil,	["foldwidth"] = nil,	["preset"] = 3,		["foldback"] = get_foldbackvalue( edgefold ),	["stamp"] = get_stampvalue( foldstamp )	},
		["programfold4"]			= { ["type"] = 9,	["foldtype"] = 8,	["imagerotation"] = nil,	["stack"] = nil,		["unit"] = nil,	["foldwidth"] = nil,	["preset"] = 4,		["foldback"] = get_foldbackvalue( edgefold ),	["stamp"] = get_stampvalue( foldstamp )	}
	}

	if( fold_detail_tbl[fold] ) then
		local imagerotation_cmd	= ( fold_detail_tbl[fold]["imagerotation"] )	and ( "/ImageRotation "	.. fold_detail_tbl[fold]["imagerotation"]						.. "\r\n" ) or ""
		local stack_cmd			= ( fold_detail_tbl[fold]["stack"] )			and ( "/Stack "			.. fold_detail_tbl[fold]["stack"]								.. "\r\n" ) or ""
		local unit_cmd			= ( fold_detail_tbl[fold]["unit"] )				and ( "/Unit "			.. fold_detail_tbl[fold]["unit"]								.. "\r\n" ) or ""
		local foldwidth_cmd		= ( fold_detail_tbl[fold]["foldwidth"] )		and ( "/FoldWidth "		.. string.format( "%.1f", fold_detail_tbl[fold]["foldwidth"] )	.. "\r\n" ) or ""
		local preset_cmd		= ( fold_detail_tbl[fold]["preset"] )			and ( "/Preset "		.. fold_detail_tbl[fold]["preset"]								.. "\r\n" ) or ""
		local foldback_cmd		= ( fold_detail_tbl[fold]["foldback"] )			and ( "/FoldBack "		.. fold_detail_tbl[fold]["foldback"]							.. "\r\n" ) or ""
		local stamp_cmd			= ( fold_detail_tbl[fold]["stamp"] )			and ( "/Stamp "			.. fold_detail_tbl[fold]["stamp"]								.. "\r\n" ) or ""

		return "1\r\n/FoldDetails <</Type " .. fold_detail_tbl[fold]["type"] .. "\r\n/FoldType " .. fold_detail_tbl[fold]["foldtype"] .. "\r\n" .. imagerotation_cmd .. stack_cmd .. unit_cmd .. foldwidth_cmd .. preset_cmd .. foldback_cmd .. stamp_cmd .. ">> "
	else
		return "0"
	end

end

-----------------------------------------------------------------------------
-- Local Function : get_fold
-----------------------------------------------------------------------------
local function get_fold( fold, foldstamp, edgefold )

	return "/Fold " .. ( 1 == get_fold_info( fold, "foldvalue", nil , nil, foldstamp ) and get_fold_details( fold, foldstamp, edgefold ) or "0")
	
end

-----------------------------------------------------------------------------
-- Local Function : output_charset
-----------------------------------------------------------------------------
local function output_charset( info_map, gb18030, utf8mode, output_utf8_rtl_language )

	local cmd_hostCharSet = "/HostCharSet " .. get_charsetvalue( gb18030, utf8mode, output_utf8_rtl_language ) .. "\r\n"
	local cmd_drivertype = ( info_map["geneinneritem_driver"] == "generic" ) and "/DriverType 7 \r\n" or "/DriverType 6 \r\n"
	
	local cmd_header = "mark {\r\n<<\r\n"
	local cmd_footer = ">>\r\n/RDeviceProcSet\r\n/ProcSet findresource\r\n/SetHostInfo get exec\r\n} stopped cleartomark\r\n"

	return cmd_header .. cmd_hostCharSet .. cmd_drivertype .. cmd_footer
	
end

-----------------------------------------------------------------------------
-- Local Function : to_oct
-----------------------------------------------------------------------------
local function to_oct( value )
	if( nil == value or "string" ~= type( value ) ) then
		return ''
	end
	local tbl = {}
	for pos = 1, string.len( value ) do
		tbl[#tbl + 1] = string.format( '\\%o', string.byte( value,pos ) )
	end
	return table.concat( tbl )
end

-----------------------------------------------------------------------------
-- Local Function : output_jobtype_defs
-----------------------------------------------------------------------------
local function output_jobtype_defs()
	return "/isEnhancedFeatures\r\n{userdict /86f6369627 known { userdict /86f6369627 get /EnhancedFeatures known} if } bind def\r\n/isGenuine{ \r\n2 dict begin\r\n/isGenuineFlag //false def\r\n//systemdict /execdict known  {\r\n//systemdict /execdict get /copyright known {\r\n/copyright //systemdict /execdict get /copyright get def\r\n{ copyright ( ) anchorsearch {\r\npop /copyright exch def\r\n}{\r\npop\r\nexit\r\n} ifelse\r\n} loop\r\ncopyright ((c) Copyright 1984-) anchorsearch {\r\npop pop \r\ncopyright ( Adobe Systems Incorporated) search {\r\npop pop pop \r\ncopyright (All Rights Reserved.) search {\r\npop pop pop\r\n/isGenuineFlag //true def\r\n}{\r\npop\r\n} ifelse\r\n}{\r\npop\r\n} ifelse\r\n}{\r\npop\r\n} ifelse\r\n} if\r\n} if\r\nisGenuineFlag\r\nend\r\n}bind def\r\n"
end

-----------------------------------------------------------------------------
-- Local Function : output_jobtype
-----------------------------------------------------------------------------
local function output_jobtype(	info_map,
								jobtype,
								userid, usercode, username,
								filename, password,
								documentserver_filename, documentserver_password,
								authentication,
								loginusername, loginpassword,
								platform,
								scheduledprint,
								scheduledprint_hour,
								scheduledprint_minute,
								documentserver_foldernumber,
								documentserver_folderpassword,
								gb18030,
								userid_type,userid_loginname,
								utf8mode,
								output_utf8_rtl_language
								)

	if( nil == jobtype ) then
		return ""
	end

	local cmd_header = "mark {\r\n<< /JobType "
	local cmd_footer = ">>\r\n>> /RDeviceProcSet /ProcSet findresource /SetJobType get exec\r\n} stopped cleartomark\r\n"
	
	local cmd_enhancedFeatureHeader = "isGenuine not isEnhancedFeatures or {\r\n"
	local cmd_enhancedFeatureHooter = "}if\r\n"

	local fixed_filename = ( "documentserver" == string.gsub( jobtype, "^documentserver.*", "documentserver" )) and ((nil == documentserver_filename) and "" or documentserver_filename) or ((nil == filename) and "" or filename)	
	local fixed_password = ( "documentserver" == string.gsub( jobtype, "^documentserver.*", "documentserver" )) and ((nil == documentserver_password) and "" or documentserver_password) or ((nil == password) and "" or password)

	fixed_password = ("on" == authentication and "2" == platform) and ((nil == loginpassword) and "" or loginpassword) or fixed_password
	
	local charcode = get_charsetvalue( gb18030, utf8mode, output_utf8_rtl_language ) == 33 and "utf8" or "osdependent"

	local cmd_userid			= "/UserID (" .. ( (nil == userid or "" == userid ) and "" or userid ) .. ")\r\n"
	local cmd_time				= "/Time (" .. os.date("%Y") .. os.date("%m") .. os.date("%d") .. os.date("%H") .. os.date("%M") .. ")\r\n"
	local cmd_usercode			= ("2" ~= platform) and ("/UserCode (" .. ((nil == usercode) and "" or usercode) .. ")\r\n") or ""
	local cmd_password			= "/Password (" .. (("2" == platform) and to_oct( Rice.Encrypt.encode("C", fixed_password ) ) or fixed_password ) .. ")\r\n"
	local cmd_filename			= "/JobID (" .. to_oct( get_trimmed_command_string( fixed_filename, charcode, 20) ) .. ")\r\n"
	local cmd_scheduledprint	= ( "on" == scheduledprint and 0 <= tonumber(scheduledprint_hour) and tonumber(scheduledprint_hour) <= 23 and 0 <= tonumber(scheduledprint_minute) and tonumber(scheduledprint_minute) <= 59 ) and ( "/PrintReservedTime(" .. string.format("%02d%02d",scheduledprint_hour , scheduledprint_minute )  .. ")" ) or ""
	local cmd_loginusername		= ("2" == platform) and ("/AuthenticationUserName (" .. ((nil ~= loginusername and "on" == authentication) and loginusername or "") .. ")\r\n") or ""
	local cmd_charset			= ("2" == platform) and ("/AuthenticationUserNameCharset " .. get_charsetvalue( gb18030, utf8mode, output_utf8_rtl_language ) .. "\r\n") or ""
	local cmd_encryption		= ("2" == platform) and "/EncryptionSetting 2\r\n/EncryptionMethod 0\r\n" or ""
	local cmd_foldernumber		= "/FolderNumber " .. ( ( nil == documentserver_foldernumber or "" == documentserver_foldernumber ) and "0" or documentserver_foldernumber ) .. "\r\n"
	local cmd_folderpassword	= "/FolderPassword (" .. ( ( nil == documentserver_folderpassword or "" == documentserver_folderpassword ) and "" or documentserver_folderpassword ) .. ")\r\n"
	local cmd_hostLoginName		= "/HostLoginName(" .. to_oct( get_trimmed_command_string( ( info_map and (info_map["windowsLogonUserName"] and info_map["windowsLogonUserName"] or "") or "" ), charcode, 20  ) ) .. ")\r\n"
	local cmd_hostName			= "/HostName(" .. to_oct( get_trimmed_command_string( ( info_map and (info_map["hostMachineName"] and info_map["hostMachineName"] or "") or "" ), charcode, 63  ) ) .. ")\r\n"
	local cmd_jobname			= "/JobName(" .. to_oct( get_trimmed_command_string( ( info_map and (info_map["jobName"] and info_map["jobName"] or "") or "" ), charcode, 232  ) ) .. ")\r\n"
	local cmd_useridtype		= (nil == userid_loginname) and "" or "/DocPrintMenuUser " .. ( ( "windowsloginname" == userid_type ) and "1" or "0")  .. "\r\n"

	local cmd_jobtype =
	{
		["normalprint"] 		= cmd_header .. "0" .. "\r\n/JobInfo <<\r\n" .. cmd_userid .. cmd_usercode .. cmd_time .. cmd_loginusername.. cmd_charset .. cmd_enhancedFeatureHeader .. cmd_jobname .. cmd_hostLoginName .. cmd_hostName .. cmd_useridtype .. cmd_enhancedFeatureHooter .. cmd_footer,
		["sampleprint"] 		= cmd_header .. "2" .. "\r\n/JobInfo <<\r\n" .. cmd_userid .. cmd_usercode .. cmd_time .. cmd_enhancedFeatureHeader .. cmd_jobname .. cmd_hostLoginName .. cmd_hostName .. cmd_useridtype .. cmd_enhancedFeatureHooter .. cmd_footer,
		["lockedprint"] 		= cmd_header .. "1" .. "\r\n/JobInfo <<\r\n" .. cmd_userid .. cmd_usercode .. cmd_time .. cmd_password .. cmd_encryption .. cmd_loginusername .. cmd_charset .. cmd_enhancedFeatureHeader .. cmd_jobname .. cmd_hostLoginName .. cmd_hostName .. cmd_useridtype .. cmd_enhancedFeatureHooter .. cmd_footer,
		["holdprint"] 			= cmd_header .. "3" .. "\r\n/JobInfo <<\r\n" .. cmd_userid .. cmd_usercode .. cmd_time .. cmd_filename .. cmd_enhancedFeatureHeader .. cmd_scheduledprint .. cmd_jobname .. cmd_hostLoginName .. cmd_hostName .. cmd_useridtype .. cmd_enhancedFeatureHooter .. cmd_loginusername .. cmd_charset .. ">>\r\n/JobAttr << /Save 0\r\n" .. cmd_footer,
		["storedprint"] 		= cmd_header .. "3" .. "\r\n/JobInfo <<\r\n" .. cmd_userid .. cmd_usercode .. cmd_time .. cmd_password .. cmd_filename .. cmd_encryption.. cmd_loginusername .. cmd_charset .. cmd_enhancedFeatureHeader .. cmd_jobname .. cmd_hostLoginName .. cmd_hostName .. cmd_useridtype .. cmd_enhancedFeatureHooter .. ">>\r\n/JobAttr << /Save 1 /Public false\r\n" .. cmd_footer,
		["storeandprint"]		= cmd_header .. "3" .. "\r\n/JobInfo <<\r\n" .. cmd_userid .. cmd_usercode .. cmd_time .. cmd_password .. cmd_filename .. cmd_encryption.. cmd_loginusername .. cmd_charset .. cmd_enhancedFeatureHeader .. cmd_jobname .. cmd_hostLoginName .. cmd_hostName .. cmd_useridtype .. cmd_enhancedFeatureHooter .. ">>\r\n/JobAttr << /Save 2 /Public false\r\n" .. cmd_footer,
		["documentserver"] 		= cmd_header .. "4" .. "\r\n/JobInfo <<\r\n" .. cmd_userid .. cmd_usercode .. cmd_time .. cmd_password .. cmd_filename .. cmd_encryption.. cmd_loginusername .. cmd_charset .. cmd_enhancedFeatureHeader .. cmd_jobname .. cmd_hostLoginName .. cmd_hostName .. cmd_useridtype .. cmd_enhancedFeatureHooter .. ">>\r\n/JobAttr <<\r\n" .. cmd_enhancedFeatureHeader .. "/dbox <<\r\n" .. cmd_foldernumber .. cmd_folderpassword .. ">>\r\n" .. cmd_enhancedFeatureHooter .. cmd_footer,
		["documentserverrgb"]	= cmd_header .. "4" .. "\r\n/JobInfo <<\r\n" .. cmd_userid .. cmd_usercode .. cmd_time .. cmd_password .. cmd_filename .. cmd_encryption.. cmd_loginusername .. cmd_charset .. cmd_enhancedFeatureHeader .. cmd_jobname .. cmd_hostLoginName .. cmd_hostName .. cmd_useridtype .. cmd_enhancedFeatureHooter .. ">>\r\n/JobAttr << /Reusable true\r\n" .. cmd_footer
	}

	return cmd_jobtype[ jobtype ]

end

-----------------------------------------------------------------------------
-- Local Function : output_bannerpageprint
-----------------------------------------------------------------------------
local function output_bannerpageprint(	info_map,
										bannerpageprint,
										inputtray,
										mediatype,
										lctmediaposition,
										traymediaposition_type,
										gb18030,
										utf8mode,
										output_utf8_rtl_language
										)

	local cmd_bannerPageMode = "/BannerPageMode " .. ("on" == bannerpageprint  and "true" or "false" ) .. "\r\n"
	
	local mediaPosition = get_inputtray(inputtray, lctmediaposition, traymediaposition_type)
	local cmd_mediaPosition = "/MediaPosition " .. ( (nil == mediaPosition) and "null" or mediaPosition) .. "\r\n"

	local mediatype_info
	if( nil ~= string.find( tostring( mediatype ), '+custom' ) ) then
		local default_mediatype = string.gsub( mediatype, "+custom", "+Custom " )
		mediatype_info = info_map and ( info_map["bannerpageprint_mediatype_name"] and info_map["bannerpageprint_mediatype_name"] or default_mediatype ) or default_mediatype
		
		if(	nil ~= info_map								and
			"raw" == info_map["mediatype_stringtype"]		) then
			mediatype_info = to_oct( mediatype_info )
		else
			local charcode = get_charsetvalue( gb18030, utf8mode, output_utf8_rtl_language ) == 33 and "utf8" or "osdependent"
			mediatype_info = to_oct( get_trimmed_command_string( mediatype_info, charcode, 65535 ) )
		end
	else
		mediatype_info = get_mediatype( mediatype )
	end
	local cmd_mediaType		= "/MediaType " .. (("none" ~= mediatype) and ("(" .. mediatype_info .. ")") or mediatype_info) .. "\r\n"

	local cmd_header = "<<\r\n"
	local cmd_footer = ">>\r\n/RDeviceProcSet\r\n/ProcSet findresource\r\n/SetBannerPage get exec\r\n"

	local command = cmd_header .. cmd_bannerPageMode .. cmd_mediaPosition .. cmd_mediaType .. cmd_footer

	return add_feature_tags("bannerpageprint" , bannerpageprint , command)

end

-----------------------------------------------------------------------------
-- Local Function : output_edgesmoothing
-----------------------------------------------------------------------------
local function output_edgesmoothing( edgesmoothing, tonersaving, economycolormode )

	if( ( "on"  == economycolormode ) or ( "on" == tonersaving ) or ( "tonersave1" == tonersaving ) or ( "tonersave2" == tonersaving ) or ( "levelcolor" == tonersaving ) or
		( ( "off"  == edgesmoothing ) and ( ( "off" == tonersaving ) or ( "off" == economycolormode ) ) )or
		( nil == edgesmoothing )		) then
		return ""
	end
	
	return add_feature_tags("edgesmoothing" , edgesmoothing , "<< /PostRenderingEnhance" .. (("on" == edgesmoothing) and (" true\r\n/PostRenderingEnhanceDetails << /Type 34 /OutputMode 0 >> >>\r\n") or (" false>>\r\n") ) .. "setpagedevice")
end

-----------------------------------------------------------------------------
-- Local Function : output_documentsize
-----------------------------------------------------------------------------
local function output_documentsize( info_map, documentsize , printsize, synchrocut, synchrocut_printsize, layout, booklet, reduceenlarge, booklet_equalmagnification, pagelayout_distribution )

	if(	nil == documentsize		or "" == documentsize	or
		nil == printsize		or "" == printsize		) then
		return ""
	end
	
	--If PrintOn value is different from PageSize value, use PrintOn value
	if("sameasoriginalsize" == printsize  or	
		true == is_printer_pagelayout( layout, booklet, pagelayout_distribution, booklet_equalmagnification, reduceenlarge) ) then
		printsize = documentsize
	end

	local printsize_tbl = get_printsize(info_map, documentsize, printsize, synchrocut, synchrocut_printsize)

	if( is_custom_paper_size( printsize ) ) then

		local custom_command = "pop pop pop\r\n<< /PageSize [ 5 -2 roll ]  /ImagingBBox null\r\n/Policies <</PageSize 2 /MediaType 2>>\r\n/DeferredMediaSelection true\r\n>> setpagedevice"

		return add_feature_tags("documentsize" , printsize , printsize_tbl['width'] ..  " " ..  printsize_tbl['height']  .. " 0 0 1 " .. custom_command)
	else
		return add_feature_tags("documentsize" , printsize , "<< /PageSize [" .. printsize_tbl['width'] .. " " .. printsize_tbl['height'] .. "] /ImagingBBox null >> setpagedevice")
	end
end

-----------------------------------------------------------------------------
-- Local Function : output_edgetoedgeprint
-----------------------------------------------------------------------------
local function output_edgetoedgeprint( edgetoedgeprint )

	if(	nil == edgetoedgeprint	or "" == edgetoedgeprint ) then
		return ""
	end

	return add_feature_tags("edgetoedgeprint" , edgetoedgeprint , (("on" == edgetoedgeprint) and "true" or "false") .. " setedgetoedge")

end

-----------------------------------------------------------------------------
-- Local Function : output_standardpages_inputtray
-----------------------------------------------------------------------------
local function output_standardpages_inputtray( standardpages_inputtray, transparency_inputtray, slipsheet, lctmediaposition, traymediaposition_type )

	local inputtray

	if ( "on" == slipsheet ) then
		if ( nil == transparency_inputtray or "" == transparency_inputtray ) then
			return ""
		end
		
		inputtray = transparency_inputtray
		
	else
	if( nil == standardpages_inputtray or "" == standardpages_inputtray ) then
		return ""
	end

		inputtray = standardpages_inputtray
	end

	local mediaPosition = get_inputtray(inputtray, lctmediaposition, traymediaposition_type )
	return ( ( nil == mediaPosition ) and "" or add_feature_tags( "standardpages_inputtray" , inputtray , "<< /MediaPosition " .. mediaPosition .. " >> setpagedevice" ) )
end

-----------------------------------------------------------------------------
-- Local Function : output_setcoversheet
-----------------------------------------------------------------------------
local function output_setcoversheet( frontcover, frontcover_inputtray, frontcover_printmode, backcover, backcover_inputtray, backcover_printmode,
									 staple, centerstaple_swapcover, lctmediaposition, traymediaposition_type )
	
	if( ( nil == frontcover or "" == frontcover ) and ( nil == backcover or "" == backcover ) ) then
		return ""
	end

	if( "off" == frontcover and "off" == backcover) then
		return ""
	end
	
	local t_front =
	{
		["leaveblank"]			=	0,
		["oneside"]				=	1,
		["bothsides"]			=	2
	}
	
	local t_front_with_booklet1 =
	{
		["leaveblank"]			=	0,
		["oneside"]				=	1,
		["bothsides"]			=	1
	}
	
	local t_front_with_booklet2 =
	{
		["leaveblank"]			=	0,
		["oneside"]				=	2,
		["bothsides"]			=	2
	}
	
	local t_back =
	{
		["leaveblank"]			=	0,
		["printonbackcover"]	=	1,
		["printonouter"]		=	2
	}

	local front_printmode	= t_front[frontcover_printmode]
	local back_printmode	= ""
	local cmd_front			= ""
	local cmd_back			= ""
	local inputtray_front	= get_inputtray( frontcover_inputtray, lctmediaposition, traymediaposition_type )
	local inputtray_back	= get_inputtray( backcover_inputtray, lctmediaposition, traymediaposition_type )
	
	if (is_frontcover_with_booklet( frontcover, staple, centerstaple_swapcover )) then
		if( "1" == centerstaple_swapcover or "2" == centerstaple_swapcover ) then
			back_printmode = t_front_with_booklet2[frontcover_printmode]
		elseif ( "3" == centerstaple_swapcover )  then
			back_printmode = t_front_with_booklet1[frontcover_printmode]
		end
		
		if( nil == back_printmode or nil == inputtray_front ) then
			return ""
		end
		
		cmd_back		= "/Back [" .. back_printmode .. " " .. inputtray_front .. "]\r\n"
	else
		back_printmode	= t_back[backcover_printmode]
		
		if( nil == front_printmode or nil == inputtray_front or nil == back_printmode or nil == inputtray_back) then
			return ""
		end
		
		cmd_front		= "/Front [" .. front_printmode .. " " .. inputtray_front .. "]\r\n"
		cmd_back		= "/Back [" .. back_printmode .. " " .. inputtray_back .. "]\r\n"
	end

	local cmd_header = "<<\r\n"
	local cmd_footer = ">>\r\n/RDeviceProcSet /ProcSet findresource /SetCoverSheet get exec\r\n"
	local command = ""
		
	if (is_frontcover_with_booklet( frontcover, staple, centerstaple_swapcover )) then
		command = cmd_header .. cmd_back .. cmd_footer
	else
		if( "on" == frontcover and "on" == backcover) then
			command = cmd_header .. cmd_front .. cmd_back .. cmd_footer
		elseif( "on" == frontcover and "off" == backcover) then
			command = cmd_header .. cmd_front .. cmd_footer
		elseif( "on" == frontcover and nil == backcover) then
			command = cmd_header .. cmd_front .. cmd_footer
		elseif( "off" == frontcover and "on" == backcover) then
			command = cmd_header .. cmd_back .. cmd_footer
		elseif( nil == frontcover and "on" == backcover) then
			command = cmd_header .. cmd_back .. cmd_footer
		end
	end
	
	return add_feature_tags("frontcover_inputtray" , frontcover_inputtray , command)
end

-----------------------------------------------------------------------------
-- Local Function : output_setslipsheet
-----------------------------------------------------------------------------
local function output_setslipsheet( slipsheet, slipsheet_inputtray, slipsheet_printmode, lctmediaposition, traymediaposition_type )

	if( nil == slipsheet or "" == slipsheet or "off" == slipsheet) then
		return ""
	end
	
	local tbl =
	{
		["leaveblank"]			=	"false",
		["printonslipsheet"]	=	"true"
	}
	
	if( tbl[slipsheet_printmode] == nil ) then
		return ""
	end
	
	local inputtray = get_inputtray( slipsheet_inputtray, lctmediaposition, traymediaposition_type )
	
	if( inputtray == nil ) then
		return ""
	end
	
	local command = "true ".. tbl[slipsheet_printmode] .. " " .. inputtray ..  " setslipsheet"
	
	return add_feature_tags("slipsheet_inputtray" , slipsheet_inputtray , command)

end

-----------------------------------------------------------------------------
-- Local Function : output_setchaptering
-----------------------------------------------------------------------------
local function output_setchaptering( chapterpages, chapterpages_setting, lctmediaposition, traymediaposition_type )

	if( nil == chapterpages or "" == chapterpages or "off" == chapterpages or nil == chapterpages_setting or "" == chapterpages_setting ) then
		return ""
	end

	local t_mode =
	{
		["leaveblank"] = "10",
		["oneside"] = "11",
		["bothsides"] = "11",
		["leadchapterpage"] = "0"
	}
	
	local t_dup =
	{
		["leaveblank"] = "false",
		["oneside"] = "false",
		["bothsides"] = "true"
	}

	local settings = CDBTableItem.decode_table( chapterpages_setting )
	local cmd_txt =""
	
	for index, settings_values in ipairs(settings) do
		local cmd_temp = ""
		if( 3 >= #settings_values ) then
	
			local pagenumberstring	= settings_values[1]
			local printmode		= settings_values[2]
			local printtray		= settings_values[3]
			
			if ( is_member( printmode, { 'leaveblank', 'oneside', 'bothsides', 'leadchapterpage' } ) ) then
				if( tonumber( pagenumberstring ) ) then
					if ("leadchapterpage" == printmode) then
						cmd_temp = "/" .. pagenumberstring .. " [" .. t_mode[printmode] .. "]"
					else
						local inputtray = get_inputtray(printtray, lctmediaposition, traymediaposition_type)
						local mode = t_mode[printmode]
						local dup = t_dup[printmode]
						
						if( ( nil ~= inputtray ) and ( nil ~= mode ) and ( nil ~= dup ) ) then
							cmd_temp = "/" .. pagenumberstring .. " [" .. mode .. " " .. inputtray .. " " .. dup .. " 1 index currentpagedevice /InputAttributes get exch get /MediaType get]"
						end
					end
				end
			end
		end
		if ( "" ~= cmd_temp ) then
			if( "" == cmd_txt ) then
				cmd_txt = cmd_temp
			else
				cmd_txt = cmd_txt .. "\r\n" .. cmd_temp
			end
		end
	end

	if( "" == cmd_txt ) then
		return ""
	end
	
	--wb_[tb^[
	local cmd_header = "<</PageInfo\r\n"
	local cmd_footer = "\r\n>> /RDeviceProcSet /ProcSet findresource /SetChaptering get exec"

	--eLXg`
	local cmd_str = "<<" .. cmd_txt .. ">>"
	local command = cmd_header .. cmd_str .. cmd_footer
	
	return add_feature_tags( "chapterpages_setting" , chapterpages_setting , command )

end

-----------------------------------------------------------------------------
-- Local Function : output_outputtray
-----------------------------------------------------------------------------
local function output_outputtray( value )

	if( nil == value or "" == value) then
		return ""
	end

	local tbl =
	{
		["printerdefault"]			=	"null",
		["standardtray"]			=	"(Standard)",
		["copytray"]				=	"(Standard)",
		["internalshifttray"]		=	"(Shift)",
		["shifttray"]				=	"(Shift)",
		["internaltray"]			=	"(Standard)",
		["internaltray2"]			=	"(1Bin)",
		["finisheruppertray"]		=	"(FinProof)",
		["finishershifttray"]		=	"(FinShift)",
		["finishershifttray1"]		=	"(FinProof)",
		["finishershifttray2"]		=	"(FinShift)",
		["finishershifttraysecond"]	=	"(FinShift2)",
		["bookletprocessortray"]	=	"(FinShift)",
		["bookletprocessortray2"]	=	"(FinBooklet)",
		["finisherbooklettray"]		=	"(FinBooklet)",
		["trimmertray"]				=	"(FinBooklet)",
		["lefttray"]				=	"(External)",
		["lefttray1"]				=	"(FinMulti1)",
		["lefttray2"]				=	"(FinMulti2)",
		["righttray"]				=	"(External)",
		["mailboxprooftray"]		=	"(MailProof)",
		["mailboxtray1"]			=	"(MailBin1)",
		["mailboxtray2"]			=	"(MailBin2)",
		["mailboxtray3"]			=	"(MailBin3)",
		["mailboxtray4"]			=	"(MailBin4)",
		["mailboxtray5"]			=	"(MailBin5)",
		["mailboxtray6"]			=	"(MailBin6)",
		["mailboxtray7"]			=	"(MailBin7)",
		["mailboxtray8"]			=	"(MailBin8)",
		["mailboxtray9"]			=	"(MailBin9)",
		["onesheetoutputtray"]		=	"(Standard2)",
		["loadingtray"]				=	"(SubFinPerf)",
		["ringstacker"]				=	"(SubFinRing)",
		["stackerprooftray"]		=	"(StackerProof)",
		["stackerprooftray2"]		=	"(Stacker2Proof)",
		["largestackertray"]		=	"(Stacker)",
		["largestackertray2"]		=	"(Stacker2)",
		["longstackertray"]			=	"(FinShift)",
		["longstackertray2"]		=	"(FinShift2)",
		["foldprooftray"]			=	"(FoldProof)",
		["upperpaperexit"]			=	"(Standard)",
		["frontpaperexit"]			=	"(Standard)",
		["lowerpaperexit"]			=	"(External)",
		["rearpaperexit"]			=	"(External)",
		["peripheralsifbox"]		=	"(FinExtFunc)",
		["twinloopbindtray"]		=	"(SubFinTwinLoop)"
	}

	return add_feature_tags("outputtray" , value , "<</OutputType " .. ((nil == tbl[value]) and "null" or tbl[value] ) .. " >>setpagedevice" )

end

-----------------------------------------------------------------------------
-- Local Function : output_orientationoverride
-----------------------------------------------------------------------------
local function output_orientationoverride( orientationoverride, orientation, layout, booklet, pagelayout_distribution )

	local function isPrinterLayout( layout, pagelayout_distribution )
		local result = false
		if(	"printer" == pagelayout_distribution	and
			is_layout_nup( layout )					) then
			result = true
		end
		
		return result
	end

	if( isPrinterLayout( layout, pagelayout_distribution )	or
		is_layout_booklet( layout, booklet ) 				) then
		return add_feature_tags("orientationoverride" , orientation , (("landscape" == orientation) and "1" or  "0" ) .. " /RDeviceProcSet /ProcSet findresource /SetOrientationOverride get exec")
	end

	if( "portrait" ~= orientationoverride and "landscape" ~= orientationoverride  ) then
		return ""
	end

	return add_feature_tags("orientationoverride" , orientationoverride , (("landscape" == orientationoverride) and "1" or  "0" ) .. " /RDeviceProcSet /ProcSet findresource /SetOrientationOverride get exec")
end

-----------------------------------------------------------------------------
-- Local Function : output_duplex
-----------------------------------------------------------------------------
local function output_duplex( duplex, booklet, orientation, layout )
	
	local fixed_duplex = ("booklet" == booklet) and "booklet" or duplex

	local duplex_cmd = function( duplex, orientation, layout )
		
		local duplex_ver2 =
		{
			["off"]					= { ["duplex"] = "false",	["tumble"] = "",			["direction"] = "" 		},
			["longedgeleftortop"]	= { ["duplex"] = "true",	["tumble"] = " /Tumble ", 	["direction"] = "false" },
			["shortedgetoporleft"]	= { ["duplex"] = "true",	["tumble"] = " /Tumble ",	["direction"] = "true"	},
			["booklet"]				= { ["duplex"] = "true",	["tumble"] = " /Tumble ",	["direction"] = "true" 	}
		}
		
		local duplex_ver1_layout_1_4_9_16 =
		{
		
			["opentoleft"]	= { ["duplex"] = "true",	["tumble"] = " /Tumble ", 	["direction"] = { ["portrait"] = "false",	["landscape"] = "true"	} },
			["opentotop"]	= { ["duplex"] = "true",	["tumble"] = " /Tumble ",	["direction"] = { ["portrait"] = "true",	["landscape"] = "false"	} }
		}

		local duplex_ver1_layout_2_6 =
		{
			["opentoleft"]	= { ["duplex"] = "true",	["tumble"] = " /Tumble ",	["direction"] = { ["portrait"] = "true",	["landscape"] = "false"	} },
			["opentotop"]	= { ["duplex"] = "true",	["tumble"] = " /Tumble ",	["direction"] = { ["portrait"] = "false",	["landscape"] = "true"	} }
		}

		local tbl 		
		if( is_member( duplex, {"off", "longedgeleftortop", "shortedgetoporleft", "booklet"} ) ) then
			
			tbl = duplex_ver2
			
			return ((nil == tbl[duplex]) and "false" or tbl[duplex]["duplex"]) .. ((nil == tbl[duplex]) and "" or tbl[duplex]["tumble"]) .. ((nil == tbl[duplex] or nil == tbl[duplex]["direction"]) and "" or (tbl[duplex]["direction"]))			
		elseif( is_member( duplex, {"opentoleft", "opentotop"} ) and is_member( orientation, {"portrait", "landscape"} ) ) then
				
			tbl = ("2pagespersheet" ~= layout and "6pagespersheet" ~= layout ) and duplex_ver1_layout_1_4_9_16 or duplex_ver1_layout_2_6
			
			return ((nil == tbl[duplex]) and "false" or tbl[duplex]["duplex"]) .. ((nil == tbl[duplex]) and "" or tbl[duplex]["tumble"]) .. ((nil == tbl[duplex] or nil == tbl[duplex]["direction"][orientation]) and "" or (tbl[duplex]["direction"][orientation]))
		
		else
			return nil
		end

	end
	
	local option_cmd = function( duplex, booklet )
	
		if( "booklet" == booklet ) then
			if( is_member( duplex, {"longedgeleftortop", "shortedgetoporleft" ,"opentoleft", "opentotop"} ) ) then

			
				return duplex .. "-" .. booklet
			else
				return "off-" .. booklet
			end
		else
			if( is_member( duplex, {"off","longedgeleftortop","shortedgetoporleft","opentoleft","opentotop"} ) ) then

				
				return duplex .. "-off"
			else
				return nil
			end
		end
	end
	

	return ( (nil == duplex_cmd( fixed_duplex, orientation, layout )) or (nil == option_cmd( duplex, booklet )) ) and "" or add_feature_tags("duplex" , option_cmd( duplex, booklet ) , "<< /Duplex " .. duplex_cmd( fixed_duplex, orientation, layout ) .. " >> setpagedevice")

end

-----------------------------------------------------------------------------
-- Local Function : output_resolution
-----------------------------------------------------------------------------
local function output_resolution( resolution )

	if( nil == resolution or "" == resolution) then
		return ""
	end

	local cmd = function( value )
		local resolutionValue = ( nil == get_resolution_info( resolution, "resolutionvalue" ) ) and 600 or get_resolution_info( resolution, "resolutionvalue" );
		local bitparpixelValue = ( nil == get_resolution_info( resolution, "bitparpixelvalue" ) ) and 1 or get_resolution_info( resolution, "bitparpixelvalue" );
		return "[" .. resolutionValue .. " " .. resolutionValue .. "] >> setpagedevice\r\n" .. bitparpixelValue
	end

	return add_feature_tags("resolution" , resolution , "<< /HWResolution " .. cmd( resolution ) .. " setbitsperpixel\r\n" )
end

-----------------------------------------------------------------------------
-- Local Function : output_gradation
-----------------------------------------------------------------------------
local function output_gradation( value )

	if( nil == value or "" == value) then
		return ""
	end

	local tbl =
	{
		['fast']		= 1,
		['standard']	= 2,
		['fine']		= 4
	}

	return add_feature_tags("gradation" , value , ((nil == tbl[value]) and "2" or tbl[value]) .. " setbitsperpixel")

end

-----------------------------------------------------------------------------
-- Local Function : output_tonersaving
-----------------------------------------------------------------------------
local function output_tonersaving( tonersaving, edgesmoothing, economycolormode )

	if( ( "on" == economycolormode )							or
		( nil == tonersaving )									or
		( "off" == tonersaving and "off" == economycolormode  ) or
		( "off" == tonersaving and "on" == edgesmoothing  )	) 	then
		return ""
	end

	local tbl =
	{
		['off'] 		= "false\r\n",
		['on'] 			= "true\r\n/PostRenderingEnhanceDetails\r\n<< /Type 34\r\n/OutputMode 3\r\n>>\r\n",
		['tonersave1'] 	= "true\r\n/PostRenderingEnhanceDetails\r\n<< /Type 34\r\n/OutputMode 3\r\n>>\r\n",
		['tonersave2'] 	= "true\r\n/PostRenderingEnhanceDetails\r\n<< /Type 34\r\n/OutputMode 4\r\n>>\r\n",
		['levelcolor'] 	= "T.B.D.",
	}

	return add_feature_tags("tonersaving" , tonersaving , "<<\r\n/PostRenderingEnhance " .. ((nil == tbl[tonersaving]) and "false\r\n" or tbl[tonersaving]) .. ">> setpagedevice" )
end

-----------------------------------------------------------------------------
-- Local Function : output_economycolormode
-----------------------------------------------------------------------------
local function output_economycolormode( economycolormode, tonersaving, edgesmoothing )

	if( nil == economycolormode																								or
		( "off" == economycolormode and ( is_member( tonersaving, {"on", "tonersave1", "tonersave2", "levelcolor"} ) ) )	or
		( "off" == economycolormode and "on" == edgesmoothing )																) then
		return ""
	end

	local tbl =
	{
		['off'] 	= "false\r\n",
		['on'] 		= "true\r\n/PostRenderingEnhanceDetails\r\n<< /Type 34\r\n/OutputMode 3\r\n>>\r\n",
	}

	return add_feature_tags("economycolormode" , economycolormode , "<<\r\n/PostRenderingEnhance " .. ((nil == tbl[economycolormode]) and tbl['off'] or tbl[economycolormode]) .. ">> setpagedevice" )
end

-----------------------------------------------------------------------------
-- Local Function : output_colorbw
-----------------------------------------------------------------------------
local function output_colorbw( value )

	if( nil == value or "" == value) then
		return ""
	end

	return add_feature_tags("colorbw" , value , (("color" == value)  and "(cmyk)" or "(gray)" ) .. " RCsetdevicecolor")
end

-----------------------------------------------------------------------------
-- Local Function : output_blackoverprint
-----------------------------------------------------------------------------
local function output_blackoverprint( value )

	if( nil == value or "" == value) then
		return ""
	end

	return add_feature_tags("blackoverprint" , value , (("on"== value)  and "(on)" or "(off)" ) ..  " RCsetblackoverprint\r\n")

end

-----------------------------------------------------------------------------
-- Local Function : output_rotated
-----------------------------------------------------------------------------
local function output_rotated( value )

	if( nil == value or "" == value ) then
		return ""
	end

	return add_feature_tags("rotated" , value , (( "on" == value ) and "1" or  "0") .. " /RDeviceProcSet /ProcSet findresource /SetImageRotation get exec")
end

-----------------------------------------------------------------------------
-- Local Function : output_standardpages_mediatype
-----------------------------------------------------------------------------
local function output_standardpages_mediatype( setting_info, standardpages_mediatype, transparency_mediatype, slipsheet, gb18030, utf8mode, output_utf8_rtl_language )

	local mediatype
	
	if ( "on" == slipsheet ) then
		if( nil == transparency_mediatype or "" == transparency_mediatype) then
			return ""
		end

		mediatype = transparency_mediatype
	else
		if( nil == standardpages_mediatype or "" == standardpages_mediatype) then
		return ""
	end

		mediatype = standardpages_mediatype
	end

	local charcode = get_charsetvalue( gb18030, utf8mode, output_utf8_rtl_language ) == 33 and "utf8" or "osdependent"

	local mediatype_info
	if( nil ~= string.find( tostring( mediatype ), '+custom' ) ) then
		local default_mediatype = string.gsub( mediatype, "+custom", "+Custom " )
		mediatype_info = setting_info and ( setting_info["standardpages_mediatype_name"] and setting_info["standardpages_mediatype_name"] or default_mediatype ) or default_mediatype
		
		if(	nil ~= setting_info							 and
			"raw" == setting_info["mediatype_stringtype"]		) then
			mediatype_info = to_oct( mediatype_info )
		else
			local charcode = get_charsetvalue( gb18030, utf8mode, output_utf8_rtl_language ) == 33 and "utf8" or "osdependent"
			mediatype_info = to_oct( get_trimmed_command_string( mediatype_info, charcode, 65535 ) )
		end
		
	else
		mediatype_info = get_mediatype( mediatype )
	end

	return add_feature_tags("standardpages_mediatype" , mediatype , "<< /MediaType " .. ( ("none" ~= mediatype) and ("(" .. mediatype_info .. ")") or mediatype_info ) .. " >> setpagedevice")

end

-----------------------------------------------------------------------------
-- Local Function : output_staple
-----------------------------------------------------------------------------
local function output_staple( staple )

	if( nil == staple or "" == staple) then
		return ""
	end

	local cmd = function( value )
		local tbl =
		{
			["off"] 					= "<<\r\n/Staple 0 >> setpagedevice\r\n",
			["topleftslant"] 			= "/Staple 2 /StapleDetails << /Type 14 /Angle 3 /Position 0 >>\r\n",
			["toprightslant"] 			= "/Staple 2 /StapleDetails << /Type 14 /Angle 3 /Position 6 >>\r\n",
			["topleft"] 				= "/Staple 2 /StapleDetails << /Type 14 /Angle 0 /Position 0 >>\r\n",
			["topright"] 				= "/Staple 2 /StapleDetails << /Type 14 /Angle 0 /Position 6 >>\r\n",
			["2attop"] 					= "/Staple 2 /StapleDetails << /Type 14 /Angle 0 /Position 7 >>\r\n",
			["2atleft"] 				= "/Staple 2 /StapleDetails << /Type 14 /Angle 0 /Position 1 >>\r\n",
			["2atright"] 				= "/Staple 2 /StapleDetails << /Type 14 /Angle 0 /Position 5 >>\r\n",
			["2atcenter"] 				= "/Staple 2 /StapleDetails << /Type 14 /Angle 0 /Position 8 >>\r\n",
			["topleftslantstapleless"]	= "/Staple 2 /StapleDetails << /Type 14 /Angle 0 /Position 0 /Stapless true >>\r\n",
			["toprightslantstapleless"]	= "/Staple 2 /StapleDetails << /Type 14 /Angle 0 /Position 6 /Stapless true >>\r\n"
		 }
		 return (nil == tbl[value]) and tbl["off"] or tbl[value]
	end

	return add_feature_tags("staple" , staple , "<<\r\n" .. cmd( staple) .. ">> setpagedevice")

end

-----------------------------------------------------------------------------
-- Local Function : is_output_foldpunch
-----------------------------------------------------------------------------
local function is_output_foldpunch( punch, foldpunch )

	local	punch_member = is_member( punch, {	"off",
												"2atleft",
												"2attop",
												"2atright",
												"3atleft",
												"3attop",
												"3atright",
												"4atleft",
												"4attop",
												"4atright",
												"multiatleftortop",
												"multiatrightortop",
												"multiatleft",
												"multiattop",
												"multiatright",
												"doubleatleftortop",
												"doubleatrightortop" } )

	local	foldpunch_member = is_member( foldpunch, {	"off", "on" } )

	if( ( not punch_member	and		foldpunch_member )	or
		( "off" == punch 	and		"on" == foldpunch )	) then
		return true
	else
		return false
	end
end

-----------------------------------------------------------------------------
-- Local Function : output_punch
-----------------------------------------------------------------------------
local function output_punch( punch, foldpunch, orientation, layout )

	if(	nil == punch		or	"" == punch			or
		nil == orientation	or	"" == orientation	or
		nil == layout		or	"" == layout		or
		true == is_output_foldpunch( punch, foldpunch ) ) then
		return ""
	end

	local tbl =
	{
		["off"] 	 		= "<< /Punch 0 >> setpagedevice",
		["2atleft"]  		= "<< /Punch 2 /PunchDetails << /Type 4 /Position 0 /NumHoles 2 >> >> setpagedevice",
		["3atleft"]  		= "<< /Punch 2 /PunchDetails << /Type 4 /Position 0 /NumHoles 3 >> >> setpagedevice",
		["4atleft"]  		= "<< /Punch 2 /PunchDetails << /Type 4 /Position 0 /NumHoles 4 >> >> setpagedevice",
		["2atright"] 		= "<< /Punch 2 /PunchDetails << /Type 4 /Position 2 /NumHoles 2 >> >> setpagedevice",
		["3atright"] 		= "<< /Punch 2 /PunchDetails << /Type 4 /Position 2 /NumHoles 3 >> >> setpagedevice",
		["4atright"] 		= "<< /Punch 2 /PunchDetails << /Type 4 /Position 2 /NumHoles 4 >> >> setpagedevice",
		["2attop"] 	 		= "<< /Punch 2 /PunchDetails << /Type 4 /Position 3 /NumHoles 2 >> >> setpagedevice",
		["3attop"] 	 		= "<< /Punch 2 /PunchDetails << /Type 4 /Position 3 /NumHoles 3 >> >> setpagedevice",
		["4attop"] 	 		= "<< /Punch 2 /PunchDetails << /Type 4 /Position 3 /NumHoles 4 >> >> setpagedevice",
		["multiatleftortop"]	= "<< /Punch 2 /PunchDetails << /Type 3 /Position 0>> >> setpagedevice",
		["multiatrightortop"]	= "<< /Punch 2 /PunchDetails << /Type 3 /Position 2>> >> setpagedevice",
		["multiatleft"]			= "<< /Punch 2 /PunchDetails << /Type 3 /Position 0>> >> setpagedevice",
		["multiattop"]			= "<< /Punch 2 /PunchDetails << /Type 3 /Position 3>> >> setpagedevice",
		["multiatright"]		= "<< /Punch 2 /PunchDetails << /Type 3 /Position 2>> >> setpagedevice",
		["doubleatleftortop"]	= "<< /Punch 2 /PunchDetails << /Type 3 /Position 7>> >> setpagedevice",
		["doubleatrightortop"]	= "<< /Punch 2 /PunchDetails << /Type 3 /Position 7>> >> setpagedevice"
	}

	local physicalpaper_orientation = get_physicalpaper_orientation( orientation, layout )

	if("landscape" == physicalpaper_orientation) then
		tbl["multiatleftortop"]  = "<< /Punch 2 /PunchDetails << /Type 3 /Position 3>> >> setpagedevice"
		tbl["multiatrightortop"] = "<< /Punch 2 /PunchDetails << /Type 3 /Position 3>> >> setpagedevice"
		tbl["doubleatleftortop"] = "<< /Punch 2 /PunchDetails << /Type 3 /Position 4>> >> setpagedevice"
		tbl["doubleatrightortop"] = "<< /Punch 2 /PunchDetails << /Type 3 /Position 6>> >> setpagedevice"
	end

	return add_feature_tags("punch" , punch , ((nil == tbl[punch] ) and tbl["off"] or tbl[punch] ))

end

-----------------------------------------------------------------------------
-- Local Function : output_foldpunch
-----------------------------------------------------------------------------
local function output_foldpunch( punch, foldpunch )

	if(	nil == foldpunch	or	"" == foldpunch	 or
		false == is_output_foldpunch( punch, foldpunch ) ) then
		return ""
	end

	local tbl =
	{
		["off"] 	 		= "<< /Punch 0 >> setpagedevice",
		["on"]  			= "<< /Punch 2 /PunchDetails << /Type 4 /Position 0 /NumHoles 2 >> >> setpagedevice",
	}

	return add_feature_tags("foldpunch" , foldpunch , ( ( nil == tbl[foldpunch] ) and tbl["off"] or tbl[foldpunch] ) )

end

-----------------------------------------------------------------------------
-- Local Function : output_printer_fittoprintsize
-----------------------------------------------------------------------------
local function output_printer_fittoprintsize( value )

	if( nil == value or "" == value) then
		return ""
	end

	local tbl =
	{
		['promptuser']			= {['select'] = 'true',	['size'] = 2},
		['nearestsizeandscale']	= {['select'] = 'false', ['size'] = 3},
		['nearestsizeandcrop']	= {['select'] = 'false', ['size'] = 5},
	}

	local select = (nil == tbl[value]) and "true" or tbl[value]['select']
	local size = (nil == tbl[value]) and "2" or tbl[value]['size']

    local command = "<</DeferredMediaSelection " .. select .. " /Policies << /PageSize " .. size .. " /MediaType 2 >> >> setpagedevice"

	return add_feature_tags("printer_fittoprintsize" , value , command)
end

-----------------------------------------------------------------------------
-- Local Function : output_collate
-----------------------------------------------------------------------------
local function output_collate( devmode, collate, enablecollate )

	if( nil == devmode			or "" == devmode		or
		nil == collate			or "" == collate		or
		nil == enablecollate	or "" == enablecollate	) then
		return ""
	end

	local value = ("off" == enablecollate) and "false" or (("application" == collate) and (("0" == devmode) and "false" or "true") or "true")

	return add_feature_tags("collate" , collate , "<</Collate " .. value .. " >> setpagedevice")
end

-----------------------------------------------------------------------------
-- Local Function : output_joboffset
-----------------------------------------------------------------------------
local function output_joboffset( devmode, collate, enablecollate, joboffset )

	if( nil == devmode			or "" == devmode		or
		nil == collate			or "" == collate		or
		nil == enablecollate	or "" == enablecollate	or
		nil == joboffset		or "" == joboffset		) then
		return ""
	end

	local value = ("off" == enablecollate) and "false" or (("application" == collate) and (("0" == devmode) and "false" or "true") or "true")

	if( "false" == value )then
		return ""
	end

	local tbl =
	{
		["normal"]	= "<<\r\n/CollateDetails\r\n<</Type 6 /AlignSet true\r\n>>\r\n>> setpagedevice",
		["rotate"]	= "<<\r\n/CollateDetails\r\n<</Type 6 /AlignSet false /AlignSetType 1\r\n>>\r\n>> setpagedevice",
		["shift"]	= "<<\r\n/CollateDetails\r\n<</Type 6 /AlignSet false /AlignSetType 0\r\n>>\r\n>> setpagedevice"
	}

	return add_feature_tags("joboffset" , joboffset, ((nil == tbl[joboffset]) and tbl['normal'] or tbl[joboffset]) )

end

-----------------------------------------------------------------------------
-- Local Function : output_imagesmoothing
-----------------------------------------------------------------------------
local function output_imagesmoothing( value )

	if( nil == value or "" == value) then
		return ""
	end

	local tbl =
	{
		["off"] 	= "(off)",
		["on"] 		= "(on)",
		["auto"] 	= "(auto)",
		["90ppi"] 	= "90",
		["150ppi"] 	= "150",
		["200ppi"] 	= "200",
		["300ppi"] 	= "300"
	}

	return add_feature_tags("imagesmoothing" , value , ((nil == tbl[value]) and "(off)" or tbl[value] ) .. "/RDeviceProcSet /ProcSet findresource /SetImageInterpolate get exec")

end

-----------------------------------------------------------------------------
-- Local Function : output_rgbrevision
-----------------------------------------------------------------------------
local function output_rgbrevision( value )

	if( nil == value or "" == value) then
		return ""
	end

	local tbl =
	{
		["off"] 		=	"none",
		["fine"] 		=	"precision-std",
		["superfine"] 	=	"precision-dark"
	}

	return add_feature_tags("rgbrevision" , value , "(" .. ((nil == tbl[value]) and "none" or tbl[value]) .. ") RCsetrgbrevision")

end

-----------------------------------------------------------------------------
-- Local Function : output_colorprofile
-----------------------------------------------------------------------------
local function output_colorprofile( info_map, value )

	if( nil == value or "" == value) then
		return ""
	end

	local tbl =
	{
		["automatic"]			= "(auto) RCsetcolorprofile",
		["automaticbright"]		= "(auto-light) RCsetcolorprofile",
		["automaticdeep"]		= "(auto-dark) RCsetcolorprofile",
		["textbright"]			= "(business-light) RCsetcolorprofile",
		["textdeep"]			= "(business-dark) RCsetcolorprofile",
		["photographic"]		= "(photo) RCsetcolorprofile",
		["photographicbright"]	= "(photo-light) RCsetcolorprofile",
		["photographicdeep"]	= "(photo-dark) RCsetcolorprofile",
		["presentation"]		= "(business) RCsetcolorprofile",
		["presentationbright"]	= "(colorimetric-light) RCsetcolorprofile",
		["presentationdeep"]	= "(colorimetric-dark) RCsetcolorprofile",
		["solidcolor"]			= "(colorimetric) RCsetcolorprofile",
		["pop"]					= "(pop) RCsetcolorprofile",
		["usersetting"] 		= "(user) RCsetcolorprofile",
		["clpsimulation1"]		= "(hybrid10) RCsetcolorprofile",
		["clpsimulation2"]		= "(hybrid11) RCsetcolorprofile",
		["clpsimulation3"]		= "(hybrid12) RCsetcolorprofile",
		["clpsimulation4"]		= "(hybrid14) RCsetcolorprofile",
		["clpsimulation5"]		= "(hybrid15) RCsetcolorprofile",
		["clpsimulation6"]		= "(hybrid16) RCsetcolorprofile",
		["clpsimulation7"]		= "(hybrid13) RCsetcolorprofile",
		["clpsimulation8"]		= "(hybrid17) RCsetcolorprofile",
		["clpsimulation9"]		= "(hybrid17) RCsetcolorprofile",
		["custom1photograph"] 	= "(user) RCsetcolorprofile\r\n/Custom1_PhotoColorRendering\r\n/ColorRendering findresource setcolorrendering",
		["custom1business"] 	= "(user) RCsetcolorprofile\r\n/Custom1_BusinessColorRendering\r\n/ColorRendering findresource setcolorrendering",
		["custom1colorimetric"] = "(user) RCsetcolorprofile\r\n/Custom1_ColorimetricColorRendering\r\n/ColorRendering findresource setcolorrendering",
		["custom2photograph"] 	= "(user) RCsetcolorprofile\r\n/Custom2_PhotoColorRendering\r\n/ColorRendering findresource setcolorrendering",
		["custom2business"] 	= "(user) RCsetcolorprofile\r\n/Custom2_BusinessColorRendering\r\n/ColorRendering findresource setcolorrendering",
		["custom2colorimetric"] = "(user) RCsetcolorprofile\r\n/Custom2_ColorimetricColorRendering\r\n/ColorRendering findresource setcolorrendering",
		["useicm"]				= ( "1" ~= info_map["icmtype"] ) and  "/RDeviceProcSet\r\n/ProcSet findresource\r\n/SetICMMode get exec" or "(auto) RCsetcolorprofile"
	}

	return add_feature_tags("colorprofile" , value , ((nil == tbl[value]) and  "(auto) RCsetcolorprofile" or tbl[value] ) )

end

-----------------------------------------------------------------------------
-- Local Function : output_dithering
-----------------------------------------------------------------------------
local function output_dithering( value )

	if( nil == value or "" == value) then
		return ""
	end

	local tbl =
	{
		["automatic"] 		= "(auto)",
		["photographic"] 	= "(photo)",
		["text"] 			= "(text)",
		["usersetting"]		= "(user)",
		["multilevel"]		= "(dispersion)"
	}

	return add_feature_tags("dithering" , value , (( nil == tbl[value] ) and "(auto)" or tbl[value] ) .. " RCsethalftonetype")

end

-----------------------------------------------------------------------------
-- Local Function : output_grayreproduction
-----------------------------------------------------------------------------
local function output_grayreproduction( value )

	if( nil == value or "" == value) then
		return ""
	end

	local tbl =
	{
		["kblack"] 							= "(gray) (gray) (off)",
		["cmyk"] 							= "(off) (off) (off)",
		["kblackfortext"] 					= "(gray) (off) (off)",
		["kblackonlypureblack"] 			= "(black) (black) (off)",
		["kblackonlypureblackfortext"]		= "(black) (off) (off)",
		["kblackbyprofile"]					= "1",
		["kblackforallobject"]				= "(gray) (gray) (gray)"
	}

	local cmd = ("kblackbyprofile" == value) and "/SetVagueJudgment get exec" or "/SetPureBlack get exec"

	return add_feature_tags("grayreproduction" , value , ((nil == tbl[value]) and tbl["kblack"] or tbl[value]) .. "\r\n/RDeviceProcSet\r\n/ProcSet findresource\r\n" .. cmd )

end

-----------------------------------------------------------------------------
-- Local Function : output_separateintocmyk
-----------------------------------------------------------------------------
local function output_separateintocmyk( cyan, magenta, yellow, black, colorbw, coloruniversaldesign, synchrocut )

	if( nil == cyan	or	"" == cyan		or
		nil == magenta	or	"" == magenta	or
		nil == yellow	or	"" == yellow	or
		nil == black				or	"" == black					or
		nil == colorbw				or	"" == colorbw				or
		"blackandwhite" == colorbw									) then
		return ""
	end

	local value =	(("on" == cyan)		and "cyan" or "")
	value = value .. (("on" == magenta)	and (("" == value) and "magenta"	or "&magenta")	or "")
	value = value .. (("on" == yellow)	and (("" == value) and "yellow"	or "&yellow")	or "")
	value = value .. (("on" == black)	and (("" == value) and "black"		or "&black")	or "")

	value = ("" == value or "cyan&magenta&yellow&black" == value) and "none" or value

	value = (	( nil ~= coloruniversaldesign 	and "on"	==	coloruniversaldesign )	or
				( nil ~= synchrocut 			and "on"	==	synchrocut			 )	)	and "none" or value

	return add_feature_tags("separateintocmyk" , value , "(" .. value .. ") RCsetprintseparation")

end

-----------------------------------------------------------------------------
-- Local Function : output_cmyksimulationprofile
-----------------------------------------------------------------------------
local function output_cmyksimulationprofile( value )

	if( nil == value or "" == value ) then
		return ""
	end

	local tbl =
	{
		["off"] 			= "(None) RCsetcmyksimulation",
		["usoffsetprint"] 	= "(USOffsetPrint) RCsetcmyksimulation",
		["euroscale"] 		= "(Euroscale) RCsetcmyksimulation",
		["japancolor"] 		= "(JapanColor) RCsetcmyksimulation",
		["jmpa"] 			= "(JMPA) RCsetcmyksimulation",
		["pantone"] 		= "(PaletteColor) RCsetcmyksimulation",
		["trans01"] 		= "(Trans01) RCsetcmyksimulation",
		["custom1"] 		= "(%disk0%/Ink/Custom1_Ink) (r) file cvx exec\r\n(Custom1_Ink) RCsetcmyksimulation",
		["custom2"] 		= "(%disk0%/Ink/Custom2_Ink) (r) file cvx exec\r\n(Custom2_Ink) RCsetcmyksimulation",
		["custom3"] 		= "(%disk0%/Ink/Custom3_Ink) (r) file cvx exec\r\n(Custom3_Ink) RCsetcmyksimulation"
	}

	return add_feature_tags("cmyksimulationprofile" , value , ((nil == tbl[value] ) and "(None)  RCsetcmyksimulation" or tbl[value]) )

end

-----------------------------------------------------------------------------
-- Local Function : output_coloruniversaldesign
-----------------------------------------------------------------------------
local function output_coloruniversaldesign( coloruniversaldesign )

	if( nil == coloruniversaldesign or "" == coloruniversaldesign ) then
		return ""
	else
		return add_feature_tags("coloruniversaldesign" , coloruniversaldesign , (("on" == coloruniversaldesign) and "1" or "0") .. " /RDeviceProcSet\r\n/ProcSet findresource\r\n/SetBFCMode get exec" )
	end
end

-----------------------------------------------------------------------------
-- Local Function : output_faceup
-----------------------------------------------------------------------------
local function output_faceup( faceup, fold )

	if( nil == faceup or "" == faceup or
		nil ~= get_fold_info( fold, "faceup", nil, nil, nil ) ) then
		return ""
	else
		return add_feature_tags("faceup" , faceup , "<< /OutputFaceUp " .. (("on" == faceup) and "true" or "false" ) .. "\r\n>>setpagedevice" )
	end
end

-----------------------------------------------------------------------------
-- Local Function : output_ringbinding
-----------------------------------------------------------------------------
local function output_ringbinding( ringbinding, orientation, layout, perfectbinding, twinloopbinding )

	if(	("on" == perfectbinding)					or
		(nil == ringbinding	or "" == ringbinding)	or
		(nil == orientation	or "" == orientation)	or
		(nil == layout		or "" == layout		)	) then
		return ""
	end

	if( is_member( twinloopbinding, { "leftortop","rightortop" } ) ) then
		return ""
	end

	local tbl=
	{
		["off"]					= { ["bind"] = 0 },
		["leftortop"] 			= { ["bind"] = 2, ["mode"] = 1, ["position"] = { ["portrait"] = 0, ["landscape"] = 3 } },
		["rightortop"]			= { ["bind"] = 2, ["mode"] = 1, ["position"] = { ["portrait"] = 2, ["landscape"] = 3 } },
		["leftortoppunchonly"]	= { ["bind"] = 2, ["mode"] = 0, ["position"] = { ["portrait"] = 0, ["landscape"] = 3 } },
		["rightortoppunchonly"]	= { ["bind"] = 2, ["mode"] = 0, ["position"] = { ["portrait"] = 2, ["landscape"] = 3 } }
	}

	local cmd = "<< /Bind " .. tbl[ringbinding]["bind"] .. "\r\n"
	
	local physicalpaper_orientation = get_physicalpaper_orientation( orientation, layout )

	if( "off" ~= ringbinding ) then
		cmd = cmd .. "/BindDetails\r\n<< /Type 4 /BindType " .. tbl[ringbinding]["mode"] .. " /Position " .. tbl[ringbinding]["position"][physicalpaper_orientation] .. " >>\r\n"
	end

	cmd = cmd .. ">> setpagedevice"

	return add_feature_tags("ringbinding", ringbinding, cmd)

end

-----------------------------------------------------------------------------
-- Local Function : output_twinloopbinding
-----------------------------------------------------------------------------
local function output_twinloopbinding( twinloopbinding, orientation, layout, perfectbinding, ringbinding )

	if(	("on" == perfectbinding)					or
		(nil == twinloopbinding	or "" == twinloopbinding)	or
		(nil == orientation	or "" == orientation)	or
		(nil == layout		or "" == layout		)	) then
		return ""
	end
	
	if( is_member( ringbinding, { "leftortop","rightortop","leftortoppunchonly","rightortoppunchonly" } ) ) then
		return ""
	end

	local tbl=
	{
		["off"]					= { ["bind"] = 0 },
		["leftortop"] 			= { ["bind"] = 2, ["mode"] = 2, ["position"] = { ["portrait"] = 0, ["landscape"] = 3 } },
		["rightortop"]			= { ["bind"] = 2, ["mode"] = 2, ["position"] = { ["portrait"] = 2, ["landscape"] = 3 } }
	}

	local cmd = "<< /Bind " .. tbl[twinloopbinding]["bind"] .. "\r\n"
	
	local physicalpaper_orientation = get_physicalpaper_orientation( orientation, layout )

	if( "off" ~= twinloopbinding ) then
		cmd = cmd .. "/BindDetails\r\n<< /Type 4 /BindType " .. tbl[twinloopbinding]["mode"] .. " /Position " .. tbl[twinloopbinding]["position"][physicalpaper_orientation] .. " >>\r\n"
	end

	cmd = cmd .. ">> setpagedevice"

	return add_feature_tags("twinloopbinding", twinloopbinding, cmd)

end

-----------------------------------------------------------------------------
-- Local Function : output_trimming
-----------------------------------------------------------------------------
local function output_trimming( trimming, perfectbinding )
	
	if( "off" == trimming and "on" ~= perfectbinding ) then
		return add_feature_tags("trimming", trimming, "currentpagedevice /Bind get 0 eq {\r\n<</Trim 0>> setpagedevice\r\n} if" )
	else
		return ""
	end

end

-----------------------------------------------------------------------------
-- Local Function : output_trimming_cutedge
-----------------------------------------------------------------------------
local function output_trimming_cutedge( trimming, trimming_cutedge_mm, disable_cutedge_option)

	if( "on" == trimming and nil ~=trimming_cutedge_mm and "1" ~= disable_cutedge_option) then
		return add_feature_tags("trimming_cutedge_mm", trimming_cutedge_mm, "<<\r\n/Trim 2 /TrimDetails <</Type 1 /TrimType 1\r\n/TrimSide " .. get_pointvalue_decimal_from_mm(trimming_cutedge_mm) .. " >>\r\n>> setpagedevice" )
	else
		return ""
	end

end

-----------------------------------------------------------------------------
-- Local Function : output_perfectbinding
-----------------------------------------------------------------------------
local function output_perfectbinding(	info_map,
										perfectbinding, documentsize, printsize,
										standardpages_inputtray, perfectbinding_inputtray, perfectbinding_direction, orientation, layout,
										perfectbinding_imgpos_along_mm, perfectbinding_imgpos_across_mm,
										perfectbinding_coversheetprint,
										perfectbinding_coversheet_imgpos_along_mm, perfectbinding_coversheet_imgpos_across_mm,
										ringbinding, lctmediaposition, traymediaposition_type, twinloopbinding )

	if(	nil == perfectbinding								or "" == perfectbinding									or
		nil == documentsize									or "" == documentsize									or
		nil == printsize									or "" == printsize										or
		nil == standardpages_inputtray						or "" == standardpages_inputtray						or
		nil == perfectbinding_inputtray						or "" == perfectbinding_inputtray						or
		nil == perfectbinding_direction						or "" == perfectbinding_direction						or
		nil == orientation									or "" == orientation									or
		nil == perfectbinding_imgpos_along_mm				or "" == perfectbinding_imgpos_along_mm					or
		nil == perfectbinding_imgpos_across_mm				or "" == perfectbinding_imgpos_across_mm				or
		nil == perfectbinding_coversheetprint				or "" == perfectbinding_coversheetprint					or
		nil == perfectbinding_coversheet_imgpos_along_mm	or "" == perfectbinding_coversheet_imgpos_along_mm		or
		nil == perfectbinding_coversheet_imgpos_across_mm	or "" == perfectbinding_coversheet_imgpos_across_mm		) then
		return ""
	end

	if( is_member( ringbinding, { "leftortop","rightortop","leftortoppunchonly","rightortoppunchonly" } ) ) then
		return ""
	end

	if( is_member( twinloopbinding, { "leftortop","rightortop" } ) ) then
		return ""
	end

	local fixed_printsize = ( "sameasoriginalsize" == printsize ) and documentsize or printsize

	local papersize = get_papersize( info_map, fixed_printsize, "paper", nil, nil )

	local physicalpaper_orientation = get_physicalpaper_orientation( orientation, layout )

	local position = function( direction, physicalpaper_orientation, papersize )
		local tbl =
		{
			["leftortop"] =	{ ["portrait"] = 0, ["landscape"] = 3 },
			["rightortop"] = { ["portrait"] = 2, ["landscape"] = 3 }
		}

		if( tonumber(papersize['width']) > tonumber(papersize['height']) ) then
			physicalpaper_orientation = ( ( "portrait" == physicalpaper_orientation ) and "landscape" or "portrait" )
		end

		if( nil == tbl[direction] ) then
			return ( nil == tbl["leftortop"][physicalpaper_orientation]) and tbl["leftortop"]["portrait"] or tbl["leftortop"][physicalpaper_orientation]
		else
			return (nil == tbl[direction][physicalpaper_orientation]) and tbl["leftortop"]["portrait"] or tbl[direction][physicalpaper_orientation]
		end
	end

	local coverprint = function( coversheetprint )
		local tbl = {
			["off"]					= 0,
			["backcover"]			= 1,
			["frontcover"]			= 2,
			["frontandbackcover"]	= 3
		}
		return (nil == tbl[coversheetprint]) and tbl["off"] or tbl[coversheetprint]
	end

	local cmd = function( value )
		local standardpages_mediaPosition
		local perfectbinding_mediaPosition
		if("on" == value) then
			standardpages_mediaPosition = get_inputtray(standardpages_inputtray, lctmediaposition, traymediaposition_type)
			perfectbinding_mediaPosition = get_inputtray(perfectbinding_inputtray, lctmediaposition, traymediaposition_type)
		end

		local tbl = {
			["off"]	=	"<< /Bind 0 >> setpagedevice\r\n",
			["on"]	=	"<< /Bind 2\r\n/BindDetails\r\n<< /Type 6" ..
						"\r\n/TransferPageSize [" .. papersize['width'] .. " " .. papersize['height'] .. "]" ..
						( ("autotrayselect" == standardpages_inputtray) and "" or ( "\r\n/TransferMediaPosition " .. ( (nil == standardpages_mediaPosition) and 89 or standardpages_mediaPosition ) ) ) ..
						"\r\n/CoverMediaPosition " .. ( (nil == perfectbinding_mediaPosition) and 89 or perfectbinding_mediaPosition ) ..
						"\r\n/Position " .. position(perfectbinding_direction, physicalpaper_orientation, papersize) ..
						"\r\n/AdjustImagePosition [" .. get_pointvalue_decimal_from_mm(perfectbinding_imgpos_across_mm) .. " " .. get_pointvalue_decimal_from_mm(perfectbinding_imgpos_along_mm) .. "]" ..
						"\r\n/CoverPrint " .. coverprint(perfectbinding_coversheetprint) ..
						"\r\n/AdjustCoverPrintPosition [" .. get_pointvalue_decimal_from_mm(perfectbinding_coversheet_imgpos_along_mm) .. " " .. get_pointvalue_decimal_from_mm(perfectbinding_coversheet_imgpos_across_mm) .. "]" ..
						"\r\n>> >> setpagedevice\r\n"
		}
		return (nil == tbl[value]) and tbl["off"] or tbl[value]
	end

	return add_feature_tags("perfectbinding" , perfectbinding , cmd(perfectbinding) )
end

-----------------------------------------------------------------------------
-- Local Function : output_perfectbinding_cuttype
-----------------------------------------------------------------------------
local function output_perfectbinding_cuttype(	perfectbinding_cuttype,
												perfectbinding_cutpos_length_mm,
												perfectbinding_cutpos_finishing_size, perfectbinding_cutpos_customsize_width_mm, perfectbinding_cutpos_customsize_length_mm,
												perfectbinding_cutpos_along_mm,
												perfectbinding_cutpos_head_mm, perfectbinding_cutpos_tail_mm, perfectbinding_cutpos_fore_mm,
												perfectbinding, trimming )

	if(	nil == perfectbinding_cuttype						or "" == perfectbinding_cuttype							or
		nil == perfectbinding_cutpos_length_mm				or "" == perfectbinding_cutpos_length_mm				or
		nil == perfectbinding_cutpos_finishing_size			or "" == perfectbinding_cutpos_finishing_size			or
		nil == perfectbinding_cutpos_customsize_width_mm	or "" == perfectbinding_cutpos_customsize_width_mm		or
		nil == perfectbinding_cutpos_customsize_length_mm	or "" == perfectbinding_cutpos_customsize_length_mm		or
		nil == perfectbinding_cutpos_along_mm				or "" == perfectbinding_cutpos_along_mm					or
		nil == perfectbinding_cutpos_head_mm				or "" == perfectbinding_cutpos_head_mm					or
		nil == perfectbinding_cutpos_tail_mm				or "" == perfectbinding_cutpos_tail_mm					or
		nil == perfectbinding_cutpos_fore_mm				or "" == perfectbinding_cutpos_fore_mm					or
		nil == perfectbinding								or "" == perfectbinding									) then
		return ""
	end

	if("off" == perfectbinding) then
		return ""
	end

	local finishing_papersize = ( "custompapersize" == perfectbinding_cutpos_finishing_size ) and  get_custompapersize_decimal_from_mm( perfectbinding_cutpos_customsize_width_mm, perfectbinding_cutpos_customsize_length_mm, "paper" ) or get_papersize( nil, perfectbinding_cutpos_finishing_size, "paper", nil, nil )

	local cmd = function( value )

		local tbl = {
			["donotcut"]						=	"<< /Trim 0 >> setpagedevice\r\n",
			["cutforeedgespecifyfinishingsize"]	=	"<< /Trim 2\r\n/TrimDetails\r\n<< /Type 1\r\n/TrimType 0\r\n/FinishedPageSize [842 " .. get_pointvalue_decimal_from_mm(perfectbinding_cutpos_length_mm) .. "]\r\n>>\r\n>> setpagedevice",
			["cutforeedge"]						=	"<< /Trim 2\r\n/TrimDetails\r\n<< /Type 1\r\n/TrimType 1\r\n/TrimSide " .. get_pointvalue_decimal_from_mm(perfectbinding_cutpos_fore_mm) .. "\r\n>>\r\n>> setpagedevice",
			["cut3edgesspecifyfinishingsize"]	=	"<< /Trim 2\r\n/TrimDetails\r\n<< /Type 1\r\n/TrimType 10\r\n/FinishedPageSize [" .. finishing_papersize["width"] .. " " .. finishing_papersize["height"] .. "]\r\n/AdjustFinishedSizePosition " .. get_pointvalue_decimal_from_mm(perfectbinding_cutpos_along_mm) .. "\r\n>>\r\n>> setpagedevice",
			["cut3edges"]						=	"<< /Trim 2\r\n/TrimDetails\r\n<< /Type 1\r\n/TrimType 11\r\n/TrimTop " .. get_pointvalue_decimal_from_mm(perfectbinding_cutpos_head_mm) .. "\r\n/TrimBottom " .. get_pointvalue_decimal_from_mm(perfectbinding_cutpos_tail_mm) .. "\r\n/TrimSide " .. get_pointvalue_decimal_from_mm(perfectbinding_cutpos_fore_mm) .. "\r\n>>\r\n>> setpagedevice",
		}
		if( "donotcut" == perfectbinding_cuttype ) then
			return ( "on" == trimming ) and "" or tbl["donotcut"]
		else
			return (nil == tbl[value]) and  ( ( "on" == trimming ) and "" or tbl["donotcut"] ) or tbl[value]
		end
	end

	return add_feature_tags("perfectbinding_cuttype" , perfectbinding_cuttype , cmd(perfectbinding_cuttype) )
end

-----------------------------------------------------------------------------
-- Local Function : output_boundingbox
-----------------------------------------------------------------------------
local function output_boundingbox( documentsize, printsize, edgetoedgeprint )

	if(	nil == documentsize		or "" == documentsize		or
		nil == printsize		or "" == printsize			) then
		return ""
	end

	local fixed_size = ( "sameasoriginalsize" == printsize ) and documentsize or printsize

	local paper_type = ("on" == edgetoedgeprint) and "paper" or "region"

	local tbl = get_papersize( nil, fixed_size, paper_type, nil, nil )

	local value = ( ("on" == edgetoedgeprint) and "0 0 " or "12 12 " ) .. tbl['width'] .. " " .. tbl['height']

	return (nil == value) and "" or ("%%PageBoundingBox: " .. value .. "\n")

end
-----------------------------------------------------------------------------
-- Local Function : output_beginpagesetup_ctm
-----------------------------------------------------------------------------
local function output_beginpagesetup_ctm( edgetoedgeprint, orientation )

	local cdb = Rice.CapsDB
	
	local offset = 12
	
	return ("on" == edgetoedgeprint) and "\r\n[ 1 0 0 1 " .. string.format( "%.5f", (("landscape" == orientation) and 1 or -1) * offset) .. " " .. string.format( "%.5f", offset) .. "] concat\r\n" or ""
	
end


-----------------------------------------------------------------------------
-- Local Function : output_ctm
-----------------------------------------------------------------------------
local function output_ctm(	documentsize, printsize,
							orientation,resolution,
							reduceenlarge, zoomvalue,
							mirroring, edgetoedgeprint,
							rotatelandscape )

	local cdb = Rice.CapsDB

	if(	nil == documentsize		or "" == documentsize		or
		nil == printsize		or "" == printsize			or
		nil == orientation		or "" == orientation		or
		nil == resolution		or "" == resolution			or
		nil == reduceenlarge	or "" == reduceenlarge		or
		nil == zoomvalue		or "" == zoomvalue			or
		nil == mirroring		or "" == mirroring			or
		nil == rotatelandscape	or "" == rotatelandscape	) then
		return ""
	end

	local fixed_size = ("sameasoriginalsize" == printsize) and documentsize or printsize

	local paper_type = ("on" == edgetoedgeprint) and "paper" or "region"

	local fixed_tbl = get_papersize( nil, fixed_size, paper_type, nil, nil )
	local doc_tbl = get_papersize( nil, documentsize, paper_type, nil, nil )

	-- current CTM scaling
	local scale = ( ( nil == get_resolution_info( resolution, "resolutionvalue" ) ) and 600 or get_resolution_info( resolution, "resolutionvalue" ) ) / 72

	-- scale_x / scale_y
	local scale_x = 1
	local scale_y = 1

	-- pos_x / pos_y
	local pos_x = string.format( "%.5f", 0 )
	local pos_y = string.format( "%.5f", 0 )

	--booklet̂Ƃlayout̒l1pagepersheet
	if( "1pagepersheet" == cdb.get("layout") and
		"booklet" ~= cdb.get("booklet")			) then
		-- zoom
		if( "zoom" == reduceenlarge ) then
			scale_x = string.format( "%.5f", zoomvalue / 100)
			scale_y = scale_x

		-- fit to print size
		elseif( "fittoprintsize" == reduceenlarge ) then
			scale_x = string.format("%.5f", ("portrait" == orientation) and fixed_tbl['width'] / doc_tbl['width'] or fixed_tbl['height'] / doc_tbl['height'] )
			scale_y = string.format("%.5f", ("portrait" == orientation) and fixed_tbl['height'] / doc_tbl['height'] or fixed_tbl['width'] / doc_tbl['width'] )

			scale_x = math.min( scale_x, scale_y )
			scale_y = scale_x
		end

		local diff_width  = (doc_tbl['width'] - fixed_tbl['width']) * scale
		local diff_height = (doc_tbl['height'] - fixed_tbl['height']) * scale
		local direction_x = -1
		local direction_y = -1

		--orientation
		if("portrait" == orientation ) then
			direction_y = direction_y * -1
			pos_y = pos_y + diff_height * direction_y
		end

		--rotatelandscape
		if( "on" == rotatelandscape ) then
			if( "landscape" == orientation ) then
				direction_x = direction_x * -1
				direction_y = direction_y * -1
				pos_x = pos_x + diff_height * direction_x
				pos_y = pos_y + diff_width * direction_y
			end
		end

		--mirroring
		if( "on" == mirroring ) then
			if( "portrait" == orientation ) then
				direction_x = direction_x * -1
				pos_x = pos_x + diff_width * direction_x
			else
				direction_y = direction_y * -1
				pos_y = pos_y + diff_width * direction_y
			end
		end

		pos_x = string.format( "%.5f", pos_x )
		pos_y = string.format( "%.5f", pos_y )
	end

	return "\r\n[" .. scale_x .. " 0 0 " .. scale_y .. " " .. pos_x .. " " .. pos_y .. "] concat\r\n"
end

-----------------------------------------------------------------------------
-- Local Function : output_drawframeborder
-----------------------------------------------------------------------------
local function output_drawframeborder( drawframeborder, booklet, page_info )

	local fixed_drawframeborder = ("on" ~= drawframeborder) and "false" or (page_info and ( ("false" ~= page_info["draw_frame_border"]) and "true" or "false" ) or "true")
	local cmd_framenup = ("booklet" == booklet) and "false" or fixed_drawframeborder
	return "mark{\r\n/DeviceRGB dup setcolorspace /colspABC exch def\r\n/framenup " .. cmd_framenup .. " def\r\n}stopped cleartomark\r\n"

end

-----------------------------------------------------------------------------
-- Local Function : output_nuprotate
-----------------------------------------------------------------------------
local function output_nuprotate()

	return "/definenup {/LH/nupshowpage ld \nnupv astore !/pgx_nup 0 store/pgy_nup 0 store \n180 neg rotate currentpagedevice /PageSize get aload pop pop neg 0 translate \n0 currentpagedevice /PageSize get aload pop exch pop neg translate } bd\n"

end

-----------------------------------------------------------------------------
-- Local Function : output_rotate180
-----------------------------------------------------------------------------
local function output_rotate180( orientation, layout, booklet, pagelayout_distribution, mirroring, documentsize, printsize, booklet_equalmagnification, reduceenlarge  )
	
	if( (	is_layout_booklet( layout, booklet )								or
			is_printer_pagelayout( layout, booklet, pagelayout_distribution, booklet_equalmagnification, reduceenlarge  )	or
			is_driver_pagelayout( layout, booklet, pagelayout_distribution, booklet_equalmagnification, reduceenlarge )	)	and
		"on" == mirroring															)	then
		return ""
	end

	local fixed_orientation = orientation

	if( "portrait" == fixed_orientation ) then
		local fixed_printsize = ( "sameasoriginalsize" == printsize ) and documentsize or printsize
		local papersize = get_papersize( nil, fixed_printsize, "paper", nil, nil )
		fixed_orientation = ( tonumber(papersize['width']) > tonumber(papersize['height']) ) and "landscape" or "portrait"
	end

	return "landscape" == fixed_orientation and "180 rotate\r\ncurrentpagedevice /PageSize get dup 0 get neg exch 1 get neg translate\r\n" or ""

end

-----------------------------------------------------------------------------
-- Local Function : output_repeatcmd
-----------------------------------------------------------------------------
local function output_repeatcmd( patternrepeat, linespace )
	
	local numberLinespace = tonumber( linespace ) and tonumber( linespace ) or 70
	local stringLinespage = tostring( (( 0 <= numberLinespace) and ( 32767 >= numberLinespace )) and numberLinespace or 70 )
	local cmd_linespace	= "/StringLoopInterval " .. stringLinespage .. "\r\n"

	local cmd_repeat = "null\r\n"
	if( "repeat" == patternrepeat or
		"repeatandrotatecarriagereturn"== patternrepeat ) then
		cmd_repeat = "<<\r\n    " .. cmd_linespace .. "/StringLoopToggle " .. (( "repeat" == patternrepeat ) and  "false"or "true") .. "\r\n>>\r\n"
	end

	return "/StringLoopAttributes  " .. cmd_repeat

end

-----------------------------------------------------------------------------
-- Local Function : output_textCmd
-----------------------------------------------------------------------------
local function output_textCmd( info_map, texttype, text )

	local textCmd
	
	local getLoginUserName = function( tbl )
		local cmd = "username"
		local loginusername = ( ( tbl ) and tbl["loginUserName"] or nil )

		if(	nil ~= loginusername and "" ~= loginusername ) then
			cmd = loginusername
		end
		return cmd
	end
	
	local getJobName = function( tbl )
		local cmd = "jobname"
		local jobname = ( ( tbl ) and tbl["jobName"] or nil )
		
		if(nil ~= jobname and "" ~= jobname) then
			cmd = jobname
		end
		return cmd
	end
	
	local getDate = function()
		--textCmd = "YYYY/MM/DD hh:mm"
		
		return os.date("%Y") .. "/" .. os.date("%m") .. "/" .. os.date("%d") .." ".. os.date("%H") .. ":" .. os.date("%M")
	end
	
	if( "loginname" == texttype) then
		textCmd = getLoginUserName( info_map )
	elseif( "printjobname" == texttype ) then
		textCmd = getJobName( info_map )
	elseif( "printjobtime" == texttype ) then
		textCmd = getDate()
	elseif( "loginnameandprintjobname" == texttype ) then
		textCmd = getLoginUserName( info_map ) .. " " .. getJobName( info_map )
	elseif( "loginnameandprintjobtime" == texttype ) then
		textCmd = getLoginUserName( info_map ) .. " " .. getDate()
	elseif( "printjobnameandprintjobtime" == texttype ) then
		textCmd = getJobName( info_map ) .. " " .. getDate()
	elseif( "loginnameandprintjobnameandprintjobtime" == texttype ) then
		textCmd = getLoginUserName( info_map ) .. " " .. getJobName( info_map ) .. " " .. getDate()
	else
		textCmd = "COPY"
		if(nil ~= text	and "" ~= text) then
			textCmd = text
		end
	end
	
	return textCmd
end

-----------------------------------------------------------------------------
-- Local Function : output_StringAttributesCmd
-----------------------------------------------------------------------------
local function output_StringAttributesCmd( 	info_map,
											texttype, text, font,
											position, size, angle,
											patternrepeat, linespace )

	local fontvalue =
	{
	--tHg
		["albertus"]							= {["command"] = "AlbertusMT",						["area"] = "European"},
		["albertus-italic"]						= {["command"] = "AlbertusMT-Italic",				["area"] = "European"},
		["albertus-light"]						= {["command"] = "AlbertusMT-Light",				["area"] = "European"},
		["antiqueolive-bold"]					= {["command"] = "AntiqueOlive-Bold",				["area"] = "European"},
		["antiqueolive-compact"]				= {["command"] = "AntiqueOlive-Compact",			["area"] = "European"},
		["antiqueolive-italic"]					= {["command"] = "AntiqueOlive-Italic",				["area"] = "European"},
		["antiqueolive-roman"]					= {["command"] = "AntiqueOlive-Roman",				["area"] = "European"},
		["apple-chancery"]						= {["command"] = "Apple-Chancery",					["area"] = "European"},
		["arial"]								= {["command"] = "ArialMT",							["area"] = "European"},
		["arial-bold"]							= {["command"] = "Arial-BoldMT",					["area"] = "European"},
		["arial-bold-italic"]					= {["command"] = "Arial-BoldItalicMT",				["area"] = "European"},
		["arial-italic"]						= {["command"] = "Arial-ItalicMT",					["area"] = "European"},
		["bodoni"]								= {["command"] = "Bodoni",							["area"] = "European"},
		["bodoni-bold"]							= {["command"] = "Bodoni-Bold",						["area"] = "European"},
		["bodoni-bold-italic"]					= {["command"] = "Bodoni-BoldItalic",				["area"] = "European"},
		["bodoni-italic"]						= {["command"] = "Bodoni-Italic",					["area"] = "European"},
		["bodoni-poster"]						= {["command"] = "Bodoni-Poster",					["area"] = "European"},
		["bodoni-poster-compressed"]			= {["command"] = "Bodoni-PosterCompressed",			["area"] = "European"},
		["carta"]								= {["command"] = "Carta",							["area"] = "European"},
		["chicago"]								= {["command"] = "Chicago",							["area"] = "European"},
		["clarendon"]							= {["command"] = "Clarendon",						["area"] = "European"},
		["clarendon-bold"]						= {["command"] = "Clarendon-Bold",					["area"] = "European"},
		["clarendon-light"]						= {["command"] = "Clarendon-Light",					["area"] = "European"},
		["cooperblack"]							= {["command"] = "CooperBlack",						["area"] = "European"},
		["cooperblack-italic"]					= {["command"] = "CooperBlack-Italic",				["area"] = "European"},
		["copperplate-gothic-32bc"]				= {["command"] = "Copperplate-ThirtyTwoBC",			["area"] = "European"},
		["copperplate-gothic-33bc"]				= {["command"] = "Copperplate-ThirtyThreeBC",		["area"] = "European"},
		["coronet"]								= {["command"] = "Coronet-Regular",					["area"] = "European"},
		["courier"]								= {["command"] = "Courier",							["area"] = "European"},
		["courier-bold"]						= {["command"] = "Courier-Bold",					["area"] = "European"},
		["courier-bold-oblique"]				= {["command"] = "Courier-BoldOblique",				["area"] = "European"},
		["courier-oblique"]						= {["command"] = "Courier-Oblique",					["area"] = "European"},
		["eurostile"]							= {["command"] = "Eurostile",						["area"] = "European"},
		["eurostile-bold"]						= {["command"] = "Eurostile-Bold",					["area"] = "European"},
		["eurostile-bold-extended-two"]			= {["command"] = "Eurostile-BoldExtendedTwo",		["area"] = "European"},
		["eurostile-extended-two"]				= {["command"] = "Eurostile-ExtendedTwo",			["area"] = "European"},
		["geneva"]								= {["command"] = "Geneva",							["area"] = "European"},
		["gillsans"]							= {["command"] = "GillSans",						["area"] = "European"},
		["gillsans-bold"]						= {["command"] = "GillSans-Bold",					["area"] = "European"},
		["gillsans-bold-italic"]				= {["command"] = "GillSans-BoldItalic",				["area"] = "European"},
		["gillsans-condensed"]					= {["command"] = "GillSans-Condensed",				["area"] = "European"},
		["gillsans-condensed-bold"]				= {["command"] = "GillSans-BoldCondensed",			["area"] = "European"},
		["gillsans-extra-bold"]					= {["command"] = "GillSans-ExtraBold",				["area"] = "European"},
		["gillsans-italic"]						= {["command"] = "GillSans-Italic",					["area"] = "European"},
		["gillsans-light"]						= {["command"] = "GillSans-Light",					["area"] = "European"},
		["gillsans-light-italic"]				= {["command"] = "GillSans-LightItalic",			["area"] = "European"},
		["goudy-bold"]							= {["command"] = "Goudy-Bold",						["area"] = "European"},
		["goudy-bold-italic"]					= {["command"] = "Goudy-BoldItalic",				["area"] = "European"},
		["goudy-extrabold"]						= {["command"] = "Goudy-ExtraBold",					["area"] = "European"},
		["goudy-oldstyle"]						= {["command"] = "Goudy",							["area"] = "European"},
		["goudy-oldstyle-italic"]				= {["command"] = "Goudy-Italic",					["area"] = "European"},
		["helvetica"]							= {["command"] = "Helvetica",						["area"] = "European"},
		["helvetica-bold"]						= {["command"] = "Helvetica-Bold",					["area"] = "European"},
		["helvetica-bold-oblique"]				= {["command"] = "Helvetica-BoldOblique",			["area"] = "European"},
		["helvetica-condensed"]					= {["command"] = "Helvetica-Condensed",				["area"] = "European"},
		["helvetica-condensed-bold"]			= {["command"] = "Helvetica-Condensed-Bold",		["area"] = "European"},
		["helvetica-condensed-bold-oblique"]	= {["command"] = "Helvetica-Condensed-BoldObl",		["area"] = "European"},
		["helvetica-condensed-oblique"]			= {["command"] = "Helvetica-Condensed-Oblique",		["area"] = "European"},
		["helvetica-narrow"]					= {["command"] = "Helvetica-Narrow",				["area"] = "European"},
		["helvetica-narrow-bold"]				= {["command"] = "Helvetica-Narrow-Bold",			["area"] = "European"},
		["helvetica-narrow-bold-oblique"]		= {["command"] = "Helvetica-Narrow-BoldOblique",	["area"] = "European"},
		["helvetica-narrow-oblique"]			= {["command"] = "Helvetica-Narrow-Oblique",		["area"] = "European"},
		["helvetica-oblique"]					= {["command"] = "Helvetica-Oblique",				["area"] = "European"},
		["hoeflertext"]							= {["command"] = "HoeflerText-Regular",				["area"] = "European"},
		["hoeflertext-black"]					= {["command"] = "HoeflerText-Black",				["area"] = "European"},
		["hoeflertext-black-italic"]			= {["command"] = "HoeflerText-BlackItalic",			["area"] = "European"},
		["hoeflertext-italic"]					= {["command"] = "HoeflerText-Italic",				["area"] = "European"},
		["hoeflertext-ornaments"]				= {["command"] = "HoeflerText-Ornaments",			["area"] = "European"},
		["itc-avantgarde-gothic-book"]			= {["command"] = "AvantGarde-Book",					["area"] = "European"},
		["itc-avantgarde-gothic-book-oblique"]	= {["command"] = "AvantGarde-BookOblique",			["area"] = "European"},
		["itc-avantgarde-gothic-demi"]			= {["command"] = "AvantGarde-Demi",					["area"] = "European"},
		["itc-avantgarde-gothic-demi-oblique"]	= {["command"] = "AvantGarde-DemiOblique",			["area"] = "European"},
		["itc-bookman-demi"]					= {["command"] = "Bookman-Demi",					["area"] = "European"},
		["itc-bookman-demi-italic"]				= {["command"] = "Bookman-DemiItalic",				["area"] = "European"},
		["itc-bookman-light"]					= {["command"] = "Bookman-Light",					["area"] = "European"},
		["itc-bookman-light-italic"]			= {["command"] = "Bookman-LightItalic",				["area"] = "European"},
		["itc-lubalin-graph-book"]				= {["command"] = "LubalinGraph-Book",				["area"] = "European"},
		["itc-lubalin-graph-book-oblique"]		= {["command"] = "LubalinGraph-BookOblique",		["area"] = "European"},
		["itc-lubalin-graph-demi"]				= {["command"] = "LubalinGraph-Demi",				["area"] = "European"},
		["itc-lubalin-graph-demi-oblique"]		= {["command"] = "LubalinGraph-DemiOblique",		["area"] = "European"},
		["itc-mona-lisa-recut"]					= {["command"] = "MonaLisa-Recut",					["area"] = "European"},
		["itc-symbol"]							= {["command"] = "Symbol",							["area"] = "European"},
		["itc-zapfchancery-mediumitalic"]		= {["command"] = "ZapfChancery-MediumItalic",		["area"] = "European"},
		["itc-zapfdingbats"]					= {["command"] = "ZapfDingbats",					["area"] = "European"},
		["joanna"]								= {["command"] = "JoannaMT",						["area"] = "European"},
		["joanna-bold"]							= {["command"] = "JoannaMT-Bold",					["area"] = "European"},
		["joanna-bold-italic"]					= {["command"] = "JoannaMT-BoldItalic",				["area"] = "European"},
		["joanna-italic"]						= {["command"] = "JoannaMT-Italic",					["area"] = "European"},
		["lettergothic"]						= {["command"] = "LetterGothic",					["area"] = "European"},
		["lettergothic-bold"]					= {["command"] = "LetterGothic-Bold",				["area"] = "European"},
		["lettergothic-bold-slanted"]			= {["command"] = "LetterGothic-BoldSlanted",		["area"] = "European"},
		["lettergothic-slanted"]				= {["command"] = "LetterGothic-Slanted",			["area"] = "European"},
		["marigold"]							= {["command"] = "Marigold",						["area"] = "European"},
		["monaco"]								= {["command"] = "Monaco",							["area"] = "European"},
		["new-century-schoolbook-bold"]			= {["command"] = "NewCenturySchlbk-Bold",			["area"] = "European"},
		["new-century-schoolbook-bold-italic"]	= {["command"] = "NewCenturySchlbk-BoldItalic",		["area"] = "European"},
		["new-century-schoolbook-italic"]		= {["command"] = "NewCenturySchlbk-Italic",			["area"] = "European"},
		["new-century-schoolbook-roman"]		= {["command"] = "NewCenturySchlbk-Roman",			["area"] = "European"},
		["newyork"]								= {["command"] = "NewYork",							["area"] = "European"},
		["optima"]								= {["command"] = "Optima",							["area"] = "European"},
		["optima-bold"]							= {["command"] = "Optima-Bold",						["area"] = "European"},
		["optima-bold-italic"]					= {["command"] = "Optima-BoldItalic",				["area"] = "European"},
		["optima-italic"]						= {["command"] = "Optima-Italic",					["area"] = "European"},
		["oxford"]								= {["command"] = "Oxford",							["area"] = "European"},
		["palatino-bold"]						= {["command"] = "Palatino-Bold",					["area"] = "European"},
		["palatino-bold-italic"]				= {["command"] = "Palatino-BoldItalic",				["area"] = "European"},
		["palatino-italic"]						= {["command"] = "Palatino-Italic",					["area"] = "European"},
		["palatino-roman"]						= {["command"] = "Palatino-Roman",					["area"] = "European"},
		["stempel-garamond-bold"]				= {["command"] = "StempelGaramond-Bold",			["area"] = "European"},
		["stempel-garamond-bold-italic"]		= {["command"] = "StempelGaramond-BoldItalic",		["area"] = "European"},
		["stempel-garamond-italic"]				= {["command"] = "StempelGaramond-Italic",			["area"] = "European"},
		["stempel-garamond-roman"]				= {["command"] = "StempelGaramond-Roman",			["area"] = "European"},
		["tekton"]								= {["command"] = "Tekton",							["area"] = "European"},
		["times-bold"]							= {["command"] = "Times-Bold",						["area"] = "European"},
		["times-bold-italic"]					= {["command"] = "Times-BoldItalic",				["area"] = "European"},
		["times-italic"]						= {["command"] = "Times-Italic",					["area"] = "European"},
		["times-new-roman"]						= {["command"] = "TimesNewRomanPSMT",				["area"] = "European"},
		["times-new-roman-bold"]				= {["command"] = "TimesNewRomanPS-BoldMT",			["area"] = "European"},
		["times-new-roman-bold-italic"]			= {["command"] = "TimesNewRomanPS-BoldItalicMT",	["area"] = "European"},
		["times-new-roman-italic"]				= {["command"] = "TimesNewRomanPS-ItalicMT",		["area"] = "European"},
		["times-roman"]							= {["command"] = "Times-Roman",						["area"] = "European"},
		["univers"]								= {["command"] = "Univers",							["area"] = "European"},
		["univers-bold"]						= {["command"] = "Univers-Bold",					["area"] = "European"},
		["univers-bold-oblique"]				= {["command"] = "Univers-BoldOblique",				["area"] = "European"},
		["universcondensed"]					= {["command"] = "Univers-Condensed",				["area"] = "European"},
		["universcondensed-bold"]				= {["command"] = "Univers-CondensedBold",			["area"] = "European"},
		["universcondensed-bold-oblique"]		= {["command"] = "Univers-CondensedBoldOblique",	["area"] = "European"},
		["universcondensed-oblique"]			= {["command"] = "Univers-CondensedOblique",		["area"] = "European"},
		["universesextended"]					= {["command"] = "Univers-Extended",				["area"] = "European"},
		["universesextended-bold"]				= {["command"] = "Univers-BoldExt",					["area"] = "European"},
		["universesextended-bold-oblique"]		= {["command"] = "Univers-BoldExtObl",				["area"] = "European"},
		["universesextended-oblique"]			= {["command"] = "Univers-ExtendedObl",				["area"] = "European"},
		["univers-light"]						= {["command"] = "Univers-Light",					["area"] = "European"},
		["univers-light-oblique"]				= {["command"] = "Univers-LightOblique",			["area"] = "European"},
		["univers-oblique"]						= {["command"] = "Univers-Oblique", 				["area"] = "European"},
		["wingdings"]							= {["command"] = "Wingdings-Regular", 				["area"] = "European"},
		["nimbussans-bold"]						= {["command"] = "NimbusSans-Bold", 				["area"] = "European"},
		["nimbusmonops-bold"]					= {["command"] = "NimbusMonoPS-Bold", 				["area"] = "European"},
		["nimbusroman-bold"]					= {["command"] = "NimbusRoman-Bold", 				["area"] = "European"},
	--atHg
		--TtHg/Adobe
		["ryumin-light-unijis-utf8-h"]			= {["command"] = "Ryumin-Light-UniJIS-UTF8-H",		["area"] = "Japanese"},
		["gothicbbb-medium-unijis-utf8-h"]		= {["command"] = "GothicBBB-Medium-UniJIS-UTF8-H",	["area"] = "Japanese"},
		--tHg/Adobe
		["heiseimin-w3-unijis-utf8-h"]			= {["command"] = "HeiseiMin-W3-UniJIS-UTF8-H",		["area"] = "Japanese"},
		["heiseikakugo-w5-unijis-utf8-h"]		= {["command"] = "HeiseiKakuGo-W5-UniJIS-UTF8-H",	["area"] = "Japanese"},
		--tHg/IRIPS
		["hgheiseimin-w3-unijis-utf8-h"]		= {["command"] = "HGHeiseiMin-W3-UniJIS-UTF8-H",	["area"] = "Japanese"},
		["hgheiseikakugo-w5-unijis-utf8-h"]		= {["command"] = "HGHeiseiKakuGo-W5-UniJIS-UTF8-H",	["area"] = "Japanese"}
	}
	
	local textCmd = output_textCmd( info_map, texttype, text )

	
	textCmd = get_trimmed_command_string( textCmd, "utf8", 64 )
	
	local positionvalue =
	{
		["center"]			= "3",
		["topleft"]			= "0",
		["centertop"]		= "1",
		["topright"]		= "2",
		["bottomleft"]		= "4",
		["centerbottom"]	= "5",
		["bottomright"]		= "6"
	}
	
	local numbersize = tonumber( size ) and tonumber( size ) or 70
	local stringsize = tostring( (( 50 <= numbersize) and ( 300 >= numbersize )) and numbersize or 70 )
	
	local numberangle = tonumber( angle ) and tonumber( angle ) or 30
	local stringangle = tostring( (( 0 <= numberangle) and ( 359 >= numberangle )) and numberangle or 30 )
	
	--local cmd_font				= "/FontName " .. fontvalue[font] .. "\r\n"
	local cmd_font				= "/FontName (" .. ( fontvalue[font] and fontvalue[font] or fontvalue["arial"] )["command"] .. ")\r\n"
	local cmd_fontsize			= "/FontSize " .. stringsize .. "\r\n"
	local cmd_string			= "/String (" .. to_oct( textCmd ) .. ")\r\n"
	local cmd_position			= "/StringPosition " .. ( positionvalue[position] and positionvalue[position] or "3" ) .. "\r\n"
	local cmd_angle				= "/StringAngle " .. stringangle .. "\r\n"
	
	
	return "/StringAttributes <<\r\n" .. cmd_font .. cmd_fontsize .. cmd_string .. cmd_position .. cmd_angle .. output_repeatcmd( patternrepeat, linespace ) .. ">>\r\n"

end

-----------------------------------------------------------------------------
-- Local Function : output_preventioncopypattern
-----------------------------------------------------------------------------
local function output_preventioncopypattern( 	info_map,
												color, density, masktype,
												patterneffects, texttype,
												text, font, position,
												size, angle, patternrepeat,
												linespace	)

	local colorvalue =
	{
		["black"]	= "0",
		["cyan"]	= "1",
		["magenta"]	= "2"
	}
	
	local densityvalue =
	{
		["verylight"]	= "1",
		["light"]		= "2",
		["middle"]		= "3",
		["dark"]		= "4",
		["verydark"]	= "5"
	}
	
	local masktypevalue =
	{
		["none"]				= "0",
		["wavecrest"]			= "1",
		["mesh"]				= "2",
		["lattice1"]			= "3",
		["lattice2"]			= "4",
		["interlockingcircles"]	= "5",
		["shokkoh"]				= "6",
		["matsukawabishi"]		= "7",
		["scale"]				= "8",
		["higaki"]				= "9",
		["hexagonal"]			= "10"
	}
	
	local patterneffectsvalue =
	{
		["normal"]								="0",
		["reversepatterns"]						="2",
		["reversepatternsandbackgroundonly"]	="3",
		["textonly"]							="4"
	}
	
	local cmd_color 			= "/MarkingColor " .. ( colorvalue[color] and colorvalue[color] or "0" ) .. "\r\n"
	local cmd_density			= "/MarkingDensity " .. ( densityvalue[density] and densityvalue[density] or "3").. "\r\n"
	local cmd_masktype			= "/MaskPattern " .. ( masktypevalue[masktype] and masktypevalue[masktype] or "0" ).. "\r\n"
	local cmd_patterneffects	= "/StringEffect " .. ( patterneffectsvalue[patterneffects] and patterneffectsvalue[patterneffects] or "0" ) .. "\r\n"

	return "mark {\r\n<< /MarkingType 2\r\n" .. cmd_color .. cmd_density .. cmd_masktype .. cmd_patterneffects .. output_StringAttributesCmd( info_map, texttype, text, font, position, size, angle, patternrepeat, linespace ) .. ">>\r\n/RDeviceProcSet /ProcSet findresource/SetMarkingType get exec\r\n} stopped cleartomark\r\n"
end

-----------------------------------------------------------------------------
-- Local Function : output_copyguard
-----------------------------------------------------------------------------
local function output_copyguard(	info_map,
									texttype, patterneffects,
									text, font,
									position, size, angle,
									patternrepeat, linespace )
	
	local patterneffectsvalue =
	{
		["normal"]			="0",
		["backgroundonly"]	="1"
	}
	
	local cmd_patterneffects	= "/StringEffect " .. ( patterneffectsvalue[patterneffects] and patterneffectsvalue[patterneffects] or "0" ) .. "\r\n"
	
	return "mark {\r\n<< /MarkingType 1\r\n" .. cmd_patterneffects .. output_StringAttributesCmd( info_map, texttype, text, font, position, size, angle, patternrepeat, linespace  ) .. ">>\r\n/RDeviceProcSet /ProcSet findresource/SetMarkingType get exec\r\n} stopped cleartomark\r\n"
end

-----------------------------------------------------------------------------
-- Local Function : output_cc_print
-----------------------------------------------------------------------------
local function output_cc_print( info_map, cc_print, cc_type )

	local cdb = Rice.CapsDB
	
	if( "on" == cc_print ) then
		local postscript_type = cdb.get("postscript_type")
		if( ( nil == postscript_type )				or
			( "adobepostscript" == postscript_type ) )then
			if( "copyguard" == cc_type) then
				return output_copyguard( 	info_map,
											cdb.get("cc_copyguard_texttype"),
											cdb.get("cc_copyguard_patterneffects"),
											cdb.get("cc_copyguard_text"),
											cdb.get("cc_copyguard_font"),
											cdb.get("cc_copyguard_position"),
											cdb.get("cc_copyguard_size"),
											cdb.get("cc_copyguard_angle"),
											cdb.get("cc_copyguard_repeat"),
											cdb.get("cc_copyguard_linespace")	)
			else
				return output_preventioncopypattern( 	info_map,
														cdb.get("cc_preventioncopypattern_color"),
														cdb.get("cc_preventioncopypattern_density"),
														cdb.get("cc_preventioncopypattern_masktype"),
														cdb.get("cc_preventioncopypattern_patterneffects"),
														cdb.get("cc_preventioncopypattern_texttype"),
														cdb.get("cc_preventioncopypattern_text"),
														cdb.get("cc_preventioncopypattern_font"),
														cdb.get("cc_preventioncopypattern_position"),
														cdb.get("cc_preventioncopypattern_size"),
														cdb.get("cc_preventioncopypattern_angle"),
														cdb.get("cc_preventioncopypattern_repeat"),
														cdb.get("cc_preventioncopypattern_linespace")	)
			end
		else
			if( "copyguard" == cc_type) then
				return output_copyguard( 	info_map,
											cdb.get("cc_copyguard_texttype"),
											cdb.get("cc_copyguard_patterneffects"),
											cdb.get("cc_copyguard_text"),
											cdb.get("cc_copyguard_font_irips"),
											cdb.get("cc_copyguard_position"),
											cdb.get("cc_copyguard_size"),
											cdb.get("cc_copyguard_angle"),
											cdb.get("cc_copyguard_repeat"),
											cdb.get("cc_copyguard_linespace")	)
			else
				return output_preventioncopypattern( 	info_map,
														cdb.get("cc_preventioncopypattern_color"),
														cdb.get("cc_preventioncopypattern_density"),
														cdb.get("cc_preventioncopypattern_masktype"),
														cdb.get("cc_preventioncopypattern_patterneffects"),
														cdb.get("cc_preventioncopypattern_texttype"),
														cdb.get("cc_preventioncopypattern_text"),
														cdb.get("cc_preventioncopypattern_font_irips"),
														cdb.get("cc_preventioncopypattern_position"),
														cdb.get("cc_preventioncopypattern_size"),
														cdb.get("cc_preventioncopypattern_angle"),
														cdb.get("cc_preventioncopypattern_repeat"),
														cdb.get("cc_preventioncopypattern_linespace")	)
			end
		end
	end
	
	return ""


end

-----------------------------------------------------------------------------
-- Local Function : output_nup
-----------------------------------------------------------------------------
local function output_nup(	layout, pageorder, borderline, booklet, pagelayout_distribution, reduceenlarge, booklet_equalmagnification )

	if( "printer" ~= pagelayout_distribution			and
		false == is_layout_booklet( layout, booklet )	and
		false == is_equalmagnification_booklet( layout, booklet, reduceenlarge, booklet_equalmagnification)	) then
		return ""
	end
	
	local nup_tbl =
	{
		["1pagepersheet"]	="1",
		["2pagespersheet"]	="2",
		["4pagespersheet"]	="4",
		["6pagespersheet"]	="6",
		["9pagespersheet"]	="9",
		["16pagespersheet"]	="16"
	}
	
	local order_tbl =
	{
		["rightthendown"]	="0",
		["downthenright"]	="1",
		["leftthendown"]	="2",
		["downthenleft"]	="3",
		["lefttoright"]		="0",
		["righttoleft"]		="2"
	
	}

	local cmd_header = "mark {\r\n<<\r\n"
	local cmd_footer = ">>\r\n/RDeviceProcSet /ProcSet findresource /SetNup get exec\r\n} stopped cleartomark\r\n"
	
	local cmd_nup = "/NupNum " .. ( nup_tbl and ( nup_tbl[layout] and nup_tbl[layout] or "1" ) or "1" ) .. "\r\n"
	local cmd_order = "/Order " .. ( order_tbl and ( order_tbl[pageorder] and order_tbl[pageorder] or "0" ) or "0" ) .. "\r\n"
	local cmd_borderline = "/BorderLine " .. ( "on" ==borderline and "true" or "false" ) .. "\r\n"
	
	return cmd_header .. cmd_nup .. cmd_order .. cmd_borderline .. cmd_footer

end

-----------------------------------------------------------------------------
-- Local Function : output_booklet
-----------------------------------------------------------------------------
local function output_booklet(	info_map, layout, booklet, booklet_pageorder, pagelayout_distribution, booklet_equalmagnification, documentsize, printsize, synchrocut, synchrocut_printsize, reduceenlarge)

	if( "printer" ~= pagelayout_distribution			and
		false == is_layout_booklet( layout, booklet )	and
		false == is_equalmagnification_booklet( layout, booklet, reduceenlarge, booklet_equalmagnification)		) then
		return ""
	end

	local booklet_pageorder_tbl =
	{
		["opentoleftortop"]		="1",
		["opentorightorbottom"]	="2"
	}
	
	--If PrintOn value is different from PageSize value, use PrintOn value
	if("sameasoriginalsize" == printsize ) then
		printsize = documentsize
	end
	
	local printsize_tbl = get_printsize(info_map, documentsize, printsize, synchrocut, synchrocut_printsize)
	
	local cmd_header = "mark {\r\n<<\r\n"
	local cmd_footer = ">>\r\n/RDeviceProcSet /ProcSet findresource /SetBooklet get exec\r\n} stopped cleartomark\r\n"
	
	local cmd_booklet = "/Booklet " .. ( ( "off" ~= booklet and booklet_pageorder_tbl ) and ( booklet_pageorder_tbl[booklet_pageorder] and booklet_pageorder_tbl[booklet_pageorder] or "0" ) or "0") .. "\r\n"

	if( nil ~= booklet_equalmagnification ) then
		local cmd_booklet_scaling = "/Scaling " .. ( ( true == is_equalmagnification_booklet( layout, booklet, reduceenlarge, booklet_equalmagnification)	) and "false" or "true" ) .. "\r\n"
		local cmd_booklet_pagesize = "/PageSize [" .. printsize_tbl['width'] .. " " .. printsize_tbl['height'] .. "]\r\n"

		return cmd_header .. cmd_booklet .. cmd_booklet_scaling .. cmd_booklet_pagesize .. cmd_footer
	else
		return cmd_header .. cmd_booklet .. cmd_footer
	end

end

-----------------------------------------------------------------------------
-- Local Function : output_fold
-----------------------------------------------------------------------------
local function output_fold(	fold, duplex, multisheetfold, foldstamp, edgefold )

	local cmd_fold = "mark{\r\n<<" .. get_fold( fold, foldstamp, edgefold ) .. ">> setpagedevice\r\n} stopped cleartomark\r\n"
	local cmd_foldtype = get_fold_info( fold, "foldtype", nil, nil, nil ) and ( "mark{\r\nuserdict /RPS_MFdict 1 dict put\r\nuserdict /RPS_MFdict get begin\r\n/RPS_MF_FOLDTYPE " .. get_fold_info( fold, "foldtype", nil, nil, nil ) .. " def end\r\n} stopped cleartomark\r\n" ) or ""
	local cmd_faceup = get_fold_info( fold, "faceup", duplex, multisheetfold, nil ) and ( "mark{\r\n<< /OutputFaceUp " .. get_fold_info( fold, "faceup", duplex, multisheetfold, nil ) .. "\r\n>> setpagedevice\r\n} stopped cleartomark\r\n" ) or ""

	return cmd_fold .. cmd_foldtype .. cmd_faceup

end

-----------------------------------------------------------------------------
-- Local Function : output_multisheetfold
-----------------------------------------------------------------------------
local function output_multisheetfold( multisheetfold, fold, physicalpaper_orientation )

	local spread_tbl =
	{
		["insidehalffold"]					= { ["portrait"] = "0",	["landscape"] = "1"	},
		["outsidehalffold"]					= { ["portrait"] = "1",	["landscape"] = "0"	},
		["rightortopinsidespiralfold"]		= { ["portrait"] = "0",	["landscape"] = "1"	},
		["leftorbottominsidespiralfold"]	= { ["portrait"] = "1",	["landscape"] = "0"	},
		["rightortopoutsidespiralfold"]		= { ["portrait"] = "0",	["landscape"] = "1"	},
		["leftorbottomoutsidespiralfold"]	= { ["portrait"] = "1",	["landscape"] = "0"	},
		["outsidezigzagfold"]				= { ["portrait"] = "0",	["landscape"] = "1"	},
		["insidezigzagfold"]				= { ["portrait"] = "1",	["landscape"] = "0"	},
		["rightortopinsidedoublefold"]		= { ["portrait"] = "0",	["landscape"] = "1"	},
		["leftorbottominsidedoublefold"]	= { ["portrait"] = "1",	["landscape"] = "0"	},
		["rightortopoutsidedoublefold"]		= { ["portrait"] = "0",	["landscape"] = "1"	},
		["leftorbottomoutsidedoublefold"]	= { ["portrait"] = "1",	["landscape"] = "0"	},
		["insidedoublegatefold"]			= { ["portrait"] = "0",	["landscape"] = "1"	},
		["outsidedoublegatefold"]			= { ["portrait"] = "1",	["landscape"] = "0"	}
	}

	local spread_value = ( fold and physicalpaper_orientation and spread_tbl[ fold ] and spread_tbl[ fold ][ physicalpaper_orientation ] ) and "/Spread " .. spread_tbl[ fold ][ physicalpaper_orientation ] .. "\r\n" or ""
	
	return "mark\r\nuserdict /RPS_MFdict known{\r\n<< /Fold 1\r\n/FoldDetails\r\n<< /Type 11\r\n/FoldType userdict /RPS_MFdict get /RPS_MF_FOLDTYPE get\r\n" .. spread_value .. "/Stack " .. (("on" == multisheetfold) and "true" or "false") .. ">>\r\n>> setpagedevice\r\n} if\r\ncleartomark\r\n"

end

-----------------------------------------------------------------------------
-- Local Function : output_squarefold
-----------------------------------------------------------------------------
local function output_squarefold( squarefold )

	if( nil == squarefold or "" == squarefold) then
		return ""
	end

	local tbl =
	{
		['on']		= "(on) /RDeviceProcSet /ProcSet findresource /SetSquareFold get exec",
		['off']		= "(off) /RDeviceProcSet /ProcSet findresource /SetSquareFold get exec"
	}

	return add_feature_tags("squarefold" , squarefold , (nil == tbl[squarefold]) and "" or tbl[squarefold] )

end

-----------------------------------------------------------------------------
-- Local Function : output_unidirectionalprinting
-----------------------------------------------------------------------------
local function output_unidirectionalprinting(	unidirectionalprinting )

	if( nil == unidirectionalprinting or "" == unidirectionalprinting ) then
		return ""
	end

	local tbl =
	{
		['off']		= "(auto) /RDeviceProcSet /ProcSet findresource /SetPrintHeadDirection get exec",
		['on']		= "(unidirectional) /RDeviceProcSet /ProcSet findresource /SetPrintHeadDirection get exec"
	}

	return add_feature_tags("unidirectionalprinting" , unidirectionalprinting , (nil == tbl[unidirectionalprinting]) and "" or tbl[unidirectionalprinting] )

end

-----------------------------------------------------------------------------
-- Local Function : output_fusersetting
-----------------------------------------------------------------------------
local function output_fusersetting( fusersetting )

	if( nil == fusersetting or "" == fusersetting) then
		return ""
	end

	local tbl =
	{
		['standard']		= "(auto) /RDeviceProcSet /ProcSet findresource /SetPrintHeadDirection get exec",
		['highquality']		= "(unidirectional) /RDeviceProcSet /ProcSet findresource /SetPrintHeadDirection get exec"
	}


	return add_feature_tags("fusersetting" , fusersetting , (nil == tbl[fusersetting]) and "" or tbl[fusersetting] )

end

-----------------------------------------------------------------------------
-- Local Function : output_qualitybalancepreset
-----------------------------------------------------------------------------
local function output_qualitybalancepreset(	qualitybalancepreset )

	if( nil == qualitybalancepreset or "" == qualitybalancepreset ) then
		return ""
	end

	local tbl =
	{
		['highspeedlinedraw']	= "(fastline) /RDeviceProcSet /ProcSet findresource /SetQualityMode get exec",
		['highspeed']			= "(fast) /RDeviceProcSet /ProcSet findresource /SetQualityMode get exec",
		['speedpriority']		= "(standard) /RDeviceProcSet /ProcSet findresource /SetQualityMode get exec",
		['qualitypriority']		= "(fine) /RDeviceProcSet /ProcSet findresource /SetQualityMode get exec"
	}

	return add_feature_tags("qualitybalancepreset" , qualitybalancepreset , (nil == tbl[qualitybalancepreset]) and "" or tbl[qualitybalancepreset] )

end

--------------------------------------------------------------------------------
-- Local Function : output_balanceadjustment
--------------------------------------------------------------------------------
local function output_balanceadjustment(	all_brightness, all_contrast, all_colorbalance_cyan, all_colorbalance_magenta, all_colorbalance_yellow, all_colorbalance_black, printer_colorspec )

	local function range_check( value )
		value = ( nil ~= tonumber(value) ) and tonumber(value) or 0
		return ((-50 <= value) and (50 >= value)) and value or 0
	end
	
	local function setup_gamma_cmd(all_colorbalance_cyan, all_colorbalance_magenta, all_colorbalance_yellow, all_colorbalance_black, printer_colorspec)

		local cyan		= ( "monochrome" == printer_colorspec	or "twotoner" == printer_colorspec )	and 0	or range_check( all_colorbalance_cyan )
		local magenta	= ( "monochrome" == printer_colorspec	or "twotoner" == printer_colorspec )	and 0	or range_check( all_colorbalance_magenta )
		local yellow	= ( "monochrome" == printer_colorspec	or "twotoner" == printer_colorspec )	and 0	or range_check( all_colorbalance_yellow )

		local cmd_header	= "/Gamma <<\r\n"
		local cmd_cyan		= " /Cyan " .. cyan .. "\r\n"
		local cmd_magenta	= " /Magenta " .. magenta .. "\r\n"
		local cmd_yellow	= " /Yellow " .. yellow .. "\r\n"
		local cmd_black		= " /Black " .. range_check( all_colorbalance_black ) .. "\r\n"
		local cmd_footer	= ">>\r\n"
		
		return cmd_header .. cmd_cyan .. cmd_magenta .. cmd_yellow .. cmd_black .. cmd_footer

	end
	
	local cmd_header			= "mark{\r\n<<\r\n"
	local cmd_all_brightness	= "/Brightness " .. range_check( all_brightness ) .. "\r\n"
	local cmd_all_contrast		= "/Contrast " .. range_check( all_contrast ) .. "\r\n"
	local cmd_gamma				= setup_gamma_cmd(	all_colorbalance_cyan, all_colorbalance_magenta, all_colorbalance_yellow, all_colorbalance_black, printer_colorspec )
	local cmd_footer			= ">>\r\n/RDeviceProcSet/ProcSet findresource/SetColorBalance get exec\r\n}stopped cleartomark\r\n\r\n"

	return cmd_header .. cmd_all_brightness .. cmd_all_contrast .. cmd_gamma .. cmd_footer

end

--------------------------------------------------------------------------------
-- Local Function : output_synchrocut
--------------------------------------------------------------------------------
local function output_synchrocut( synchrocut, synchrocut_margin_top_mm, synchrocut_margin_bottom_mm, synchrocut_margin_left_mm, synchrocut_margin_right_mm, synchrocut_position, synchrocut_feeddirection )

	if( nil == synchrocut					or "" == synchrocut						or
		nil == synchrocut_margin_top_mm		or "" == synchrocut_margin_top_mm		or
		nil == synchrocut_margin_bottom_mm	or "" == synchrocut_margin_bottom_mm	or
		nil == synchrocut_margin_left_mm	or "" == synchrocut_margin_left_mm		or
		nil == synchrocut_margin_right_mm	or "" == synchrocut_margin_right_mm		or
		nil == synchrocut_position			or "" == synchrocut_position			or
		nil == synchrocut_feeddirection		or "" == synchrocut_feeddirection		) then
		return ""
	end

	local synchrocut_enable = ( "on" == synchrocut) and true or false

	local function range_check( value )
		value = ( nil ~= tonumber(value) ) and tonumber(value) or 0
		return ((0 <= value) and (200 >= value)) and value or 0
	end

	local function setup_margin_cmd( synchrocut_enable, synchrocut_margin_top_mm, synchrocut_margin_bottom_mm, synchrocut_margin_left_mm, synchrocut_margin_right_mm )

		local cmd_header	= "/Margin <<\r\n"
		local cmd_top		= " /Top " .. ( synchrocut_enable and range_check(synchrocut_margin_top_mm) or 0 ) .. "\r\n"
		local cmd_bottom	= " /Bottom " .. ( synchrocut_enable and range_check(synchrocut_margin_bottom_mm) or 0 ) .. "\r\n"
		local cmd_left		= " /Left " .. ( synchrocut_enable and range_check(synchrocut_margin_left_mm) or 0 ) .. "\r\n"
		local cmd_right		= " /Right " .. ( synchrocut_enable and range_check(synchrocut_margin_right_mm) or 0 ) .. "\r\n"
		local cmd_footer	= ">>\r\n"

		return cmd_header .. cmd_top .. cmd_bottom .. cmd_left .. cmd_right .. cmd_footer

	end

	local position = function( synchrocut_position )
		local tbl =
		{
			["left"]	=	"0",
			["center"]	=	"1",
			["right"]	=	"2"
		}

		if( nil ~= tbl[synchrocut_position] ) then
			return tbl[synchrocut_position]
		else
			return tbl["left"]
		end
	end

	local feeddirection = function( synchrocut_feeddirection )
		local tbl =
		{
			["lef"]	=	"0",
			["sef"]	=	"1"
		}

		if( nil ~= tbl[synchrocut_feeddirection] ) then
			return tbl[synchrocut_feeddirection]
		else
			return tbl["lef"]
		end
	end

	local cmd_header		= "<<\r\n"
	local cmd_enable		= "/Enable " .. ( synchrocut_enable and "true" or "false" ) .. "\r\n"
	local cmd_margin		= setup_margin_cmd( synchrocut_enable, synchrocut_margin_top_mm, synchrocut_margin_bottom_mm, synchrocut_margin_left_mm, synchrocut_margin_right_mm )
	local cmd_cornerlock	= "/CornerLock " .. ( synchrocut_enable and position( synchrocut_position ) or "0" ) .. "\r\n"
	local cmd_leading		= "/Leading " .. ( synchrocut_enable and feeddirection( synchrocut_feeddirection ) or "0" ) .. "\r\n"
	local cmd_footer		= ">>\r\n/RDeviceProcSet /ProcSet findresource/SetSynchroCut get exec"

	local command 			= cmd_header .. cmd_enable .. cmd_margin .. cmd_cornerlock .. cmd_leading .. cmd_footer

	return add_feature_tags("synchrocut" , ( synchrocut_enable and "true" or "false" ) , command)

end

-----------------------------------------------------------------------------
-- Local Function : output_genericfunction
-----------------------------------------------------------------------------
local function output_genericfunction( genericfunction )

	if( nil == genericfunction or "" == genericfunction ) then
		return ""
	end

	local genericfunction_tbl =
	{
		["off"] 				= "/ExtFinFunction null",
		["genericfunction1"] 	= "/ExtFinFunction 1",
		["genericfunction2"] 	= "/ExtFinFunction 2",
		["genericfunction3"] 	= "/ExtFinFunction 3",
		["genericfunction4"] 	= "/ExtFinFunction 4",
		["genericfunction5"] 	= "/ExtFinFunction 5",
		["genericfunction6"] 	= "/ExtFinFunction 6",
		["genericfunction7"] 	= "/ExtFinFunction 7",
		["genericfunction8"] 	= "/ExtFinFunction 8",
		["genericfunction9"] 	= "/ExtFinFunction 9",
	}

	local cmd_header = "<<\r\n"
	local cmd_footer = "\r\n>>\r\n/RDeviceProcSet /ProcSet findresource/SetGeneralInfo get exec"

	return add_feature_tags("genericfunction" , genericfunction , ( nil == genericfunction_tbl[genericfunction]) and "" or cmd_header .. genericfunction_tbl[genericfunction] .. cmd_footer )

end

-----------------------------------------------------------------------------
-- Local Function : output_secretnumbering
-----------------------------------------------------------------------------
local function output_secretnumbering( secretnumbering, secretnumbering_size, secretnumbering_density, secretnumbering_color, secretnumbering_number )

	if(	nil == secretnumbering			or "on" ~= secretnumbering			or
		nil == secretnumbering_size		or "" == secretnumbering_size		or
		nil == secretnumbering_density	or "" == secretnumbering_density	or
		nil == secretnumbering_number	or "" == secretnumbering_number		) then
		return ""
	end

	local size = function( secretnumbering_size )
		local tbl =
		{
			["small"]	=	"0",
			["middle"]	=	"1",
			["large"]	=	"2"
		}

		if( nil ~= tbl[secretnumbering_size] ) then
			return tbl[secretnumbering_size]
		else
			return tbl["middle"]
		end
	end

	local density = function( secretnumbering_density )
		local tbl =
		{
			["light"]	=		"0",
			["middle"]	=		"1",
			["littledark"]	=	"2",
			["dark"]	=		"3"
		}

		if( nil ~= tbl[secretnumbering_density] ) then
			return tbl[secretnumbering_density]
		else
			return tbl["middle"]
		end
	end

	local color = function( secretnumbering_color )
		local tbl =
		{
			["black"]	=	"0",
			["cyan"]	=	"1",
			["magenta"]	=	"2",
			["yellow"]	=	"3",
			["red"]	=		"4",
			["green"]	=	"5",
			["blue"]	=	"6"
		}

		if( nil ~= tbl[secretnumbering_color] ) then
			return tbl[secretnumbering_color]
		else
			return tbl["black"]
		end
	end

	local cmd_header	= "<<\r\n"
	local cmd_enable	= "/Enable true\r\n"
	local cmd_fontsize	= "/FontSize " .. size( secretnumbering_size ) .. "\r\n"
	local cmd_density	= "/Density " .. density( secretnumbering_density ) .. "\r\n"
	local cmd_color		= "/Color " .. color( secretnumbering_color ) .. "\r\n"
	local cmd_number	= "/InitialNumber " .. secretnumbering_number .. "\r\n"
	local cmd_footer	= ">>\r\n/RDeviceProcSet /ProcSet findresource /SetSecurityNumbering get exec"

	local command		= cmd_header .. cmd_enable .. cmd_fontsize .. cmd_density .. cmd_color .. cmd_number .. cmd_footer

	return add_feature_tags( "secretnumbering" , secretnumbering , command )

end

-----------------------------------------------------------------------------
-- Local Function : output_postscript_type
-----------------------------------------------------------------------------
local function output_postscript_type( value )

	if( nil == value or "" == value) then
		return ""
	end

	local tbl =
	{
		["adobepostscript"]	= "0",
		["emulationpostscript"]	= "1"
	}

	return "mark {userdict /86f6369627 known { userdict /86f6369627 get /+DrvMode " .. ( nil == tbl[value] and "1" or tbl[value] ) .. " put } if } stopped cleartomark\r\n"

end

--------------------------------------------------------------------------------
--	Function	:	beginprolog_command
--------------------------------------------------------------------------------
function beginprolog_command( devmode, info_map )

	local commands = {}
	local cdb = Rice.CapsDB

	if( "eps" ~= cdb.get("psoutputoption" ) ) then
		commands[#commands+1] = output_postscript_type( cdb.get("postscript_type") )
		
		commands[#commands+1] = output_charset(info_map, 
												cdb.get("gb18030support"), cdb.get("utf8mode"), cdb.get("output_utf8_rtl_language") )

		commands[#commands+1] = output_jobtype_defs()

	    commands[#commands+1] = output_jobtype(	info_map,
												cdb.get("jobtype"),
												cdb.get("userid"), cdb.get("usercode"), cdb.get("username"),
												cdb.get("filename"), cdb.get("password"),
												cdb.get("documentserver_filename"), cdb.get("documentserver_password"),
												cdb.get("userauthentication"),
												cdb.get("loginusername"), cdb.get("loginpassword"),
												cdb.get("controllerplatformtype"),
												cdb.get("scheduledprint"),
												cdb.get("scheduledprint_hour"),
												cdb.get("scheduledprint_minute"),
												cdb.get("documentserver_foldernumber"),
												cdb.get("documentserver_folderpassword"),
												cdb.get("gb18030support"),
												cdb.get("userid_type"),cdb.get("userid_loginname"),
												cdb.get("utf8mode"),
												cdb.get("output_utf8_rtl_language")
												)

		commands[#commands+1] = output_outputtray( cdb.get("outputtray") )

		commands[#commands+1] = output_secretnumbering(	cdb.get("secretnumbering"), 
														cdb.get("secretnumbering_size"), 
														cdb.get("secretnumbering_density"), 
														cdb.get("secretnumbering_color"), 
														cdb.get("secretnumbering_number") )

		commands[#commands+1] = output_bannerpageprint(	info_map,
														cdb.get("bannerpageprint"),
														cdb.get("bannerpageprint_inputtray"),
														cdb.get("bannerpageprint_mediatype"),
														cdb.get("lctmediaposition"),
														cdb.get("traymediaposition_type"),
														cdb.get("gb18030support"),
														cdb.get("utf8mode"),
														cdb.get("output_utf8_rtl_language")
														)
	end

    return table.concat(commands)
end

--------------------------------------------------------------------------------
--	Function	:	beginsetup_command
--------------------------------------------------------------------------------
function beginsetup_command( devmode, info_map )

	local commands = {}
	local cdb = Rice.CapsDB

	if( "eps" ~= cdb.get("psoutputoption") ) then
		commands[#commands+1]  		= output_watermarkpattern( cdb.get("pwmk_print"), cdb.get("pwmk_textstyle"), cdb.get("pwmk_opaque"), cdb.get("pwmk_color"), cdb.get("pwmk_density") )

		commands[#commands+1]		= output_header_footer_pattern( cdb.get("phf_print") )
		
		commands[#commands+1] 		= output_printer_fittoprintsize( cdb.get("printer_fittoprintsize") )

		commands[#commands+1]  		= output_documentsize( info_map, cdb.get("documentsize"), cdb.get("printsize"), cdb.get("synchrocut"), cdb.get("synchrocut_printsize"), cdb.get("layout"), cdb.get("booklet"), cdb.get("reduceenlarge"), cdb.get("booklet_equalmagnification"), cdb.get("pagelayout_distribution") )

		commands[#commands+1]  		= output_edgetoedgeprint( cdb.get("edgetoedgeprint") )

		commands[#commands+1]		= output_standardpages_inputtray( cdb.get("standardpages_inputtray"), cdb.get("transparency_inputtray"), cdb.get("slipsheet"), cdb.get("lctmediaposition"), cdb.get("traymediaposition_type") )

		commands[#commands+1]		= output_setcoversheet(	cdb.get("frontcover"), cdb.get("frontcover_inputtray"),cdb.get("frontcover_printmode"),
															cdb.get("backcover"), cdb.get("backcover_inputtray"),cdb.get("backcover_printmode"),
															cdb.get("staple"), cdb.get("centerstaple_swapcover") , cdb.get("lctmediaposition"), cdb.get("traymediaposition_type") )

		commands[#commands+1]		= output_setslipsheet( cdb.get("slipsheet"), cdb.get("slipsheet_inputtray"), cdb.get("slipsheet_printmode"), cdb.get("lctmediaposition"), cdb.get("traymediaposition_type" ) )
		
		commands[#commands+1]		= output_setchaptering(	cdb.get("chapterpages"), cdb.get("chapterpages_setting"), cdb.get("lctmediaposition"), cdb.get("traymediaposition_type") )

		commands[#commands+1]		= output_resolution( cdb.get("resolution") )

		commands[#commands+1]		= output_gradation( cdb.get("gradation") )

		commands[#commands+1]		= output_tonersaving( cdb.get("tonersaving"), cdb.get("edgesmoothing"), cdb.get("economycolormode") )

		commands[#commands+1]		= output_economycolormode( cdb.get("economycolormode"), cdb.get("tonersaving"), cdb.get("edgesmoothing") )

		commands[#commands+1]		= output_edgesmoothing( cdb.get("edgesmoothing"), cdb.get("tonersaving"), cdb.get("economycolormode") )

		commands[#commands+1]		= output_duplex( cdb.get("duplex"), cdb.get("booklet"),  cdb.get("orientation"), cdb.get("layout"))

		commands[#commands+1]		= output_colorbw( cdb.get("colorbw") )

		commands[#commands+1]		= output_rgbrevision( cdb.get("rgbrevision") )

		commands[#commands+1]		= output_colorprofile( info_map, cdb.get("colorprofile") )

		commands[#commands+1]		= output_dithering( cdb.get("dithering") )

		commands[#commands+1]		= output_grayreproduction( cdb.get("grayreproduction") )

		commands[#commands+1]		= output_blackoverprint( cdb.get("blackoverprint") )

		commands[#commands+1]		= output_separateintocmyk(	cdb.get("separateintocmyk_cyan"), cdb.get("separateintocmyk_magenta"),
																cdb.get("separateintocmyk_yellow"), cdb.get("separateintocmyk_black"),
																cdb.get("colorbw"), cdb.get("coloruniversaldesign"), cdb.get("synchrocut") )

		commands[#commands+1]		= output_cmyksimulationprofile( cdb.get("cmyksimulationprofile") )

		commands[#commands+1]		= output_imagesmoothing( cdb.get("imagesmoothing") )

		commands[#commands+1]		= output_rotated( cdb.get("rotated") )

		commands[#commands+1]		= output_faceup( cdb.get("faceup"), cdb.get("fold") )

		commands[#commands+1]		= output_orientationoverride( cdb.get("orientationoverride"), cdb.get("orientation"), cdb.get("layout"), cdb.get("booklet"), cdb.get("pagelayout_distribution") )

		commands[#commands+1]		= output_standardpages_mediatype( info_map, cdb.get("standardpages_mediatype"), cdb.get("transparency_mediatype"), cdb.get("slipsheet"), cdb.get("gb18030support"), cdb.get("utf8mode"), cdb.get("output_utf8_rtl_language") )

		commands[#commands+1]		= output_collate( devmode['dmCollate'], cdb.get("collate"), cdb.get("enablecollate") )

		commands[#commands+1]		= output_joboffset( devmode['dmCollate'], cdb.get("collate"), cdb.get("enablecollate"), cdb.get("joboffset") )

		commands[#commands+1]		= output_staple( cdb.get("staple") )

		commands[#commands+1]		= output_punch( cdb.get("punch"), cdb.get("foldpunch"), cdb.get("orientation"), cdb.get("layout") )

		commands[#commands+1]		= output_foldpunch( cdb.get("punch"), cdb.get("foldpunch") )

		commands[#commands+1]		= output_ringbinding( cdb.get("ringbinding"), cdb.get("orientation"), cdb.get("layout"), cdb.get("perfectbinding"), cdb.get("twinloopbinding") )

		commands[#commands+1]		= output_twinloopbinding( cdb.get("twinloopbinding"), cdb.get("orientation"), cdb.get("layout"), cdb.get("perfectbinding"), cdb.get("ringbinding") )

		commands[#commands+1]		= output_trimming( cdb.get("trimming"), cdb.get("perfectbinding") )

		commands[#commands+1]		= output_trimming_cutedge( cdb.get("trimming"), cdb.get("trimming_cutedge_mm") , cdb.get("disable_cutedge_option"))

		commands[#commands+1]		= output_perfectbinding(	info_map,
																cdb.get("perfectbinding"), cdb.get("documentsize"), cdb.get("printsize"),
																cdb.get("standardpages_inputtray"),
																cdb.get("perfectbinding_inputtray"), cdb.get("perfectbinding_direction"), cdb.get("orientation"), cdb.get("layout"),
																cdb.get("perfectbinding_imgpos_along_mm"), cdb.get("perfectbinding_imgpos_across_mm"), cdb.get("perfectbinding_coversheetprint"),
																cdb.get("perfectbinding_coversheet_imgpos_along_mm"), cdb.get("perfectbinding_coversheet_imgpos_across_mm"),
																cdb.get("ringbinding"), cdb.get("lctmediaposition"), cdb.get("traymediaposition_type"), cdb.get("twinloopbinding") )

		commands[#commands+1]		= output_perfectbinding_cuttype(	cdb.get("perfectbinding_cuttype"),
																		cdb.get("perfectbinding_cutpos_length_mm"),
																		cdb.get("perfectbinding_cutpos_finishing_size"), cdb.get("perfectbinding_cutpos_customsize_width_mm"), cdb.get("perfectbinding_cutpos_customsize_length_mm"),
																		cdb.get("perfectbinding_cutpos_along_mm"),
																		cdb.get("perfectbinding_cutpos_head_mm"), cdb.get("perfectbinding_cutpos_tail_mm"), cdb.get("perfectbinding_cutpos_fore_mm"),
																		cdb.get("perfectbinding"), cdb.get("trimming") )

		commands[#commands+1]		= output_coloruniversaldesign( cdb.get("coloruniversaldesign") )

		commands[#commands+1]		= output_cc_print( info_map, cdb.get("cc_print"), cdb.get("cc_type") )

		commands[#commands+1]		= output_nup( cdb.get("layout"), cdb.get("pageorder"), cdb.get("drawframeborder"), cdb.get("booklet"), cdb.get("pagelayout_distribution"), cdb.get("reduceenlarge"), cdb.get("booklet_equalmagnification") )

		commands[#commands+1]		= output_booklet(  info_map, cdb.get("layout"), cdb.get("booklet"), cdb.get("booklet_pageorder"), cdb.get("pagelayout_distribution"), cdb.get("booklet_equalmagnification"), cdb.get("documentsize"), cdb.get("printsize"), cdb.get("synchrocut"), cdb.get("synchrocut_printsize"), cdb.get("reduceenlarge") ) 

		commands[#commands+1]		= output_fold( cdb.get("fold"), cdb.get("duplex"), cdb.get("multisheetfold"), cdb.get("foldstamp"), cdb.get("edgefold") ) 

		commands[#commands+1]		= output_multisheetfold( cdb.get("multisheetfold"), cdb.get("fold"), get_physicalpaper_orientation( cdb.get("orientation"), cdb.get("layout") ) )

		commands[#commands+1]		= output_squarefold( cdb.get("squarefold") )

		commands[#commands+1]		= output_unidirectionalprinting( cdb.get("unidirectionalprinting") )

		commands[#commands+1]		= output_fusersetting( cdb.get("fusersetting") )

		commands[#commands+1]		= output_qualitybalancepreset( cdb.get("qualitybalancepreset") )

	    commands[#commands+1]		= output_balanceadjustment( cdb.get ("all_brightness"), cdb.get("all_contrast"), cdb.get("all_colorbalance_cyan"), cdb.get("all_colorbalance_magenta"), cdb.get("all_colorbalance_yellow"), cdb.get("all_colorbalance_black"), cdb.get("printer_colorspec") )

	    commands[#commands+1]		= output_synchrocut(	cdb.get("synchrocut"),
															cdb.get("synchrocut_margin_top_mm"),
															cdb.get("synchrocut_margin_bottom_mm"),
															cdb.get("synchrocut_margin_left_mm"),
															cdb.get("synchrocut_margin_right_mm"),
															cdb.get("synchrocut_position"),
															cdb.get("synchrocut_feeddirection") )

		commands[#commands+1]		= output_genericfunction( cdb.get("genericfunction") )

	end

	return table.concat(commands)
end

--------------------------------------------------------------------------------
--	Function	:	endsetup_command
--------------------------------------------------------------------------------
function endsetup_command( devmode )

	local commands = {}
	local cdb = Rice.CapsDB

	if( "eps" ~= cdb.get("psoutputoption" ) ) then
		if( "landscape" == cdb.get("orientation") ) then
			if( "4pagespersheet" == cdb.get("layout") or
				"9pagespersheet" == cdb.get("layout") or
				"16pagespersheet" == cdb.get("layout")) then

				commands[#commands+1]	= output_nuprotate()

			end
		end
	end

    return table.concat(commands)
end

--------------------------------------------------------------------------------
--	Local Function	:	beginpagesetup_resetting_command
--------------------------------------------------------------------------------
local function beginpagesetup_resetting_command( commands, page_info )
	local cdb = Rice.CapsDB

	commands[#commands+1]  		= output_edgetoedgeprint( cdb.get("edgetoedgeprint") )

	commands[#commands+1]		= output_resolution( cdb.get("resolution") )

	commands[#commands+1]		= output_gradation( cdb.get("gradation") )

	commands[#commands+1]		= output_tonersaving( cdb.get("tonersaving"), cdb.get("edgesmoothing"), cdb.get("economycolormode") )

	commands[#commands+1]		= output_economycolormode( cdb.get("economycolormode"), cdb.get("tonersaving"), cdb.get("edgesmoothing") )

	commands[#commands+1]		= output_edgesmoothing( cdb.get("edgesmoothing"), cdb.get("tonersaving"), cdb.get("economycolormode") )

	commands[#commands+1]		= output_colorbw( cdb.get("colorbw") )

	commands[#commands+1]		= output_rgbrevision( cdb.get("rgbrevision") )

	commands[#commands+1]		= output_colorprofile( page_info, cdb.get("colorprofile") )

	commands[#commands+1]		= output_dithering( cdb.get("dithering") )

	commands[#commands+1]		= output_grayreproduction( cdb.get("grayreproduction") )

	commands[#commands+1]		= output_blackoverprint( cdb.get("blackoverprint") )

	commands[#commands+1]		= output_separateintocmyk(	cdb.get("separateintocmyk_cyan"), cdb.get("separateintocmyk_magenta"),
															cdb.get("separateintocmyk_yellow"), cdb.get("separateintocmyk_black"),
															cdb.get("colorbw"), cdb.get("coloruniversaldesign"), cdb.get("synchrocut") )

	commands[#commands+1]		= output_cmyksimulationprofile( cdb.get("cmyksimulationprofile") )

	commands[#commands+1]		= output_imagesmoothing( cdb.get("imagesmoothing") )

	commands[#commands+1]		= output_rotated( cdb.get("rotated") )

	commands[#commands+1]		= output_orientationoverride( cdb.get("orientationoverride"), cdb.get("orientation"), cdb.get("layout"), cdb.get("booklet") )

	commands[#commands+1]		= output_balanceadjustment( cdb.get ("all_brightness"), cdb.get("all_contrast"), cdb.get("all_colorbalance_cyan"), cdb.get("all_colorbalance_magenta"), cdb.get("all_colorbalance_yellow"), cdb.get("all_colorbalance_black"), cdb.get("printer_colorspec") )

    return ""
end

--------------------------------------------------------------------------------
--	Local Function	:	beginpagesetup_papersizechanged_command
--------------------------------------------------------------------------------
local function beginpagesetup_papersizechanged_command( page_info, commands )
	local cdb = Rice.CapsDB

	if( "sameasoriginalsize" ~= cdb.get("printsize")	and 
		false == is_printer_pagelayout( cdb.get("layout"), cdb.get("booklet"), cdb.get("pagelayout_distribution"), cdb.get("booklet_equalmagnification"), cdb.get("reduceenlarge")) ) then
		return ""
	end

	commands[#commands+1]  		= output_documentsize( page_info, cdb.get("documentsize"), cdb.get("printsize"), cdb.get("synchrocut"), cdb.get("synchrocut_printsize"), cdb.get("layout"), cdb.get("booklet"), cdb.get("reduceenlarge"), cdb.get("booklet_equalmagnification"), cdb.get("pagelayout_distribution") )

    return ""
end

--------------------------------------------------------------------------------
--	Local Function	:	beginpagesetup_synchrocutchanged_command
--------------------------------------------------------------------------------
local function beginpagesetup_synchrocutchanged_command( commands )
	local cdb = Rice.CapsDB

	commands[#commands+1]		= output_synchrocut(	cdb.get("synchrocut"),
														cdb.get("synchrocut_margin_top_mm"),
														cdb.get("synchrocut_margin_bottom_mm"),
														cdb.get("synchrocut_margin_left_mm"),
														cdb.get("synchrocut_margin_right_mm"),
														cdb.get("synchrocut_position"),
														cdb.get("synchrocut_feeddirection") )

    return ""
end

--------------------------------------------------------------------------------
--	Local Function	:	beginpagesetup_inputtraychanged_command
--------------------------------------------------------------------------------
local function beginpagesetup_inputtraychanged_command( commands )
	local cdb = Rice.CapsDB

	commands[#commands+1]		= output_standardpages_inputtray( cdb.get("standardpages_inputtray"), cdb.get("transparency_inputtray"), cdb.get("slipsheet"), cdb.get("lctmediaposition"), cdb.get("traymediaposition_type") )

    return ""
end

--------------------------------------------------------------------------------
--	Local Function	:	beginpagesetup_mediatypechanged_command
--------------------------------------------------------------------------------
local function beginpagesetup_mediatypechanged_command( page_info, commands )
	local cdb = Rice.CapsDB

	commands[#commands+1]		= output_standardpages_mediatype( page_info, cdb.get("standardpages_mediatype"), cdb.get("transparency_mediatype"), cdb.get("slipsheet"), cdb.get("gb18030support"), cdb.get("utf8mode"), cdb.get("output_utf8_rtl_language") )

    return ""
end

--------------------------------------------------------------------------------
--	Local Function	:	beginpagesetup_duplexchanged_command
--------------------------------------------------------------------------------
local function beginpagesetup_duplexchanged_command( commands )
	local cdb = Rice.CapsDB

	commands[#commands+1]		= output_duplex( cdb.get("duplex"), cdb.get("booklet"),  cdb.get("orientation"), cdb.get("layout"))

    return ""
end

--------------------------------------------------------------------------------
--	Local Function	:	beginpagesetup_watermark_command
--------------------------------------------------------------------------------
local function beginpagesetup_watermark_command( commands )
	local cdb = Rice.CapsDB

	if( "on" == cdb.get("pwmk_print") ) then
		commands[#commands+1] = "userdict begin\r\n" .. output_watermarkpattern( cdb.get("pwmk_print"), cdb.get("pwmk_textstyle"), cdb.get("pwmk_opaque"), cdb.get("pwmk_color"), cdb.get("pwmk_density") ) .. "end\r\n"
	end

    return ""
end

--------------------------------------------------------------------------------
--	Local Function	:	beginpagesetup_header_footer_command
--------------------------------------------------------------------------------
local function beginpagesetup_header_footer_command( commands )
	local cdb = Rice.CapsDB

	if( "on" == cdb.get("phf_print") ) then
		commands[#commands+1] = "userdict begin\r\n" .. output_header_footer_pattern( cdb.get("phf_print") ) .. "end\r\n"
	end

    return ""
end

--------------------------------------------------------------------------------
--	Function	:	beginpagesetup_command
--------------------------------------------------------------------------------
function beginpagesetup_command( devmode, page_info )
	local cdb = Rice.CapsDB

	local commands = {}

	local psoutputoption = cdb.get("psoutputoption")

	if( false == is_driver_pagelayout( cdb.get("layout"), cdb.get("booklet"), cdb.get("pagelayout_distribution"), cdb.get("booklet_equalmagnification"), cdb.get("reduceenlarge") ) ) then
		if( "eps" ~= psoutputoption ) then
			if( ("false" == page_info['is_proc_endpage'])  and ("true" == page_info['outputAppendRenderingPattern']) ) then
				beginpagesetup_watermark_command( commands )
				beginpagesetup_header_footer_command( commands )
			end

			if( "1pagepersheet" == cdb.get("layout")								and
				"booklet" ~= cdb.get("booklet")										and
				false == is_layout_booklet( cdb.get("layout"), cdb.get("booklet") )	)then

				if( ("false" == page_info['is_proc_endpage'])  and ("true" == page_info['inputTrayChanged']) ) then
					beginpagesetup_inputtraychanged_command( commands )
				end

				if( ("false" == page_info['is_proc_endpage'])  and ("true" == page_info['mediaTypeChanged']) ) then
					beginpagesetup_mediatypechanged_command( page_info, commands )
				end

				if( ("false" == page_info['is_proc_endpage'])  and (("true" == page_info['paperSizeChanged']) or ("true" == page_info['inputTrayChanged']) or ("true" == page_info['mediaTypeChanged'])) ) then
					beginpagesetup_resetting_command( commands, page_info )
				end

				if( ("false" == page_info['is_proc_endpage'])  and ("true" == page_info['duplexChanged']) ) then
					beginpagesetup_duplexchanged_command( commands )
				end
			end

			if( ("false" == page_info['is_proc_endpage'])  and ("true" == page_info['paperSizeChanged']) ) then
				beginpagesetup_papersizechanged_command( page_info, commands )
			end

			if( ("false" == page_info['is_proc_endpage'])  and ("true" == page_info['synchrocutChanged']) ) then
				beginpagesetup_synchrocutchanged_command( commands )
			end
		end

		commands[#commands+1]		= output_beginpagesetup_ctm( cdb.get("edgetoedgeprint"),
																 cdb.get("orientation")		)

		if( "eps" ~= psoutputoption ) then
			commands[#commands+1]		= output_rotate180( cdb.get("orientation"),
															cdb.get("layout"),
															cdb.get("booklet"),
															cdb.get("pagelayout_distribution"),
															cdb.get("mirroring"),
															cdb.get("documentsize"),
															cdb.get("printsize"),
															cdb.get("booklet_equalmagnification"),
															cdb.get("reduceenlarge") )
		end
	else
		commands[#commands+1]		= output_drawframeborder(	cdb.get("drawframeborder"),
																cdb.get("booklet"),
																page_info )
	end

    return table.concat(commands)
end

--------------------------------------------------------------------------------
--	Function	:	endpagesetup_command
--------------------------------------------------------------------------------
function endpagesetup_command( devmode )

	local commands = {}
	local cdb = Rice.CapsDB

	commands[#commands+1]	= output_ctm(	cdb.get("documentsize"), cdb.get("printsize"), cdb.get("orientation"),
											cdb.get("resolution"), cdb.get("reduceenlarge"),
											cdb.get("zoomvalue"),
											cdb.get("mirroring"), cdb.get("edgetoedgeprint"), cdb.get("rotatelandscape") )

    return table.concat(commands)
end

--------------------------------------------------------------------------------
--	Function	:	pageboundingbox_command
--------------------------------------------------------------------------------
function pageboundingbox_command( devmode )

	local commands = {}
	local cdb = Rice.CapsDB

	commands[#commands+1]	= output_boundingbox(	cdb.get("documentsize"), cdb.get("printsize"), cdb.get("edgetoedgeprint") )

	-- PageBoundinxBox can't extend.

    return table.concat(commands)
end

return true

