Rice.require 'TableConverter'
Rice.require 'ExtensionLoader'
Rice.require 'extable'

------------------------------------------------------------------------------
-- Conversion Table : PSD -> PPD
------------------------------------------------------------------------------
local psd_to_ppd_table = {

  orientation = {
    item = '%Orientation',
    values = function(value,psd)
		return ('portrait' == value) and 'Portrait' or ( ( 'on' == psd['rotatelandscape'] ) and 'RotatedLandscape' or 'Landscape' )
	end
  },

  layout = {
    item = function(value,psd) return ('booklet' ~= psd['booklet']) and '%PagePerSheet' or nil end,
    values = function(value,psd)
		local tbl = 
		{
			['1pagepersheet']   = '1',
			['2pagespersheet']  = '2',
			['4pagespersheet']  = '4',
			['6pagespersheet']  = '6',
			['9pagespersheet']  = '9',
			['16pagespersheet'] = '16'
		}
		return ( "printer" == psd['pagelayout_distribution'] or "PDF" == psd['<pdl_name>'] ) and '1' or tbl[psd['layout']]
	end
  },

  booklet = {
	item = function(value) return ('booklet' == value) and '%PagePerSheet' or nil end,
	values = function(value,psd)
		if( 'printer' == psd['pagelayout_distribution'] or 
			'PDF' == psd['<pdl_name>'] 					) then
			
			return '1'
		
		elseif( 'booklet' == value					and
				'1pagepersheet' == psd['layout']	and
				'off'	== psd['reduceenlarge']		and
				'on' == psd['booklet_equalmagnification'] 	) then 
				
			return ('true' == psd['<enableyapdpp>']) and '1' or 'Booklet'
		
		elseif( 'booklet' == value					and
				'1pagepersheet' == psd['layout'] 	) then 
				
			return ('true' == psd['<enableyapdpp>']) and '2' or 'Booklet'
		
		elseif( 'booklet' == value ) then
		
			return ('true' == psd['<enableyapdpp>']) and '1' or 'Booklet'
			
		else
			return ''
		end
	end
  },

  duplex = {
    item = 'Duplex',
    values = function(value,psd)
    	if(	'off' == value or
			nil	== value	) then
    		return( 'booklet' == psd['booklet'] and (nil == psd['<enableyapdpp>'] or 'false' == psd['<enableyapdpp>'])) and 'DuplexTumble' or 'None'
 		
		elseif( 'longedgeleftortop' == value or
				'shortedgetoporleft' == value	) then
			local tbl = 
			{
				["longedgeleftortop"]	= "DuplexNoTumble",
				["shortedgetoporleft"]	= "DuplexTumble"
			}
			
			return tbl['duplex']
			
 		elseif( 'opentotop'	== value or
				'opentoleft' == value	) then
			local duplex_cmd = function( duplex, orientation, layout )
				local layout_1_4_9_16 =
				{
					["opentotop"]	= { ["portrait"] = "DuplexTumble",		["landscape"] = "DuplexNoTumble"	},
					["opentoleft"]	= { ["portrait"] = "DuplexNoTumble",	["landscape"] = "DuplexTumble"		}
				}

				local layout_2_6 =
				{
					["opentotop"]	= { ["portrait"] = "DuplexNoTumble",	["landscape"] = "DuplexTumble"		},
					["opentoleft"]	= { ["portrait"] = "DuplexTumble",		["landscape"] = "DuplexNoTumble"	}
				}

				local tbl = ("2pagespersheet" ~= layout and "6pagespersheet" ~= layout ) and layout_1_4_9_16 or layout_2_6

				return tbl[duplex][ (nil == orientation) and 'portrait' or orientation ]
			end

			return duplex_cmd( psd['duplex'], psd['orientation'], psd['layout'] )
			
		else
			return 'None'
		end
	end
  },

   resolution = {
    item = 'Resolution',
    values = {
      ['300']   	= '300dpi',
      ['600']		= '600dpi',
      ['1200x600']  = '600dpi',
      ['600x1200']  = '600dpi',
      ['2400x600']  = '600dpi',
      ['9600x600']	= '600dpi',
      ['38400x600']	= '600dpi',
	  ['1200']		= '1200dpi',
	  ['4800x1200']	= '1200dpi'
    }
  },

  custompapersize_width_inch = {
  	item = '%CustomPageSize',
  	values = function(value,psd)
  		if( 'custompapersize' == psd['documentsize'] ) then
	  		local width = string.format( "%d", value * 72 / 100 + 0.5 )
	  		local height = string.format( "%d", psd['custompapersize_length_inch'] * 72 / 100 + 0.5 )
	  		local edge = ( width > height ) and 'LongEdge' or 'ShortEdge'
	  		return width .. ' ' .. height .. ' 0 0 ' .. edge
	  	else
	  		return ''
	  	end
  	end
  },

  reverseorder = {
    item = '%PageOrder',
    values = {
    	on = 'BackToFront',
   		off = 'FrontToBack'
   	}
  },

  emfspool = {
    item = '%MetafileSpooling',
    values = {
      on  = 'True',
      off = 'False'
    }
  },

  mirroring = {
    item = '%Mirroring',
    values = {
      on  = 'True',
      off = 'False'
    }
  },

  negaprinting = {
    item = '%Negative',
    values = {
      on  = 'True',
      off = 'False'
    }
  },

  pserrorhandler = {
    item = '%PSErrorHandler',
    values = {
      on  = 'True',
      off = 'False'
    }
  },

  psoutputoption = {
    item = '%OutputFormat',
    values = {
      speed       = 'Speed',
      portability = 'Portability',
      eps         = 'EPS',
      archive     = 'Archive'
    }
  },

  truetypefont_download = {
    item = '%TTDownloadFormat',
    values = {
      automatic      = 'Automatic',
      outline        = 'Outline',
      bitmap         = 'Bitmap',
      nativetruetype = 'NativeTrueType'
    }
  },

  pslanguagelevel = {
    item = '%OutputPSLevel',
    values = function(value) return (nil ~= tonumber(value) ) and value or '' end
  },

  ps_addeuro = {
    item = '%AddEuro',
    values = {
      on  = 'True',
      off = 'False'
    }
  },

  ps_ctldafter = {
    item = '%CtrlDAfter',
    values = {
      on  = 'True',
      off = 'False'
    }
  },

  ps_ctldbefore = {
    item = '%CtrlDBefore',
    values = {
      on  = 'True',
      off = 'False'
    }
  },

  ps_graphicstruegray = {
  	item = '%GraphicsTrueGray',
    values = {
      on  = 'True',
      off = 'False'
    }
  },

  ps_jobtimeout = {
  	item = '%JobTimeout',
  	values = function(value) return (nil ~= tonumber(value) ) and value or '' end
  },

  ps_maxfontsizeasbitmap = {
  	item = '%MaxFontSizeAsBitmap',
  	values = function(value) return (nil ~= tonumber(value) ) and value or '' end
  },

  ps_memory = {
  	item = '%PSMemory',
  	values = function(value) return (nil ~= tonumber(value) ) and value or '' end
  },

  ps_minfontsizeasoutline = {
  	item = '%MinFontSizeAsOutline',
  	values = function(value) return (nil ~= tonumber(value) ) and value or '' end
  },

  ps_outputprotocol = {
    item = '%OutputProtocol',
    values = {
      ascii  = 'ASCII',
      tbcp   = 'TBCP',
      binary = 'Binary'
    }
  },

  ps_texttruegray = {
  	item = '%TextTrueGray',
    values = {
      on  = 'True',
      off = 'False'
    }
  },

  ps_waittimeout = {
  	item = '%WaitTimeout',
  	values = function(value) return (nil ~= tonumber(value) ) and value or '' end
  }

}

-----------------------------------------------------------------------------
-- Export Function : Convert PSD to PPD
------------------------------------------------------------------------------
function psd_to_mini(psd)
  local converter = TableConverter(psd_to_ppd_table)
  local ppd = converter:run(psd)

  local loader = ExtensionLoader(_MODULE, 'psd_to_ppd')
  for name, func in loader:extensions() do
    table.cat(ppd, func(psd))
  end

  return ppd
end

return true
