Rice.require 'Rice.CapsDB'
Rice.require 'Rice.LangDB'
Rice.require 'OrderedHash'
Rice.require 'exstring'

-- GetCurrentItem
function get_current_value( item_name )
	local capsdb = Rice.CapsDB
	
	local current_value = ''
	if capsdb and item_name then
		current_value = capsdb.get( item_name )
	end
	
	return current_value and current_value or ''
end

-- GetWording
function get_display_wording( item_name , element_name )
	local langdb = Rice.LangDB
	
	local wording = ''
	if langdb and item_name then
		if element_name then
			wording = langdb.get_elementwording( item_name , element_name )
		else
			wording = langdb.get( item_name )
		end
	end

	return wording and wording or ''
end

-- GetAvailableCustomMediaNumber
function get_available_custommedia_number()

	local result_tbl = {}
	local custommedia_info = get_current_value( 'custommediatype_info' ):split("\n")
	
	if custommedia_info then
		for count = 0 , 200 do
			local name = custommedia_info[count*3 + 1]
			
			if( not name ) then
				break
			end
			
			if( string.len(name) > 0 ) then
				table.insert(result_tbl, tostring(count+1))
			end
		end
	end
	
	return result_tbl
end

-- Create PickOne Table
local function standard_option_generator(item_table, element_table)
	local result_tbl = {}

	if( not item_table or not element_table ) then
		return result_tbl
	end

	-- PickOne Tag
	result_tbl[ #result_tbl + 1 ] = {
		tag			= 	'psf:Property',
		attribute	=
		{
			key		=	'name',
			value	=	'psf:SelectionType'
		},
		children =
		{
			tag			= 	'psf:Value',
			attribute	=
			{
				key		=	'xsi:type',
				value	=	'xsd:QName'
			},
			text		=	'psk:PickOne'
		}
	}

	-- PickOne Item
	for i, k, v in OrderedHash_each( element_table ) do
		if( i and k and v ) then
			local option_value = ( 'string' == type( v ) ) and v or v.value
			if( option_value ) then
				local isSupported = false
				for index, value in pairs(item_table) do
					if( k == value[ 'name' ] ) then
						isSupported = true
						break
					end
				end

				if( isSupported ) then
					result_tbl[ #result_tbl + 1 ] =
					{
						tag		=	'psf:Option',
						attribute	=
						{
							key		=	'name',
							value	=	option_value
						}
					}

					-- ScoredProperty
					if( v.scoredproperty and #(v.scoredproperty) >= 1 ) then
						result_tbl[ #result_tbl ][ 'children' ] = {}
						for si, sv in pairs( v.scoredproperty ) do
							if( sv and sv['name'] and sv['def'] ) then
								result_tbl[ #result_tbl ][ 'children' ][ #( result_tbl[ #result_tbl ][ 'children' ] ) + 1 ] =
								{
									tag			= 	'psf:ScoredProperty',
									attribute	=
									{
										key		=	'name',
										value	=	sv['name']
									},
									children = {
									tag			= 	'psf:ParameterRef',
									attribute	=
										{
											key		=	'name',
											value	=	sv['def']
										}
									}
								}
							end
						end
					end

					-- DisplayName 
					local item = v.item and v.item or v.wording
					local element = v.item and v.wording or nil

					local item_name = get_display_wording(item, element)
					item_name = item_name and item_name or ''
					
					if( item_name and '' ~= item_name) then
						if( not result_tbl[ #result_tbl ][ 'children' ] )then
							result_tbl[ #result_tbl ][ 'children' ] = {}
						end
						
						result_tbl[ #result_tbl ][ 'children' ][ #(result_tbl[ #result_tbl ][ 'children' ]) + 1 ] =
						{
							tag			= 	'psf:Property',
							attribute	=
							{
								key		=	'name',
								value	=	'psk:DisplayName'
							},
							children = {
								tag			= 	'psf:Value',
								attribute	=
								{
									key		=	'xsi:type',
									value	=	'xsd:string'
								},
								text		=	item_name
							}
						}
					end
				end
			end
		end
	end

	return result_tbl
end

-- Create Range/String Item Table
local function paramdef_generator(param_type, item_table, unittype, multiple)
	local result_tbl = {}

	if(nil == item_table)then
		return {}
	end

	--RangeACeiPʁFj --
	if('integer' == param_type or 'integer_unconditional' == param_type or 'integer_optional' == param_type) then

		local mandatory_table =
		{
			['integer']					=	'psk:Conditional',
			['integer_unconditional']	=	'psk:Unconditional',
			['integer_optional']		=	'psk:Optional',
		}

		result_tbl =
		{
			{
				tag			=	'psf:Property',
				attribute	=
				{
					key		=	'name',
					value	=	'psf:DataType'
				},
				children	=
				{
					tag 		=	'psf:Value',
					attribute	=
					{
						key 	=	'xsi:type',
						value	=	'xsd:QName'
					},
					text		=	'xsd:integer'
				}
			},
			{
				tag			=	'psf:Property',
				attribute	=
				{
					key		=	'name',
					value	=	'psf:UnitType'
				},
				children	=
				{
					tag 		=	'psf:Value',
					attribute	=
					{
						key 	=	'xsi:type',
						value	=	'xsd:string'
					},
					text		=	unittype
				}
			},
			{
				tag			=	'psf:Property',
				attribute	=
				{
					key		=	'name',
					value	=	'psf:Multiple'
				},
				children	=
				{
					tag 		=	'psf:Value',
					attribute	=
					{
						key 	=	'xsi:type',
						value	=	'xsd:integer'
					},
					text		=	tostring(multiple)
				}
			},
			{
				tag			=	'psf:Property',
				attribute	=
				{
					key		=	'name',
					value	=	'psf:MaxValue'
				},
				children	=
				{
					tag 		=	'psf:Value',
					attribute	=
					{
						key 	=	'xsi:type',
						value	=	'xsd:integer'
					},
					text		=	tostring(item_table['max'])
				}
			},
			{
				tag			=	'psf:Property',
				attribute	=
				{
					key		=	'name',
					value	=	'psf:MinValue'
				},
				children	=
				{
					tag 		=	'psf:Value',
					attribute	=
					{
						key 	=	'xsi:type',
						value	=	'xsd:integer'
					},
					text		=	tostring(item_table['min'])
				}
			},
			{
				tag			=	'psf:Property',
				attribute	=
				{
					key		=	'name',
					value	=	'psf:DefaultValue'
				},
				children	=
				{
					tag 		=	'psf:Value',
					attribute	=
					{
						key 	=	'xsi:type',
						value	=	'xsd:integer'
					},
					text		=	tostring(item_table['default'])
				}
			},
			{
				tag			=	'psf:Property',
				attribute	=
				{
					key		=	'name',
					value	=	'psf:Mandatory'
				},
				children	=
				{
					tag 		=	'psf:Value',
					attribute	=
					{
						key 	=	'xsi:type',
						value	=	'xsd:QName'
					},
					text		=	mandatory_table[param_type]
				}
			}
		}

	--StringACe --
	elseif('string' == param_type or 'string_unconditional' == param_type or 'string_optional' == param_type) then

		local mandatory_table =
		{
			['string']					=	'psk:Conditional',
			['string_unconditional']	=	'psk:Unconditional',
			['string_optional']			=	'psk:Optional',
		}

		result_tbl =
		{
			{
				tag			=	'psf:Property',
				attribute	=
				{
					key		=	'name',
					value	=	'psf:DataType'
				},
				children	=
				{
					tag 		=	'psf:Value',
					attribute	=
					{
						key 	=	'xsi:type',
						value	=	'xsd:QName'
					},
					text		=	'xsd:string'
				}
			},
			{
				tag			=	'psf:Property',
				attribute	=
				{
					key		=	'name',
					value	=	'psf:UnitType'
				},
				children	=
				{
					tag 		=	'psf:Value',
					attribute	=
					{
						key 	=	'xsi:type',
						value	=	'xsd:string'
					},
					text		=	unittype
				}
			},
			{
				tag			=	'psf:Property',
				attribute	=
				{
					key		=	'name',
					value	=	'psf:MaxLength'
				},
				children	=
				{
					tag 		=	'psf:Value',
					attribute	=
					{
						key 	=	'xsi:type',
						value	=	'xsd:integer'
					},
					text		=	tostring(item_table['max'])
				}
			},
			{
				tag			=	'psf:Property',
				attribute	=
				{
					key		=	'name',
					value	=	'psf:MinLength'
				},
				children	=
				{
					tag 		=	'psf:Value',
					attribute	=
					{
						key 	=	'xsi:type',
						value	=	'xsd:integer'
					},
					text		=	tostring(item_table['min'])
				}
			},
			{
				tag			=	'psf:Property',
				attribute	=
				{
					key		=	'name',
					value	=	'psf:DefaultValue'
				},
				children	=
				{
					tag 		=	'psf:Value',
					attribute	=
					{
						key 	=	'xsi:type',
						value	=	'xsd:string'
					},
					text		=	tostring(item_table['default'])
				}
			},
			{
				tag			=	'psf:Property',
				attribute	=
				{
					key		=	'name',
					value	=	'psf:Mandatory'
				},
				children	=
				{
					tag 		=	'psf:Value',
					attribute	=
					{
						key 	=	'xsi:type',
						value	=	'xsd:QName'
					},
					text		=	mandatory_table[param_type]
				}
			}
		}
	end

	return result_tbl
end

pc_table =
{
-- [01 Job]
	-- 01 Job Type
	jobtype	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:JobRicPrintJob'
		},
		children	=	function(tbl)
			local element_table	= OrderedHash(
				{ 'normalprint',			{value = 'ns0000:RicPrintNormal', scoredproperty = {}}},
				{ 'sampleprint',			{value = 'ns0000:RicPrintSample', scoredproperty = {}}},
				{ 'lockedprint',			{value = 'ns0000:RicPrintLocked', scoredproperty = {}}},
				{ 'holdprint',				{value = 'ns0000:RicPrintHold', scoredproperty = {}}},
				{ 'storedprint',			{value = 'ns0000:RicPrintStored', scoredproperty = {}}},
				{ 'storedprintshared',		{value = 'ns0000:RicPrintStoredShared', scoredproperty = {}}},
				{ 'storeandprint',			{value = 'ns0000:RicPrintStoreAndPrint', scoredproperty = {}}},
				{ 'storeandprintshared',	{value = 'ns0000:RicPrintStoreAndPrintShared', scoredproperty = {}}},
				{ 'documentserver',			{value = 'ns0000:RicDocumentServer', scoredproperty = {}}},
				{ 'saveoverlaydata',		{value = 'ns0000:RicSaveOverlayData', scoredproperty = {}}},
				{ 'storeoverlaydata',		{value = 'ns0000:RicStoreOverlayDataToPrinterStorage', scoredproperty = {}}},
				{ 'storefile',				{value = 'ns0000:RicStoreFile', scoredproperty = {}}}
			)
			--Set ScoredProperty Info--
			---User ID--
			if(nil ~= tbl['userid'])then
				table.insert(element_table['normalprint']['scoredproperty'],{name = 'ns0000:RicUserID', def = 'ns0000:JobRicUserIDDef'})
				table.insert(element_table['sampleprint']['scoredproperty'],{name = 'ns0000:RicUserID', def = 'ns0000:JobRicUserIDDef'})
				table.insert(element_table['lockedprint']['scoredproperty'],{name = 'ns0000:RicUserID', def = 'ns0000:JobRicUserIDDef'})
				table.insert(element_table['holdprint']['scoredproperty'],{name = 'ns0000:RicUserID', def = 'ns0000:JobRicUserIDDef'})
				table.insert(element_table['storedprint']['scoredproperty'],{name = 'ns0000:RicUserID', def = 'ns0000:JobRicUserIDDef'})
				table.insert(element_table['storedprintshared']['scoredproperty'],{name = 'ns0000:RicUserID', def = 'ns0000:JobRicUserIDDef'})
				table.insert(element_table['storeandprint']['scoredproperty'],{name = 'ns0000:RicUserID', def = 'ns0000:JobRicUserIDDef'})
				table.insert(element_table['storeandprintshared']['scoredproperty'],{name = 'ns0000:RicUserID', def = 'ns0000:JobRicUserIDDef'})
				table.insert(element_table['documentserver']['scoredproperty'],{name = 'ns0000:RicUserID', def = 'ns0000:JobRicUserIDDef'})
				table.insert(element_table['storefile']['scoredproperty'],{name = 'ns0000:RicUserID', def = 'ns0000:JobRicUserIDDef'})
			end
			
			---Password--
			if(nil ~= tbl['password'])then
				table.insert(element_table['lockedprint']['scoredproperty'],{name = 'ns0000:RicPassword', def = 'ns0000:JobRicPasswordDef'})
				table.insert(element_table['storedprint']['scoredproperty'],{name = 'ns0000:RicPassword', def = 'ns0000:JobRicPasswordDef'})
				table.insert(element_table['storedprintshared']['scoredproperty'],{name = 'ns0000:RicPassword', def = 'ns0000:JobRicPasswordDef'})
				table.insert(element_table['storeandprint']['scoredproperty'],{name = 'ns0000:RicPassword', def = 'ns0000:JobRicPasswordDef'})
				table.insert(element_table['storeandprintshared']['scoredproperty'],{name = 'ns0000:RicPassword', def = 'ns0000:JobRicPasswordDef'})
				table.insert(element_table['documentserver']['scoredproperty'],{name = 'ns0000:RicPassword', def = 'ns0000:JobRicPasswordDef'})
				table.insert(element_table['storefile']['scoredproperty'],{name = 'ns0000:RicPassword', def = 'ns0000:JobRicPasswordDef'})
			end
			
			---filename--
			if(nil ~= tbl['filename'])then
				table.insert(element_table['holdprint']['scoredproperty'],{name = 'ns0000:RicJobID', def = 'ns0000:JobRicFileNameDef'})
				table.insert(element_table['storedprint']['scoredproperty'],{name = 'ns0000:RicJobID', def = 'ns0000:JobRicFileNameDef'})
				table.insert(element_table['storedprintshared']['scoredproperty'],{name = 'ns0000:RicJobID', def = 'ns0000:JobRicFileNameDef'})
				table.insert(element_table['storeandprint']['scoredproperty'],{name = 'ns0000:RicJobID', def = 'ns0000:JobRicFileNameDef'})
				table.insert(element_table['storeandprintshared']['scoredproperty'],{name = 'ns0000:RicJobID', def = 'ns0000:JobRicFileNameDef'})
				table.insert(element_table['documentserver']['scoredproperty'],{name = 'ns0000:RicJobID', def = 'ns0000:JobRicFileNameDef'})
				table.insert(element_table['storefile']['scoredproperty'],{name = 'ns0000:RicJobID', def = 'ns0000:JobRicFileNameDef'})
			end
			
			---username--
			if(nil ~= tbl['username'])then
				table.insert(element_table['documentserver']['scoredproperty'],{name = 'ns0000:RicUserName', def = 'ns0000:JobRicUserNameDef'})
				table.insert(element_table['storefile']['scoredproperty'],{name = 'ns0000:RicUserName', def = 'ns0000:JobRicUserNameDef'})
			end
			
			---saveoverlaydata_filepath--
			if(nil ~= tbl['saveoverlaydata_filepath'])then
				table.insert(element_table['saveoverlaydata']['scoredproperty'],{name = 'ns0000:RicRPCSOverlayData', def = 'ns0000:JobRPCSOverlayDataNameDef'})
			end
			---storeoverlaydata_number--
			if(nil ~= tbl['storeoverlaydata_number'])then
				table.insert(element_table['storeoverlaydata']['scoredproperty'],{name = 'ns0000:RicRPCSOverlayNumber', def = 'ns0000:JobImageOverlayRegistrationNumber'})
			end
			
			---storeoverlaydata_name--
			if(nil ~= tbl['storeoverlaydata_name'])then
				table.insert(element_table['storeoverlaydata']['scoredproperty'],{name = 'ns0000:RicRPCSOverlayName', def = 'ns0000:JobImageOverlayRegistrationName'})
			end
			
			---documentserver_foldernumber---
			if(nil ~= tbl['documentserver_foldernumber'])then
				table.insert(element_table['documentserver']['scoredproperty'],{name = 'ns0000:RicFolderNumber', def = 'ns0000:JobRicFolderNumberDef'})
			end
			
			---documentserver_folderpassword---
			if(nil ~= tbl['documentserver_folderpassword'])then
				table.insert(element_table['documentserver']['scoredproperty'],{name = 'ns0000:RicFolderPassword', def = 'ns0000:JobRicFolderPasswordDef'})
			end
			
			local result_tbl = standard_option_generator(tbl['jobtype'], element_table)

			--------User ID Type--------
			if(nil ~= tbl['userid_type'])then
				local userid_type_element_table	= OrderedHash(
					{ 'userdefined',		'ns0000:UserDefined'},
					{ 'windowsloginname',	'ns0000:WindowsLoginName'}
				)
				local userid_type_table = 
				{
					tag			= 	'psf:Feature',
					attribute	=
					{
						key		=	'name',
						value	=	'ns0000:RicUserIDType'
					},
					children = standard_option_generator(tbl['userid_type'], userid_type_element_table)
				}
				table.insert(result_tbl, userid_type_table)
			end
			
			--------Overlay Direction--------
			if(nil ~= tbl['storeoverlaydata_direction'])then
				local direction_element_table	= OrderedHash(
					{ 'lef',	'ns0000:FitToFeedFromLongEdge'},
					{ 'sef',	'ns0000:FitToFeedFromShortEdge'}
				)

				local direction_table = 
				{
					tag			= 	'psf:Feature',
					attribute	=
					{
						key		=	'name',
						value	=	'ns0000:ImageOverlayDirection'
					},
					children	=	standard_option_generator(tbl['storeoverlaydata_direction'], direction_element_table)
				}
				table.insert(result_tbl, direction_table)
			end
			
			--------Scheduled Print--------
			if(nil ~= tbl['scheduledprint'])then
				local scheduled_element_table	= OrderedHash(
					{ 'off',	{value = 'ns0000:Off', scoredproperty = {}}},
					{ 'on',		{value = 'ns0000:On', scoredproperty = {}}}
				)
				--Set ScoredProperty Info--
				---Hour--
				if(nil ~= tbl['scheduledprint_hour'])then
					table.insert(scheduled_element_table['on']['scoredproperty'],{name = 'ns0000:Hour', def = 'ns0000:JobScheduledPrintHourDef'})
				end
				---Minute--
				if(nil ~= tbl['scheduledprint_minute'])then
					table.insert(scheduled_element_table['on']['scoredproperty'],{name = 'ns0000:Minute', def = 'ns0000:JobScheduledPrintMinuteDef'})
				end

				local scheduled_table = 
				{
					tag			= 	'psf:Feature',
					attribute	=
					{
						key		=	'name',
						value	=	'ns0000:ScheduledPrint'
					},
					children	=	standard_option_generator(tbl['scheduledprint'], scheduled_element_table)
				}
				table.insert(result_tbl, scheduled_table)
			end
			
			return result_tbl
		end
	},
	userid	=
	{
		tag		=	'psf:ParameterDef',
		attribute	=
			{
				key		=	'name',
				value	=	'ns0000:JobRicUserIDDef'
			},
		children	=	function(tbl)
							return paramdef_generator('string', tbl['userid'], 'characters', nil)
						end
	},
	password	=
	{
		tag		=	'psf:ParameterDef',
		attribute	=
			{
				key		=	'name',
				value	=	'ns0000:JobRicPasswordDef'
			},
		children	=	function(tbl)
							return paramdef_generator('string', tbl['password'], 'characters', nil)
						end
	},
	filename	=
	{
		tag		=	'psf:ParameterDef',
		attribute	=
			{
				key		=	'name',
				value	=	'ns0000:JobRicFileNameDef'
			},
		children	=	function(tbl)
							return paramdef_generator('string', tbl['filename'], 'characters', nil)
						end
	},
	username	=
	{
		tag		=	'psf:ParameterDef',
		attribute	=
			{
				key		=	'name',
				value	=	'ns0000:JobRicUserNameDef'
			},
		children	=	function(tbl)
							return paramdef_generator('string', tbl['username'], 'characters', nil)
						end
	},
	saveoverlaydata_filepath	=
	{
		tag		=	'psf:ParameterDef',
		attribute	=
			{
				key		=	'name',
				value	=	'ns0000:JobRPCSOverlayDataNameDef'
			},
		children	=	function(tbl)
							return paramdef_generator('string', tbl['saveoverlaydata_filepath'], 'characters', nil)
						end
	},
	storeoverlaydata_number	=
	{
		tag		=	'psf:ParameterDef',
		attribute	=
			{
				key		=	'name',
				value	=	'ns0000:JobImageOverlayRegistrationNumber'
			},
		children	=	function(tbl)
							return paramdef_generator('integer', tbl['storeoverlaydata_number'], 'reg', 1)
						end
	},
	storeoverlaydata_name	=
	{
		tag		=	'psf:ParameterDef',
		attribute	=
			{
				key		=	'name',
				value	=	'ns0000:JobImageOverlayRegistrationName'
			},
		children	=	function(tbl)
							return paramdef_generator('string', tbl['storeoverlaydata_name'], 'characters', nil)
						end
	},
	documentserver_foldernumber	=
	{
		tag		=	'psf:ParameterDef',
		attribute	=
			{
				key		=	'name',
				value	=	'ns0000:JobRicFolderNumberDef'
			},
		children	=	function(tbl)
							return paramdef_generator('integer', tbl['documentserver_foldernumber'], 'reg', 1)
						end
	},
	documentserver_folderpassword	=
	{
		tag		=	'psf:ParameterDef',
		attribute	=
			{
				key		=	'name',
				value	=	'ns0000:JobRicFolderPasswordDef'
			},
		children	=	function(tbl)
							return paramdef_generator('string', tbl['documentserver_folderpassword'], 'characters', nil)
						end  
	},
	loginusername =
	{
		{
			tag			=	'psf:Feature',
			attribute	=
			{
				key		=	'name',
				value	=	'ns0000:JobUserAuthentication'
			},
			children	=	function(tbl)
				local result_tbl = {}

				result_tbl[ #result_tbl + 1 ] =
				{
					tag			= 	'psf:Property',
					attribute	=
					{
						key		=	'name',
						value	=	'psf:SelectionType'
					},
					children =
					{
						tag			= 	'psf:Value',
						attribute	=
						{
							key		=	'xsi:type',
							value	=	'xsd:QName'
						},
						text		=	'psk:PickOne'
					}
				}

				result_tbl[ #result_tbl + 1 ] =
				{
					tag			= 	'psf:Option',
					children	=
					{
						{
							tag			= 	'psf:ScoredProperty',
							attribute	=
							{
								key		=	'name',
								value	=	'ns0000:LoginUserName'
							},
							children =
							{
								tag			= 	'psf:ParameterRef',
								attribute	=
								{
									key		=	'name',
									value	=	'ns0000:JobUserAuthenticationLoginUserName'
								}
							}
						}
					}
				}

				if( tbl['loginpassword'] ) then
					result_tbl[ #result_tbl ][ 'children' ][ #(result_tbl[ #result_tbl ][ 'children' ]) + 1 ] =
					{
						tag			= 	'psf:ScoredProperty',
						attribute	=
						{
							key		=	'name',
							value	=	'ns0000:LoginPassword'
						},
						children =
						{
							tag			= 	'psf:ParameterRef',
							attribute	=
							{
								key		=	'name',
								value	=	'ns0000:JobUserAuthenticationLoginPassword'
							}
						}
					}
				end

				-- loginusername_type
				if( tbl['loginusername_type'] )then
					local loginnametype_element_table	= OrderedHash(
						{ 'userdefined',		'ns0000:UserDefined'},
						{ 'windowsloginname',	'ns0000:WindowsLoginName'}
					)
					result_tbl[ #result_tbl + 1 ] =
					{
						tag			= 	'psf:Feature',
						attribute	=
						{
							key		=	'name',
							value	=	'ns0000:UserAuthenticationLoginUserNameType'
						},
						children	=	standard_option_generator(tbl['loginusername_type'], loginnametype_element_table)
					}
				end

				return result_tbl
			end
		},
		{
			tag		=	'psf:ParameterDef',
			attribute	=
			{
				key		=	'name',
				value	=	'ns0000:JobUserAuthenticationLoginUserName'
			},
			children	=	function(tbl)
				return paramdef_generator('string', tbl['loginusername'], 'characters', nil)
			end
		}
	},
	loginpassword	=
	{
		tag		=	'psf:ParameterDef',
		attribute	=
			{
				key		=	'name',
				value	=	'ns0000:JobUserAuthenticationLoginPassword'
			},
		children	=	function(tbl)
							return paramdef_generator('string', tbl['loginpassword'], 'characters', nil)
						end
	},
	usercode =
	{
		{
			tag			=	'psf:Feature',
			attribute	=
			{
				key		=	'name',
				value	=	'ns0000:JobRicLogJob'
			},
			children	=
				{
					{
						tag			= 	'psf:Property',
						attribute	=
						{
							key		=	'name',
							value	=	'psf:SelectionType'
						},
						children = {
								tag			= 	'psf:Value',
								attribute	=
								{
									key		=	'xsi:type',
									value	=	'xsd:QName'
								},
								text		=	'psk:PickOne'
							}
					},
					{
						tag			= 	'psf:Option',
						children	=	
							{
								tag			= 	'psf:ScoredProperty',
								attribute	=
								{
									key		=	'name',
									value	=	'ns0000:RicUserCode'
								},
								children = {
									tag			= 	'psf:ParameterRef',
									attribute	=
										{
											key		=	'name',
											value	=	'ns0000:JobRicUserCodeDef'
										}
									}
							}
					}
				}
		},
		{
			tag		=	'psf:ParameterDef',
			attribute	=
				{
					key		=	'name',
					value	=	'ns0000:JobRicUserCodeDef'
				},
			children	=	function(tbl)
								return paramdef_generator('string', tbl['usercode'], 'characters', nil)
							end
		},
	},
	scheduledprint_hour	=
	{
		tag		=	'psf:ParameterDef',
		attribute	=
			{
				key		=	'name',
				value	=	'ns0000:JobScheduledPrintHourDef'
			},
		children	=	function(tbl)
							return paramdef_generator('integer', tbl['scheduledprint_hour'], 'hours', 1)
						end
	},
	scheduledprint_minute	=
	{
		tag		=	'psf:ParameterDef',
		attribute	=
			{
				key		=	'name',
				value	=	'ns0000:JobScheduledPrintMinuteDef'
			},
		children	=	function(tbl)
							return paramdef_generator('integer', tbl['scheduledprint_minute'], 'minutes', 1)
						end
	},
	userdomainname =
	{
		{
			tag			=	'psf:Feature',
			attribute	=
			{
				key		=	'name',
				value	=	'ns0000:JobUserDomain'
			},
			children	=	function(tbl)
				local result_tbl = {}

				result_tbl[ #result_tbl + 1 ] =
				{
					tag			= 	'psf:Property',
					attribute	=
					{
						key		=	'name',
						value	=	'psf:SelectionType'
					},
					children =
					{
						tag			= 	'psf:Value',
						attribute	=
						{
							key		=	'xsi:type',
							value	=	'xsd:QName'
						},
						text		=	'psk:PickOne'
					}
				}

				result_tbl[ #result_tbl + 1 ] =
				{
					tag			= 	'psf:Option',
					children	=
					{
						{
							tag			= 	'psf:ScoredProperty',
							attribute	=
							{
								key		=	'name',
								value	=	'ns0000:JobUserDomainText'
							},
							children =
							{
								tag			= 	'psf:ParameterRef',
								attribute	=
								{
									key		=	'name',
									value	=	'ns0000:JobUserDomainTextDef'
								}
							}
						}
					}
				}

				-- userdomainname_type
				if( tbl['userdomainname_type'] )then
					local userdomainname_type_element_table	= OrderedHash(
						{ 'userdefined',	'ns0000:UserDefined'},
						{ 'auto',			'ns0000:Auto'}
					)
					result_tbl[ #result_tbl + 1 ] =
					{
						tag			= 	'psf:Feature',
						attribute	=
						{
							key		=	'name',
							value	=	'ns0000:JobUserDomainType'
						},
						children	=	standard_option_generator(tbl['userdomainname_type'], userdomainname_type_element_table)
					}
				end

				return result_tbl
			end
		},
		{
			tag		=	'psf:ParameterDef',
			attribute	=
			{
				key		=	'name',
				value	=	'ns0000:JobUserDomainTextDef'
			},
			children	=	function(tbl)
				return paramdef_generator('string', tbl['userdomainname'], 'characters', nil)
			end
		}
	},

-- [02 Finishing]
	-- 01 Duplex
	duplex	=
	{
		{
			tag		=	'psf:Feature',
			attribute	=
			{
				key		=	'name',
				value	=	'ns0000:JobDuplexAllDocumentsDirection'
			},
			children	=	function(tbl)
				local element_table = OrderedHash(
					{ 'off',					{value = 'ns0000:Off', scoredproperty = {}}},
					{ 'longedgeleftortop',		{value = 'ns0000:LongEdgeLeftOrTop', scoredproperty = {}}},
					{ 'longedgerightortop',		{value = 'ns0000:LongEdgeRightOrTop', scoredproperty = {}}},
					{ 'shortedgetoporleft',		{value = 'ns0000:ShortEdgeTopOrLeft', scoredproperty = {}}},
					{ 'shortedgetoporright',	{value = 'ns0000:ShortEdgeTopOrRight', scoredproperty = {}}}
				)
				--Set ScoredProperty Info--

				---FrontSideMargin--
				if(nil ~= tbl['duplex_bindingmarginfront_mm'])then
					table.insert(element_table['longedgeleftortop']['scoredproperty'],{name = 'ns0000:DuplexFrontSideMargin', def = 'ns0000:JobDuplexMarginFrontSide'})
					table.insert(element_table['longedgerightortop']['scoredproperty'],{name = 'ns0000:DuplexFrontSideMargin', def = 'ns0000:JobDuplexMarginFrontSide'})
					table.insert(element_table['shortedgetoporleft']['scoredproperty'],{name = 'ns0000:DuplexFrontSideMargin', def = 'ns0000:JobDuplexMarginFrontSide'})
					table.insert(element_table['shortedgetoporright']['scoredproperty'],{name = 'ns0000:DuplexFrontSideMargin', def = 'ns0000:JobDuplexMarginFrontSide'})
				end

				---BackSideMargin--
				if(nil ~= tbl['duplex_bindingmarginback_mm'])then
					table.insert(element_table['longedgeleftortop']['scoredproperty'],{name = 'ns0000:DuplexBackSideMargin', def = 'ns0000:JobDuplexMarginBackSide'})
					table.insert(element_table['longedgerightortop']['scoredproperty'],{name = 'ns0000:DuplexBackSideMargin', def = 'ns0000:JobDuplexMarginBackSide'})
					table.insert(element_table['shortedgetoporleft']['scoredproperty'],{name = 'ns0000:DuplexBackSideMargin', def = 'ns0000:JobDuplexMarginBackSide'})
					table.insert(element_table['shortedgetoporright']['scoredproperty'],{name = 'ns0000:DuplexBackSideMargin', def = 'ns0000:JobDuplexMarginBackSide'})
				end

				return standard_option_generator(tbl['duplex'], element_table)
			end
		},
	},
	supportduplex	=
	{
		function(tbl)
			local capability_table = {
				tag		=	'psf:Feature',
				attribute	=
				{
					key		=	'name',
					value	=	'psk:JobDuplexAllDocumentsContiguously'
				},
				children	=
				{
					{
						tag			= 	'psf:Property',
						attribute	=
						{
							key		=	'name',
							value	=	'psf:SelectionType'
						},
						children =
						{
							tag			= 	'psf:Value',
							attribute	=
							{
								key		=	'xsi:type',
								value	=	'xsd:QName'
							},
							text		=	'psk:PickOne'
						}
					},
					{
						tag			= 	'psf:Option',
						attribute	=
						{
							key		=	'name',
							value	=	'psk:OneSided'
						}
					},
					{
						tag			= 	'psf:Option',
						attribute	=
						{
							key		=	'name',
							value	=	'psk:TwoSidedLongEdge'
						}
					},
					{
						tag			= 	'psf:Option',
						attribute	=
						{
							key		=	'name',
							value	=	'psk:TwoSidedShortEdge'
						}
					}
				}
			}
			
			if('on' == tbl['supportduplex']['current']['name'])then
				return capability_table
			else
				return nil
			end
		end
	},
	duplex_bindingmarginfront_mm	=
	{
		tag		=	'psf:ParameterDef',
		attribute	=
			{
				key		=	'name',
				value	=	'ns0000:JobDuplexMarginFrontSide'
			},
		children	=	function(tbl)
							--default
							local default_val = tonumber(tbl['duplex_bindingmarginfront_mm']['default'])
							if(nil ~= default_val)then
								tbl['duplex_bindingmarginfront_mm']['default'] = tostring(default_val * 100)
							else
								tbl['duplex_bindingmarginfront_mm']['default'] = '0'
							end
							
							--max
							local max_val = tonumber(tbl['duplex_bindingmarginfront_mm']['max'])
							if(nil ~= max_val)then
								tbl['duplex_bindingmarginfront_mm']['max'] = tostring(max_val * 100)
							else
								tbl['duplex_bindingmarginfront_mm']['max'] = '0'
							end
							
							--min
							local min_val = tonumber(tbl['duplex_bindingmarginfront_mm']['min'])
							if(nil ~= min_val)then
								tbl['duplex_bindingmarginfront_mm']['min'] = tostring(min_val * 100)
							else
								tbl['duplex_bindingmarginfront_mm']['min'] = '0'
							end
							
							return paramdef_generator('integer', tbl['duplex_bindingmarginfront_mm'], 'microns', 100)
						end
	},
	duplex_bindingmarginback_mm	=
	{
		tag		=	'psf:ParameterDef',
		attribute	=
			{
				key		=	'name',
				value	=	'ns0000:JobDuplexMarginBackSide'
			},
		children	=	function(tbl)
							--default
							local default_val = tonumber(tbl['duplex_bindingmarginback_mm']['default'])
							if(nil ~= default_val)then
								tbl['duplex_bindingmarginback_mm']['default'] = tostring(default_val * 100)
							else
								tbl['duplex_bindingmarginback_mm']['default'] = '0'
							end
							
							--max
							local max_val = tonumber(tbl['duplex_bindingmarginback_mm']['max'])
							if(nil ~= max_val)then
								tbl['duplex_bindingmarginback_mm']['max'] = tostring(max_val * 100)
							else
								tbl['duplex_bindingmarginback_mm']['max'] = '0'
							end
							
							--min
							local min_val = tonumber(tbl['duplex_bindingmarginback_mm']['min'])
							if(nil ~= min_val)then
								tbl['duplex_bindingmarginback_mm']['min'] = tostring(min_val * 100)
							else
								tbl['duplex_bindingmarginback_mm']['min'] = '0'
							end
							
							return paramdef_generator('integer', tbl['duplex_bindingmarginback_mm'], 'microns', 100)
						end
	},

	-- 02 Enable Printer Stamp Setting
	enableprinterstampsetting	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:JobPrinterStamp'
		},
		children	=	function(tbl)
			return standard_option_generator(
				tbl['enableprinterstampsetting'],
				OrderedHash(
					{ 'off',	'ns0000:Off'},
					{ 'on',		'ns0000:On'}
				) )
		end
	},

	-- 03 Collate
	supportcollate	=
	{
		function(tbl)
			local capability_table = {
					tag			= 	'psf:Feature',
					attribute	=
					{
						key		=	'name',
						value	=	'psk:DocumentCollate'
					},
					children	=
						{
							{
								tag			= 	'psf:Property',
								attribute	=
								{
									key		=	'name',
									value	=	'psf:SelectionType'
								},
								children =
								{
									tag			= 	'psf:Value',
									attribute	=
									{
										key		=	'xsi:type',
										value	=	'xsd:QName'
									},
									text		=	'psk:PickOne'
								}
							},
							{
								tag			= 	'psf:Option',
								attribute	=
								{
									key		=	'name',
									value	=	'psk:Uncollated'
								}
							},
							{
								tag			= 	'psf:Option',
								attribute	=
								{
									key		=	'name',
									value	=	'psk:Collated'
								}
							}
						}
				}
			
			if('on' == tbl['supportcollate']['current']['name'])then
				return capability_table
			else
				return nil
			end
		end
	},
	collate	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:DocumentCollateDistribution'
		},
		children	=	function(tbl)
			return standard_option_generator(
				tbl['collate'],
				OrderedHash(
					{ 'application',	'ns0000:ApplicationCollate'},
					{ 'printer',		'ns0000:PrinterCollate'}
				) )
		end
	},
	-- 03-01 Job Offset
	joboffset	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:JobOffset'
		},
		children	=	function(tbl)
			return standard_option_generator(
				tbl['joboffset'],
				OrderedHash(
					{ 'off',	'ns0000:Normal'},
					{ 'rotate',	'ns0000:Rotate'},
					{ 'shift',	'ns0000:Shift'}
				) )
		end
	},
	-- 04 Crop Marks
	cropmarks	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:JobCrops'
		},
		children	=	function(tbl)
			local element_table = OrderedHash(
				{ 'off',		{value = 'ns0000:Off', scoredproperty = {}}},
				{ 'on',			{value = 'ns0000:AllPages', scoredproperty = {}}},
				{ 'firstpage',	{value = 'ns0000:FirstPageOnly', scoredproperty = {}}}
			)

			local cropmarks_table = tbl['cropmarks']
			
			if(nil ~= tbl['cropmarks_firstpageonly'])then
				table.insert(cropmarks_table,{ name = 'firstpage', condition = {}})
			end
			
			--Set ScoredProperty Info--
			---CropMarksLength--
			if(nil ~= tbl['cropmarks_length_mm'])then
				table.insert(element_table['on']['scoredproperty'],{name = 'ns0000:CropMarksLength', def = 'ns0000:JobCropMarksLengthDef'})
				table.insert(element_table['firstpage']['scoredproperty'],{name = 'ns0000:CropMarksLength', def = 'ns0000:JobCropMarksLengthDef'})
			end

			return standard_option_generator(cropmarks_table, element_table)
		end
	},
	cropmarks_length_mm 	=
	{
		tag		=	'psf:ParameterDef',
		attribute	=
			{
				key		=	'name',
				value	=	'ns0000:JobCropMarksLengthDef'
			},
		children	=	function(tbl)
							--default
							local default_val = tonumber(tbl['cropmarks_length_mm']['default'])
							if(nil ~= default_val)then
								tbl['cropmarks_length_mm']['default'] = tostring(default_val * 100)
							else
								tbl['cropmarks_length_mm']['default'] = '0'
							end
							
							--max
							local max_val = tonumber(tbl['cropmarks_length_mm']['max'])
							if(nil ~= max_val)then
								tbl['cropmarks_length_mm']['max'] = tostring(max_val * 100)
							else
								tbl['cropmarks_length_mm']['max'] = '0'
							end
							
							--min
							local min_val = tonumber(tbl['cropmarks_length_mm']['min'])
							if(nil ~= min_val)then
								tbl['cropmarks_length_mm']['min'] = tostring(min_val * 100)
							else
								tbl['cropmarks_length_mm']['min'] = '0'
							end
							
							return paramdef_generator('integer', tbl['cropmarks_length_mm'], 'microns', 100)
						end
	},
	-- 05 Punch
	punch	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'psk:JobHolePunch'
		},
		children	=	function(tbl)
			return standard_option_generator(
				tbl['punch'],
				OrderedHash(
					{ 'off',				'psk:None'},
					{ 'left',				'ns0000:Left'},
					{ 'top',				'ns0000:Top'},
					{ 'right',				'ns0000:Right'},
					{ '1atlefttop',			'ns0000:LeftTop'},
					{ '2atleft',			'ns0000:Left2'},
					{ '2attop',				'ns0000:Top2'},
					{ '2atright',			'ns0000:Right2'},
					{ '3atleft',			'ns0000:Left3'},
					{ '3attop',				'ns0000:Top3'},
					{ '3atright',			'ns0000:Right3'},
					{ '4atleft',			'ns0000:Left4'},
					{ '4attop',				'ns0000:Top4'},
					{ '4atright',			'ns0000:Right4'},
					{ 'multiatleftortop',	'ns0000:LeftOrTopMulti'},
					{ 'multiatrightortop',	'ns0000:RightOrTopMulti'},
					{ 'multiatleft',		'ns0000:LeftMulti'},
					{ 'multiattop',			'ns0000:TopMulti'},
					{ 'multiatright',		'ns0000:RightMulti'},
					{ 'doubleatleftortop',	'ns0000:LeftOrTopDouble'},
					{ 'doubleatrightortop',	'ns0000:RightOrTopDouble'}
				) )
		end
	},
	-- 06 Staple
	staple	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'psk:JobStapleAllDocuments'
		},
		children	=	function(tbl)
			return standard_option_generator(
				tbl['staple'],
				OrderedHash(
					{ 'off',						'psk:None'},
					{ 'topleftslant',				'ns0000:TopLeftSlant'},
					{ 'toprightslant',				'ns0000:TopRightSlant'},
					{ 'topleft',					'psk:StapleTopLeft'},
					{ 'topright',					'psk:StapleTopRight'},
					{ '2attop',						'psk:StapleDualTop'},
					{ '2atleft',					'psk:StapleDualLeft'},
					{ '2atright',					'psk:StapleDualRight'},
					{ '2atcenter',					'psk:SaddleStitch'},
					{ 'topleftslantstapleless',		'ns0000:TopLeftSlantStapleless'},
					{ 'toprightslantstapleless',	'ns0000:TopRightSlantStapleless'}
				) )
		end
	},
	-- 07 Output Tray
	outputtray	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'psk:JobOutputBin'
		},
		children	=	function(tbl)
			return standard_option_generator(
				tbl['outputtray'],
				OrderedHash(
					{ 'printerdefault',			'ns0000:PrinterDefault'},
					{ 'standardtray',			'ns0000:Standard'},
					{ 'copytray',				'ns0000:Copy'},
					{ 'internalshifttray',		'ns0000:InternalShift'},
					{ 'shifttray',				'ns0000:Shift'},
					{ 'internaltray',			'ns0000:Internal'},
					{ 'internaltray2',			'ns0000:Internal2'},
					{ 'finisheruppertray',		'ns0000:FinisherUpper'},
					{ 'finishershifttray',		'ns0000:FinisherShift'},
					{ 'finishershifttray1',		'ns0000:FinisherShift1'},
					{ 'finishershifttray2',		'ns0000:FinisherShift2'},
					{ 'finishershifttraysecond',	'ns0000:FinisherShiftSecond'},
					{ 'bookletprocessortray',	'ns0000:BookletProcessorTray'},
					{ 'bookletprocessortray2',	'ns0000:BookletProcessorTray2'},
					{ 'finisherbooklettray',	'ns0000:FinisherBooklet'},
					{ 'trimmertray',			'ns0000:TrimmerTray'},
					{ 'lefttray',				'ns0000:LeftTray'},
					{ 'lefttray1',				'ns0000:LeftTray1'},
					{ 'lefttray2',				'ns0000:LeftTray2'},
					{ 'righttray',				'ns0000:RightTray'},
					{ 'mailboxprooftray',		'ns0000:MailboxProofTray'},
					{ 'mailboxtray1',			'ns0000:MailBoxBin1'},
					{ 'mailboxtray2',			'ns0000:MailBoxBin2'},
					{ 'mailboxtray3',			'ns0000:MailBoxBin3'},
					{ 'mailboxtray4',			'ns0000:MailBoxBin4'},
					{ 'mailboxtray5',			'ns0000:MailBoxBin5'},
					{ 'mailboxtray6',			'ns0000:MailBoxBin6'},
					{ 'mailboxtray7',			'ns0000:MailBoxBin7'},
					{ 'mailboxtray8',			'ns0000:MailBoxBin8'},
					{ 'mailboxtray9',			'ns0000:MailBoxBin9'},
					{ 'onesheetoutputtray',		'ns0000:OneSheetOutputTray'},
					{ 'loadingtray',			'ns0000:LoadingTray'},
					{ 'ringstacker',			'ns0000:RingStacker'},
					{ 'stackerprooftray',		'ns0000:StackerProof'},
					{ 'stackerprooftray2',			'ns0000:StackerProofTray2'},
					{ 'largestackertray',		'ns0000:LargeStacker'},
					{ 'largestackertray2',		'ns0000:LargeStacker2'},
					{ 'longstackertray',			'ns0000:LongStackerTray'},
					{ 'longstackertray2',			'ns0000:LongStackerTray2'},
					{ 'foldprooftray',			'ns0000:FoldProof'},
					{ 'upperpaperexit',			'ns0000:UpperPaper'},
					{ 'frontpaperexit',			'ns0000:FrontPaper'},
					{ 'lowerpaperexit',			'ns0000:LowerPaper'},
					{ 'rearpaperexit',			'ns0000:RearPaper'},
					{ 'peripheralsifbox',		'ns0000:PeripheralsIfBox'},
					{ 'twinloopbindtray',		'ns0000:TwinLoopBindTray'}
				) )
		end
	},
	-- 08 Copies
	copies	=
	{
		tag			=	'psf:ParameterDef',
		attribute	=
		{
			key		=	'name',
			value	=	'psk:JobCopiesAllDocuments'
		},
		children	=	function(tbl)
							return paramdef_generator('integer_unconditional', tbl['copies'], 'copies',1)
						end
	},
	-- 09 Insert Separation Sheet
	insertseparationsheet	=
	{
		{
			tag		= 	'psf:Feature',
			attribute	=
			{
				key		=	'name',
				value	=	'psk:DocumentSeparatorSheet'
			},
			children	=	function(tbl)
				
				local element_table = OrderedHash(
					{ 'off',	{value = 'psk:None', scoredproperty = {}}},
					{ 'on',		{value = 'psk:StartSheet', scoredproperty = {{name = 'ns0000:Copies', def = 'ns0000:DocumentSeparatorSheetCopies'}}}}
					)
				
				local result_tbl = standard_option_generator(tbl['insertseparationsheet'], element_table)
				
				--------Insert Separation Sheeet : Input Tray--------
				if(nil ~= tbl['insertseparationsheet_inputtray'])then
					local tray_element_table	= OrderedHash(
						{ 'tray1',					'ns0000:Tray1'},
						{ 'tray1roll',				'ns0000:Tray1_Roll'},
						{ 'tray2',					'ns0000:Tray2'},
						{ 'tray2roll',				'ns0000:Tray2_Roll'},
						{ 'tray3',					'ns0000:Tray3'},
						{ 'tray3lct',				'ns0000:Tray3_LCT'},
						{ 'tray3roll',				'ns0000:Tray3_Roll'},
						{ 'tray4',					'ns0000:Tray4'},
						{ 'tray4roll',				'ns0000:Tray4_Roll'},
						{ 'tray5',					'ns0000:Tray5'},
						{ 'tray6',					'ns0000:Tray6'},
						{ 'tray7',					'ns0000:Tray7'},
						{ 'tray8',					'ns0000:Tray8'},
						{ 'tray9',					'ns0000:Tray9'},
						{ 'tray10',					'ns0000:Tray10'},
						{ 'tray11',					'ns0000:Tray11'},
						{ 'traya',					'ns0000:TrayA'},
						{ 'bypasstray',				'ns0000:BypassTray'},
						{ 'frontbypasstray',		'ns0000:FrontBypassTray'},
						{ 'largecapacitytray',		'ns0000:LCT'},
						{ 'interposer',				'ns0000:Inserter'},
						{ 'interposeruppertray',	'ns0000:InserterUpper'},
						{ 'interposerlowertray',	'ns0000:InserterLower'},
						{ 'largecapacityinterposer1',		'ns0000:LargeCapacityInserter1'},
						{ 'largecapacityinterposer2',		'ns0000:LargeCapacityInserter2'},
						{ 'paperfeedtray',			'ns0000:PaperFeedTray'}
					)
					local tray_table = 
					{
						tag			= 	'psf:Feature',
						attribute	=
						{
							key		=	'name',
							value	=	'ns0000:SeparationSheetTray'
						},
						children	=	standard_option_generator(tbl['insertseparationsheet_inputtray'], tray_element_table)
					}
					table.insert(result_tbl, tray_table)
				end
				
				return result_tbl

			end
		},
		{
			tag			=	'psf:ParameterDef',
			attribute	=
			{
				key		=	'name',
				value	=	'ns0000:DocumentSeparatorSheetCopies'
			},
			children	=	function(tbl)
				return paramdef_generator('integer', tbl['insertseparationsheet_percopies'], 'copies',1)
			end
		}
	},
	-- 10 Reverse Order
	reverseorder	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'psk:JobPageOrder'
		},
		children	=	function(tbl)
			return standard_option_generator(
				tbl['reverseorder'],
				OrderedHash(
					{ 'off',	'psk:Standard'},
					{ 'on',		'psk:Reverse'}
				) )
		end
	},
	-- 11 Ring Binding
	ringbinding	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:JobRingBind'
		},
		children	=	function(tbl)
			return standard_option_generator(
				tbl['ringbinding'],
				OrderedHash(
					{ 'off',					'ns0000:Off'},
					{ 'leftortop',				'ns0000:LeftOrTopPunchAndRingBooklet'},
					{ 'rightortop',				'ns0000:RightOrTopPunchAndRingBooklet'},
					{ 'leftortoppunchonly',		'ns0000:LeftOrTopPunchOnly'},
					{ 'rightortoppunchonly',	'ns0000:RightOrTopPunchOnly'}
				) )
		end
	},
	-- 12 Fold
	fold	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:JobFold'
		},
		children	=	function(tbl)
			return standard_option_generator(
				tbl['fold'],
				OrderedHash(
					{ 'off',							'ns0000:Off'},
					{ 'rightzfold',						'ns0000:RightZFold'},
					{ 'leftzfold',						'ns0000:LeftZFold'},
					{ 'bottomzfold',					'ns0000:BottomZFold'},
					{ 'insidefold',						'ns0000:InsideFold'},
					{ 'outsidefold',					'ns0000:OutsideFold'},
					{ 'superimposedfoldinside',			'ns0000:SuperimposedFoldInside'},
					{ 'superimposedfoldoutside',		'ns0000:SuperimposedFoldOutside'},
					{ 'insidehalffold',					'ns0000:InsideHalfFold'},
					{ 'outsidehalffold',				'ns0000:OutsideHalfFold'},
					{ 'rightortopinsidespiralfold',		'ns0000:RightOrTopInsideSpiralFold'},
					{ 'leftorbottominsidespiralfold',	'ns0000:LeftOrBottomInsideSpiralFold'},
					{ 'rightortopoutsidespiralfold',	'ns0000:RightOrTopOutsideSpiralFold'},
					{ 'leftorbottomoutsidespiralfold',	'ns0000:LeftOrBottomOutsideSpiralFold'},
					{ 'outsidezigzagfold',				'ns0000:OutsideZigzagFold'},
					{ 'insidezigzagfold',				'ns0000:InsideZigzagFold'},
					{ 'rightortopinsidedoublefold',		'ns0000:RightOrTopInsideDoubleFold'},
					{ 'leftorbottominsidedoublefold',	'ns0000:LeftOrBottomInsideDoubleFold'},
					{ 'rightortopoutsidedoublefold',	'ns0000:RightOrTopOutsideDoubleFold'},
					{ 'leftorbottomoutsidedoublefold',	'ns0000:LeftOrBottomOutsideDoubleFold'},
					{ 'insidedoublegatefold',			'ns0000:InsideDoubleGateFold'},
					{ 'outsidedoublegatefold',			'ns0000:OutsideDoubleGateFold'},
					{ 'standardfold',					'ns0000:StandardFold'},
					{ 'narrowstandardfold',				'ns0000:NarrowStandardFold'},
					{ 'marginfold',						'ns0000:MarginFold'},
					{ 'fanfold140mm',					'ns0000:FanFold140mm'},
					{ 'fanfold170mm',					'ns0000:FanFold170mm'},
					{ 'fanfold210mm',					'ns0000:FanFold210mm'},
					{ 'fanfold297mm',					'ns0000:FanFold297mm'},
					{ 'fanfold8.5inch',					'ns0000:FanFold8.5inch'},
					{ 'fanfold9inch',					'ns0000:FanFold9inch'},
					{ 'fanfold11inch',					'ns0000:FanFold11inch'},
					{ 'fanfold12inch',					'ns0000:FanFold12inch'},
					{ 'fanfoldwithmargin',				'ns0000:FanFoldWithMargin'},
					{ 'specialfanfold1',				'ns0000:SpecialFanFold1'},
					{ 'specialfanfold2',				'ns0000:SpecialFanFold2'},
					{ 'specialfold1',					'ns0000:SpecialFold1'},
					{ 'specialfold2',					'ns0000:SpecialFold2'},
					{ 'programfold1',					'ns0000:ProgramFold1'},
					{ 'programfold2',					'ns0000:ProgramFold2'},
					{ 'programfold3',					'ns0000:ProgramFold3'},
					{ 'programfold4',					'ns0000:ProgramFold4'}
				) )
		end
	},
	-- 12-01 Multi-sheet fold
	multisheetfold	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:JobMultiSheetFold'
		},
		children	=	function(tbl)
			return standard_option_generator(
				tbl['multisheetfold'],
				OrderedHash(
					{ 'on',		'ns0000:On'},
					{ 'off',	'ns0000:Off'}
				) )
		end
	},
	-- 12-02 Fold Stamp
	foldstamp	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:JobFoldStamp'
		},
		children	=	function(tbl)
			return standard_option_generator(
				tbl['foldstamp'],
				OrderedHash(
					{ 'on',		'ns0000:On'},
					{ 'off',	'ns0000:Off'}
				) )
		end
	},
	-- 12-03 Fold Punch
	foldpunch	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:JobFoldPunch'
		},
		children	=	function(tbl)
			return standard_option_generator(
				tbl['foldpunch'],
				OrderedHash(
					{ 'on',		'ns0000:On'},
					{ 'off',	'ns0000:Off'}
				) )
		end
	},
	-- 12-04 Edge Fold
	edgefold	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:JobEdgeFold'
		},
		children	=	function(tbl)
			return standard_option_generator(
				tbl['edgefold'],
				OrderedHash(
					{ 'on',		'ns0000:On'},
					{ 'off',	'ns0000:Off'}
				) )
		end
	},
	-- 13 Trimming
	trimming	=
	{
		tag		=	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:JobTrimming'
		},
		children	=	function(tbl)
			local element_table = OrderedHash(
				{ 'off',	{value = 'ns0000:Off', scoredproperty = {}}},
				{ 'on',		{value = 'ns0000:On', scoredproperty = {}}}
			)

			--Set ScoredProperty Info--
			---CutEdge--
			if(nil ~= tbl['trimming_cutedge_mm'])then
				table.insert(element_table['on']['scoredproperty'],{name = 'ns0000:TrimmingCutEdge', def = 'ns0000:JobTrimmingCutEdgeDef'})
			end
			return standard_option_generator(tbl['trimming'], element_table)
		end
	},
	trimming_cutedge_mm 	=
	{
		tag		=	'psf:ParameterDef',
		attribute	=
			{
				key		=	'name',
				value	=	'ns0000:JobTrimmingCutEdgeDef'
			},
		children	=	function(tbl)
							--default
							local default_val = tonumber(tbl['trimming_cutedge_mm']['default'])
							if(nil ~= default_val)then
								tbl['trimming_cutedge_mm']['default'] = tostring(default_val * 100)
							else
								tbl['trimming_cutedge_mm']['default'] = '0'
							end
							
							--max
							local max_val = tonumber(tbl['trimming_cutedge_mm']['max'])
							if(nil ~= max_val)then
								tbl['trimming_cutedge_mm']['max'] = tostring(max_val * 100)
							else
								tbl['trimming_cutedge_mm']['max'] = '0'
							end
							
							--min
							local min_val = tonumber(tbl['trimming_cutedge_mm']['min'])
							if(nil ~= min_val)then
								tbl['trimming_cutedge_mm']['min'] = tostring(min_val * 100)
							else
								tbl['trimming_cutedge_mm']['min'] = '0'
							end
							
							return paramdef_generator('integer', tbl['trimming_cutedge_mm'], 'microns', 100)
						end
	},
	-- 14 Face Up
	faceup	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:JobEjectFaceUp'
		},
		children	=	function(tbl)
			return standard_option_generator(
				tbl['faceup'],
				OrderedHash(
					{ 'on',		'ns0000:On'},
					{ 'off',	'ns0000:Off'}
				) )
		end
	},
	-- 15 Do not print blank pages
	donotprintblankpages	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:JobWhitePaperSuppress'
		},
		children	=	function(tbl)
			return standard_option_generator(
				tbl['donotprintblankpages'],
				OrderedHash(
					{ 'on',		'ns0000:On'},
					{ 'off',	'ns0000:Off'}
				) )
		end
	},
	-- 16 Shift Paper
	shiftpaper	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:PageShiftPaper'
		},
		children	=	function(tbl)
			return standard_option_generator(
				tbl['shiftpaper'],
				OrderedHash(
					{ 'on',		'ns0000:On'},
					{ 'off',	'ns0000:Off'}
				) )
		end
	},
	-- 17 Orientation Override
	orientationoverride	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:DocumentOrientationOvrerride'
		},
		children	=	function(tbl)
			return standard_option_generator(
				tbl['orientationoverride'],
				OrderedHash(
					{ 'off',		'ns0000:Off'},
					{ 'landscape',	'ns0000:Landscape'},
					{ 'portrait',	'ns0000:Portrait'}
				) )
		end
	},
	-- 18 Perfect Binding
	perfectbinding	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:JobPerfectBind'
		},
		children	=	function(tbl)
			local result_tbl = {}
			local element_table = OrderedHash(
				{ 'off',		{value = 'ns0000:Off', scoredproperty = {}}},
				{ 'leftortop',	{value = 'ns0000:LeftOrTop', scoredproperty = {}}},
				{ 'rightortop',	{value = 'ns0000:RightOrTop', scoredproperty = {}}}
			)

			local perfectbinding_table = tbl['perfectbinding_direction']
			
			if(nil ~= tbl['perfectbinding_direction'])then
				table.insert(perfectbinding_table,{ name = 'off', condition = {}})
			end 

			--Set ScoredProperty Info--
			--Image Shift--
			if(nil ~= tbl['perfectbinding_imgpos_along_mm'])then
				table.insert(element_table['leftortop']['scoredproperty'],{name = 'ns0000:VerticalImageShift', def = 'ns0000:JobPerfectBindVerticalShift'})
				table.insert(element_table['rightortop']['scoredproperty'],{name = 'ns0000:VerticalImageShift', def = 'ns0000:JobPerfectBindVerticalShift'})
			end
			
			if(nil ~= tbl['perfectbinding_imgpos_across_mm'])then
				table.insert(element_table['leftortop']['scoredproperty'],{name = 'ns0000:HorizontalImageShift', def = 'ns0000:JobPerfectBindHorizontalShift'})
				table.insert(element_table['rightortop']['scoredproperty'],{name = 'ns0000:HorizontalImageShift', def = 'ns0000:JobPerfectBindHorizontalShift'})
			end
			result_tbl = standard_option_generator(perfectbinding_table, element_table)
							
			-- Binding Cut Type
			if(nil ~= tbl['perfectbinding_cuttype'])then
				local cuttype_element_table	= OrderedHash(
					{ 'donotcut',							{value = 'ns0000:Off', scoredproperty = {}}},
					{ 'cutforeedgespecifyfinishingsize',	{value = 'ns0000:SingleCutFinishSize', scoredproperty = {}}},
					{ 'cutforeedge',						{value = 'ns0000:SingleCut', scoredproperty = {}}},
					{ 'cut3edgesspecifyfinishingsize',		{value = 'ns0000:TripleCutFinishSize', scoredproperty = {}}},
					{ 'cut3edges',							{value = 'ns0000:TripleCut', scoredproperty = {}}}
				)
	
				--Cut Position Fine Adjustment--
				if(nil ~= tbl['perfectbinding_cutpos_along_mm'])then
					table.insert(cuttype_element_table['cut3edgesspecifyfinishingsize']['scoredproperty'],{name = 'ns0000:CutAdjustmentShift', def = 'ns0000:JobPerfectBindCutAdjustmentShift'})
				end
				
				--Cut Head Edge--
				if(nil ~= tbl['perfectbinding_cutpos_head_mm'])then
					table.insert(cuttype_element_table['cut3edges']['scoredproperty'],{name = 'ns0000:TopCutLength', def = 'ns0000:JobPerfectBindTopCutLength'})
				end
				
				--Cut Tail Edge--
				if(nil ~= tbl['perfectbinding_cutpos_tail_mm'])then
					table.insert(cuttype_element_table['cut3edges']['scoredproperty'],{name = 'ns0000:BottomCutLength', def = 'ns0000:JobPerfectBindBottomCutLength'})
				end
				
				--Cut Fore Edge--
				if(nil ~= tbl['perfectbinding_cutpos_fore_mm'])then
					table.insert(cuttype_element_table['cutforeedge']['scoredproperty'],{name = 'ns0000:SideEdgeCutLength', def = 'ns0000:JobPerfectBindSideEdgeCutLength'})
					table.insert(cuttype_element_table['cut3edges']['scoredproperty'],{name = 'ns0000:SideEdgeCutLength', def = 'ns0000:JobPerfectBindSideEdgeCutLength'})
				end
								
				--Length Finish Size--
				if(nil ~= tbl['perfectbinding_cutpos_length_mm'])then
					table.insert(cuttype_element_table['cutforeedgespecifyfinishingsize']['scoredproperty'],{name = 'ns0000:PageWidthCutLength', def = 'ns0000:JobPerfectBindCutAdjustmentByPageWidth'})
				end
				
				local cuttype_table = standard_option_generator(tbl['perfectbinding_cuttype'], cuttype_element_table)
				table.insert(result_tbl, {
					tag			= 	'psf:Feature',
					attribute	=
					{
						key		=	'name',
						value	=	'ns0000:PerfectBindTrimMode'
					},
					children	=	cuttype_table
				})
			end

			-- Specify Cut Size
			if(nil ~= tbl['perfectbinding_cutpos_finishing_size'])then
				local cutsize_element_table	= OrderedHash(
					{ '12x18',					{value = 'ns0000:_12X18', scoredproperty = {}}},
					{ '11x17',					{value = 'ns0000:NorthAmerica11x17', scoredproperty = {}}},
					{ 'legal',					{value = 'ns0000:NorthAmericaLegal', scoredproperty = {}}},
					{ 'letter',					{value = 'ns0000:NorthAmericaLetter', scoredproperty = {}}},
					{ 'executive',				{value = 'ns0000:NorthAmericaExecutive', scoredproperty = {}}},
					{ '5.5x8.5',				{value = 'ns0000:NorthAmericaStatement', scoredproperty = {}}},
					{ 'a0',						{value = 'ns0000:ISOA0', scoredproperty = {}}},
					{ 'a1',						{value = 'ns0000:ISOA1', scoredproperty = {}}},
					{ 'a2',						{value = 'ns0000:ISOA2', scoredproperty = {}}},
					{ 'a3',						{value = 'ns0000:ISOA3', scoredproperty = {}}},
					{ 'a4',						{value = 'ns0000:ISOA4', scoredproperty = {}}},
					{ 'a5',						{value = 'ns0000:ISOA5', scoredproperty = {}}},
					{ 'a6',						{value = 'ns0000:ISOA6', scoredproperty = {}}},
					{ 'b1',						{value = 'ns0000:JISB1', scoredproperty = {}}},
					{ 'b2',						{value = 'ns0000:JISB2', scoredproperty = {}}},
					{ 'b3',						{value = 'ns0000:JISB3', scoredproperty = {}}},
					{ 'b4jis',					{value = 'ns0000:JISB4', scoredproperty = {}}},
					{ 'b5jis',					{value = 'ns0000:JISB5', scoredproperty = {}}},
					{ 'b6',						{value = 'ns0000:JISB6', scoredproperty = {}}},
					{ 'b7',						{value = 'ns0000:JISB7', scoredproperty = {}}},
					{ '8x13',					{value = 'ns0000:_8X13', scoredproperty = {}}},
					{ '8.25x13',				{value = 'ns0000:_8_25X13', scoredproperty = {}}},
					{ '8.5x13',					{value = 'ns0000:FOLIO', scoredproperty = {}}},
					{ '8.5x13.4',				{value = 'ns0000:_8_5X13_4', scoredproperty = {}}},
					{ 'com10env',				{value = 'ns0000:NorthAmericaNumber10Envelope', scoredproperty = {}}},
					{ 'monarchenv',				{value = 'ns0000:NorthAmericaMonarchEnvelope', scoredproperty = {}}},
					{ 'dlenv',					{value = 'ns0000:ISODLEnvelope', scoredproperty = {}}},
					{ 'c6env',					{value = 'ns0000:ISOC6Envelope', scoredproperty = {}}},
					{ 'c5env',					{value = 'ns0000:ISOC5Envelope', scoredproperty = {}}},
					{ 'chou3jenv',				{value = 'ns0000:JapanChou3Envelope', scoredproperty = {}}},
					{ 'chou4jenv',				{value = 'ns0000:JapanChou4Envelope', scoredproperty = {}}},
					{ 'you4jenv',				{value = 'ns0000:JapanYou4Envelope', scoredproperty = {}}},
					{ 'youchou3jenv',			{value = 'ns0000:JapanYou4EnvelopeRotated', scoredproperty = {}}},
					{ 'kaku1jenv',				{value = 'ns0000:JapanKaku1Envelope', scoredproperty = {}}},
					{ 'kaku2jenv',				{value = 'ns0000:JapanKaku2Envelope', scoredproperty = {}}},
					{ 'bd',						{value = 'ns0000:BD', scoredproperty = {}}},
					{ 'lineslider1',			{value = 'ns0000:LineSlider1', scoredproperty = {}}},
					{ 'lineslider2',			{value = 'ns0000:LineSlider2', scoredproperty = {}}},
					{ 'glt',					{value = 'ns0000:_8X10_5', scoredproperty = {}}},
					{ '10x14',					{value = 'ns0000:NorthAmerica10x14', scoredproperty = {}}},
					{ '10x15',					{value = 'ns0000:_10X15', scoredproperty = {}}},
					{ '11x15',					{value = 'ns0000:_11X15', scoredproperty = {}}},
					{ '15x20',					{value = 'ns0000:_15X20', scoredproperty = {}}},
					{ '8k',						{value = 'ns0000:_8kai', scoredproperty = {}}},
					{ '16k',					{value = 'ns0000:_16kai', scoredproperty = {}}},
					{ 'postcard',				{value = 'ns0000:JapanHagakiPostcard', scoredproperty = {}}},
					{ 'doublepostcard',			{value = 'ns0000:JapanDoubleHagakiPostcardRotated', scoredproperty = {}}},
					{ 'doublepostcardrotated',	{value = 'ns0000:JapanDoubleHagakiPostcard', scoredproperty = {}}},
					{ 'governmentlg',			{value = 'ns0000:_8_25X14', scoredproperty = {}}},
					{ 'engquatro',				{value = 'ns0000:NorthAmerica8x10', scoredproperty = {}}},
					{ 'sra3',					{value = 'ns0000:_320X450', scoredproperty = {}}},
					{ 'sra4',					{value = 'ns0000:_225X320', scoredproperty = {}}},
					{ '11x14',					{value = 'ns0000:_11X14', scoredproperty = {}}},
					{ '13x19.2',				{value = 'ns0000:_13X19_2', scoredproperty = {}}},
					{ '13x19',					{value = 'ns0000:_13X19', scoredproperty = {}}},
					{ '12.6x19.2',				{value = 'ns0000:_12_6X19_2', scoredproperty = {}}},
					{ '12.6x18.5',				{value = 'ns0000:_12_6X18_5', scoredproperty = {}}},
					{ '13x18',					{value = 'ns0000:_13X18', scoredproperty = {}}},
					{ '226x310mm',				{value = 'ns0000:_226X310', scoredproperty = {}}},
					{ '310x432mm',				{value = 'ns0000:_310X432', scoredproperty = {}}},
					{ '8.5x12',					{value = 'ns0000:NorthAmericaGermanStandardFanfold', scoredproperty = {}}},
					{ '17x22',					{value = 'ns0000:Ansic', scoredproperty = {}}},
					{ '9x12',					{value = 'ns0000:_9X12', scoredproperty = {}}},
					{ '18x24',					{value = 'ns0000:_18X24', scoredproperty = {}}},
					{ '21x30',					{value = 'ns0000:_21X30', scoredproperty = {}}},
					{ '8.5x13.5',				{value = 'ns0000:_8_5X13_5', scoredproperty = {}}},
					{ 'kaku3jenv',				{value = 'ns0000:JapanKaku3Envelope', scoredproperty = {}}},
					{ '22x34',					{value = 'ns0000:_22X34', scoredproperty = {}}},
					{ '24x36',					{value = 'ns0000:_24X36', scoredproperty = {}}},
					{ '30x42',					{value = 'ns0000:_30X42', scoredproperty = {}}},
					{ '34x44',					{value = 'ns0000:_34X44', scoredproperty = {}}},
					{ '36x48',					{value = 'ns0000:_36X48', scoredproperty = {}}},
					{ 'postcardstandard',		{value = 'ns0000:PostcardStandard', scoredproperty = {}}},
					{ 'kiku4',					{value = 'ns0000:Kiku4', scoredproperty = {}}},
					{ '329x483mm',				{value = 'ns0000:_329X483', scoredproperty = {}}},
					{ 'custompapersize',		{value = 'ns0000:CustomMediaSize', scoredproperty = {}}},
					{ 'custompapersize1',		{value = 'ns0000:CustomSize1', scoredproperty = {}}},
					{ 'custompapersize2',		{value = 'ns0000:CustomSize2', scoredproperty = {}}},
					{ 'custompapersize3',		{value = 'ns0000:CustomSize3', scoredproperty = {}}},
					{ 'custompapersize4',		{value = 'ns0000:CustomSize4', scoredproperty = {}}},
					{ 'custompapersize5',		{value = 'ns0000:CustomSize5', scoredproperty = {}}},
					{ 'custompapersize6',		{value = 'ns0000:CustomSize6', scoredproperty = {}}},
					{ 'custompapersize7',		{value = 'ns0000:CustomSize7', scoredproperty = {}}},
					{ 'custompapersize8',		{value = 'ns0000:CustomSize8', scoredproperty = {}}},
					{ 'custompapersize9',		{value = 'ns0000:CustomSize9', scoredproperty = {}}},
					{ 'custompapersize10',		{value = 'ns0000:CustomSize10', scoredproperty = {}}},
					{ 'custompapersize11',		{value = 'ns0000:CustomSize11', scoredproperty = {}}},
					{ 'custompapersize12',		{value = 'ns0000:CustomSize12', scoredproperty = {}}},
					{ 'custompapersize13',		{value = 'ns0000:CustomSize13', scoredproperty = {}}},
					{ 'custompapersize14',		{value = 'ns0000:CustomSize14', scoredproperty = {}}},
					{ 'custompapersize15',		{value = 'ns0000:CustomSize15', scoredproperty = {}}},
					{ 'custompapersize16',		{value = 'ns0000:CustomSize16', scoredproperty = {}}},
					{ 'custompapersize17',		{value = 'ns0000:CustomSize17', scoredproperty = {}}},
					{ 'custompapersize18',		{value = 'ns0000:CustomSize18', scoredproperty = {}}},
					{ 'custompapersize19',		{value = 'ns0000:CustomSize19', scoredproperty = {}}},
					{ 'custompapersize20',		{value = 'ns0000:CustomSize20', scoredproperty = {}}},
					{ 'custompapersize21',		{value = 'ns0000:CustomSize21', scoredproperty = {}}},
					{ 'custompapersize22',		{value = 'ns0000:CustomSize22', scoredproperty = {}}},
					{ 'custompapersize23',		{value = 'ns0000:CustomSize23', scoredproperty = {}}},
					{ 'custompapersize24',		{value = 'ns0000:CustomSize24', scoredproperty = {}}},
					{ 'custompapersize25',		{value = 'ns0000:CustomSize25', scoredproperty = {}}},
					{ 'custompapersize26',		{value = 'ns0000:CustomSize26', scoredproperty = {}}},
					{ 'custompapersize27',		{value = 'ns0000:CustomSize27', scoredproperty = {}}},
					{ 'custompapersize28',		{value = 'ns0000:CustomSize28', scoredproperty = {}}},
					{ 'custompapersize29',		{value = 'ns0000:CustomSize29', scoredproperty = {}}},
					{ 'custompapersize30',		{value = 'ns0000:CustomSize30', scoredproperty = {}}},
					{ 'custompapersize31',		{value = 'ns0000:CustomSize31', scoredproperty = {}}},
					{ 'custompapersize32',		{value = 'ns0000:CustomSize32', scoredproperty = {}}},
					{ 'custompapersize33',		{value = 'ns0000:CustomSize33', scoredproperty = {}}},
					{ 'custompapersize34',		{value = 'ns0000:CustomSize34', scoredproperty = {}}},
					{ 'custompapersize35',		{value = 'ns0000:CustomSize35', scoredproperty = {}}},
					{ 'custompapersize36',		{value = 'ns0000:CustomSize36', scoredproperty = {}}},
					{ 'custompapersize37',		{value = 'ns0000:CustomSize37', scoredproperty = {}}},
					{ 'custompapersize38',		{value = 'ns0000:CustomSize38', scoredproperty = {}}},
					{ 'custompapersize39',		{value = 'ns0000:CustomSize39', scoredproperty = {}}},
					{ 'custompapersize40',		{value = 'ns0000:CustomSize40', scoredproperty = {}}},
					{ 'custompapersize41',		{value = 'ns0000:CustomSize41', scoredproperty = {}}},
					{ 'custompapersize42',		{value = 'ns0000:CustomSize42', scoredproperty = {}}},
					{ 'custompapersize43',		{value = 'ns0000:CustomSize43', scoredproperty = {}}},
					{ 'custompapersize44',		{value = 'ns0000:CustomSize44', scoredproperty = {}}},
					{ 'custompapersize45',		{value = 'ns0000:CustomSize45', scoredproperty = {}}},
					{ 'custompapersize46',		{value = 'ns0000:CustomSize46', scoredproperty = {}}},
					{ 'custompapersize47',		{value = 'ns0000:CustomSize47', scoredproperty = {}}},
					{ 'custompapersize48',		{value = 'ns0000:CustomSize48', scoredproperty = {}}},
					{ 'custompapersize49',		{value = 'ns0000:CustomSize49', scoredproperty = {}}},
					{ 'custompapersize50',		{value = 'ns0000:CustomSize50', scoredproperty = {}}},
					{ 'custompapersize51',		{value = 'ns0000:CustomSize51', scoredproperty = {}}},
					{ 'custompapersize52',		{value = 'ns0000:CustomSize52', scoredproperty = {}}},
					{ 'custompapersize53',		{value = 'ns0000:CustomSize53', scoredproperty = {}}},
					{ 'custompapersize54',		{value = 'ns0000:CustomSize54', scoredproperty = {}}},
					{ 'custompapersize55',		{value = 'ns0000:CustomSize55', scoredproperty = {}}},
					{ 'custompapersize56',		{value = 'ns0000:CustomSize56', scoredproperty = {}}},
					{ 'custompapersize57',		{value = 'ns0000:CustomSize57', scoredproperty = {}}},
					{ 'custompapersize58',		{value = 'ns0000:CustomSize58', scoredproperty = {}}},
					{ 'custompapersize59',		{value = 'ns0000:CustomSize59', scoredproperty = {}}},
					{ 'custompapersize60',		{value = 'ns0000:CustomSize60', scoredproperty = {}}},
					{ 'custompapersize61',		{value = 'ns0000:CustomSize61', scoredproperty = {}}},
					{ 'custompapersize62',		{value = 'ns0000:CustomSize62', scoredproperty = {}}},
					{ 'custompapersize63',		{value = 'ns0000:CustomSize63', scoredproperty = {}}},
					{ 'custompapersize64',		{value = 'ns0000:CustomSize64', scoredproperty = {}}},
					{ 'custompapersize65',		{value = 'ns0000:CustomSize65', scoredproperty = {}}},
					{ 'custompapersize66',		{value = 'ns0000:CustomSize66', scoredproperty = {}}},
					{ 'custompapersize67',		{value = 'ns0000:CustomSize67', scoredproperty = {}}},
					{ 'custompapersize68',		{value = 'ns0000:CustomSize68', scoredproperty = {}}},
					{ 'custompapersize69',		{value = 'ns0000:CustomSize69', scoredproperty = {}}},
					{ 'custompapersize70',		{value = 'ns0000:CustomSize70', scoredproperty = {}}},
					{ 'custompapersize71',		{value = 'ns0000:CustomSize71', scoredproperty = {}}},
					{ 'custompapersize72',		{value = 'ns0000:CustomSize72', scoredproperty = {}}},
					{ 'custompapersize73',		{value = 'ns0000:CustomSize73', scoredproperty = {}}},
					{ 'custompapersize74',		{value = 'ns0000:CustomSize74', scoredproperty = {}}},
					{ 'custompapersize75',		{value = 'ns0000:CustomSize75', scoredproperty = {}}},
					{ 'custompapersize76',		{value = 'ns0000:CustomSize76', scoredproperty = {}}},
					{ 'custompapersize77',		{value = 'ns0000:CustomSize77', scoredproperty = {}}},
					{ 'custompapersize78',		{value = 'ns0000:CustomSize78', scoredproperty = {}}},
					{ 'custompapersize79',		{value = 'ns0000:CustomSize79', scoredproperty = {}}},
					{ 'custompapersize80',		{value = 'ns0000:CustomSize80', scoredproperty = {}}},
					{ 'custompapersize81',		{value = 'ns0000:CustomSize81', scoredproperty = {}}},
					{ 'custompapersize82',		{value = 'ns0000:CustomSize82', scoredproperty = {}}},
					{ 'custompapersize83',		{value = 'ns0000:CustomSize83', scoredproperty = {}}},
					{ 'custompapersize84',		{value = 'ns0000:CustomSize84', scoredproperty = {}}},
					{ 'custompapersize85',		{value = 'ns0000:CustomSize85', scoredproperty = {}}},
					{ 'custompapersize86',		{value = 'ns0000:CustomSize86', scoredproperty = {}}},
					{ 'custompapersize87',		{value = 'ns0000:CustomSize87', scoredproperty = {}}},
					{ 'custompapersize88',		{value = 'ns0000:CustomSize88', scoredproperty = {}}},
					{ 'custompapersize89',		{value = 'ns0000:CustomSize89', scoredproperty = {}}},
					{ 'custompapersize90',		{value = 'ns0000:CustomSize90', scoredproperty = {}}},
					{ 'custompapersize91',		{value = 'ns0000:CustomSize91', scoredproperty = {}}},
					{ 'custompapersize92',		{value = 'ns0000:CustomSize92', scoredproperty = {}}},
					{ 'custompapersize93',		{value = 'ns0000:CustomSize93', scoredproperty = {}}},
					{ 'custompapersize94',		{value = 'ns0000:CustomSize94', scoredproperty = {}}},
					{ 'custompapersize95',		{value = 'ns0000:CustomSize95', scoredproperty = {}}},
					{ 'custompapersize96',		{value = 'ns0000:CustomSize96', scoredproperty = {}}},
					{ 'custompapersize97',		{value = 'ns0000:CustomSize97', scoredproperty = {}}},
					{ 'custompapersize98',		{value = 'ns0000:CustomSize98', scoredproperty = {}}},
					{ 'custompapersize99',		{value = 'ns0000:CustomSize99', scoredproperty = {}}},
					{ 'custompapersize100',		{value = 'ns0000:CustomSize100', scoredproperty = {}}}
				)

				--Custom Binding Paper Size--
				if(nil ~= tbl['perfectbinding_cutpos_customsize_width_mm'])then
					table.insert(cutsize_element_table['custompapersize']['scoredproperty'],{name = 'ns0000:CutSizeWidth', def = 'ns0000:JobPerfectBindOriginalSizeCutWidth'})
				end
				if(nil ~= tbl['perfectbinding_cutpos_customsize_length_mm'])then
					table.insert(cutsize_element_table['custompapersize']['scoredproperty'],{name = 'ns0000:CutSizeHeight', def = 'ns0000:JobPerfectBindOriginalSizeCutLehgth'})
				end
				
				local cutsize_table = standard_option_generator(tbl['perfectbinding_cutpos_finishing_size'], cutsize_element_table)
				table.insert(result_tbl, {
						tag			= 	'psf:Feature',
						attribute	=
						{
							key		=	'name',
							value	=	'ns0000:PerfectBindCutAdjustmentByPaperSize'
						},
						children	=	cutsize_table
					}
				)
			end

			-- Binding Cover Sheet Input Tray
			if(nil ~= tbl['perfectbinding_inputtray'])then
				local tray_element_table	= OrderedHash(
					{ 'tray1',					'ns0000:Tray1'},
					{ 'tray1roll',				'ns0000:Tray1_Roll'},
					{ 'tray2',					'ns0000:Tray2'},
					{ 'tray2roll',				'ns0000:Tray2_Roll'},
					{ 'tray3',					'ns0000:Tray3'},
					{ 'tray3lct',				'ns0000:Tray3_LCT'},
					{ 'tray3roll',				'ns0000:Tray3_Roll'},
					{ 'tray4',					'ns0000:Tray4'},
					{ 'tray4roll',				'ns0000:Tray4_Roll'},
					{ 'tray5',					'ns0000:Tray5'},
					{ 'tray6',					'ns0000:Tray6'},
					{ 'tray7',					'ns0000:Tray7'},
					{ 'tray8',					'ns0000:Tray8'},
					{ 'tray9',					'ns0000:Tray9'},
					{ 'tray10',					'ns0000:Tray10'},
					{ 'tray11',					'ns0000:Tray11'},
					{ 'traya',					'ns0000:TrayA'},
					{ 'onesheetbypass',			'ns0000:OneSheetBypassTray'},
					{ 'bypasstray',				'ns0000:BypassTray'},
					{ 'frontbypasstray',		'ns0000:FrontBypassTray'},
					{ 'largecapacitytray',		'ns0000:LCT'},
					{ 'interposer',				'ns0000:Inserter'},
					{ 'interposeruppertray',	'ns0000:InserterUpper'},
					{ 'interposerlowertray',	'ns0000:InserterLower'},
					{ 'bindinginterposer1',		'ns0000:BindInserter'},
					{ 'bindinginterposer2',		'ns0000:BindInserter2'},
					{ 'paperfeedtray',			'ns0000:PaperFeedTray'}
				)
				
				local tray_table = standard_option_generator(tbl['perfectbinding_inputtray'], tray_element_table)
				table.insert(result_tbl, {
						tag			= 	'psf:Feature',
						attribute	=
						{
							key		=	'name',
							value	=	'ns0000:PerfectBindCoverTray'
						},
						children	=	tray_table
					})
			end

			-- Perfect Binding Cover Sheet Print
			if(nil ~= tbl['perfectbinding_coversheetprint'])then
				local coverprint_element_table	= OrderedHash(
					{ 'off',				{value = 'ns0000:Off', scoredproperty = {}}},
					{ 'backcover',			{value = 'ns0000:BackCover', scoredproperty = {}}},
					{ 'frontcover',			{value = 'ns0000:FrontCover', scoredproperty = {}}},
					{ 'frontandbackcover',	{value = 'ns0000:FrontCoverAndBackCover', scoredproperty = {}}}
				)
					
				--Cover Sheet Image Position--
				if(nil ~= tbl['perfectbinding_coversheet_imgpos_along_mm'])then
					table.insert(coverprint_element_table['backcover']['scoredproperty'],{name = 'ns0000:ShiftAlong', def = 'ns0000:JobPerfectBindCoverPrintShiftAlong'})
					table.insert(coverprint_element_table['frontcover']['scoredproperty'],{name = 'ns0000:ShiftAlong', def = 'ns0000:JobPerfectBindCoverPrintShiftAlong'})
					table.insert(coverprint_element_table['frontandbackcover']['scoredproperty'],{name = 'ns0000:ShiftAlong', def = 'ns0000:JobPerfectBindCoverPrintShiftAlong'})
				end
				
				if(nil ~= tbl['perfectbinding_coversheet_imgpos_across_mm'])then
					table.insert(coverprint_element_table['backcover']['scoredproperty'],{name = 'ns0000:ShiftAcross', def = 'ns0000:JobPerfectBindCoverPrintShiftAcross'})
					table.insert(coverprint_element_table['frontcover']['scoredproperty'],{name = 'ns0000:ShiftAcross', def = 'ns0000:JobPerfectBindCoverPrintShiftAcross'})
					table.insert(coverprint_element_table['frontandbackcover']['scoredproperty'],{name = 'ns0000:ShiftAcross', def = 'ns0000:JobPerfectBindCoverPrintShiftAcross'})
				end
				
				
				local coverprint_table = standard_option_generator(tbl['perfectbinding_coversheetprint'], coverprint_element_table)
				table.insert(result_tbl, {
					tag			= 	'psf:Feature',
					attribute	=
					{
						key		=	'name',
						value	=	'ns0000:PerfectBindCoverPrint'
					},
					children	=	coverprint_table
				})
			end
			return result_tbl
		end
	},
	--Image Shift--
	perfectbinding_imgpos_along_mm 	=
	{
		tag		=	'psf:ParameterDef',
		attribute	=
			{
				key		=	'name',
				value	=	'ns0000:JobPerfectBindVerticalShift'
			},
		children	=	function(tbl)
							--default
							local default_val = tonumber(tbl['perfectbinding_imgpos_along_mm']['default'])
							if(nil ~= default_val)then
								tbl['perfectbinding_imgpos_along_mm']['default'] = tostring(default_val * 100)
							else
								tbl['perfectbinding_imgpos_along_mm']['default'] = '0'
							end
							
							--max
							local max_val = tonumber(tbl['perfectbinding_imgpos_along_mm']['max'])
							if(nil ~= max_val)then
								tbl['perfectbinding_imgpos_along_mm']['max'] = tostring(max_val * 100)
							else
								tbl['perfectbinding_imgpos_along_mm']['max'] = '0'
							end
							
							--min
							local min_val = tonumber(tbl['perfectbinding_imgpos_along_mm']['min'])
							if(nil ~= min_val)then
								tbl['perfectbinding_imgpos_along_mm']['min'] = tostring(min_val * 100)
							else
								tbl['perfectbinding_imgpos_along_mm']['min'] = '0'
							end
							
							return paramdef_generator('integer', tbl['perfectbinding_imgpos_along_mm'], 'microns', 100)
						end
	},
	perfectbinding_imgpos_across_mm 	=
	{
		tag		=	'psf:ParameterDef',
		attribute	=
			{
				key		=	'name',
				value	=	'ns0000:JobPerfectBindHorizontalShift'
			},
		children	=	function(tbl)
							--default
							local default_val = tonumber(tbl['perfectbinding_imgpos_across_mm']['default'])
							if(nil ~= default_val)then
								tbl['perfectbinding_imgpos_across_mm']['default'] = tostring(default_val * 100)
							else
								tbl['perfectbinding_imgpos_across_mm']['default'] = '0'
							end
							
							--max
							local max_val = tonumber(tbl['perfectbinding_imgpos_across_mm']['max'])
							if(nil ~= max_val)then
								tbl['perfectbinding_imgpos_across_mm']['max'] = tostring(max_val * 100)
							else
								tbl['perfectbinding_imgpos_across_mm']['max'] = '0'
							end
							
							--min
							local min_val = tonumber(tbl['perfectbinding_imgpos_across_mm']['min'])
							if(nil ~= min_val)then
								tbl['perfectbinding_imgpos_across_mm']['min'] = tostring(min_val * 100)
							else
								tbl['perfectbinding_imgpos_across_mm']['min'] = '0'
							end
							
							return paramdef_generator('integer', tbl['perfectbinding_imgpos_across_mm'], 'microns', 100)
						end
	},
	--Custom Binding Paper Size--
	perfectbinding_customsize_width_mm 	=
	{
		tag		=	'psf:ParameterDef',
		attribute	=
			{
				key		=	'name',
				value	=	'ns0000:JobPerfectBindOriginalSizeWidth'
			},
		children	=	function(tbl)
							--default
							local default_val = tonumber(tbl['perfectbinding_customsize_width_mm']['default'])
							if(nil ~= default_val)then
								tbl['perfectbinding_customsize_width_mm']['default'] = tostring(default_val * 100)
							else
								tbl['perfectbinding_customsize_width_mm']['default'] = '0'
							end
							
							--max
							local max_val = tonumber(tbl['perfectbinding_customsize_width_mm']['max'])
							if(nil ~= max_val)then
								tbl['perfectbinding_customsize_width_mm']['max'] = tostring(max_val * 100)
							else
								tbl['perfectbinding_customsize_width_mm']['max'] = '0'
							end
							
							--min
							local min_val = tonumber(tbl['perfectbinding_customsize_width_mm']['min'])
							if(nil ~= min_val)then
								tbl['perfectbinding_customsize_width_mm']['min'] = tostring(min_val * 100)
							else
								tbl['perfectbinding_customsize_width_mm']['min'] = '0'
							end
							
							return paramdef_generator('integer', tbl['perfectbinding_customsize_width_mm'], 'microns', 100)
						end
	},
	perfectbinding_customsize_length_mm 	=
	{
		tag		=	'psf:ParameterDef',
		attribute	=
			{
				key		=	'name',
				value	=	'ns0000:JobPerfectBindOriginalSizeHeight'
			},
		children	=	function(tbl)
							--default
							local default_val = tonumber(tbl['perfectbinding_customsize_length_mm']['default'])
							if(nil ~= default_val)then
								tbl['perfectbinding_customsize_length_mm']['default'] = tostring(default_val * 100)
							else
								tbl['perfectbinding_customsize_length_mm']['default'] = '0'
							end
							
							--max
							local max_val = tonumber(tbl['perfectbinding_customsize_length_mm']['max'])
							if(nil ~= max_val)then
								tbl['perfectbinding_customsize_length_mm']['max'] = tostring(max_val * 100)
							else
								tbl['perfectbinding_customsize_length_mm']['max'] = '0'
							end
							
							--min
							local min_val = tonumber(tbl['perfectbinding_customsize_length_mm']['min'])
							if(nil ~= min_val)then
								tbl['perfectbinding_customsize_length_mm']['min'] = tostring(min_val * 100)
							else
								tbl['perfectbinding_customsize_length_mm']['min'] = '0'
							end
							
							return paramdef_generator('integer', tbl['perfectbinding_customsize_length_mm'], 'microns', 100)
						end
	},
	--Cut Position Fine Adjustment--
	perfectbinding_cutpos_along_mm 	=
	{
		tag		=	'psf:ParameterDef',
		attribute	=
			{
				key		=	'name',
				value	=	'ns0000:JobPerfectBindCutAdjustmentShift'
			},
		children	=	function(tbl)
							--default
							local default_val = tonumber(tbl['perfectbinding_cutpos_along_mm']['default'])
							if(nil ~= default_val)then
								tbl['perfectbinding_cutpos_along_mm']['default'] = tostring(default_val * 100)
							else
								tbl['perfectbinding_cutpos_along_mm']['default'] = '0'
							end
							
							--max
							local max_val = tonumber(tbl['perfectbinding_cutpos_along_mm']['max'])
							if(nil ~= max_val)then
								tbl['perfectbinding_cutpos_along_mm']['max'] = tostring(max_val * 100)
							else
								tbl['perfectbinding_cutpos_along_mm']['max'] = '0'
							end
							
							--min
							local min_val = tonumber(tbl['perfectbinding_cutpos_along_mm']['min'])
							if(nil ~= min_val)then
								tbl['perfectbinding_cutpos_along_mm']['min'] = tostring(min_val * 100)
							else
								tbl['perfectbinding_cutpos_along_mm']['min'] = '0'
							end
							
							return paramdef_generator('integer', tbl['perfectbinding_cutpos_along_mm'], 'microns', 100)
						end
	},
	--Cut Head Edge--
	perfectbinding_cutpos_head_mm 	=
	{
		tag		=	'psf:ParameterDef',
		attribute	=
			{
				key		=	'name',
				value	=	'ns0000:JobPerfectBindTopCutLength'
			},
		children	=	function(tbl)
							--default
							local default_val = tonumber(tbl['perfectbinding_cutpos_head_mm']['default'])
							if(nil ~= default_val)then
								tbl['perfectbinding_cutpos_head_mm']['default'] = tostring(default_val * 100)
							else
								tbl['perfectbinding_cutpos_head_mm']['default'] = '0'
							end
							
							--max
							local max_val = tonumber(tbl['perfectbinding_cutpos_head_mm']['max'])
							if(nil ~= max_val)then
								tbl['perfectbinding_cutpos_head_mm']['max'] = tostring(max_val * 100)
							else
								tbl['perfectbinding_cutpos_head_mm']['max'] = '0'
							end
							
							--min
							local min_val = tonumber(tbl['perfectbinding_cutpos_head_mm']['min'])
							if(nil ~= min_val)then
								tbl['perfectbinding_cutpos_head_mm']['min'] = tostring(min_val * 100)
							else
								tbl['perfectbinding_cutpos_head_mm']['min'] = '0'
							end
							
							return paramdef_generator('integer', tbl['perfectbinding_cutpos_head_mm'], 'microns', 100)
						end
	},
	--Cut Tail Edge--
	perfectbinding_cutpos_tail_mm 	=
	{
		tag		=	'psf:ParameterDef',
		attribute	=
			{
				key		=	'name',
				value	=	'ns0000:JobPerfectBindBottomCutLength'
			},
		children	=	function(tbl)
							--default
							local default_val = tonumber(tbl['perfectbinding_cutpos_tail_mm']['default'])
							if(nil ~= default_val)then
								tbl['perfectbinding_cutpos_tail_mm']['default'] = tostring(default_val * 100)
							else
								tbl['perfectbinding_cutpos_tail_mm']['default'] = '0'
							end
							
							--max
							local max_val = tonumber(tbl['perfectbinding_cutpos_tail_mm']['max'])
							if(nil ~= max_val)then
								tbl['perfectbinding_cutpos_tail_mm']['max'] = tostring(max_val * 100)
							else
								tbl['perfectbinding_cutpos_tail_mm']['max'] = '0'
							end
							
							--min
							local min_val = tonumber(tbl['perfectbinding_cutpos_tail_mm']['min'])
							if(nil ~= min_val)then
								tbl['perfectbinding_cutpos_tail_mm']['min'] = tostring(min_val * 100)
							else
								tbl['perfectbinding_cutpos_tail_mm']['min'] = '0'
							end
							
							return paramdef_generator('integer', tbl['perfectbinding_cutpos_tail_mm'], 'microns', 100)
						end
	},
	--Cut Fore Edge--
	perfectbinding_cutpos_fore_mm 	=
	{
		tag		=	'psf:ParameterDef',
		attribute	=
			{
				key		=	'name',
				value	=	'ns0000:JobPerfectBindSideEdgeCutLength'
			},
		children	=	function(tbl)
							--default
							local default_val = tonumber(tbl['perfectbinding_cutpos_fore_mm']['default'])
							if(nil ~= default_val)then
								tbl['perfectbinding_cutpos_fore_mm']['default'] = tostring(default_val * 100)
							else
								tbl['perfectbinding_cutpos_fore_mm']['default'] = '0'
							end
							
							--max
							local max_val = tonumber(tbl['perfectbinding_cutpos_fore_mm']['max'])
							if(nil ~= max_val)then
								tbl['perfectbinding_cutpos_fore_mm']['max'] = tostring(max_val * 100)
							else
								tbl['perfectbinding_cutpos_fore_mm']['max'] = '0'
							end
							
							--min
							local min_val = tonumber(tbl['perfectbinding_cutpos_fore_mm']['min'])
							if(nil ~= min_val)then
								tbl['perfectbinding_cutpos_fore_mm']['min'] = tostring(min_val * 100)
							else
								tbl['perfectbinding_cutpos_fore_mm']['min'] = '0'
							end
							
							return paramdef_generator('integer', tbl['perfectbinding_cutpos_fore_mm'], 'microns', 100)
						end
	},
	--Length Finish Size--
	perfectbinding_cutpos_length_mm 	=
	{
		tag		=	'psf:ParameterDef',
		attribute	=
			{
				key		=	'name',
				value	=	'ns0000:JobPerfectBindCutAdjustmentByPageWidth'
			},
		children	=	function(tbl)
							--default
							local default_val = tonumber(tbl['perfectbinding_cutpos_length_mm']['default'])
							if(nil ~= default_val)then
								tbl['perfectbinding_cutpos_length_mm']['default'] = tostring(default_val * 100)
							else
								tbl['perfectbinding_cutpos_length_mm']['default'] = '0'
							end
							
							--max
							local max_val = tonumber(tbl['perfectbinding_cutpos_length_mm']['max'])
							if(nil ~= max_val)then
								tbl['perfectbinding_cutpos_length_mm']['max'] = tostring(max_val * 100)
							else
								tbl['perfectbinding_cutpos_length_mm']['max'] = '0'
							end
							
							--min
							local min_val = tonumber(tbl['perfectbinding_cutpos_length_mm']['min'])
							if(nil ~= min_val)then
								tbl['perfectbinding_cutpos_length_mm']['min'] = tostring(min_val * 100)
							else
								tbl['perfectbinding_cutpos_length_mm']['min'] = '0'
							end
							
							return paramdef_generator('integer', tbl['perfectbinding_cutpos_length_mm'], 'microns', 100)
						end
	},
	--Custom Binding Paper Size--
	perfectbinding_cutpos_customsize_width_mm 	=
	{
		tag		=	'psf:ParameterDef',
		attribute	=
			{
				key		=	'name',
				value	=	'ns0000:JobPerfectBindOriginalSizeCutWidth'
			},
		children	=	function(tbl)
							--default
							local default_val = tonumber(tbl['perfectbinding_cutpos_customsize_width_mm']['default'])
							if(nil ~= default_val)then
								tbl['perfectbinding_cutpos_customsize_width_mm']['default'] = tostring(default_val * 100)
							else
								tbl['perfectbinding_cutpos_customsize_width_mm']['default'] = '0'
							end
							
							--max
							local max_val = tonumber(tbl['perfectbinding_cutpos_customsize_width_mm']['max'])
							if(nil ~= max_val)then
								tbl['perfectbinding_cutpos_customsize_width_mm']['max'] = tostring(max_val * 100)
							else
								tbl['perfectbinding_cutpos_customsize_width_mm']['max'] = '0'
							end
							
							--min
							local min_val = tonumber(tbl['perfectbinding_cutpos_customsize_width_mm']['min'])
							if(nil ~= min_val)then
								tbl['perfectbinding_cutpos_customsize_width_mm']['min'] = tostring(min_val * 100)
							else
								tbl['perfectbinding_cutpos_customsize_width_mm']['min'] = '0'
							end
							
							return paramdef_generator('integer', tbl['perfectbinding_cutpos_customsize_width_mm'], 'microns', 100)
						end
	},
	perfectbinding_cutpos_customsize_length_mm 	=
	{
		tag		=	'psf:ParameterDef',
		attribute	=
			{
				key		=	'name',
				value	=	'ns0000:JobPerfectBindOriginalSizeCutLehgth'
			},
		children	=	function(tbl)
							--default
							local default_val = tonumber(tbl['perfectbinding_cutpos_customsize_length_mm']['default'])
							if(nil ~= default_val)then
								tbl['perfectbinding_cutpos_customsize_length_mm']['default'] = tostring(default_val * 100)
							else
								tbl['perfectbinding_cutpos_customsize_length_mm']['default'] = '0'
							end
							
							--max
							local max_val = tonumber(tbl['perfectbinding_cutpos_customsize_length_mm']['max'])
							if(nil ~= max_val)then
								tbl['perfectbinding_cutpos_customsize_length_mm']['max'] = tostring(max_val * 100)
							else
								tbl['perfectbinding_cutpos_customsize_length_mm']['max'] = '0'
							end
							
							--min
							local min_val = tonumber(tbl['perfectbinding_cutpos_customsize_length_mm']['min'])
							if(nil ~= min_val)then
								tbl['perfectbinding_cutpos_customsize_length_mm']['min'] = tostring(min_val * 100)
							else
								tbl['perfectbinding_cutpos_customsize_length_mm']['min'] = '0'
							end
							
							return paramdef_generator('integer', tbl['perfectbinding_cutpos_customsize_length_mm'], 'microns', 100)
						end
	},
	--Cover Sheet Image Position--
	perfectbinding_coversheet_imgpos_along_mm 	=
	{
		tag		=	'psf:ParameterDef',
		attribute	=
			{
				key		=	'name',
				value	=	'ns0000:JobPerfectBindCoverPrintShiftAlong'
			},
		children	=	function(tbl)
							--default
							local default_val = tonumber(tbl['perfectbinding_coversheet_imgpos_along_mm']['default'])
							if(nil ~= default_val)then
								tbl['perfectbinding_coversheet_imgpos_along_mm']['default'] = tostring(default_val * 100)
							else
								tbl['perfectbinding_coversheet_imgpos_along_mm']['default'] = '0'
							end
							
							--max
							local max_val = tonumber(tbl['perfectbinding_coversheet_imgpos_along_mm']['max'])
							if(nil ~= max_val)then
								tbl['perfectbinding_coversheet_imgpos_along_mm']['max'] = tostring(max_val * 100)
							else
								tbl['perfectbinding_coversheet_imgpos_along_mm']['max'] = '0'
							end
							
							--min
							local min_val = tonumber(tbl['perfectbinding_coversheet_imgpos_along_mm']['min'])
							if(nil ~= min_val)then
								tbl['perfectbinding_coversheet_imgpos_along_mm']['min'] = tostring(min_val * 100)
							else
								tbl['perfectbinding_coversheet_imgpos_along_mm']['min'] = '0'
							end
							
							return paramdef_generator('integer', tbl['perfectbinding_coversheet_imgpos_along_mm'], 'microns', 100)
						end
	},
	perfectbinding_coversheet_imgpos_across_mm 	=
	{
		tag		=	'psf:ParameterDef',
		attribute	=
			{
				key		=	'name',
				value	=	'ns0000:JobPerfectBindCoverPrintShiftAcross'
			},
		children	=	function(tbl)
							--default
							local default_val = tonumber(tbl['perfectbinding_coversheet_imgpos_across_mm']['default'])
							if(nil ~= default_val)then
								tbl['perfectbinding_coversheet_imgpos_across_mm']['default'] = tostring(default_val * 100)
							else
								tbl['perfectbinding_coversheet_imgpos_across_mm']['default'] = '0'
							end
							
							--max
							local max_val = tonumber(tbl['perfectbinding_coversheet_imgpos_across_mm']['max'])
							if(nil ~= max_val)then
								tbl['perfectbinding_coversheet_imgpos_across_mm']['max'] = tostring(max_val * 100)
							else
								tbl['perfectbinding_coversheet_imgpos_across_mm']['max'] = '0'
							end
							
							--min
							local min_val = tonumber(tbl['perfectbinding_coversheet_imgpos_across_mm']['min'])
							if(nil ~= min_val)then
								tbl['perfectbinding_coversheet_imgpos_across_mm']['min'] = tostring(min_val * 100)
							else
								tbl['perfectbinding_coversheet_imgpos_across_mm']['min'] = '0'
							end
							
							return paramdef_generator('integer', tbl['perfectbinding_coversheet_imgpos_across_mm'], 'microns', 100)
						end
	},
	-- 19 Sheet-dry print delay in duplex/booklet
	duplex_sheetdryprintdelay	=
	{
		tag			=	'psf:ParameterDef',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:JobDelayTimeForDuplex'
		},
		children	=	function(tbl)
							return paramdef_generator('integer_optional', tbl['duplex_sheetdryprintdelay'], 'seconds',1)
						end
	},
	-- 20 Generic Function
	genericfunction	=
	{
		tag			=	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:JobGenericFunction'
		},
		children	=	function(tbl)
			return standard_option_generator(
				tbl['genericfunction'],
				OrderedHash(
					{ 'off',				'ns0000:Off' },
					{ 'genericfunction1',	'ns0000:GenericFunction1' },
					{ 'genericfunction2',	'ns0000:GenericFunction2' },
					{ 'genericfunction3',	'ns0000:GenericFunction3' },
					{ 'genericfunction4',	'ns0000:GenericFunction4' },
					{ 'genericfunction5',	'ns0000:GenericFunction5' },
					{ 'genericfunction6',	'ns0000:GenericFunction6' },
					{ 'genericfunction7',	'ns0000:GenericFunction7' },
					{ 'genericfunction8',	'ns0000:GenericFunction8' },
					{ 'genericfunction9',	'ns0000:GenericFunction9' }
				) )
		end
	},
	-- 21 TwinLoop Binding
	twinloopbinding	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:JobTwinLoopBind'
		},
		children	=	function(tbl)
			return standard_option_generator(
				tbl['twinloopbinding'],
				OrderedHash(
					{ 'off',					'ns0000:Off'},
					{ 'leftortop',				'ns0000:LeftOrTopPunchAndTwinLoopBooklet'},
					{ 'rightortop',				'ns0000:RightOrTopPunchAndTwinLoopBooklet'}
				) )
		end
	},
	-- 22 Square Fold
	squarefold	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:JobSquareFold'
		},
		children	=	function(tbl)
			return standard_option_generator(
				tbl['squarefold'],
				OrderedHash(
					{ 'off',			'ns0000:Off'},
					{ 'on',				'ns0000:On'}
				) )
		end
	},
-- [03 Paper Feed]
	-- 01-01 Standard Pages : Input Tray
	standardpages_inputtray	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'psk:JobInputBin'
		},
		children	=	function(tbl)
			local result_tbl = {}

			local element_table = OrderedHash(
				{ 'autotrayselect',		{value = 'ns0000:AUTO',				wording = '*_inputtray_autotrayselect'}},
				{ 'tray1',				{value = 'ns0000:Tray1',			wording = '*_inputtray_tray1'}},
				{ 'tray1roll',			{value = 'ns0000:Tray1_Roll',		wording = '*_inputtray_tray1roll'}},
				{ 'tray2',				{value = 'ns0000:Tray2',			wording = '*_inputtray_tray2'}},
				{ 'tray2roll',			{value = 'ns0000:Tray2_Roll',		wording = '*_inputtray_tray2roll'}},
				{ 'tray3',				{value = 'ns0000:Tray3',			wording = '*_inputtray_tray3'}},
				{ 'tray3lct',			{value = 'ns0000:Tray3_LCT',		wording = '*_inputtray_tray3lct'}},
				{ 'tray3roll',			{value = 'ns0000:Tray3_Roll',		wording = '*_inputtray_tray3roll'}},
				{ 'tray4',				{value = 'ns0000:Tray4',			wording = '*_inputtray_tray4'}},
				{ 'tray4roll',			{value = 'ns0000:Tray4_Roll',		wording = '*_inputtray_tray4roll'}},
				{ 'tray5',				{value = 'ns0000:Tray5',			wording = '*_inputtray_tray5'}},
				{ 'tray6',				{value = 'ns0000:Tray6',			wording = '*_inputtray_tray6'}},
				{ 'tray7',				{value = 'ns0000:Tray7',			wording = '*_inputtray_tray7'}},
				{ 'tray8',				{value = 'ns0000:Tray8',			wording = '*_inputtray_tray8'}},
				{ 'tray9',				{value = 'ns0000:Tray9',			wording = '*_inputtray_tray9'}},
				{ 'tray10',				{value = 'ns0000:Tray10',			wording = '*_inputtray_tray10'}},
				{ 'tray11',				{value = 'ns0000:Tray11',			wording = '*_inputtray_tray11'}},
				{ 'traya',				{value = 'ns0000:TrayA',			wording = '*_inputtray_traya'}},
				{ 'bypasstray',			{value = 'ns0000:BypassTray',		wording = '*_inputtray_bypasstray'}},
				{ 'onesheetbypass',		{value = 'ns0000:OneSheetBypassTray',wording = '*_inputtray_onesheetbypass'}},
				{ 'frontbypasstray',	{value = 'ns0000:FrontBypassTray',	wording = '*_inputtray_frontbypasstray'}},
				{ 'largecapacitytray',	{value = 'ns0000:LCT',				wording = '*_inputtray_largecapacitytray'}},
				{ 'paperfeedtray',		{value = 'ns0000:PaperFeedTray',	wording = '*_inputtray_paperfeedtray'}}
			)

			if('on' == tbl['slipsheet'])then
				result_tbl = standard_option_generator(tbl['transparency_inputtray'], element_table)
			else
				result_tbl = standard_option_generator(tbl['standardpages_inputtray'], element_table)
			end
			
			local func_name = get_display_wording( '*_inputtray' )
			if(nil ~= func_name and "" ~= func_name)then
				table.insert(result_tbl,
					{
						tag			= 	'psf:Property',
						attribute	=
						{
							key		=	'name',
							value	=	'psk:DisplayName'
						},
						children = {
							tag			= 	'psf:Value',
							attribute	=
							{
								key		=	'xsi:type',
								value	=	'xsd:string'
							},
							text		=	func_name
						}
					}
				)
			end
			
			return result_tbl
		end
	},
	-- 01-02 Standard Pages : Media Type
	standardpages_mediatype	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'psk:PageMediaType'
		},
		children	=	function(tbl)
			local result_tbl = {}

			local element_table = OrderedHash(
				{ 'plainandrecycled',		{value = 'ns0000:Standard_Recycled',		wording = '*_mediatype_plainandrecycled'}},
				{ 'plain',					{value = 'psk:Plain',						wording = '*_mediatype_plain'}},
				{ 'plain2',					{value = 'ns0000:Plain2',					wording = '*_mediatype_plain2'}},
				{ 'recycled',				{value = 'ns0000:Recycled',					wording = '*_mediatype_recycled'}},
				{ 'inkjetplain',			{value = 'ns0000:InkjetPlain',				wording = '*_mediatype_inkjetplain'}},
				{ 'special',				{value = 'ns0000:Special',	 				wording = '*_mediatype_special'}},
				{ 'special2',				{value = 'ns0000:Special2',			 	 	wording = '*_mediatype_special2'}},
				{ 'special3',				{value = 'ns0000:Special3',					wording = '*_mediatype_special3'}},
				{ 'special4',				{value = 'ns0000:Special4',					wording = '*_mediatype_special4'}},
				{ 'special5',				{value = 'ns0000:Special5',					wording = '*_mediatype_special5'}},
				{ 'special6',				{value = 'ns0000:Special6',					wording = '*_mediatype_special6'}},
				{ 'thick',					{value = 'ns0000:Thick',					wording = '*_mediatype_thick'}},
				{ 'middlethick',			{value = 'ns0000:MiddleThick',				wording = '*_mediatype_middlethick'}},
				{ 'thick2',					{value = 'ns0000:Thick2',					wording = '*_mediatype_thick2'}},
				{ 'thick3',					{value = 'ns0000:Thick3',					wording = '*_mediatype_thick3'}},
				{ 'thick4',					{value = 'ns0000:Thick4',					wording = '*_mediatype_thick4'}},
				{ 'thin',					{value = 'ns0000:Thin',						wording = '*_mediatype_thin'}},
				{ 'transparency',			{value = 'psk:Transparency',				wording = '*_mediatype_transparency'}},
				{ 'translucent',			{value = 'ns0000:Translucent',				wording = '*_mediatype_translucent'}},
				{ 'color',					{value = 'ns0000:Color',					wording = '*_mediatype_color'}},
				{ 'color1',					{value = 'ns0000:Color1',					wording = '*_mediatype_color1'}},
				{ 'color2',					{value = 'ns0000:Color2',					wording = '*_mediatype_color2'}},
				{ 'letterhead',				{value = 'ns0000:Letterhead',				wording = '*_mediatype_letterhead'}},
				{ 'preprinted',				{value = 'ns0000:Preprinted',				wording = '*_mediatype_preprinted'}},
				{ 'plainduplexbackside',	{value = 'ns0000:Plain_Duplex_Backside',	wording = '*_mediatype_plainduplexbackside'}},
				{ 'thickduplexbackside',	{value = 'ns0000:Thick_Duplex_Backside',	wording = '*_mediatype_thickduplexbackside'}},
				{ 'thick2duplexbackside',	{value = 'ns0000:Thick_Duplex_Backside2',	wording = '*_mediatype_thick2duplexbackside'}},
				{ 'thick3duplexbackside',	{value = 'ns0000:Thick_Duplex_Backside3',	wording = '*_mediatype_thick3duplexbackside'}},
				{ 'thick4duplexbackside',	{value = 'ns0000:Thick_Duplex_Backside4',	wording = '*_mediatype_thick4duplexbackside'}},
				{ 'prepunched',				{value = 'ns0000:Prepunched',				wording = '*_mediatype_prepunched'}},
				{ 'labels',					{value = 'psk:Label',						wording = '*_mediatype_labels'}},
				{ 'bond',					{value = 'psk:Bond',						wording = '*_mediatype_bond'}},
				{ 'cardstock',				{value = 'psk:CardStock',					wording = '*_mediatype_cardstock'}},
				{ 'tabstock',				{value = 'psk:TabStockPreCut',				wording = '*_mediatype_tabstock'}},
				{ 'glossytabstock',			{value = 'ns0000:GlossyTabStock',			wording = '*_mediatype_glossytabstock'}},
				{ 'mattabstock',			{value = 'ns0000:MatTabStock',				wording = '*_mediatype_mattabstock'}},
				{ 'moreglossytabstock',		{value = 'ns0000:MoreGlossyTabStock',		wording = '*_mediatype_moreglossytabstock'}},
				{ 'yellow',					{value = 'ns0000:Yellow',					wording = '*_mediatype_yellow'}},
				{ 'green',					{value = 'ns0000:Green',			 	 	wording = '*_mediatype_green'}},
				{ 'blue',					{value = 'ns0000:Blue',						wording = '*_mediatype_blue'}},
				{ 'purple',					{value = 'ns0000:Purple',					wording = '*_mediatype_purple'}},
				{ 'ivory',					{value = 'ns0000:Ivory',					wording = '*_mediatype_ivory'}},
				{ 'orange',					{value = 'ns0000:Orange',					wording = '*_mediatype_orange'}},
				{ 'pink',					{value = 'ns0000:Pink',						wording = '*_mediatype_pink'}},
				{ 'red',					{value = 'ns0000:Red',						wording = '*_mediatype_red'}},
				{ 'gray',					{value = 'ns0000:Gray',						wording = '*_mediatype_gray'}},
				{ 'glossy',					{value = 'psk:PhotographicGlossy',			wording = '*_mediatype_glossy'}},
				{ 'semiglossy',				{value = 'ns0000:SemiGlossy',				wording = '*_mediatype_semiglossy'}},
				{ 'glossycoated',			{value = 'ns0000:GlossyCoated',				wording = '*_mediatype_glossycoated'}},
				{ 'matcoated',				{value = 'ns0000:MatCoated',				wording = '*_mediatype_matcoated'}},
				{ 'matfilm',				{value = 'ns0000:MatFilm',					wording = '*_mediatype_matfilm'}},
				{ 'thickmatcoated',			{value = 'ns0000:ThickMatCoated',			wording = '*_mediatype_thickmatcoated'}},
				{ 'hgplain',				{value = 'ns0000:HGPlain',					wording = '*_mediatype_hgplain'}},
				{ 'envelope',				{value = 'psk:EnvelopePlain',				wording = '*_mediatype_envelope'}},
				{ 'glossyenvelope',			{value = 'ns0000:GlossyEnvelope',			wording = '*_mediatype_glossyenvelope'}},
				{ 'matenvelope',			{value = 'ns0000:MatEnvelope',				wording = '*_mediatype_matenvelope'}},
				{ 'moreglossyenvelope',		{value = 'ns0000:MoreGlossyEnvelope',		wording = '*_mediatype_moreglossyenvelope'}},
				{ 'postcard',				{value = 'ns0000:Jpost',					wording = '*_mediatype_postcard'}},
				{ 'inkjetpostcard',			{value = 'ns0000:InkjetJpost',				wording = '*_mediatype_inkjetpostcard'}},
				{ 'coated',					{value = 'ns0000:Coated',					wording = '*_mediatype_coated'}},
				{ 'cadcoated',				{value = 'ns0000:CadCoated',				wording = '*_mediatype_cadcoated'}},
				{ 'postcardduplexbackside',	{value = 'ns0000:Jpost_Duplex_Backside',	wording = '*_mediatype_postcardduplexbackside'}},
				{ 'thickglossycoated',		{value = 'ns0000:ThickGlossyCoated',		wording = '*_mediatype_thickglossycoated'}},
				{ 'waterproof',				{value = 'ns0000:WaterProof',				wording = '*_mediatype_waterproof'}},
				{ 'film',					{value = 'ns0000:Film',						wording = '*_mediatype_film'}},
				{ 'thickglossy',			{value = 'ns0000:ThickGlossy',				wording = '*_mediatype_thickglossy'}},
				{ 'black',					{value = 'ns0000:Black',					wording = '*_mediatype_black'}},
				{ 'clearfile',				{value = 'ns0000:ClearFile',				wording = '*_mediatype_clearfile'}},
				{ 'magnet',					{value = 'ns0000:Magnet',					wording = '*_mediatype_magnet'}},
				{ 'metallicpearl',			{value = 'ns0000:MetallicPearl',			wording = '*_mediatype_metallicpearl'}},
				{ 'synthetic',				{value = 'ns0000:Synthetic',				wording = '*_mediatype_synthetic'}},
				{ 'texture',				{value = 'ns0000:Texture',					wording = '*_mediatype_texture'}},
				{ 'ncr',					{value = 'ns0000:NCR',						wording = '*_mediatype_ncr'}},
				{ 'auto',					{value = 'psk:AutoSelect',					wording = '*_mediatype_auto'}},
				{ 'none',					{value = 'ns0000:None',						wording = '*_mediatype_none'}}
			)
			
			-- Custom Media Type 1-200
			for key, num in pairs( get_available_custommedia_number() ) do
				element_table['+custom' .. num] = {value = 'ns0000:Custom' .. num, wording = '+custom' .. num, item = 'standardpages_mediatype'}
			end
			--for count = 1 , 200 , 1 do
			--	local num = tostring(count)
			--	element_table['+custom' .. num] = {value = 'ns0000:Custom' .. num, wording = '+custom' .. num, item = 'standardpages_mediatype'}
			--	
			--end
			if(nil ~= tbl['slipsheet'] and 'on' == tbl['slipsheet'])then
				result_tbl = standard_option_generator(tbl['transparency_mediatype'], element_table)
			else
				result_tbl = standard_option_generator(tbl['standardpages_mediatype'], element_table)
			end
			
			local func_name = get_display_wording( '*_mediatype' )
			if(nil ~= func_name and "" ~= func_name)then
				table.insert(result_tbl,
					{
						tag			= 	'psf:Property',
						attribute	=
						{
							key		=	'name',
							value	=	'psk:DisplayName'
						},
						children = {
							tag			= 	'psf:Value',
							attribute	=
							{
								key		=	'xsi:type',
								value	=	'xsd:string'
							},
							text		=	func_name
						}
					}
				)
			end
			return result_tbl
		end
	},
	-- 01-05 Tab Stock Printing
	standardpages_tabstockprinting	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:JobTabStockPrinting'
		},
		children	=	function(tbl)
			return standard_option_generator(
				tbl['standardpages_tabstockprinting'],
				OrderedHash(
					{ 'off',	'ns0000:Off'},
					{ 'on',		'ns0000:On'}
				) )
		end
	},
	-- 01-06 Shift Print Position for [Tab Stock] Printing
	standardpages_shiftprintposition	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:JobShftPositionForTabStock'
		},
		children	=	function(tbl)
			return standard_option_generator(
				tbl['standardpages_shiftprintposition'],
				OrderedHash(
					{ 'off',	'ns0000:Off'},
					{ 'on',		'ns0000:On'}
				) )
		end
	},
	-- 02 Front Cover
	frontcover	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:JobCoverFront'
		},
		children	=	function(tbl)
			local element_table = OrderedHash(
				{ 'off',		'ns0000:NoCover'},
				{ 'leaveblank',	'ns0000:BlankCover'},
				{ 'oneside',	'ns0000:PrintSimplex'},
				{ 'bothsides',	'ns0000:PrintDuplex'}
			)

			local frontcover_table = tbl['frontcover']
			
			if(nil ~= tbl['frontcover_printmode'])then
				for i, v in pairs(tbl['frontcover_printmode']) do
					table.insert(frontcover_table,v)
				end
			end

			local result_tbl = standard_option_generator(frontcover_table, element_table)

			--------Front Cover Input Tray--------
			if(nil ~= tbl['frontcover_inputtray'])then
				local tray_element_table	= OrderedHash(
					{ 'tray1',					'ns0000:Tray1'},
					{ 'tray1roll',				'ns0000:Tray1_Roll'},
					{ 'tray2',					'ns0000:Tray2'},
					{ 'tray2roll',				'ns0000:Tray2_Roll'},
					{ 'tray3',					'ns0000:Tray3'},
					{ 'tray3lct',				'ns0000:Tray3_LCT'},
					{ 'tray3roll',				'ns0000:Tray3_Roll'},
					{ 'tray4',					'ns0000:Tray4'},
					{ 'tray4roll',				'ns0000:Tray4_Roll'},
					{ 'tray5',					'ns0000:Tray5'},
					{ 'tray6',					'ns0000:Tray6'},
					{ 'tray7',					'ns0000:Tray7'},
					{ 'tray8',					'ns0000:Tray8'},
					{ 'tray9',					'ns0000:Tray9'},
					{ 'tray10',					'ns0000:Tray10'},
					{ 'tray11',					'ns0000:Tray11'},
					{ 'traya',					'ns0000:TrayA'},
					{ 'bypasstray',				'ns0000:BypassTray'},
					{ 'frontbypasstray',		'ns0000:FrontBypassTray'},
					{ 'largecapacitytray',		'ns0000:LCT'},
					{ 'interposer',				'ns0000:Inserter'},
					{ 'interposeruppertray',	'ns0000:InserterUpper'},
					{ 'interposerlowertray',	'ns0000:InserterLower'},
					{ 'largecapacityinterposer1',		'ns0000:LargeCapacityInserter1'},
					{ 'largecapacityinterposer2',		'ns0000:LargeCapacityInserter2'},
					{ 'paperfeedtray',			'ns0000:PaperFeedTray'}
				)

				local tray_table = 
				{
					tag			= 	'psf:Feature',
					attribute	=
					{
						key		=	'name',
						value	=	'ns0000:CoverFrontInputBin'
					},
					children	=	standard_option_generator(tbl['frontcover_inputtray'], tray_element_table)
				}
				table.insert(result_tbl, tray_table)
			end
			
			return result_tbl
		end
	},
	-- 03 Back Cover
	backcover	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:JobCoverBack'
		},
		children	=	function(tbl)
			local element_table	= OrderedHash(
				{ 'off',				'ns0000:NoCover'},
				{ 'leaveblank',			'ns0000:BlankCover'},
				{ 'printonbackcover',	'ns0000:PrintInner'},
				{ 'printonouter',		'ns0000:PrintOuter'}
			)
			local backcover_table = tbl['backcover']
			
			if(nil ~= tbl['backcover_printmode'])then
				for i, v in pairs(tbl['backcover_printmode']) do
					table.insert(backcover_table,v)
				end
			end

			local result_tbl = standard_option_generator(backcover_table, element_table)

			--------Back Cover Input Tray--------
			if(nil ~= tbl['backcover_inputtray'])then
				local tray_element_table	= OrderedHash(
					{ 'tray1',					'ns0000:Tray1'},
					{ 'tray1roll',				'ns0000:Tray1_Roll'},
					{ 'tray2',					'ns0000:Tray2'},
					{ 'tray2roll',				'ns0000:Tray2_Roll'},
					{ 'tray3',					'ns0000:Tray3'},
					{ 'tray3lct',				'ns0000:Tray3_LCT'},
					{ 'tray3roll',				'ns0000:Tray3_Roll'},
					{ 'tray4',					'ns0000:Tray4'},
					{ 'tray4roll',				'ns0000:Tray4_Roll'},
					{ 'tray5',					'ns0000:Tray5'},
					{ 'tray6',					'ns0000:Tray6'},
					{ 'tray7',					'ns0000:Tray7'},
					{ 'tray8',					'ns0000:Tray8'},
					{ 'tray9',					'ns0000:Tray9'},
					{ 'tray10',					'ns0000:Tray10'},
					{ 'tray11',					'ns0000:Tray11'},
					{ 'traya',					'ns0000:TrayA'},
					{ 'bypasstray',				'ns0000:BypassTray'},
					{ 'frontbypasstray',		'ns0000:FrontBypassTray'},
					{ 'largecapacitytray',		'ns0000:LCT'},
					{ 'interposer',				'ns0000:Inserter'},
					{ 'interposeruppertray',	'ns0000:InserterUpper'},
					{ 'interposerlowertray',	'ns0000:InserterLower'},
					{ 'largecapacityinterposer1',		'ns0000:LargeCapacityInserter1'},
					{ 'largecapacityinterposer2',		'ns0000:LargeCapacityInserter2'},
					{ 'paperfeedtray',			'ns0000:PaperFeedTray'}
				)
				local tray_table = 
				{
					tag			= 	'psf:Feature',
					attribute	=
					{
						key		=	'name',
						value	=	'ns0000:CoverBackInputBin'
					},
					children	=	standard_option_generator(tbl['backcover_inputtray'], tray_element_table)
				}
				table.insert(result_tbl, tray_table)
			end
			
			return result_tbl
		end
	},
	-- 04 Slip Sheet
	slipsheet	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:JobSlipSheet'
		},
		children	=	function(tbl)
			local element_table	= OrderedHash(
				{ 'off',	'ns0000:Off'},
				{ 'on',		'ns0000:On'}
			)
			local result_tbl = standard_option_generator(tbl['slipsheet'], element_table)
			
			--------Slip Sheet Print Modey--------
			if(nil ~= tbl['slipsheet_printmode'])then
				local printmode_element_table	= OrderedHash(
					{ 'leaveblank',			'ns0000:NotPrintOnSlipSheet'},
					{ 'printonslipsheet',	'ns0000:PrintOnSlipSheet'}
				)
				local mode_table = 
				{
					tag			= 	'psf:Feature',
					attribute	=
					{
						key		=	'name',
						value	=	'ns0000:PrintOnSlipSheet'
					},
					children	=	standard_option_generator(tbl['slipsheet_printmode'], printmode_element_table)
				}
				table.insert(result_tbl, mode_table)
			end
			
			--------Slip Sheet Input Tray--------
			if(nil ~= tbl['slipsheet_inputtray'])then
				local tray_element_table	= OrderedHash(
					{ 'tray1',					'ns0000:Tray1'},
					{ 'tray1roll',				'ns0000:Tray1_Roll'},
					{ 'tray2',					'ns0000:Tray2'},
					{ 'tray2roll',				'ns0000:Tray2_Roll'},
					{ 'tray3',					'ns0000:Tray3'},
					{ 'tray3lct',				'ns0000:Tray3_LCT'},
					{ 'tray3roll',				'ns0000:Tray3_Roll'},
					{ 'tray4',					'ns0000:Tray4'},
					{ 'tray4roll',				'ns0000:Tray4_Roll'},
					{ 'tray5',					'ns0000:Tray5'},
					{ 'tray6',					'ns0000:Tray6'},
					{ 'tray7',					'ns0000:Tray7'},
					{ 'tray8',					'ns0000:Tray8'},
					{ 'tray9',					'ns0000:Tray9'},
					{ 'tray10',					'ns0000:Tray10'},
					{ 'tray11',					'ns0000:Tray11'},
					{ 'traya',					'ns0000:TrayA'},
					{ 'bypasstray',				'ns0000:BypassTray'},
					{ 'frontbypasstray',		'ns0000:FrontBypassTray'},
					{ 'largecapacitytray',		'ns0000:LCT'},
					{ 'interposer',				'ns0000:Inserter'},
					{ 'interposeruppertray',	'ns0000:InserterUpper'},
					{ 'interposerlowertray',	'ns0000:InserterLower'},
					{ 'largecapacityinterposer1',		'ns0000:LargeCapacityInserter1'},
					{ 'largecapacityinterposer2',		'ns0000:LargeCapacityInserter2'},
					{ 'paperfeedtray',			'ns0000:PaperFeedTray'}
				)
				local tray_table = 
				{
					tag			= 	'psf:Feature',
					attribute	=
					{
						key		=	'name',
						value	=	'ns0000:SlipSheetInputBin'
					},
					children	=	standard_option_generator(tbl['slipsheet_inputtray'], tray_element_table)
				}
				table.insert(result_tbl, tray_table)
			end
			
			return result_tbl
		end
	},
	-- 05 Chapter Pages
	chapterpages	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:JobChapterPrint'
		},
		children	=	function(tbl)
			local element_table	= OrderedHash(
				{ 'off',	{value = 'ns0000:Off', scoredproperty = {}}},
				{ 'on',		{value = 'ns0000:On', scoredproperty = {}}}
			)
			
			--Chapter Page Setting--
			if(nil ~= tbl['chapterpages_setting'])then
				table.insert(element_table['on']['scoredproperty'],{name = 'ns0000:ChapterSettings', def = 'ns0000:JobChapterPrintSettings'})
			end

			return standard_option_generator(tbl['chapterpages'], element_table)
		end
	},
	chapterpages_setting	=
	{
		tag		=	'psf:ParameterDef',
		attribute	=
			{
				key		=	'name',
				value	=	'ns0000:JobChapterPrintSettings'
			},
		children	=	function(tbl)
							local chapterpages_table = 
							{
								['max'] = '5000',
								['min'] = '0',
								['default'] = tbl['chapterpages_setting']['default'],
								['current'] = tbl['chapterpages_setting']['current']
							}
							return paramdef_generator('string', chapterpages_table, 'characters', nil)
						end
	},
	-- 06 Bypass Tray's Media Type Common for Functional Paper
	functionalpaper_bypasstray_mediatype	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:JobFunctionalPaperMediaType'
		},
		children	=	function(tbl)
			local functionalpaper_bypasstray_mediatype_table = OrderedHash(
				{ 'plainandrecycled',			'ns0000:Standard_Recycled'},
				{ 'plain',						'ns0000:Plain'},
				{ 'plain2',						'ns0000:Plain2'},
				{ 'recycled',					'ns0000:Recycled'},
				{ 'inkjetplain',				'ns0000:InkjetPlain'},
				{ 'special',					'ns0000:Special'},
				{ 'special2',					'ns0000:Special2'},
				{ 'special3',					'ns0000:Special3'},
				{ 'special4',					'ns0000:Special4'},
				{ 'special5',					'ns0000:Special5'},
				{ 'special6',					'ns0000:Special6'},
				{ 'thick',						'ns0000:Thick'},
				{ 'middlethick',				'ns0000:MiddleThick'},
				{ 'thick2',						'ns0000:Thick2'},
				{ 'thick3',						'ns0000:Thick3'},
				{ 'thick4',						'ns0000:Thick4'},
				{ 'thin',						'ns0000:Thin'},
				{ 'transparency',				'ns0000:Transparency'},
				{ 'translucent',				'ns0000:Translucent'},
				{ 'color',						'ns0000:Color'},
				{ 'color1',						'ns0000:Color1'},
				{ 'color2',						'ns0000:Color2'},
				{ 'letterhead',					'ns0000:Letterhead'},
				{ 'preprinted',					'ns0000:Preprinted'},
				{ 'plainduplexbackside',		'ns0000:Plain_Duplex_Backside'},
				{ 'thickduplexbackside',		'ns0000:Thick_Duplex_Backside'},
				{ 'thick2duplexbackside',		'ns0000:Thick_Duplex_Backside2'},
				{ 'thick3duplexbackside',		'ns0000:Thick_Duplex_Backside3'},
				{ 'thick4duplexbackside',		'ns0000:Thick_Duplex_Backside4'},
				{ 'prepunched',					'ns0000:Prepunched'},
				{ 'labels',						'ns0000:Label'},
				{ 'bond',						'ns0000:Bond'},
				{ 'cardstock',					'ns0000:CardStock'},
				{ 'tabstock',					'ns0000:TabStockPreCut'},
				{ 'glossytabstock',				'ns0000:GlossyTabStock'},
				{ 'mattabstock',				'ns0000:MatTabStock'},
				{ 'moreglossytabstock',			'ns0000:MoreGlossyTabStock'},
				{ 'yellow',						'ns0000:Yellow'},
				{ 'green',						'ns0000:Green'},
				{ 'blue',						'ns0000:Blue'},
				{ 'purple',						'ns0000:Purple'},
				{ 'ivory',						'ns0000:Ivory'},
				{ 'orange',						'ns0000:Orange'},
				{ 'pink',						'ns0000:Pink'},
				{ 'red',						'ns0000:Red'},
				{ 'gray',						'ns0000:Gray'},
				{ 'glossy',						'ns0000:PhotographicGlossy'},
				{ 'semiglossy',					'ns0000:SemiGlossy'},
				{ 'glossycoated',				'ns0000:GlossyCoated'},
				{ 'matcoated',					'ns0000:MatCoated'},
				{ 'matfilm',					'ns0000:MatFilm'},
				{ 'thickmatcoated',				'ns0000:ThickMatCoated'},
				{ 'hgplain',					'ns0000:HGPlain'},
				{ 'envelope',					'ns0000:EnvelopePlain'},
				{ 'glossyenvelope',				'ns0000:GlossyEnvelope'},
				{ 'matenvelope',				'ns0000:MatEnvelope'},
				{ 'moreglossyenvelope',			'ns0000:MoreGlossyEnvelope'},
				{ 'postcard',					'ns0000:Jpost'},
				{ 'inkjetpostcard',				'ns0000:InkjetJpost'},
				{ 'coated',						'ns0000:Coated'},
				{ 'cadcoated',					'ns0000:CadCoated'},
				{ 'postcardduplexbackside',		'ns0000:Jpost_Duplex_Backside'},
				{ 'thickglossycoated',			'ns0000:ThickGlossyCoated'},
				{ 'waterproof',					'ns0000:WaterProof'},
				{ 'film',						'ns0000:Film'},
				{ 'thickglossy',				'ns0000:ThickGlossy'},
				{ 'black',						'ns0000:Black'},
				{ 'clearfile',					'ns0000:ClearFile'},
				{ 'magnet',						'ns0000:Magnet'},
				{ 'metallicpearl',				'ns0000:MetallicPearl'},
				{ 'synthetic',					'ns0000:Synthetic'},
				{ 'texture',					'ns0000:Texture'},
				{ 'ncr',						'ns0000:NCR'},
				{ 'auto',						'ns0000:AutoSelect'},
				{ 'none',						'ns0000:None'}
			)

			-- Custom Media Type 1-200
			for key, num in pairs( get_available_custommedia_number() ) do
				functionalpaper_bypasstray_mediatype_table['+custom' .. num] = ('ns0000:Custom' .. num)
			end

			return standard_option_generator( tbl['functionalpaper_bypasstray_mediatype'] , functionalpaper_bypasstray_mediatype_table )
		end
	},
	-- 07-01 Force Paper Direction in Bypass Tray
 	bypasstray_forcepaperdirection	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:JobBypassTrayForcePaperDirection'
		},
		children	=	function(tbl)
			local element_table = OrderedHash(
				{ 'off',	'ns0000:Off'},
				{ 'lef',	'ns0000:FeedFromLongEdge'},
				{ 'sef',	'ns0000:FeedFromShortEdge'}
			)

			local item_table = tbl['bypasstray_forcepaperdirection_feed']
			table.insert( item_table, { name = 'off', condition = {} } )

			return standard_option_generator( item_table, element_table)
		end
	},
	-- 11 Use Preprinted/Prepunched Paper
	usepreprintedprepunchedpaper	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:JobUsePrePrintedPrePunchedPaper'
		},
		children	=	function(tbl)
			return standard_option_generator(
				tbl['usepreprintedprepunchedpaper'],
				OrderedHash(
					{ 'on',		'ns0000:On'},
					{ 'off',	'ns0000:Off'}
				) )
		end
	},
	-- 12 Adjust paper feed timing
	quickpaperfeed	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:JobQuickPaperFeed'
		},
		children	=	function(tbl)
			return standard_option_generator(
				tbl['quickpaperfeed'],
				OrderedHash(
					{ 'on',		'ns0000:On'},
					{ 'off',	'ns0000:Off'}
				) )
		end
	},
	bannerpageprint	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:JobBannerPage'
		},
		children	=	function(tbl)
			local element_table	= OrderedHash(
				{ 'off',	'ns0000:Off'},
				{ 'on',		'ns0000:On'}
			)
			local result_tbl = standard_option_generator(tbl['bannerpageprint'], element_table)
			
			if(nil ~= tbl['bannerpageprint_inputtray'])then
				local tray_element_table	= OrderedHash(
					{ 'autotrayselect',		'ns0000:AUTO'},
					{ 'tray1',				'ns0000:Tray1'},
					{ 'tray1roll',			'ns0000:Tray1_Roll'},
					{ 'tray2',				'ns0000:Tray2'},
					{ 'tray2roll',			'ns0000:Tray2_Roll'},
					{ 'tray3',				'ns0000:Tray3'},
					{ 'tray3lct',			'ns0000:Tray3_LCT'},
					{ 'tray3roll',			'ns0000:Tray3_Roll'},
					{ 'tray4',				'ns0000:Tray4'},
					{ 'tray4roll',			'ns0000:Tray4_Roll'},
					{ 'tray5',				'ns0000:Tray5'},
					{ 'tray6',				'ns0000:Tray6'},
					{ 'tray7',				'ns0000:Tray7'},
					{ 'tray8',				'ns0000:Tray8'},
					{ 'tray9',				'ns0000:Tray9'},
					{ 'tray10',				'ns0000:Tray10'},
					{ 'tray11',				'ns0000:Tray11'},
					{ 'traya',				'ns0000:TrayA'},
					{ 'bypasstray',			'ns0000:BypassTray'},
					{ 'frontbypasstray',	'ns0000:FrontBypassTray'},
					{ 'onesheetbypass',		'ns0000:OneSheetBypassTray'},
					{ 'largecapacitytray',	'ns0000:LCT'},
					{ 'paperfeedtray',		'ns0000:PaperFeedTray'}
				)
				local tray_table = 
				{
					tag			= 	'psf:Feature',
					attribute	=
					{
						key		=	'name',
						value	=	'ns0000:BannerPageInputBin'
					},
					children	=	standard_option_generator(tbl['bannerpageprint_inputtray'], tray_element_table)
				}
				table.insert(result_tbl, tray_table)
			end
			
			if(nil ~= tbl['bannerpageprint_mediatype'])then
				local mediatype_element_table	= OrderedHash(
					{ 'plainandrecycled',			'ns0000:Standard_Recycled'},
					{ 'plain',						'ns0000:Plain'},
					{ 'plain2',						'ns0000:Plain2'},
					{ 'recycled',					'ns0000:Recycled'},
					{ 'inkjetplain',				'ns0000:InkjetPlain'},
					{ 'special',					'ns0000:Special'},
					{ 'special2',					'ns0000:Special2'},
					{ 'special3',					'ns0000:Special3'},
					{ 'special4',					'ns0000:Special4'},
					{ 'special5',					'ns0000:Special5'},
					{ 'special6',					'ns0000:Special6'},
					{ 'thick',						'ns0000:Thick'},
					{ 'middlethick',				'ns0000:MiddleThick'},
					{ 'thick2',						'ns0000:Thick2'},
					{ 'thick3',						'ns0000:Thick3'},
					{ 'thick4',						'ns0000:Thick4'},
					{ 'thin',						'ns0000:Thin'},
					{ 'transparency',				'ns0000:Transparency'},
					{ 'translucent',				'ns0000:Translucent'},
					{ 'color',						'ns0000:Color'},
					{ 'color1',						'ns0000:Color1'},
					{ 'color2',						'ns0000:Color2'},
					{ 'letterhead',					'ns0000:Letterhead'},
					{ 'preprinted',					'ns0000:Preprinted'},
					{ 'plainduplexbackside',		'ns0000:Plain_Duplex_Backside'},
					{ 'thickduplexbackside',		'ns0000:Thick_Duplex_Backside'},
					{ 'thick2duplexbackside',		'ns0000:Thick_Duplex_Backside2'},
					{ 'thick3duplexbackside',		'ns0000:Thick_Duplex_Backside3'},
					{ 'thick4duplexbackside',		'ns0000:Thick_Duplex_Backside4'},
					{ 'prepunched',					'ns0000:Prepunched'},
					{ 'labels',						'ns0000:Label'},
					{ 'bond',						'ns0000:Bond'},
					{ 'cardstock',					'ns0000:CardStock'},
					{ 'tabstock',					'ns0000:TabStockPreCut'},
					{ 'glossytabstock',				'ns0000:GlossyTabStock'},
					{ 'mattabstock',				'ns0000:MatTabStock'},
					{ 'moreglossytabstock',			'ns0000:MoreGlossyTabStock'},
					{ 'yellow',						'ns0000:Yellow'},
					{ 'green',						'ns0000:Green'},
					{ 'blue',						'ns0000:Blue'},
					{ 'purple',						'ns0000:Purple'},
					{ 'ivory',						'ns0000:Ivory'},
					{ 'orange',						'ns0000:Orange'},
					{ 'pink',						'ns0000:Pink'},
					{ 'red',						'ns0000:Red'},
					{ 'gray',						'ns0000:Gray'},
					{ 'glossy',						'ns0000:PhotographicGlossy'},
					{ 'semiglossy',					'ns0000:SemiGlossy'},
					{ 'glossycoated',				'ns0000:GlossyCoated'},
					{ 'matcoated',					'ns0000:MatCoated'},
					{ 'matfilm',					'ns0000:MatFilm'},
					{ 'thickmatcoated',				'ns0000:ThickMatCoated'},
					{ 'hgplain',					'ns0000:HGPlain'},
					{ 'envelope',					'ns0000:EnvelopePlain'},
					{ 'glossyenvelope',				'ns0000:GlossyEnvelope'},
					{ 'matenvelope',				'ns0000:MatEnvelope'},
					{ 'moreglossyenvelope',			'ns0000:MoreGlossyEnvelope'},
					{ 'postcard',					'ns0000:Jpost'},
					{ 'inkjetpostcard',				'ns0000:InkjetJpost'},
					{ 'coated',						'ns0000:Coated'},
					{ 'cadcoated',					'ns0000:CadCoated'},
					{ 'postcardduplexbackside',		'ns0000:Jpost_Duplex_Backside'},
					{ 'thickglossycoated',			'ns0000:ThickGlossyCoated'},
					{ 'waterproof',					'ns0000:WaterProof'},
					{ 'film',						'ns0000:Film'},
					{ 'thickglossy',				'ns0000:ThickGlossy'},
					{ 'black',						'ns0000:Black'},
					{ 'clearfile',					'ns0000:ClearFile'},
					{ 'magnet',						'ns0000:Magnet'},
					{ 'metallicpearl',				'ns0000:MetallicPearl'},
					{ 'synthetic',					'ns0000:Synthetic'},
					{ 'texture',					'ns0000:Texture'},
					{ 'ncr',						'ns0000:NCR'},
					{ 'auto',						'ns0000:AutoSelect'},
					{ 'none',						'ns0000:None'}
				)
				
				-- Custom Media Type 1-200
				for key, num in pairs( get_available_custommedia_number() ) do
					mediatype_element_table['+custom' .. num] = ('ns0000:Custom' .. num)
				end
				
				local mediatype_table = 
				{
					tag			= 	'psf:Feature',
					attribute	=
					{
						key		=	'name',
						value	=	'ns0000:BannerPageMediaType'
					},
					children	=	standard_option_generator(tbl['bannerpageprint_mediatype'], mediatype_element_table)
				}
				table.insert(result_tbl, mediatype_table)
			end
			
			return result_tbl
		end
	},
	-- 14 Letterhead Mode
	letterheadmode	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:JobLetterheadMode'
		},
		children	=	function(tbl)
			return standard_option_generator(
				tbl['letterheadmode'],
				OrderedHash(
					{ 'printerdefault',	'ns0000:PrinterDefault'},
					{ 'off',					'ns0000:Off'},
					{ 'auto',					'ns0000:Auto'},
					{ 'always',					'ns0000:Always'}
				) )
		end
	},
	-- 15 Duplex Mode for one sided print
	duplexmode	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:JobDuplexModeOneSidedPrint'
		},
		children	=	function(tbl)
			return standard_option_generator(
				tbl['duplexmode'],
				OrderedHash(
					{ 'off',	'ns0000:Off'},
					{ 'on',		'ns0000:On'}
				) )
		end
	},
-- [04 Printing Area]
	-- 01 Document Size
	documentsize	=
	{
		{
			tag			= 	'psf:Feature',
			attribute	=
			{
				key		=	'name',
				value	=	'psk:PageMediaSize'
			},
			children	=	function(tbl, option)
				local element_table	= OrderedHash(
					{ '12x18',						{value = 'ns0000:_12X18', wording = '*_12x18'}},
					{ '11x17',						{value = 'psk:NorthAmerica11x17', wording = '*_11x17'}},
					{ 'legal',						{value = 'psk:NorthAmericaLegal', wording = '*_legal'}},
					{ 'letter',						{value = 'psk:NorthAmericaLetter', wording = '*_letter'}},
					{ 'executive',					{value = 'psk:NorthAmericaExecutive', wording = '*_executive'}},
					{ '5.5x8.5',					{value = 'psk:NorthAmericaStatement', wording = '*_5.5x8.5'}},
					{ 'a0',							{value = 'psk:ISOA0', wording = '*_a0'}},
					{ 'a1',							{value = 'psk:ISOA1', wording = '*_a1'}},
					{ 'a2',							{value = 'psk:ISOA2', wording = '*_a2'}},
					{ 'a3',							{value = 'psk:ISOA3', wording = '*_a3'}},
					{ 'a4',							{value = 'psk:ISOA4', wording = '*_a4'}},
					{ 'a5',							{value = 'psk:ISOA5', wording = '*_a5'}},
					{ 'a6',							{value = 'psk:ISOA6', wording = '*_a6'}},
					{ 'b1',							{value = 'psk:JISB1', wording = '*_b1'}},
					{ 'b2',							{value = 'psk:JISB2', wording = '*_b2'}},
					{ 'b3',							{value = 'psk:JISB3', wording = '*_b3'}},
					{ 'b4jis',						{value = 'psk:JISB4', wording = '*_b4jis'}},
					{ 'b5jis',						{value = 'psk:JISB5', wording = '*_b5jis'}},
					{ 'b6',							{value = 'psk:JISB6', wording = '*_b6'}},
					{ 'b7',							{value = 'psk:JISB7', wording = '*_b7'}},
					{ '8x13',						{value = 'ns0000:_8X13', wording = '*_8x13'}},
					{ '8.25x13',					{value = 'ns0000:_8_25X13', wording = '*_8.25x13'}},
					{ '8.5x13',						{value = 'ns0000:FOLIO', wording = '*_8.5x13'}},
					{ '8.5x13.4',					{value = 'ns0000:_8_5X13_4', wording = '*_8.5x13.4'}},
					{ 'com10env',					{value = 'psk:NorthAmericaNumber10Envelope', wording = '*_com10env'}},
					{ 'monarchenv',					{value = 'psk:NorthAmericaMonarchEnvelope', wording = '*_monarchenv'}},
					{ 'dlenv',						{value = 'psk:ISODLEnvelope', wording = '*_dlenv'}},
					{ 'c6env',						{value = 'psk:ISOC6Envelope', wording = '*_c6env'}},
					{ 'c5env',						{value = 'psk:ISOC5Envelope', wording = '*_c5env'}},
					{ 'chou3jenv',					{value = 'psk:JapanChou3Envelope', wording = '*_chou3jenv'}},
					{ 'chou4jenv',					{value = 'psk:JapanChou4Envelope', wording = '*_chou4jenv'}},
					{ 'you4jenv',					{value = 'psk:JapanYou4Envelope', wording = '*_you4jenv'}},
					{ 'youchou3jenv',				{value = 'psk:JapanYou4EnvelopeRotated', wording = '*_youchou3jenv'}},
					{ 'kaku1jenv',					{value = 'ns0000:JapanKaku1Envelope', wording = '*_kaku1jenv'}},
					{ 'kaku2jenv',					{value = 'psk:JapanKaku2Envelope', wording = '*_kaku2jenv'}},
					{ 'bd',							{value = 'ns0000:BD', wording = '*_bd'}},
					{ 'lineslider1',				{value = 'ns0000:LineSlider1', wording = '*_lineslider1'}},
					{ 'lineslider2',				{value = 'ns0000:LineSlider2', wording = '*_lineslider2'}},
					{ 'glt',						{value = 'ns0000:_8X10_5', wording = '*_glt'}},
					{ '10x14',						{value = 'psk:NorthAmerica10x14', wording = '*_10x14'}},
					{ '10x15',						{value = 'ns0000:_10X15', wording = '*_10x15'}},
					{ '11x15',						{value = 'ns0000:_11X15', wording = '*_11x15'}},
					{ '15x20',						{value = 'ns0000:_15X20', wording = '*_15x20'}},
					{ '8k',							{value = 'ns0000:_8kai', wording = '*_8k'}},
					{ '16k',						{value = 'ns0000:_16kai', wording = '*_16k'}},
					{ 'postcard',					{value = 'psk:JapanHagakiPostcard', wording = '*_postcard'}},
					{ 'doublepostcard',				{value = 'psk:JapanDoubleHagakiPostcardRotated', wording = '*_doublepostcard'}},
					{ 'doublepostcardrotated',		{value = 'psk:JapanDoubleHagakiPostcard', wording = '*_doublepostcardrotated'}},
					{ 'governmentlg',				{value = 'ns0000:_8_25X14', wording = '*_governmentlg'}},
					{ 'engquatro',					{value = 'psk:NorthAmerica8x10', wording = '*_engquatro'}},
					{ 'sra3',						{value = 'ns0000:_320X450', wording = '*_sra3'}},
					{ 'sra4',						{value = 'ns0000:_225X320', wording = '*_sra4'}},
					{ '11x14',						{value = 'ns0000:_11X14', wording = '*_11x14'}},
					{ '13x19.2',					{value = 'ns0000:_13X19_2', wording = '*_13x19.2'}},
					{ '13x19',						{value = 'ns0000:_13X19', wording = '*_13x19'}},
					{ '12.6x19.2',					{value = 'ns0000:_12_6X19_2', wording = '*_12.6x19.2'}},
					{ '12.6x18.5',					{value = 'ns0000:_12_6X18_5', wording = '*_12.6x18.5'}},
					{ '13x18',						{value = 'ns0000:_13X18', wording = '*_13x18'}},
					{ '226x310mm',					{value = 'ns0000:_226X310', wording = '*_226x310mm'}},
					{ '310x432mm',					{value = 'ns0000:_310X432', wording = '*_310x432mm'}},
					{ '8.5x12',						{value = 'psk:NorthAmericaGermanStandardFanfold', wording = '*_8.5x12'}},
					{ '17x22',						{value = 'ns0000:Ansic', wording = '*_17x22'}},
					{ '9x12',						{value = 'ns0000:_9X12', wording = '*_9x12'}},
					{ '18x24',						{value = 'ns0000:_18X24', wording = '*_18x24'}},
					{ '21x30',						{value = 'ns0000:_21X30', wording = '*_21x30'}},
					{ '22x34',						{value = 'ns0000:_22X34', wording = '*_22x34'}},
					{ '24x36',						{value = 'ns0000:_24X36', wording = '*_24x36'}},
					{ '30x42',						{value = 'ns0000:_30X42', wording = '*_30x42'}},
					{ '34x44',						{value = 'ns0000:_34X44', wording = '*_34x44'}},
					{ '36x48',						{value = 'ns0000:_36X48', wording = '*_36x48'}},
					{ 'postcardstandard',			{value = 'ns0000:PostcardStandard', wording = '*_postcardstandard'}},
					{ 'kiku4',						{value = 'ns0000:Kiku4', wording = '*_kiku4'}},
					{ '329x483mm',					{value = 'ns0000:_329X483', wording = '*_329x483mm'}},
					{ 'photol',						{value = 'ns0000:PhotoL', wording = '*_photol'}},
					{ '8.5x13.5',					{value = 'ns0000:_8_5X13_5', wording = '*_8.5x13.5'}},
					{ 'kaku3jenv',					{value = 'psk:JapanKaku3Envelope', wording = '*_kaku3jenv'}},
					{ 'synchrocut_a0width',			{value = 'ns0000:CustomSizeWidthA0', wording = '*_synchrocut_a0width' }},
					{ 'synchrocut_a1width',			{value = 'ns0000:CustomSizeWidthA1', wording = '*_synchrocut_a1width' }},
					{ 'synchrocut_a2width',			{value = 'ns0000:CustomSizeWidthA2', wording = '*_synchrocut_a2width' }},
					{ 'synchrocut_a3width',			{value = 'ns0000:CustomSizeWidthA3', wording = '*_synchrocut_a3width' }},
					{ 'synchrocut_a4width',			{value = 'ns0000:CustomSizeWidthA4', wording = '*_synchrocut_a4width' }},
					{ 'synchrocut_customwidth',		{value = 'ns0000:CustomSizeSynchroCut', wording = '*_synchrocut_customwidth' }},
					{ 'custompapersize',			{value = 'psk:CustomMediaSize', wording = '*_custompapersize'}},
					{ 'custompapersize1',			{value = 'ns0000:CustomSize1', wording = 'custompapersize1', item = 'documentsize'}},
					{ 'custompapersize2',			{value = 'ns0000:CustomSize2', wording = 'custompapersize2', item = 'documentsize'}},
					{ 'custompapersize3',			{value = 'ns0000:CustomSize3', wording = 'custompapersize3', item = 'documentsize'}},
					{ 'custompapersize4',			{value = 'ns0000:CustomSize4', wording = 'custompapersize4', item = 'documentsize'}},
					{ 'custompapersize5',			{value = 'ns0000:CustomSize5', wording = 'custompapersize5', item = 'documentsize'}},
					{ 'custompapersize6',			{value = 'ns0000:CustomSize6', wording = 'custompapersize6', item = 'documentsize'}},
					{ 'custompapersize7',			{value = 'ns0000:CustomSize7', wording = 'custompapersize7', item = 'documentsize'}},
					{ 'custompapersize8',			{value = 'ns0000:CustomSize8', wording = 'custompapersize8', item = 'documentsize'}},
					{ 'custompapersize9',			{value = 'ns0000:CustomSize9', wording = 'custompapersize9', item = 'documentsize'}},
					{ 'custompapersize10',			{value = 'ns0000:CustomSize10', wording = 'custompapersize10', item = 'documentsize'}},
					{ 'custompapersize11',			{value = 'ns0000:CustomSize11', wording = 'custompapersize11', item = 'documentsize'}},
					{ 'custompapersize12',			{value = 'ns0000:CustomSize12', wording = 'custompapersize12', item = 'documentsize'}},
					{ 'custompapersize13',			{value = 'ns0000:CustomSize13', wording = 'custompapersize13', item = 'documentsize'}},
					{ 'custompapersize14',			{value = 'ns0000:CustomSize14', wording = 'custompapersize14', item = 'documentsize'}},
					{ 'custompapersize15',			{value = 'ns0000:CustomSize15', wording = 'custompapersize15', item = 'documentsize'}},
					{ 'custompapersize16',			{value = 'ns0000:CustomSize16', wording = 'custompapersize16', item = 'documentsize'}},
					{ 'custompapersize17',			{value = 'ns0000:CustomSize17', wording = 'custompapersize17', item = 'documentsize'}},
					{ 'custompapersize18',			{value = 'ns0000:CustomSize18', wording = 'custompapersize18', item = 'documentsize'}},
					{ 'custompapersize19',			{value = 'ns0000:CustomSize19', wording = 'custompapersize19', item = 'documentsize'}},
					{ 'custompapersize20',			{value = 'ns0000:CustomSize20', wording = 'custompapersize20', item = 'documentsize'}},
					{ 'custompapersize21',			{value = 'ns0000:CustomSize21', wording = 'custompapersize21', item = 'documentsize'}},
					{ 'custompapersize22',			{value = 'ns0000:CustomSize22', wording = 'custompapersize22', item = 'documentsize'}},
					{ 'custompapersize23',			{value = 'ns0000:CustomSize23', wording = 'custompapersize23', item = 'documentsize'}},
					{ 'custompapersize24',			{value = 'ns0000:CustomSize24', wording = 'custompapersize24', item = 'documentsize'}},
					{ 'custompapersize25',			{value = 'ns0000:CustomSize25', wording = 'custompapersize25', item = 'documentsize'}},
					{ 'custompapersize26',			{value = 'ns0000:CustomSize26', wording = 'custompapersize26', item = 'documentsize'}},
					{ 'custompapersize27',			{value = 'ns0000:CustomSize27', wording = 'custompapersize27', item = 'documentsize'}},
					{ 'custompapersize28',			{value = 'ns0000:CustomSize28', wording = 'custompapersize28', item = 'documentsize'}},
					{ 'custompapersize29',			{value = 'ns0000:CustomSize29', wording = 'custompapersize29', item = 'documentsize'}},
					{ 'custompapersize30',			{value = 'ns0000:CustomSize30', wording = 'custompapersize30', item = 'documentsize'}},
					{ 'custompapersize31',			{value = 'ns0000:CustomSize31', wording = 'custompapersize31', item = 'documentsize'}},
					{ 'custompapersize32',			{value = 'ns0000:CustomSize32', wording = 'custompapersize32', item = 'documentsize'}},
					{ 'custompapersize33',			{value = 'ns0000:CustomSize33', wording = 'custompapersize33', item = 'documentsize'}},
					{ 'custompapersize34',			{value = 'ns0000:CustomSize34', wording = 'custompapersize34', item = 'documentsize'}},
					{ 'custompapersize35',			{value = 'ns0000:CustomSize35', wording = 'custompapersize35', item = 'documentsize'}},
					{ 'custompapersize36',			{value = 'ns0000:CustomSize36', wording = 'custompapersize36', item = 'documentsize'}},
					{ 'custompapersize37',			{value = 'ns0000:CustomSize37', wording = 'custompapersize37', item = 'documentsize'}},
					{ 'custompapersize38',			{value = 'ns0000:CustomSize38', wording = 'custompapersize38', item = 'documentsize'}},
					{ 'custompapersize39',			{value = 'ns0000:CustomSize39', wording = 'custompapersize39', item = 'documentsize'}},
					{ 'custompapersize40',			{value = 'ns0000:CustomSize40', wording = 'custompapersize40', item = 'documentsize'}},
					{ 'custompapersize41',			{value = 'ns0000:CustomSize41', wording = 'custompapersize41', item = 'documentsize'}},
					{ 'custompapersize42',			{value = 'ns0000:CustomSize42', wording = 'custompapersize42', item = 'documentsize'}},
					{ 'custompapersize43',			{value = 'ns0000:CustomSize43', wording = 'custompapersize43', item = 'documentsize'}},
					{ 'custompapersize44',			{value = 'ns0000:CustomSize44', wording = 'custompapersize44', item = 'documentsize'}},
					{ 'custompapersize45',			{value = 'ns0000:CustomSize45', wording = 'custompapersize45', item = 'documentsize'}},
					{ 'custompapersize46',			{value = 'ns0000:CustomSize46', wording = 'custompapersize46', item = 'documentsize'}},
					{ 'custompapersize47',			{value = 'ns0000:CustomSize47', wording = 'custompapersize47', item = 'documentsize'}},
					{ 'custompapersize48',			{value = 'ns0000:CustomSize48', wording = 'custompapersize48', item = 'documentsize'}},
					{ 'custompapersize49',			{value = 'ns0000:CustomSize49', wording = 'custompapersize49', item = 'documentsize'}},
					{ 'custompapersize50',			{value = 'ns0000:CustomSize50', wording = 'custompapersize50', item = 'documentsize'}},
					{ 'custompapersize51',			{value = 'ns0000:CustomSize51', wording = 'custompapersize51', item = 'documentsize'}},
					{ 'custompapersize52',			{value = 'ns0000:CustomSize52', wording = 'custompapersize52', item = 'documentsize'}},
					{ 'custompapersize53',			{value = 'ns0000:CustomSize53', wording = 'custompapersize53', item = 'documentsize'}},
					{ 'custompapersize54',			{value = 'ns0000:CustomSize54', wording = 'custompapersize54', item = 'documentsize'}},
					{ 'custompapersize55',			{value = 'ns0000:CustomSize55', wording = 'custompapersize55', item = 'documentsize'}},
					{ 'custompapersize56',			{value = 'ns0000:CustomSize56', wording = 'custompapersize56', item = 'documentsize'}},
					{ 'custompapersize57',			{value = 'ns0000:CustomSize57', wording = 'custompapersize57', item = 'documentsize'}},
					{ 'custompapersize58',			{value = 'ns0000:CustomSize58', wording = 'custompapersize58', item = 'documentsize'}},
					{ 'custompapersize59',			{value = 'ns0000:CustomSize59', wording = 'custompapersize59', item = 'documentsize'}},
					{ 'custompapersize60',			{value = 'ns0000:CustomSize60', wording = 'custompapersize60', item = 'documentsize'}},
					{ 'custompapersize61',			{value = 'ns0000:CustomSize61', wording = 'custompapersize61', item = 'documentsize'}},
					{ 'custompapersize62',			{value = 'ns0000:CustomSize62', wording = 'custompapersize62', item = 'documentsize'}},
					{ 'custompapersize63',			{value = 'ns0000:CustomSize63', wording = 'custompapersize63', item = 'documentsize'}},
					{ 'custompapersize64',			{value = 'ns0000:CustomSize64', wording = 'custompapersize64', item = 'documentsize'}},
					{ 'custompapersize65',			{value = 'ns0000:CustomSize65', wording = 'custompapersize65', item = 'documentsize'}},
					{ 'custompapersize66',			{value = 'ns0000:CustomSize66', wording = 'custompapersize66', item = 'documentsize'}},
					{ 'custompapersize67',			{value = 'ns0000:CustomSize67', wording = 'custompapersize67', item = 'documentsize'}},
					{ 'custompapersize68',			{value = 'ns0000:CustomSize68', wording = 'custompapersize68', item = 'documentsize'}},
					{ 'custompapersize69',			{value = 'ns0000:CustomSize69', wording = 'custompapersize69', item = 'documentsize'}},
					{ 'custompapersize70',			{value = 'ns0000:CustomSize70', wording = 'custompapersize70', item = 'documentsize'}},
					{ 'custompapersize71',			{value = 'ns0000:CustomSize71', wording = 'custompapersize71', item = 'documentsize'}},
					{ 'custompapersize72',			{value = 'ns0000:CustomSize72', wording = 'custompapersize72', item = 'documentsize'}},
					{ 'custompapersize73',			{value = 'ns0000:CustomSize73', wording = 'custompapersize73', item = 'documentsize'}},
					{ 'custompapersize74',			{value = 'ns0000:CustomSize74', wording = 'custompapersize74', item = 'documentsize'}},
					{ 'custompapersize75',			{value = 'ns0000:CustomSize75', wording = 'custompapersize75', item = 'documentsize'}},
					{ 'custompapersize76',			{value = 'ns0000:CustomSize76', wording = 'custompapersize76', item = 'documentsize'}},
					{ 'custompapersize77',			{value = 'ns0000:CustomSize77', wording = 'custompapersize77', item = 'documentsize'}},
					{ 'custompapersize78',			{value = 'ns0000:CustomSize78', wording = 'custompapersize78', item = 'documentsize'}},
					{ 'custompapersize79',			{value = 'ns0000:CustomSize79', wording = 'custompapersize79', item = 'documentsize'}},
					{ 'custompapersize80',			{value = 'ns0000:CustomSize80', wording = 'custompapersize80', item = 'documentsize'}},
					{ 'custompapersize81',			{value = 'ns0000:CustomSize81', wording = 'custompapersize81', item = 'documentsize'}},
					{ 'custompapersize82',			{value = 'ns0000:CustomSize82', wording = 'custompapersize82', item = 'documentsize'}},
					{ 'custompapersize83',			{value = 'ns0000:CustomSize83', wording = 'custompapersize83', item = 'documentsize'}},
					{ 'custompapersize84',			{value = 'ns0000:CustomSize84', wording = 'custompapersize84', item = 'documentsize'}},
					{ 'custompapersize85',			{value = 'ns0000:CustomSize85', wording = 'custompapersize85', item = 'documentsize'}},
					{ 'custompapersize86',			{value = 'ns0000:CustomSize86', wording = 'custompapersize86', item = 'documentsize'}},
					{ 'custompapersize87',			{value = 'ns0000:CustomSize87', wording = 'custompapersize87', item = 'documentsize'}},
					{ 'custompapersize88',			{value = 'ns0000:CustomSize88', wording = 'custompapersize88', item = 'documentsize'}},
					{ 'custompapersize89',			{value = 'ns0000:CustomSize89', wording = 'custompapersize89', item = 'documentsize'}},
					{ 'custompapersize90',			{value = 'ns0000:CustomSize90', wording = 'custompapersize90', item = 'documentsize'}},
					{ 'custompapersize91',			{value = 'ns0000:CustomSize91', wording = 'custompapersize91', item = 'documentsize'}},
					{ 'custompapersize92',			{value = 'ns0000:CustomSize92', wording = 'custompapersize92', item = 'documentsize'}},
					{ 'custompapersize93',			{value = 'ns0000:CustomSize93', wording = 'custompapersize93', item = 'documentsize'}},
					{ 'custompapersize94',			{value = 'ns0000:CustomSize94', wording = 'custompapersize94', item = 'documentsize'}},
					{ 'custompapersize95',			{value = 'ns0000:CustomSize95', wording = 'custompapersize95', item = 'documentsize'}},
					{ 'custompapersize96',			{value = 'ns0000:CustomSize96', wording = 'custompapersize96', item = 'documentsize'}},
					{ 'custompapersize97',			{value = 'ns0000:CustomSize97', wording = 'custompapersize97', item = 'documentsize'}},
					{ 'custompapersize98',			{value = 'ns0000:CustomSize98', wording = 'custompapersize98', item = 'documentsize'}},
					{ 'custompapersize99',			{value = 'ns0000:CustomSize99', wording = 'custompapersize99', item = 'documentsize'}},
					{ 'custompapersize100',			{value = 'ns0000:CustomSize100', wording = 'custompapersize100', item = 'documentsize'}}
				)

				local result_tbl = {}

				result_tbl[ #result_tbl + 1 ] =
				{
					tag			= 	'psf:Property',
					attribute	=
					{
						key		=	'name',
						value	=	'psf:SelectionType'
					},
					children = {
						tag			= 	'psf:Value',
						attribute	=
						{
							key		=	'xsi:type',
							value	=	'xsd:QName'
						},
						text		=	'psk:PickOne'
					}
				}

				local func_name = get_display_wording( 'documentsize' )
				if(nil ~= func_name and "" ~= func_name) then
					result_tbl[ #result_tbl + 1 ] = 
					{
						tag			= 	'psf:Property',
						attribute	=
						{
							key		=	'name',
							value	=	'psk:DisplayName'
						},
						children = {
							tag			= 	'psf:Value',
							attribute	=
							{
								key		=	'xsi:type',
								value	=	'xsd:string'
							},
							text		=	func_name
						}
					}
				end

				-- PickOne Item
				for i, k, v in OrderedHash_each( element_table ) do
					if( i and k and v ) then
						local option_value = ( 'string' == type( v ) ) and v or v.value
						if( option_value and option and option['paper_info'] ) then
							local isSupported = false
							for index, value in pairs( tbl.documentsize ) do
								if( k == value[ 'name' ] ) then
									isSupported = true
									break
								end
							end

							if( isSupported ) then
								result_tbl[ #result_tbl + 1 ] =
								{
									tag		=	'psf:Option',
									attribute	=
									{
										key		=	'name',
										value	=	option_value
									}
								}

								local current_paper = option['paper_info'][ k ]
								if(	current_paper					and
									current_paper['mm']				and
									current_paper['mm']['width']	and
									current_paper['mm']['height']	) then

									local paper_width = tostring(current_paper['mm']['width'] * 100)
									local paper_height = tostring(current_paper['mm']['height'] * 100)

									if(	paper_width			and
										'' ~= paper_width	and
										paper_height		and
										'' ~= paper_height	) then

										if( 'custompapersize' ~= k ) then
											result_tbl[ #result_tbl ]['children'] =
											{
												{
													tag			= 	'psf:ScoredProperty',
													attribute	=
													{
														key		=	'name',
														value	=	'psk:MediaSizeWidth'
													},
													children =
													{
														tag			= 	'psf:Value',
														attribute	=
														{
															key		=	'xsi:type',
															value	=	'xsd:integer'
														},
														text = paper_width
													}
												},
												{
													tag			= 	'psf:ScoredProperty',
													attribute	=
													{
														key		=	'name',
														value	=	'psk:MediaSizeHeight'
													},
													children =
													{
														tag			= 	'psf:Value',
														attribute	=
														{
															key		=	'xsi:type',
															value	=	'xsd:integer'
														},
														text = paper_height
													}
												}
											}
										else
											result_tbl[ #result_tbl ]['children'] =
											{
												{
													tag			= 	'psf:ScoredProperty',
													attribute	=
													{
														key		=	'name',
														value	=	'psk:MediaSizeWidth'
													},
													children =
													{
														tag			= 	'psf:ParameterRef',
														attribute	=
														{
															key		=	'name',
															value	=	'psk:PageMediaSizeMediaSizeWidth'
														}
													}
												},
												{
													tag			= 	'psf:ScoredProperty',
													attribute	=
													{
														key		=	'name',
														value	=	'psk:MediaSizeHeight'
													},
													children =
													{
														tag			= 	'psf:ParameterRef',
														attribute	=
														{
															key		=	'name',
															value	=	'psk:PageMediaSizeMediaSizeHeight'
														}
													}
												}
											}
										end
									end
								end

								-- DisplayName 
								local item = v.item and v.item or v.wording
								local element = v.item and v.wording or nil

								local item_name = get_display_wording(item, element)
								item_name = item_name and item_name or ''
								
								if( item_name and '' ~= item_name) then
									if( not result_tbl[ #result_tbl ][ 'children' ] )then
										result_tbl[ #result_tbl ][ 'children' ] = {}
									end
									
									result_tbl[ #result_tbl ][ 'children' ][ #(result_tbl[ #result_tbl ][ 'children' ]) + 1 ] =
									{
										tag			= 	'psf:Property',
										attribute	=
										{
											key		=	'name',
											value	=	'psk:DisplayName'
										},
										children = {
											tag			= 	'psf:Value',
											attribute	=
											{
												key		=	'xsi:type',
												value	=	'xsd:string'
											},
											text		=	item_name
										}
									}
								end
							end
						end
					end
				end

				return result_tbl
			end
		},
		{
			tag			= 	'psf:Property',
			attribute	=
			{
				key		=	'name',
				value	=	'psk:PageImageableSize'
			},
			children	=	function(tbl, option)
				local paper_width		= '0'
				local paper_height		= '0'
				local origin_width		= '0'
				local origin_height		= '0'
				local area_width		= '0'
				local area_height		= '0'
				
				if(	tbl['documentsize']												and
					tbl['documentsize']['current']									and
					tbl['documentsize']['current']['name']							and
					option															and
					option['paper_info']											) then

					local current_paper = option['paper_info'][tbl['documentsize']['current']['name']]
					
					if(	current_paper						and
						current_paper['mm']					and
						current_paper['mm']['width']		and
						current_paper['mm']['height']		and
						current_paper['mm']['page_width']	and
						current_paper['mm']['page_height']	and
						current_paper['mm']['offset_x']		and
						current_paper['mm']['offset_y']		) then
						
						if('portrait' == tbl['orientation']['current']['name'])then
							paper_width = tostring(current_paper['mm']['width'] * 100)
							paper_height = tostring(current_paper['mm']['height'] * 100)
							area_width = tostring(current_paper['mm']['page_width'] * 100)
							area_height = tostring(current_paper['mm']['page_height'] * 100)
							origin_width = tostring(current_paper['mm']['offset_x'] * 100)
							origin_height = tostring(current_paper['mm']['offset_y'] * 100)
						else
							paper_width = tostring(current_paper['mm']['height'] * 100)
							paper_height = tostring(current_paper['mm']['width'] * 100)
							area_width = tostring(current_paper['mm']['page_height'] * 100)
							area_height = tostring(current_paper['mm']['page_width'] * 100)
							origin_width = tostring(current_paper['mm']['offset_y'] * 100)
							origin_height = tostring(current_paper['mm']['offset_x'] * 100)
						end
					end
				end

				return {
					{
						tag			= 	'psf:Property',
						attribute	=
						{
							key		=	'name',
							value	=	'psk:ImageableSizeWidth'
						},
						children	=
						{
							tag			=	'psf:Value',
							attribute	=
								{
									key		=	'xsi:type',
									value	=	'xsd:integer'
								},
							text		=	paper_width
						}
					},
					{
						tag			= 	'psf:Property',
						attribute	=
						{
							key		=	'name',
							value	=	'psk:ImageableSizeHeight'
						},
						children	=
						{
							tag			=	'psf:Value',
							attribute	=
								{
									key		=	'xsi:type',
									value	=	'xsd:integer'
								},
							text		=	paper_height
						}
					},
					{
						tag			= 	'psf:Property',
						attribute	=
						{
							key		=	'name',
							value	=	'psk:ImageableArea'
						},
						children	=
						{
							{
								tag			= 	'psf:Property',
								attribute	=
								{
									key		=	'name',
									value	=	'psk:OriginWidth'
								},
								children	=
								{
									tag			=	'psf:Value',
									attribute	=
										{
											key		=	'xsi:type',
											value	=	'xsd:integer'
										},
									text		=	origin_width
								}
							},
							{
								tag			= 	'psf:Property',
								attribute	=
								{
									key		=	'name',
									value	=	'psk:OriginHeight'
								},
								children	=
								{
									tag			=	'psf:Value',
									attribute	=
										{
											key		=	'xsi:type',
											value	=	'xsd:integer'
										},
									text		=	origin_height
								}
							},
							{
								tag			= 	'psf:Property',
								attribute	=
								{
									key		=	'name',
									value	=	'psk:ExtentWidth'
								},
								children	=
								{
									tag			=	'psf:Value',
									attribute	=
										{
											key		=	'xsi:type',
											value	=	'xsd:integer'
										},
									text		=	area_width
								}
							},
							{
								tag			= 	'psf:Property',
								attribute	=
								{
									key		=	'name',
									value	=	'psk:ExtentHeight'
								},
								children	=
								{
									tag			=	'psf:Value',
									attribute	=
										{
											key		=	'xsi:type',
											value	=	'xsd:integer'
										},
									text		=	area_height
								}
							},
						}
					},
				}
			end
		},
	},
	custompapersize_width_mm	=
	{
		tag		=	'psf:ParameterDef',
		attribute	=
			{
				key		=	'name',
				value	=	'psk:PageMediaSizeMediaSizeWidth'
			},
		children	=	function(tbl)
							--default
							local default_val = tonumber(tbl['custompapersize_width_mm']['default'])
							if(nil ~= default_val)then
								tbl['custompapersize_width_mm']['default'] = tostring(default_val * 100)
							else
								tbl['custompapersize_width_mm']['default'] = '0'
							end
							
							--max
							local max_val = tonumber(tbl['custompapersize_width_mm']['max'])
							if(nil ~= max_val)then
								tbl['custompapersize_width_mm']['max'] = tostring(max_val * 100)
							else
								tbl['custompapersize_width_mm']['max'] = '0'
							end
							
							--min
							local min_val = tonumber(tbl['custompapersize_width_mm']['min'])
							if(nil ~= min_val)then
								tbl['custompapersize_width_mm']['min'] = tostring(min_val * 100)
							else
								tbl['custompapersize_width_mm']['min'] = '0'
							end
							return paramdef_generator('integer', tbl['custompapersize_width_mm'], 'microns', 1)
						end
	},
	custompapersize_length_mm	=
	{
		tag		=	'psf:ParameterDef',
		attribute	=
			{
				key		=	'name',
				value	=	'psk:PageMediaSizeMediaSizeHeight'
			},
		children	=	function(tbl)
							
							--default
							local default_val = tonumber(tbl['custompapersize_length_mm']['default'])
							if(nil ~= default_val)then
								tbl['custompapersize_length_mm']['default'] = tostring(default_val * 100)
							else
								tbl['custompapersize_length_mm']['default'] = '0'
							end
							
							--max
							local max_val = tonumber(tbl['custompapersize_length_mm']['max'])
							if(nil ~= max_val)then
								tbl['custompapersize_length_mm']['max'] = tostring(max_val * 100)
							else
								tbl['custompapersize_length_mm']['max'] = '0'
							end
							
							--min
							local min_val = tonumber(tbl['custompapersize_length_mm']['min'])
							if(nil ~= min_val)then
								tbl['custompapersize_length_mm']['min'] = tostring(min_val * 100)
							else
								tbl['custompapersize_length_mm']['min'] = '0'
							end
							
							return paramdef_generator('integer', tbl['custompapersize_length_mm'], 'microns', 1)
						end
	},
	-- Disable Custom Paper Size
	disablecustompapersize 	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:JobAroundDefiniteSize'
		},
		children	=	function(tbl)
			return standard_option_generator(
				tbl['disablecustompapersize'],
				OrderedHash(
					{ 'off',	'ns0000:Off'},
					{ 'on',		'ns0000:On'}
				) )
		end
	},
	-- 02 Print Size
	printsize	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:PagePrintPaperSize'
		},
		children	=	function(tbl, option)
			local element_table	= OrderedHash(
				{ 'sameasoriginalsize',		'ns0000:SameAsPageMediaSize'},
				{ '12x18',					'ns0000:_12X18'},
				{ '11x17',					'ns0000:NorthAmerica11x17'},
				{ 'legal',					'ns0000:NorthAmericaLegal'},
				{ 'letter',					'ns0000:NorthAmericaLetter'},
				{ 'executive',				'ns0000:NorthAmericaExecutive'},
				{ '5.5x8.5',				'ns0000:NorthAmericaStatement'},
				{ 'a0',						'ns0000:ISOA0'},
				{ 'a1',						'ns0000:ISOA1'},
				{ 'a2',						'ns0000:ISOA2'},
				{ 'a3',						'ns0000:ISOA3'},
				{ 'a4',						'ns0000:ISOA4'},
				{ 'a5',						'ns0000:ISOA5'},
				{ 'a6',						'ns0000:ISOA6'},
				{ 'b1',						'ns0000:JISB1'},
				{ 'b2',						'ns0000:JISB2'},
				{ 'b3',						'ns0000:JISB3'},
				{ 'b4jis',					'ns0000:JISB4'},
				{ 'b5jis',					'ns0000:JISB5'},
				{ 'b6',						'ns0000:JISB6'},
				{ 'b7',						'ns0000:JISB7'},
				{ '8x13',					'ns0000:_8X13'},
				{ '8.25x13',				'ns0000:_8_25X13'},
				{ '8.5x13',					'ns0000:FOLIO'},
				{ '8.5x13.4',				'ns0000:_8_5X13_4'},
				{ 'com10env',				'ns0000:NorthAmericaNumber10Envelope'},
				{ 'monarchenv',				'ns0000:NorthAmericaMonarchEnvelope'},
				{ 'dlenv',					'ns0000:ISODLEnvelope'},
				{ 'c6env',					'ns0000:ISOC6Envelope'},
				{ 'c5env',					'ns0000:ISOC5Envelope'},
				{ 'chou3jenv',				'ns0000:JapanChou3Envelope'},
				{ 'chou4jenv',				'ns0000:JapanChou4Envelope'},
				{ 'you4jenv',				'ns0000:JapanYou4Envelope'},
				{ 'youchou3jenv',			'ns0000:JapanYou4EnvelopeRotated'},
				{ 'kaku1jenv',				'ns0000:JapanKaku1Envelope'},
				{ 'kaku2jenv',				'ns0000:JapanKaku2Envelope'},
				{ 'bd',						'ns0000:BD'},
				{ 'lineslider1',			'ns0000:LineSlider1'},
				{ 'lineslider2',			'ns0000:LineSlider2'},
				{ 'glt',					'ns0000:_8X10_5'},
				{ '10x14',					'ns0000:NorthAmerica10x14'},
				{ '10x15',					'ns0000:_10X15'},
				{ '11x15',					'ns0000:_11X15'},
				{ '15x20',					'ns0000:_15X20'},
				{ '8k',						'ns0000:_8kai'},
				{ '16k',					'ns0000:_16kai'},
				{ 'postcard',				'ns0000:JapanHagakiPostcard'},
				{ 'doublepostcard',			'ns0000:JapanDoubleHagakiPostcardRotated'},
				{ 'doublepostcardrotated',	'ns0000:JapanDoubleHagakiPostcard'},
				{ 'governmentlg',			'ns0000:_8_25X14'},
				{ 'engquatro',				'ns0000:NorthAmerica8x10'},
				{ 'sra3',					'ns0000:_320X450'},
				{ 'sra4',					'ns0000:_225X320'},
				{ '11x14',					'ns0000:_11X14'},
				{ '13x19.2',				'ns0000:_13X19_2'},
				{ '13x19',					'ns0000:_13X19'},
				{ '12.6x19.2',				'ns0000:_12_6X19_2'},
				{ '12.6x18.5',				'ns0000:_12_6X18_5'},
				{ '13x18',					'ns0000:_13X18'},
				{ '226x310mm',				'ns0000:_226X310'},
				{ '310x432mm',				'ns0000:_310X432'},
				{ '8.5x12',					'ns0000:NorthAmericaGermanStandardFanfold'},
				{ '17x22',					'ns0000:Ansic'},
				{ '9x12',					'ns0000:_9X12'},
				{ '18x24',					'ns0000:_18X24'},
				{ '21x30',					'ns0000:_21X30'},
				{ '22x34',					'ns0000:_22X34'},
				{ '24x36',					'ns0000:_24X36'},
				{ '30x42',					'ns0000:_30X42'},
				{ '34x44',					'ns0000:_34X44'},
				{ '36x48',					'ns0000:_36X48'},
				{ 'postcardstandard',		'ns0000:PostcardStandard'},
				{ 'kiku4',					'ns0000:Kiku4'},
				{ '329x483mm',				'ns0000:_329X483'},
				{ 'photol',					'ns0000:PhotoL'},
				{ '8.5x13.5',				'ns0000:_8_5X13_5'},
				{ 'kaku3jenv',				'ns0000:JapanKaku3Envelope'},
				{ 'synchrocut_a0width',		'ns0000:CustomSizeWidthA0'},
				{ 'synchrocut_a1width',		'ns0000:CustomSizeWidthA1'},
				{ 'synchrocut_a2width',		'ns0000:CustomSizeWidthA2'},
				{ 'synchrocut_a3width',		'ns0000:CustomSizeWidthA3'},
				{ 'synchrocut_a4width',		'ns0000:CustomSizeWidthA4'},
				{ 'synchrocut_customwidth',	'ns0000:CustomSizeSynchroCut'},
				{ 'custompapersize',		'ns0000:CustomMediaSize'},
				{ 'custompapersize1',		'ns0000:CustomSize1'},
				{ 'custompapersize2',		'ns0000:CustomSize2'},
				{ 'custompapersize3',		'ns0000:CustomSize3'},
				{ 'custompapersize4',		'ns0000:CustomSize4'},
				{ 'custompapersize5',		'ns0000:CustomSize5'},
				{ 'custompapersize6',		'ns0000:CustomSize6'},
				{ 'custompapersize7',		'ns0000:CustomSize7'},
				{ 'custompapersize8',		'ns0000:CustomSize8'},
				{ 'custompapersize9',		'ns0000:CustomSize9'},
				{ 'custompapersize10',		'ns0000:CustomSize10'},
				{ 'custompapersize11',		'ns0000:CustomSize11'},
				{ 'custompapersize12',		'ns0000:CustomSize12'},
				{ 'custompapersize13',		'ns0000:CustomSize13'},
				{ 'custompapersize14',		'ns0000:CustomSize14'},
				{ 'custompapersize15',		'ns0000:CustomSize15'},
				{ 'custompapersize16',		'ns0000:CustomSize16'},
				{ 'custompapersize17',		'ns0000:CustomSize17'},
				{ 'custompapersize18',		'ns0000:CustomSize18'},
				{ 'custompapersize19',		'ns0000:CustomSize19'},
				{ 'custompapersize20',		'ns0000:CustomSize20'},
				{ 'custompapersize21',		'ns0000:CustomSize21'},
				{ 'custompapersize22',		'ns0000:CustomSize22'},
				{ 'custompapersize23',		'ns0000:CustomSize23'},
				{ 'custompapersize24',		'ns0000:CustomSize24'},
				{ 'custompapersize25',		'ns0000:CustomSize25'},
				{ 'custompapersize26',		'ns0000:CustomSize26'},
				{ 'custompapersize27',		'ns0000:CustomSize27'},
				{ 'custompapersize28',		'ns0000:CustomSize28'},
				{ 'custompapersize29',		'ns0000:CustomSize29'},
				{ 'custompapersize30',		'ns0000:CustomSize30'},
				{ 'custompapersize31',		'ns0000:CustomSize31'},
				{ 'custompapersize32',		'ns0000:CustomSize32'},
				{ 'custompapersize33',		'ns0000:CustomSize33'},
				{ 'custompapersize34',		'ns0000:CustomSize34'},
				{ 'custompapersize35',		'ns0000:CustomSize35'},
				{ 'custompapersize36',		'ns0000:CustomSize36'},
				{ 'custompapersize37',		'ns0000:CustomSize37'},
				{ 'custompapersize38',		'ns0000:CustomSize38'},
				{ 'custompapersize39',		'ns0000:CustomSize39'},
				{ 'custompapersize40',		'ns0000:CustomSize40'},
				{ 'custompapersize41',		'ns0000:CustomSize41'},
				{ 'custompapersize42',		'ns0000:CustomSize42'},
				{ 'custompapersize43',		'ns0000:CustomSize43'},
				{ 'custompapersize44',		'ns0000:CustomSize44'},
				{ 'custompapersize45',		'ns0000:CustomSize45'},
				{ 'custompapersize46',		'ns0000:CustomSize46'},
				{ 'custompapersize47',		'ns0000:CustomSize47'},
				{ 'custompapersize48',		'ns0000:CustomSize48'},
				{ 'custompapersize49',		'ns0000:CustomSize49'},
				{ 'custompapersize50',		'ns0000:CustomSize50'},
				{ 'custompapersize51',		'ns0000:CustomSize51'},
				{ 'custompapersize52',		'ns0000:CustomSize52'},
				{ 'custompapersize53',		'ns0000:CustomSize53'},
				{ 'custompapersize54',		'ns0000:CustomSize54'},
				{ 'custompapersize55',		'ns0000:CustomSize55'},
				{ 'custompapersize56',		'ns0000:CustomSize56'},
				{ 'custompapersize57',		'ns0000:CustomSize57'},
				{ 'custompapersize58',		'ns0000:CustomSize58'},
				{ 'custompapersize59',		'ns0000:CustomSize59'},
				{ 'custompapersize60',		'ns0000:CustomSize60'},
				{ 'custompapersize61',		'ns0000:CustomSize61'},
				{ 'custompapersize62',		'ns0000:CustomSize62'},
				{ 'custompapersize63',		'ns0000:CustomSize63'},
				{ 'custompapersize64',		'ns0000:CustomSize64'},
				{ 'custompapersize65',		'ns0000:CustomSize65'},
				{ 'custompapersize66',		'ns0000:CustomSize66'},
				{ 'custompapersize67',		'ns0000:CustomSize67'},
				{ 'custompapersize68',		'ns0000:CustomSize68'},
				{ 'custompapersize69',		'ns0000:CustomSize69'},
				{ 'custompapersize70',		'ns0000:CustomSize70'},
				{ 'custompapersize71',		'ns0000:CustomSize71'},
				{ 'custompapersize72',		'ns0000:CustomSize72'},
				{ 'custompapersize73',		'ns0000:CustomSize73'},
				{ 'custompapersize74',		'ns0000:CustomSize74'},
				{ 'custompapersize75',		'ns0000:CustomSize75'},
				{ 'custompapersize76',		'ns0000:CustomSize76'},
				{ 'custompapersize77',		'ns0000:CustomSize77'},
				{ 'custompapersize78',		'ns0000:CustomSize78'},
				{ 'custompapersize79',		'ns0000:CustomSize79'},
				{ 'custompapersize80',		'ns0000:CustomSize80'},
				{ 'custompapersize81',		'ns0000:CustomSize81'},
				{ 'custompapersize82',		'ns0000:CustomSize82'},
				{ 'custompapersize83',		'ns0000:CustomSize83'},
				{ 'custompapersize84',		'ns0000:CustomSize84'},
				{ 'custompapersize85',		'ns0000:CustomSize85'},
				{ 'custompapersize86',		'ns0000:CustomSize86'},
				{ 'custompapersize87',		'ns0000:CustomSize87'},
				{ 'custompapersize88',		'ns0000:CustomSize88'},
				{ 'custompapersize89',		'ns0000:CustomSize89'},
				{ 'custompapersize90',		'ns0000:CustomSize90'},
				{ 'custompapersize91',		'ns0000:CustomSize91'},
				{ 'custompapersize92',		'ns0000:CustomSize92'},
				{ 'custompapersize93',		'ns0000:CustomSize93'},
				{ 'custompapersize94',		'ns0000:CustomSize94'},
				{ 'custompapersize95',		'ns0000:CustomSize95'},
				{ 'custompapersize96',		'ns0000:CustomSize96'},
				{ 'custompapersize97',		'ns0000:CustomSize97'},
				{ 'custompapersize98',		'ns0000:CustomSize98'},
				{ 'custompapersize99',		'ns0000:CustomSize99'},
				{ 'custompapersize100',		'ns0000:CustomSize100'}

				
			)

			local result_tbl = {}

			result_tbl[ #result_tbl + 1 ] =
			{
				tag			= 	'psf:Property',
				attribute	=
				{
					key		=	'name',
					value	=	'psf:SelectionType'
				},
				children = {
					tag			= 	'psf:Value',
					attribute	=
					{
						key		=	'xsi:type',
						value	=	'xsd:QName'
					},
					text		=	'psk:PickOne'
				}
			}

			-- PickOne Item
			for i, k, v in OrderedHash_each( element_table ) do
				if( i and k and v ) then
					local option_value = ( 'string' == type( v ) ) and v or v.value
					if( option_value and option and option['paper_info'] ) then
						local isSupported = false
						for index, value in pairs( tbl.printsize ) do
							if( k == value[ 'name' ] ) then
								isSupported = true
								break
							end
						end

						if( isSupported ) then
							result_tbl[ #result_tbl + 1 ] =
							{
								tag		=	'psf:Option',
								attribute	=
								{
									key		=	'name',
									value	=	option_value
								}
							}

							if( 'sameasoriginalsize' ~= k ) then
								local current_paper = option['paper_info'][ k ]
								if(	current_paper					and
									current_paper['mm']				and
									current_paper['mm']['width']	and
									current_paper['mm']['height']	) then

									local paper_width = tostring(current_paper['mm']['width'] * 100)
									local paper_height = tostring(current_paper['mm']['height'] * 100)

									if(	paper_width			and
										'' ~= paper_width	and
										paper_height		and
										'' ~= paper_height	) then
										result_tbl[ #result_tbl ]['children'] =
										{
											{
												tag			= 	'psf:ScoredProperty',
												attribute	=
												{
													key		=	'name',
													value	=	'ns0000:MediaSizeWidth'
												},
												children =
												{
													tag			= 	'psf:Value',
													attribute	=
													{
														key		=	'xsi:type',
														value	=	'xsd:integer'
													},
													text = paper_width
												}
											},
											{
												tag			= 	'psf:ScoredProperty',
												attribute	=
												{
													key		=	'name',
													value	=	'ns0000:MediaSizeHeight'
												},
												children =
												{
													tag			= 	'psf:Value',
													attribute	=
													{
														key		=	'xsi:type',
														value	=	'xsd:integer'
													},
													text = paper_height
												}
											}
										}
									end
								end
							end
						end
					end
				end
			end

			return result_tbl
		end
	},
	-- 05 Reduce/Enlarge
	reduceenlarge	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'psk:PageScaling'
		},
		children	=	function(tbl)
			local element_table	= OrderedHash(
				{ 'off',				{value = 'psk:None', scoredproperty = {}}},
				{ 'fittoprintsize',		{value = 'ns0000:FitToPaperSize', scoredproperty = {}}},
				{ 'zoom',				{value = 'psk:CustomSquare', scoredproperty = {}}},
				{ 'directionalzoom',	{value = 'psk:Custom', scoredproperty = {}}}
			)
			--Set ScoredProperty Info--
			---Zoom--
			if(nil ~= tbl['zoomvalue'])then
				table.insert(element_table['zoom']['scoredproperty'],{name = 'psk:Scale', def = 'psk:PageScalingScale'})
			end
			
			---Directional Zoom--
			if(nil ~= tbl['directionalzoom_verticalvalue'])then
				table.insert(element_table['directionalzoom']['scoredproperty'],{name = 'psk:ScaleWidth', def = 'psk:PageScalingScaleWidth'})
			end
			
			if(nil ~= tbl['directionalzoom_horizontalvalue'])then
				table.insert(element_table['directionalzoom']['scoredproperty'],{name = 'psk:ScaleHeight', def = 'psk:PageScalingScaleHeight'})
			end

			return standard_option_generator(tbl['reduceenlarge'], element_table)
		end
	},
	-- Zoom
	zoomvalue	=
	{
		tag		=	'psf:ParameterDef',
		attribute	=
			{
				key		=	'name',
				value	=	'psk:PageScalingScale'
			},
		children	=	function(tbl)
							
							--default
							local default_val = tonumber(tbl['zoomvalue']['default'])
							if(nil ~= default_val)then
								tbl['zoomvalue']['default'] = tostring(default_val)
							else
								tbl['zoomvalue']['default'] = '0'
							end
							
							--max
							local max_val = tonumber(tbl['zoomvalue']['max'])
							if(nil ~= max_val)then
								tbl['zoomvalue']['max'] = tostring(max_val)
							else
								tbl['zoomvalue']['max'] = '0'
							end
							
							--min
							local min_val = tonumber(tbl['zoomvalue']['min'])
							if(nil ~= min_val)then
								tbl['zoomvalue']['min'] = tostring(min_val)
							else
								tbl['zoomvalue']['min'] = '0'
							end
							
							return paramdef_generator('integer', tbl['zoomvalue'], 'percent', 1)
						end
	},
	-- Directional Zoom
	directionalzoom_horizontalvalue	=
	{
		tag		=	'psf:ParameterDef',
		attribute	=
			{
				key		=	'name',
				value	=	'psk:PageScalingScaleWidth'
			},
		children	=	function(tbl)
							
							--default
							local default_val = tonumber(tbl['directionalzoom_horizontalvalue']['default'])
							if(nil ~= default_val)then
								tbl['directionalzoom_horizontalvalue']['default'] = tostring(default_val)
							else
								tbl['directionalzoom_horizontalvalue']['default'] = '100'
							end
							
							--max
							local max_val = tonumber(tbl['directionalzoom_horizontalvalue']['max'])
							if(nil ~= max_val)then
								tbl['directionalzoom_horizontalvalue']['max'] = tostring(max_val)
							else
								tbl['directionalzoom_horizontalvalue']['max'] = '100'
							end
							
							--min
							local min_val = tonumber(tbl['directionalzoom_horizontalvalue']['min'])
							if(nil ~= min_val)then
								tbl['directionalzoom_horizontalvalue']['min'] = tostring(min_val)
							else
								tbl['directionalzoom_horizontalvalue']['min'] = '100'
							end
							
							return paramdef_generator('integer', tbl['directionalzoom_horizontalvalue'], 'percent', 1)
						end
	},
	directionalzoom_verticalvalue	=
	{
		tag		=	'psf:ParameterDef',
		attribute	=
			{
				key		=	'name',
				value	=	'psk:PageScalingScaleHeight'
			},
		children	=	function(tbl)
							
							--default
							local default_val = tonumber(tbl['directionalzoom_verticalvalue']['default'])
							if(nil ~= default_val)then
								tbl['directionalzoom_verticalvalue']['default'] = tostring(default_val)
							else
								tbl['directionalzoom_verticalvalue']['default'] = '100'
							end
							
							--max
							local max_val = tonumber(tbl['directionalzoom_verticalvalue']['max'])
							if(nil ~= max_val)then
								tbl['directionalzoom_verticalvalue']['max'] = tostring(max_val)
							else
								tbl['directionalzoom_verticalvalue']['max'] = '100'
							end
							
							--min
							local min_val = tonumber(tbl['directionalzoom_verticalvalue']['min'])
							if(nil ~= min_val)then
								tbl['directionalzoom_verticalvalue']['min'] = tostring(min_val)
							else
								tbl['directionalzoom_verticalvalue']['min'] = '100'
							end
							
							return paramdef_generator('integer', tbl['directionalzoom_verticalvalue'], 'percent', 1)
						end
	},
	-- 06 Adjust image position
	adjustimgpos_direction	=
	{
		tag		=	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:JobAdjustImagePosition'
		},
		children	=	function(tbl)
			local scored_tbl =
			{
				{ name = 'ns0000:ShiftMarginX', def = 'ns0000:JobShiftMarginX' },
				{ name = 'ns0000:ShiftMarginY', def = 'ns0000:JobShiftMarginY' }
			}

			return standard_option_generator(
				tbl['adjustimgpos_direction'],
				OrderedHash(
					{ 'downwardright',		{ value = 'ns0000:DownwardRight',	scoredproperty = scored_tbl }},
					{ 'downwardleft',		{ value = 'ns0000:DownwardLeft',	scoredproperty = scored_tbl }},
					{ 'upwardright',		{ value = 'ns0000:UpwardRight',		scoredproperty = scored_tbl }},
					{ 'upwardleft',			{ value = 'ns0000:UpwardLeft',		scoredproperty = scored_tbl }}
				) )
		end
	},
 	adjustimgpos_leftrightmargin_mm	=
	{
		tag		=	'psf:ParameterDef',
		attribute	=
			{
				key		=	'name',
				value	=	'ns0000:JobShiftMarginX'
			},
		children	=	function(tbl)
							
							--default
							local default_val = tonumber(tbl['adjustimgpos_leftrightmargin_mm']['default'])
							if(nil ~= default_val)then
								tbl['adjustimgpos_leftrightmargin_mm']['default'] = tostring(default_val * 100)
							else
								tbl['adjustimgpos_leftrightmargin_mm']['default'] = '0'
							end
							
							--max
							local max_val = tonumber(tbl['adjustimgpos_leftrightmargin_mm']['max'])
							if(nil ~= max_val)then
								tbl['adjustimgpos_leftrightmargin_mm']['max'] = tostring(max_val * 100)
							else
								tbl['adjustimgpos_leftrightmargin_mm']['max'] = '0'
							end
							
							--min
							local min_val = tonumber(tbl['adjustimgpos_leftrightmargin_mm']['min'])
							if(nil ~= min_val)then
								tbl['adjustimgpos_leftrightmargin_mm']['min'] = tostring(min_val * 100)
							else
								tbl['adjustimgpos_leftrightmargin_mm']['min'] = '0'
							end
							
							return paramdef_generator('integer', tbl['adjustimgpos_leftrightmargin_mm'], 'microns', 100)
						end
	},
 	adjustimgpos_topbottommargin_mm	=
	{
		tag		=	'psf:ParameterDef',
		attribute	=
			{
				key		=	'name',
				value	=	'ns0000:JobShiftMarginY'
			},
		children	=	function(tbl)
							
							--default
							local default_val = tonumber(tbl['adjustimgpos_topbottommargin_mm']['default'])
							if(nil ~= default_val)then
								tbl['adjustimgpos_topbottommargin_mm']['default'] = tostring(default_val * 100)
							else
								tbl['adjustimgpos_topbottommargin_mm']['default'] = '0'
							end
							
							--max
							local max_val = tonumber(tbl['adjustimgpos_topbottommargin_mm']['max'])
							if(nil ~= max_val)then
								tbl['adjustimgpos_topbottommargin_mm']['max'] = tostring(max_val * 100)
							else
								tbl['adjustimgpos_topbottommargin_mm']['max'] = '0'
							end
							
							--min
							local min_val = tonumber(tbl['adjustimgpos_topbottommargin_mm']['min'])
							if(nil ~= min_val)then
								tbl['adjustimgpos_topbottommargin_mm']['min'] = tostring(min_val * 100)
							else
								tbl['adjustimgpos_topbottommargin_mm']['min'] = '0'
							end
							
							return paramdef_generator('integer', tbl['adjustimgpos_topbottommargin_mm'], 'microns', 100)
						end
	},
	-- 07 Shift image
 	shiftimage	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:JobShiftImageToEdgeCorner'
		},
		children	=	function(tbl)
			return standard_option_generator(
				tbl['shiftimage'],
				OrderedHash(
					{ 'off',					'ns0000:Off'},
					{ 'topleft_nooverlap',		'ns0000:TopLeftNoOverlap'},
					{ 'topcenter',				'ns0000:TopCenter'},
					{ 'topleft',				'ns0000:TopLeft'},
					{ 'topright',				'ns0000:TopRight'},
					{ 'bottomcenter',			'ns0000:BottomCenter'},
					{ 'bottomleft',				'ns0000:BottomLeft'},
					{ 'bottomright',			'ns0000:BottomRight'},
					{ 'center',					'ns0000:Center'},
					{ 'centerleft',				'ns0000:CenterLeft'},
					{ 'centerright',			'ns0000:CenterRight'},
					{ 'topleft_symmetric',		'ns0000:TopLeftSymmetric'},
					{ 'topright_symmetric',		'ns0000:TopRightSymmetric'},
					{ 'bottomleft_symmetric',	'ns0000:BottomLeftSymmetric'},
					{ 'bottomright_symmetric',	'ns0000:BottomRightSymmetric'},
					{ 'centerright_symmetric',	'ns0000:CenterRightSymmetric'},
					{ 'center_imposition',		'ns0000:CenterImposition'},
					{ 'topleft_samepos',		'ns0000:TopLeftSamePosition'},
					{ 'topright_samepos',		'ns0000:TopRightSamePosition'},
					{ 'bottomleft_samepos',		'ns0000:BottomLeftSamePosition'},
					{ 'bottomright_samepos',	'ns0000:BottomRightSamePosition'},
					{ 'topcenter_samepos',		'ns0000:TopCenterSamePosition'},
					{ 'bottomcenter_samepos',	'ns0000:BottomCenterSamePosition'},
					{ 'centerleft_samepos',		'ns0000:CenterLeftSamePosition'},
					{ 'centerright_samepos',	'ns0000:CenterRightSamePosition'}
				) )
		end
	},
	-- 08 Printable Area
	printablearea 	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:JobPrintableArea'
		},
		children	=	function(tbl)
			return standard_option_generator(
				tbl['printablearea'],
				OrderedHash(
					{ 'standard',			'ns0000:Regular'},
					{ 'maximum',			'ns0000:Maximum'},
					{ 'rpdlcompatible',		'ns0000:RPDL_Compatible'},
					{ 'standard2',			'ns0000:Regular2'},
					{ 'maximum2',			'ns0000:Maximum2'},
					{ 'pcl5compatible',		'ns0000:PCL5_Compatible'},
					{ 'enlargedmargin',		'ns0000:EnlargedMargin'}
				) )
		end
	},
	-- 11 Printer Fit to Print Size
	printer_fittoprintsize	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:JobFitToPrint'
		},
		children	=	function(tbl)
			return standard_option_generator(
				tbl['printer_fittoprintsize'],
				OrderedHash(
					{ 'promptuser',				'ns0000:PromptUser'},
					{ 'nearestsizeandscale',	'ns0000:NearestSizeAndScale'},
					{ 'nearestsizeandcrop',		'ns0000:NearestSizeAndCrop'}
				) )
		end
	},
	-- 12 Edge to Edge Print
	edgetoedgeprint	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:JobEdgeToEdgePrint'
		},
		children	=	function(tbl)
			return standard_option_generator(
				tbl['edgetoedgeprint'],
				OrderedHash(
					{ 'off',	'ns0000:Off'},
					{ 'on',		'ns0000:On'}
				) )
		end
	},
	-- 13 Synchro Cut
	synchrocut	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:PageSynchroCut'
		},
		children	=	function(tbl)
			local result_tbl = {}

			local option_scored_table = {}
			if(nil ~= tbl['synchrocut_customwidth_mm']) then
					table.insert(option_scored_table, {
								tag			= 	'psf:ScoredProperty',
								attribute	=
								{
									key		=	'name',
									value	=	'ns0000:CustomSynchroCutWidth'
								},
								children =
								{
									tag			= 	'psf:ParameterRef',
									attribute	=
										{
											key		=	'name',
											value	=	'ns0000:PageSynchroCutCustomSynchroCutWidth'
										}
								}
							}
						)
			end

			if(nil ~= tbl['synchrocut_margin_top_mm']) then
					table.insert(option_scored_table, {
								tag			= 	'psf:ScoredProperty',
								attribute	=
								{
									key		=	'name',
									value	=	'ns0000:MarginAdjustmentTop'
								},
								children =
								{
									tag			= 	'psf:ParameterRef',
									attribute	=
										{
											key		=	'name',
											value	=	'ns0000:PageSynchroCutMarginAdjustmentTop'
										}
								}
							}
						)
			end

			if(nil ~= tbl['synchrocut_margin_bottom_mm']) then
					table.insert(option_scored_table, {
								tag			= 	'psf:ScoredProperty',
								attribute	=
								{
									key		=	'name',
									value	=	'ns0000:MarginAdjustmentBottom'
								},
								children =
								{
									tag			= 	'psf:ParameterRef',
									attribute	=
										{
											key		=	'name',
											value	=	'ns0000:PageSynchroCutMarginAdjustmentBottom'
										}
								}
							}
						)
			end

			if(nil ~= tbl['synchrocut_margin_left_mm']) then
					table.insert(option_scored_table, {
								tag			= 	'psf:ScoredProperty',
								attribute	=
								{
									key		=	'name',
									value	=	'ns0000:MarginAdjustmentLeft'
								},
								children =
								{
									tag			= 	'psf:ParameterRef',
									attribute	=
										{
											key		=	'name',
											value	=	'ns0000:PageSynchroCutMarginAdjustmentLeft'
										}
								}
							}
						)
			end

			if(nil ~= tbl['synchrocut_margin_right_mm']) then
					table.insert(option_scored_table, {
								tag			= 	'psf:ScoredProperty',
								attribute	=
								{
									key		=	'name',
									value	=	'ns0000:MarginAdjustmentRight'
								},
								children =
								{
									tag			= 	'psf:ParameterRef',
									attribute	=
										{
											key		=	'name',
											value	=	'ns0000:PageSynchroCutMarginAdjustmentRight'
										}
								}
							}
						)
			end

			table.insert(result_tbl, {
						tag			=	'psf:Option',
						children	=	option_scored_table
						--children	=	nil
					}
				)

			-- Set Max Cut Size
			if(nil ~= tbl['synchrocut_setmaxcutsize']) then
				local setcutsize_element_table	= OrderedHash(
					{ 'off',				{value = 'ns0000:Off', scoredproperty = {} }},
					{ 'on',					{value = 'ns0000:On', scoredproperty = {} }}
				)
				if(nil ~= tbl['synchrocut_maxcutsize_mm'] ) then
					table.insert(setcutsize_element_table['on']['scoredproperty'],
						{ name = 'ns0000:MaximumCutSize', def = 'ns0000:PageSynchroCutMaximumCutSize' }
						)
				end

				local setcutsize_table = standard_option_generator(tbl['synchrocut_setmaxcutsize'], setcutsize_element_table)
				table.insert(result_tbl,
					{
						tag			= 	'psf:Feature',
						attribute	=
						{
							key		=	'name',
							value	=	'ns0000:SetMaximumCutSize'
						},
						children	=	setcutsize_table
					}
				)
			end

			-- Print Position
			if(nil ~= tbl['synchrocut_position']) then
				local position_element_table	= OrderedHash(
					{ 'left',				{value = 'ns0000:Left', scoredproperty = {}}},
					{ 'center',				{value = 'ns0000:Center', scoredproperty = {}}},
					{ 'right',				{value = 'ns0000:Right', scoredproperty = {}}}
				)

				local position_table = standard_option_generator(tbl['synchrocut_position'], position_element_table)
				table.insert(result_tbl,
					{
						tag			= 	'psf:Feature',
						attribute	=
						{
							key		=	'name',
							value	=	'ns0000:PrintPosition'
						},
						children	=	position_table
					}
				)
			end

			-- Priority Image Direction
			if(nil ~= tbl['synchrocut_feeddirection']) then
				local direction_element_table	= OrderedHash(
					{ 'lef',				{value = 'ns0000:Lef', scoredproperty = {}}},
					{ 'sef',				{value = 'ns0000:Sef', scoredproperty = {}}}
				)

				local direction_table = standard_option_generator(tbl['synchrocut_feeddirection'], direction_element_table)
				table.insert(result_tbl,
					{
						tag			= 	'psf:Feature',
						attribute	=
						{
							key		=	'name',
							value	=	'ns0000:PriorityImageDirection'
						},
						children	=	direction_table
					}
				)
			end

			return result_tbl
		end
	},
	synchrocut_customwidth_mm = 
	{
		tag		=	'psf:ParameterDef',
		attribute	=
			{
				key		=	'name',
				value	=	'ns0000:PageSynchroCutCustomSynchroCutWidth'
			},
		children	=	function(tbl)
							--default
							local default_val = tonumber(tbl['synchrocut_customwidth_mm']['default'])
							if(nil ~= default_val)then
								tbl['synchrocut_customwidth_mm']['default'] = tostring(default_val * 100)
							else
								tbl['synchrocut_customwidth_mm']['default'] = '0'
							end
							
							--max
							local max_val = tonumber(tbl['synchrocut_customwidth_mm']['max'])
							if(nil ~= max_val)then
								tbl['synchrocut_customwidth_mm']['max'] = tostring(max_val * 100)
							else
								tbl['synchrocut_customwidth_mm']['max'] = '0'
							end
							
							--min
							local min_val = tonumber(tbl['synchrocut_customwidth_mm']['min'])
							if(nil ~= min_val)then
								tbl['synchrocut_customwidth_mm']['min'] = tostring(min_val * 100)
							else
								tbl['synchrocut_customwidth_mm']['min'] = '0'
							end
							
							return paramdef_generator('integer', tbl['synchrocut_customwidth_mm'], 'microns', 100)
						end
	},
	synchrocut_margin_top_mm = 
	{
		tag		=	'psf:ParameterDef',
		attribute	=
			{
				key		=	'name',
				value	=	'ns0000:PageSynchroCutMarginAdjustmentTop'
			},
		children	=	function(tbl)
							--default
							local default_val = tonumber(tbl['synchrocut_margin_top_mm']['default'])
							if(nil ~= default_val)then
								tbl['synchrocut_margin_top_mm']['default'] = tostring(default_val * 1000)
							else
								tbl['synchrocut_margin_top_mm']['default'] = '0'
							end
							
							--max
							local max_val = tonumber(tbl['synchrocut_margin_top_mm']['max'])
							if(nil ~= max_val)then
								tbl['synchrocut_margin_top_mm']['max'] = tostring(max_val * 1000)
							else
								tbl['synchrocut_margin_top_mm']['max'] = '0'
							end
							
							--min
							local min_val = tonumber(tbl['synchrocut_margin_top_mm']['min'])
							if(nil ~= min_val)then
								tbl['synchrocut_margin_top_mm']['min'] = tostring(min_val * 1000)
							else
								tbl['synchrocut_margin_top_mm']['min'] = '0'
							end
							
							return paramdef_generator('integer', tbl['synchrocut_margin_top_mm'], 'microns', 100)
						end
	},
	synchrocut_margin_bottom_mm = 
	{
		tag		=	'psf:ParameterDef',
		attribute	=
			{
				key		=	'name',
				value	=	'ns0000:PageSynchroCutMarginAdjustmentBottom'
			},
		children	=	function(tbl)
							--default
							local default_val = tonumber(tbl['synchrocut_margin_bottom_mm']['default'])
							if(nil ~= default_val)then
								tbl['synchrocut_margin_bottom_mm']['default'] = tostring(default_val * 1000)
							else
								tbl['synchrocut_margin_bottom_mm']['default'] = '0'
							end
							
							--max
							local max_val = tonumber(tbl['synchrocut_margin_bottom_mm']['max'])
							if(nil ~= max_val)then
								tbl['synchrocut_margin_bottom_mm']['max'] = tostring(max_val * 1000)
							else
								tbl['synchrocut_margin_bottom_mm']['max'] = '0'
							end
							
							--min
							local min_val = tonumber(tbl['synchrocut_margin_bottom_mm']['min'])
							if(nil ~= min_val)then
								tbl['synchrocut_margin_bottom_mm']['min'] = tostring(min_val * 1000)
							else
								tbl['synchrocut_margin_bottom_mm']['min'] = '0'
							end
							
							return paramdef_generator('integer', tbl['synchrocut_margin_bottom_mm'], 'microns', 100)
						end
	},
	synchrocut_margin_left_mm = 
	{
		tag		=	'psf:ParameterDef',
		attribute	=
			{
				key		=	'name',
				value	=	'ns0000:PageSynchroCutMarginAdjustmentLeft'
			},
		children	=	function(tbl)
							--default
							local default_val = tonumber(tbl['synchrocut_margin_left_mm']['default'])
							if(nil ~= default_val)then
								tbl['synchrocut_margin_left_mm']['default'] = tostring(default_val * 1000)
							else
								tbl['synchrocut_margin_left_mm']['default'] = '0'
							end
							
							--max
							local max_val = tonumber(tbl['synchrocut_margin_left_mm']['max'])
							if(nil ~= max_val)then
								tbl['synchrocut_margin_left_mm']['max'] = tostring(max_val * 1000)
							else
								tbl['synchrocut_margin_left_mm']['max'] = '0'
							end
							
							--min
							local min_val = tonumber(tbl['synchrocut_margin_left_mm']['min'])
							if(nil ~= min_val)then
								tbl['synchrocut_margin_left_mm']['min'] = tostring(min_val * 1000)
							else
								tbl['synchrocut_margin_left_mm']['min'] = '0'
							end
							
							return paramdef_generator('integer', tbl['synchrocut_margin_left_mm'], 'microns', 100)
						end
	},
	synchrocut_margin_right_mm = 
	{
		tag		=	'psf:ParameterDef',
		attribute	=
			{
				key		=	'name',
				value	=	'ns0000:PageSynchroCutMarginAdjustmentRight'
			},
		children	=	function(tbl)
							--default
							local default_val = tonumber(tbl['synchrocut_margin_right_mm']['default'])
							if(nil ~= default_val)then
								tbl['synchrocut_margin_right_mm']['default'] = tostring(default_val * 1000)
							else
								tbl['synchrocut_margin_right_mm']['default'] = '0'
							end
							
							--max
							local max_val = tonumber(tbl['synchrocut_margin_right_mm']['max'])
							if(nil ~= max_val)then
								tbl['synchrocut_margin_right_mm']['max'] = tostring(max_val * 1000)
							else
								tbl['synchrocut_margin_right_mm']['max'] = '0'
							end
							
							--min
							local min_val = tonumber(tbl['synchrocut_margin_right_mm']['min'])
							if(nil ~= min_val)then
								tbl['synchrocut_margin_right_mm']['min'] = tostring(min_val * 1000)
							else
								tbl['synchrocut_margin_right_mm']['min'] = '0'
							end
							
							return paramdef_generator('integer', tbl['synchrocut_margin_right_mm'], 'microns', 100)
						end
	},
	synchrocut_maxcutsize_mm = 
	{
		tag		=	'psf:ParameterDef',
		attribute	=
			{
				key		=	'name',
				value	=	'ns0000:PageSynchroCutMaximumCutSize'
			},
		children	=	function(tbl)
							--default
							local default_val = tonumber(tbl['synchrocut_maxcutsize_mm']['default'])
							if(nil ~= default_val)then
								tbl['synchrocut_maxcutsize_mm']['default'] = tostring(default_val * 100)
							else
								tbl['synchrocut_maxcutsize_mm']['default'] = '0'
							end
							
							--max
							local max_val = tonumber(tbl['synchrocut_maxcutsize_mm']['max'])
							if(nil ~= max_val)then
								tbl['synchrocut_maxcutsize_mm']['max'] = tostring(max_val * 100)
							else
								tbl['synchrocut_maxcutsize_mm']['max'] = '0'
							end
							
							--min
							local min_val = tonumber(tbl['synchrocut_maxcutsize_mm']['min'])
							if(nil ~= min_val)then
								tbl['synchrocut_maxcutsize_mm']['min'] = tostring(min_val * 100)
							else
								tbl['synchrocut_maxcutsize_mm']['min'] = '0'
							end
							
							return paramdef_generator('integer', tbl['synchrocut_maxcutsize_mm'], 'microns', 100)
						end
	},

	-- 14 Create Margin
	createmargin	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:JobCreateMargin'
		},
		children	=	function(tbl)
			return standard_option_generator(
				tbl['createmargin'],
				OrderedHash(
					{ 'off',	'ns0000:Off'},
					{ 'on',		'ns0000:On'}
				) )
		end
	},

-- [05 Page Layout]
	-- 01 Orientation
	orientation	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'psk:PageOrientation'
		},
		children	=	function(tbl)
			local element_table	= OrderedHash(
				{ 'portrait',	'psk:Portrait'},
				{ 'landscape',	'psk:Landscape'},
				{ 'on',			'psk:ReverseLandscape'}
			)
			local orientation_table = tbl['orientation']

			if(tbl['rotatelandscape']) then 
				for i,v in pairs( tbl['rotatelandscape'] ) do
					if( v and ('on' == v.name) ) then
						table.insert(orientation_table, v)
						break
					end
				end
			end

			return standard_option_generator(orientation_table, element_table)
		end
	},
	rotatemixedoriginalorientation	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:JobMixedDirectionRotation'
		},
		children	=	function(tbl)
			return standard_option_generator(
				tbl['rotatemixedoriginalorientation'],
				OrderedHash(
					{ 'automatic',					'ns0000:Automatic'},
					{ 'printerdefault',				'ns0000:PrinterDefaults'},
					{ 'reverseprinterdefault',		'ns0000:ReversePrinterDefaults'}
				) )
		end
	},
	-- 02 Rotated
	rotated	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:JobImageDirection'
		},
		children	=	function(tbl)
			return standard_option_generator(
				tbl['rotated'],
				OrderedHash(
					{ 'off',	'ns0000:Normal'},
					{ 'on',		'ns0000:Reverse'}
				) )
		end
	},
	-- 03 Mirroring
	mirroring	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'psk:PageMirrorImage'
		},
		children	=	function(tbl)
			return standard_option_generator(
				tbl['mirroring'],
				OrderedHash(
					{ 'off',	'psk:None'},
					{ 'on',		'psk:MirrorImageWidth'}
				) )
		end
	},
	-- 04 Layout
	layout =
	{
		tag			=	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'psk:JobNUpAllDocumentsContiguously'
		},
		children	=	function(tbl)
			local result_tbl = {}

			result_tbl[ #result_tbl + 1 ] =
			{
				tag			= 	'psf:Property',
				attribute	=
				{
					key		=	'name',
					value	=	'psf:SelectionType'
				},
				children = {
					tag			= 	'psf:Value',
					attribute	=
					{
						key		=	'xsi:type',
						value	=	'xsd:QName'
					},
					text		=	'psk:PickOne'
				}
			}

			local element_table	= OrderedHash(
				{ '1pagepersheet',		'1'},
				{ '2pagespersheet',		'2'},
				{ '4pagespersheet',		'4'},
				{ '6pagespersheet',		'6'},
				{ '8pagespersheet',		'8'},
				{ '9pagespersheet',		'9'},
				{ '16pagespersheet',	'16'}
			)

			-- PickOne Item
			for i, k, v in OrderedHash_each( element_table ) do
				if( i and k and v ) then
					local option_value = ( 'string' == type( v ) ) and v or v.value
					if( option_value ) then
						local isSupported = false
						for index, value in pairs( tbl.layout ) do
							if( k == value[ 'name' ] ) then
								isSupported = true
								break
							end
						end

						if( isSupported ) then
							result_tbl[ #result_tbl + 1 ] =
							{
								tag		=	'psf:Option',
								children =
								{
									tag			= 	'psf:ScoredProperty',
									attribute	=
									{
										key		=	'name',
										value	=	'psk:PagesPerSheet'
									},
									children =
									{
										tag			= 	'psf:Value',
										attribute	=
										{
											key		=	'xsi:type',
											value	=	'xsd:integer'
										},
										text		=	option_value
									}
								}
							}
						end
					end
				end
			end

			-- pageorder
			if(nil ~= tbl['pageorder'])then
				local pageorder_element_table	= OrderedHash(
					{ 'none',			'ns0000:None'},
					{ 'rightthendown',	'psk:RightBottom'},
					{ 'downthenright',	'psk:BottomRight'},
					{ 'leftthendown',	'psk:LeftBottom'},
					{ 'downthenleft',	'psk:BottomLeft'},
					{ 'lefttoright',	'ns0000:LeftRightOrTopBottom'},
					{ 'righttoleft',	'ns0000:RightLeftOrTopBottom'}
				)
				table.insert(result_tbl, {
						tag			= 	'psf:Feature',
						attribute	=
						{
							key		=	'name',
							value	=	'psk:PresentationDirection'
						},
						children	=	standard_option_generator(tbl['pageorder'], pageorder_element_table)
					}
				)
			end
			
			-- drawframeborder
			if(nil ~= tbl['drawframeborder'])then
				local frameborder_element_table	= OrderedHash(
					{ 'off',	'ns0000:Off'},
					{ 'on',		'ns0000:On'}
				)
				table.insert(result_tbl, {
						tag			= 	'psf:Feature',
						attribute	=
						{
							key		=	'name',
							value	=	'ns0000:Frame'
						},
						children	=	standard_option_generator(tbl['drawframeborder'], frameborder_element_table)
					}
				)
			end
			
			-- drawborderline
			if(nil ~= tbl['drawborderline'])then
				local borderline_element_table	= OrderedHash(
					{ 'off',	'ns0000:Off'},
					{ 'on',		'ns0000:On'}
				)
				table.insert(result_tbl, {
						tag			= 	'psf:Feature',
						attribute	=
						{
							key		=	'name',
							value	=	'ns0000:Border'
						},
						children	=	standard_option_generator(tbl['drawborderline'], borderline_element_table)
					}
				)
			end
			
			-- layout_rotatemixedoriginalorientation
			if(nil ~= tbl['layout_rotatemixedoriginalorientation'])then
				local mixed_element_table	= OrderedHash(
					{ 'rotate',		'ns0000:RotateNaturally'},
					{ 'reduce',		'ns0000:ReduceWithoutRotation'},
					{ 'pagebreak',	'ns0000:NotMix'}
				)
				table.insert(result_tbl, {
						tag			= 	'psf:Feature',
						attribute	=
						{
							key		=	'name',
							value	=	'ns0000:NUpMixedDirectionLayout'
						},
						children	=	standard_option_generator(tbl['layout_rotatemixedoriginalorientation'], mixed_element_table)
					}
				)
			end
			
			return result_tbl
		end
	},
	-- 05 Booklet
	booklet	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:JobBooklet'
		},
		children	=	function(tbl)
			local element_table	= OrderedHash(
				{ 'off',		{value = 'ns0000:Off', scoredproperty = {}}},
				{ 'booklet',	{value = 'ns0000:Booklet', scoredproperty = {}}},
				{ 'minibook',	{value = 'ns0000:MiniBook', scoredproperty = {}}}
			)
			--Set ScoredProperty Info--
			---Creep inside--
			if(nil ~= tbl['booklet_creepinsidemargin_mm'])then
				table.insert(element_table['booklet']['scoredproperty'],{name = 'ns0000:InsideMargin', def = 'ns0000:JobBookletCreepInsidePage'})
			end

			---Creep outside--
			if(nil ~= tbl['booklet_creepoutsidemargin_mm'])then
				table.insert(element_table['booklet']['scoredproperty'],{name = 'ns0000:OutsideMargin', def = 'ns0000:JobBookletCreepOutsidePage'})
			end
			
			local result_tbl = standard_option_generator(tbl['booklet'], element_table)
			
			--------Booklet Page Order--------
			if(nil ~= tbl['booklet_pageorder'])then
				local order_element_table	= OrderedHash(
					{ 'opentoleftortop',		'ns0000:OpenToLeftOrTop'},
					{ 'opentorightorbottom',	'ns0000:OpenToRightOrBottom'},
					{ 'opentorightortop',		'ns0000:OpenToRightOrTop'}
				)
				local order_table = 
				{
					tag			= 	'psf:Feature',
					attribute	=
					{
						key		=	'name',
						value	=	'ns0000:BookletPageOrder'
					},
					children = standard_option_generator(tbl['booklet_pageorder'], order_element_table)
				}
				table.insert(result_tbl, order_table)
			end
			
			--------Booklet Mixed Direction--------
			if(nil ~= tbl['booklet_rotatemixedoriginalorientation'])then
				local direction_element_table	= OrderedHash(
					{ 'rotate',			'ns0000:RotateNaturally'},
					{ 'reduce',			'ns0000:ReduceWithoutRotation'}
				)
					
				local direction_table = 
				{
					tag			= 	'psf:Feature',
					attribute	=
					{
						key		=	'name',
						value	=	'ns0000:BookletMixedDirectionLayout'
					},
					children	=	standard_option_generator(tbl['booklet_rotatemixedoriginalorientation'], direction_element_table)
				}
				table.insert(result_tbl, direction_table)
			end
			
			--------Booklet EqualMagnification--------
			if(nil ~= tbl['booklet_equalmagnification'])then
				local booklet_equalmagnification_element_table	= OrderedHash(
					{ 'off',		'ns0000:Off'},
					{ 'on',			'ns0000:On'}
				)
					
				local booklet_equalmagnification_table = 
				{
					tag			= 	'psf:Feature',
					attribute	=
					{
						key		=	'name',
						value	=	'ns0000:BookletEqualMagnification'
					},
					children	=	standard_option_generator(tbl['booklet_equalmagnification'], booklet_equalmagnification_element_table)
				}
				table.insert(result_tbl, booklet_equalmagnification_table)
			end
			return result_tbl
		end
	},
	-- Booklet Creep inside
	booklet_creepinsidemargin_mm	=
	{
		tag		=	'psf:ParameterDef',
		attribute	=
			{
				key		=	'name',
				value	=	'ns0000:JobBookletCreepInsidePage'
			},
		children	=	function(tbl)
							
							--default
							local default_val = tonumber(tbl['booklet_creepinsidemargin_mm']['default'])
							if(nil ~= default_val)then
								tbl['booklet_creepinsidemargin_mm']['default'] = tostring(default_val * 100)
							else
								tbl['booklet_creepinsidemargin_mm']['default'] = '0'
							end
							
							--max
							local max_val = tonumber(tbl['booklet_creepinsidemargin_mm']['max'])
							if(nil ~= max_val)then
								tbl['booklet_creepinsidemargin_mm']['max'] = tostring(max_val * 100)
							else
								tbl['booklet_creepinsidemargin_mm']['max'] = '0'
							end
							
							--min
							local min_val = tonumber(tbl['booklet_creepinsidemargin_mm']['min'])
							if(nil ~= min_val)then
								tbl['booklet_creepinsidemargin_mm']['min'] = tostring(min_val * 100)
							else
								tbl['booklet_creepinsidemargin_mm']['min'] = '0'
							end
							
							return paramdef_generator('integer', tbl['booklet_creepinsidemargin_mm'], 'microns', 100)
						end
	},
	-- Booklet Creep outside
	booklet_creepoutsidemargin_mm	=
	{
		tag		=	'psf:ParameterDef',
		attribute	=
			{
				key		=	'name',
				value	=	'ns0000:JobBookletCreepOutsidePage'
			},
		children	=	function(tbl)
							
							--default
							local default_val = tonumber(tbl['booklet_creepoutsidemargin_mm']['default'])
							if(nil ~= default_val)then
								tbl['booklet_creepoutsidemargin_mm']['default'] = tostring(default_val * 100)
							else
								tbl['booklet_creepoutsidemargin_mm']['default'] = '0'
							end
							
							--max
							local max_val = tonumber(tbl['booklet_creepoutsidemargin_mm']['max'])
							if(nil ~= max_val)then
								tbl['booklet_creepoutsidemargin_mm']['max'] = tostring(max_val * 100)
							else
								tbl['booklet_creepoutsidemargin_mm']['max'] = '0'
							end
							
							--min
							local min_val = tonumber(tbl['booklet_creepoutsidemargin_mm']['min'])
							if(nil ~= min_val)then
								tbl['booklet_creepoutsidemargin_mm']['min'] = tostring(min_val * 100)
							else
								tbl['booklet_creepoutsidemargin_mm']['min'] = '0'
							end
							
							return paramdef_generator('integer', tbl['booklet_creepoutsidemargin_mm'], 'microns', 100)
						end
	},
	-- 06 Poster
	poster	=
	{
		tag		=	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'psk:PagePoster'
		},
		children	=	function(tbl)
			local result_tbl = {}
			result_tbl[ #result_tbl + 1 ] = 
			{
				tag			= 	'psf:Property',
				attribute	=
					{
						key		=	'name',
						value	=	'psf:SelectionType'
					},
				children = {
					tag			= 	'psf:Value',
					attribute	=
						{
							key		=	'xsi:type',
							value	=	'xsd:QName'
						},
					text		=	'psk:PickOne'
				}
			}

			local element_table	= OrderedHash(
				{ 'off',				{ scoredproperty = '1'	}},
				{ '1pageinto2sheets',	{ scoredproperty = '2'	}},
				{ '1pageinto4sheets',	{ scoredproperty = '4'	}},
				{ '1pageinto9sheets',	{ scoredproperty = '9'	}}
			)

			-- PickOne Item
			for i, k, v in OrderedHash_each( element_table ) do
				if( i and k and v ) then
					local isSupported = false
					for index, value in pairs( tbl.poster ) do
						if( k == value[ 'name' ] ) then
							isSupported = true
							break
						end
					end

					if( isSupported ) then
						result_tbl[ #result_tbl + 1 ]	=
						{
							tag			=	'psf:Option',
							children	=
							{
								tag			= 	'psf:ScoredProperty',
								attribute	=
								{
									key		=	'name',
									value	=	'psk:SheetsPerPage'
								},
								children	= 
								{
									tag 		=	'psf:Value',
									attribute	=
									{
										key 	=	'xsi:type',
										value	=	'xsd:integer'
									},
									text		=	v.scoredproperty
								}
							}
						}
					end
				end
			end

			return result_tbl
		end
	},
	-- 07 Speed
	speed	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:JobSpeedPrintContiguously'
		},
		children	=	function(tbl)
			return standard_option_generator(
				tbl['speed'],
				OrderedHash(
					{ 'off',						'ns0000:Off'},
					{ 'lefttorightortoptobottom',	'ns0000:LeftToRightOrTopToBottom'},
					{ 'righttoleftortoptobottom',	'ns0000:RightToLeftOrTopToBottom'}
				) )
		end
	},
	-- 08 Double
	double	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:JobDoublePrint'
		},
		children	=	function(tbl)
			return standard_option_generator(
				tbl['double'],
				OrderedHash(
					{ 'off',	'ns0000:Off'},
					{ 'on',		'ns0000:On'}
				) )
		end
	},
	-- 09 GuideBook Fold
	guidebookfold	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:JobGuideBookFold'
		},
		children	=	function(tbl)
			local element_table	= OrderedHash(
				{ 'off',	'ns0000:Off'},
				{ 'on',		'ns0000:On'}
			)

			local result_tbl = standard_option_generator(tbl['guidebookfold'], element_table)

			--------09-01 GuideBook Fold::Draw Border Line--------
			if(nil ~= tbl['guidebookfold_drawborderline'])then
				local border_element_table	= OrderedHash(
					{ 'off',	'ns0000:Off'},
					{ 'on',		'ns0000:On'}
				)
				local border_table = 
				{
					tag			= 	'psf:Feature',
					attribute	=
					{
						key		=	'name',
						value	=	'ns0000:Border'
					},
					children = standard_option_generator(tbl['guidebookfold_drawborderline'], border_element_table)
				}
				table.insert(result_tbl, border_table)
			end
			return result_tbl
		end
	},
	-- 11 Repeat
	['repeat']	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:JobRepeatPrint'
		},
		children	=	function(tbl)
			local element_table	= OrderedHash(
				{ 'off',		'ns0000:Off'},
				{ '2repeat',	'ns0000:Repeat2'},
				{ '4repeat',	'ns0000:Repeat4'},
				{ '9repeat',	'ns0000:Repeat9'},
				{ '16repeat',	'ns0000:Repeat16'}
			)

			local result_tbl = standard_option_generator(tbl['repeat'], element_table)

			--------11-01 Repeat::Draw Border Line for Repeat--------
			if(nil ~= tbl['repeat_drawborderline'])then
				local repeat_borderline_element_table	= OrderedHash(
					{ 'off',	'ns0000:Off'},
					{ 'on',		'ns0000:On'}
				)
				local repeat_borderline_table = 
				{
					tag			= 	'psf:Feature',
					attribute	=
					{
						key		=	'name',
						value	=	'ns0000:RepeatFrameBorder'
					},
					children = standard_option_generator(tbl['repeat_drawborderline'], repeat_borderline_element_table)
				}
				table.insert(result_tbl, repeat_borderline_table)
			end
			return result_tbl
		end
	},
	-- 12 Leaflet Fold
	leafletfold	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:JobLeafletFold'
		},
		children	=	function(tbl)
			local element_table	= OrderedHash(
				{ 'off',				'ns0000:Off'},
				{ '1_4pagespersheet',	'ns0000:1_4PagesPerSheet'},
				{ '4_4pagespersheet',	'ns0000:4_4PagesPerSheet'}
			)

			local result_tbl = standard_option_generator(tbl['leafletfold'], element_table)

			--------12-01 Leaflet Fold::Draw Border Line--------
			if(nil ~= tbl['leafletfold_drawborderline'])then
				local border_element_table	= OrderedHash(
					{ 'off',	'ns0000:Off'},
					{ 'on',		'ns0000:On'}
				)
				local border_table = 
				{
					tag			= 	'psf:Feature',
					attribute	=
					{
						key		=	'name',
						value	=	'ns0000:Border'
					},
					children = standard_option_generator(tbl['leafletfold_drawborderline'], border_element_table)
				}
				table.insert(result_tbl, border_table)
			end
			return result_tbl
		end
	},
	-- 13 Manual Duplex
	manualduplex	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:JobManualDuplex'
		},
		children	=	function(tbl)
			return standard_option_generator(
				tbl['manualduplex'],
				OrderedHash(
					{ 'off',			'ns0000:Off'},
					{ 'evenpageprint',	'ns0000:EvenPagePrint'},
					{ 'oddpageprint',	'ns0000:OddPagePrint'}
				) )
		end
	},
-- [06 Image]
	-- 01 Watermark
	pwmk_print	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:PageWatermark'
		},
		children	=	function(tbl)
			local result_tbl = {}

			result_tbl[ #result_tbl + 1 ] =
			{
				tag			= 	'psf:Property',
				attribute	=
				{
					key		=	'name',
					value	=	'psf:SelectionType'
				},
				children =
				{
					tag			= 	'psf:Value',
					attribute	=
					{
						key		=	'xsi:type',
						value	=	'xsd:QName'
					},
					text		=	'psk:PickOne'
				}
			}

			for i, v in pairs(tbl['pwmk_print']) do
				if('off' == v['name']) then
					result_tbl[ #result_tbl + 1 ] =
					{
						tag		=	'psf:Option',
						attribute	=
						{
							key		=	'name',
							value	=	'ns0000:Off'
						}
					}
				elseif('on' == v['name'] and nil ~= tbl['pwmk_namelist'])then
					for index, value in pairs(tbl['pwmk_namelist']) do
						if( value['name'] )then
							result_tbl[ #result_tbl + 1 ] =
							{
								tag		=	'psf:Option',
								attribute	=
								{
									key		=	'name',
									value	=	'ns0000:On'
								},
								children	=
								{
									tag			= 	'psf:ScoredProperty',
									attribute	=
									{
										key		=	'name',
										value	=	'ns0000:WatermarkItem'
									},
									children =
									{
										tag			= 	'psf:Value',
										attribute	=
										{
											key		=	'xsi:type',
											value	=	'xsd:string'
										},
										text		=	value['name']
									}
								}
							}
						end
					end
				end
			end

			return result_tbl
		end
	},
	-- 02 Header/Footer
	phf_print	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:PageHeaderFooter'
		},
		children	=	function(tbl)
			local result_tbl = {}
			
			local element_table	= OrderedHash(
				{ 'off',	{value = 'ns0000:Off', scoredproperty = {}}},
				{ 'on',		{value = 'ns0000:On', scoredproperty = {}}}
			)
			--Set ScoredProperty Info--
			---Font--
			if(nil ~= tbl['phf_font'])then
				table.insert(element_table['on']['scoredproperty'],{name = 'ns0000:FontName', def = 'ns0000:PageHeaderFooterFontName'})
			end

			---Font Size--
			if(nil ~= tbl['phf_fontsize'])then
				table.insert(element_table['on']['scoredproperty'],{name = 'ns0000:FontSize', def = 'ns0000:PageHeaderFooterFontSize'})
			end

			local result_tbl = standard_option_generator(tbl['phf_print'], element_table)

			-- phf_perlogicalpage
			if(nil ~= tbl['phf_perlogicalpage'])then
				local logicalpage_element_table	= OrderedHash(
					{ 'off',	'ns0000:PhysicalPage'},
					{ 'on',		'ns0000:OriginalPage'}
				)
				table.insert(result_tbl, {
							tag			= 	'psf:Feature',
							attribute	=
							{
								key		=	'name',
								value	=	'ns0000:HeaderFooterPrintPer'
							},
							children	=	standard_option_generator(tbl['phf_perlogicalpage'], logicalpage_element_table)
						}
					)
			end
			
			-- phf_dateandtime_print
			if(nil ~= tbl['phf_dateandtime_print'])then
				local datetaime_element_table	= OrderedHash(
					{ 'off',	'ns0000:Off'},
					{ 'on',		'ns0000:On'}
				)
				local datetaime_table = standard_option_generator(tbl['phf_dateandtime_print'], datetaime_element_table)

				-- phf_dateandtime_position
				if(nil ~= tbl['phf_dateandtime_position'])then
					local datetimeposition_element_table	= OrderedHash(
						{ 'topright',		'ns0000:TopRight'},
						{ 'topleft',		'ns0000:TopLeft'},
						{ 'topcenter',		'ns0000:TopCenter'},
						{ 'bottomright',	'ns0000:BottomRight'},
						{ 'bottomleft',		'ns0000:BottomLeft'},
						{ 'bottomcenter',	'ns0000:BottomCenter'}
					)
					table.insert(datetaime_table, {
								tag			= 	'psf:Feature',
								attribute	=
								{
									key		=	'name',
									value	=	'ns0000:HeaderFooterDatePosition'
								},
								children	=	standard_option_generator(tbl['phf_dateandtime_position'], datetimeposition_element_table)
							}
						)
				end

				-- phf_dateandtime_format
				if(nil ~= tbl['phf_dateandtime_format'])then
					local datetimeformat_element_table	= OrderedHash(
						{ 'mmddyyyy_slash',					'ns0000:MMDDYYYY_SlashFormat'},
						{ 'mmddyyyy_dot',					'ns0000:MMDDYYYY_DotFormat'},
						{ 'ddmmyyyy_slash',					'ns0000:DDMMYYYY_SlashFormat'},
						{ 'ddmmyyyy_dot',					'ns0000:DDMMYYYY_DotFormat'},
						{ 'yyyymmdd_dot',					'ns0000:YYYYMMDD_DotFormat'},
						{ 'mmddyyyy_slash_hhmm_colon',		'ns0000:MMDDYYYYhhmm_SlashFormat'},
						{ 'mmddyyyy_dot_hhmm_colon',		'ns0000:MMDDYYYYhhmm_DotFormat'},
						{ 'ddmmyyyy_slash_hhmm_colon',		'ns0000:DDMMYYYYhhmm_SlashFormat'},
						{ 'ddmmyyyy_dot_hhmm_colon',		'ns0000:YYYYMMDDhhmm_DotFormat'},
						{ 'yyyymmdd_dot_hhmm_colon',		'ns0000:DDMMYYYYhhmm_DotFormat'},
						{ 'yyyymmdd_jpn',					'ns0000:YYYYMMDD_JapaneseFormat'},
						{ 'yyyymmdd_jpn_hhmm_jpn',			'ns0000:YYYYMMDDhhmm_JapaneseFormat'}
					)
					table.insert(datetaime_table, {
								tag			= 	'psf:Feature',
								attribute	=
								{
									key		=	'name',
									value	=	'ns0000:HeaderFooterDateFormat'
								},
								children	=	standard_option_generator(tbl['phf_dateandtime_format'], datetimeformat_element_table)
							}
						)
				end
				
				table.insert(result_tbl, {
							tag			= 	'psf:Feature',
							attribute	=
							{
								key		=	'name',
								value	=	'ns0000:HeaderFooterDate'
							},
							children	=	datetaime_table
						}
					)
			end

			-- phf_pagenumber_print
			if(nil ~= tbl['phf_pagenumber_print'])then
				local number_element_table	= OrderedHash(
					{ 'off',	'ns0000:Off'},
					{ 'on',		'ns0000:On'}
				)
				local number_table = standard_option_generator(tbl['phf_pagenumber_print'], number_element_table)
				
				-- phf_pagenumber_position
				if(nil ~= tbl['phf_pagenumber_position'])then
					local numberposition_element_table	= OrderedHash(
						{ 'topright',			'ns0000:TopRight'},
						{ 'topleft',			'ns0000:TopLeft'},
						{ 'topcenter',			'ns0000:TopCenter'},
						{ 'bottomright',		'ns0000:BottomRight'},
						{ 'bottomleft',			'ns0000:BottomLeft'},
						{ 'bottomcenter',		'ns0000:BottomCenter'},
						{ 'insidetopleft',		'ns0000:InsideTopOrLeft'},
						{ 'insidetopright',		'ns0000:InsideTopOrRight'},
						{ 'insidebottomleft',	'ns0000:InsideBottomOrLeft'},
						{ 'insidebottomright',	'ns0000:InsideBottomOrRight'},
						{ 'outsidetopleft',		'ns0000:OutsideTopOrLeft'},
						{ 'outsidetopright',	'ns0000:OutsideTopOrRight'},
						{ 'outsidebottomleft',	'ns0000:OutsideBottomOrLeft'},
						{ 'outsidebottomright',	'ns0000:OutsideBottomOrRight'}
					)
					table.insert(number_table, {
								tag			= 	'psf:Feature',
								attribute	=
								{
									key		=	'name',
									value	=	'ns0000:HeaderFooterNumberPosition'
								},
								children	=	standard_option_generator(tbl['phf_pagenumber_position'], numberposition_element_table)
							}
						)
				end
				
				-- phf_dateandtime_format
				if(nil ~= tbl['phf_pagenumber_format'])then
					local numberformat_element_table	= OrderedHash(
						{ 'p_only',			'ns0000:PNumber'},
						{ 'hyphen',			'ns0000:NumberHyphen'},
						{ 'p_dot',			'ns0000:PDotNumber'},
						{ 'none',			'ns0000:OnlyNumber'},
						{ 'jpn_page',		'ns0000:JapanesePageNumber'}
					)

					table.insert(number_table, {
								tag			= 	'psf:Feature',
								attribute	=
								{
									key		=	'name',
									value	=	'ns0000:HeaderFooterNumberFormat'
								},
								children	=	standard_option_generator(tbl['phf_pagenumber_format'], numberformat_element_table)
							}
						)
				end
				
				table.insert(result_tbl, {
							tag			= 	'psf:Feature',
							attribute	=
							{
								key		=	'name',
								value	=	'ns0000:HeaderFooterNumber'
							},
							children	=	number_table
						}
					)
			end
			
			-- phf_text_print
			if(nil ~= tbl['phf_text_print'])then
				local text_element_table	= OrderedHash(
					{ 'off',	'ns0000:Off'},
					{ 'on',		'ns0000:On'}
				)
				local text_table = standard_option_generator(tbl['phf_text_print'], text_element_table)
				
				-- phf_text_position
				if(nil ~= tbl['phf_text_position'])then
					local textposition_element_table	= OrderedHash(
						{ 'topright',			'ns0000:TopRight'},
						{ 'topleft',			'ns0000:TopLeft'},
						{ 'topcenter',			'ns0000:TopCenter'},
						{ 'bottomright',		'ns0000:BottomRight'},
						{ 'bottomleft',			'ns0000:BottomLeft'},
						{ 'bottomcenter',		'ns0000:BottomCenter'}
					)

					table.insert(text_table, {
								tag			= 	'psf:Feature',
								attribute	=
								{
									key		=	'name',
									value	=	'ns0000:HeaderFooterTextPosition'
								},
								children	=	standard_option_generator(tbl['phf_text_position'], textposition_element_table)
							}
						)
				end
				
				-- phf_text_type
				if(nil ~= tbl['phf_text_type'])then
					local textformat_element_table	= OrderedHash(
						{ 'filename',			{value = 'ns0000:FileName', scoredproperty = {}}},
						{ 'logonname',			{value = 'ns0000:LoginName', scoredproperty = {}}},
						{ 'userdefinedtext',	{value = 'ns0000:Custom', scoredproperty = {}}}
					)

					--Set ScoredProperty Info--
					---Text--
					if(nil ~= tbl['phf_text_userdefined'])then
						table.insert(textformat_element_table['userdefinedtext']['scoredproperty'],{name = 'ns0000:CustomText', def = 'ns0000:PageHeaderFooterCustomText'})
					end

					table.insert(text_table, {
								tag			= 	'psf:Feature',
								attribute	=
								{
									key		=	'name',
									value	=	'ns0000:HeaderFooterTextFormat'
								},
								children	=	standard_option_generator(tbl['phf_text_type'], textformat_element_table)
							}
						)
				end
				
				table.insert(result_tbl, {
							tag			= 	'psf:Feature',
							attribute	=
							{
								key		=	'name',
								value	=	'ns0000:HeaderFooterText'
							},
							children	=	text_table
						}
					)
			end
			return result_tbl
		end
	},
	-- Header/Footer Font
	phf_font	=
	{
		tag		=	'psf:ParameterDef',
		attribute	=
			{
				key		=	'name',
				value	=	'ns0000:PageHeaderFooterFontName'
			},
		children	=	function(tbl)
							local font_spec_table	=
								{
									max = '256',
									min = '1',
									default = tbl['phf_font']['default'],
									current = tbl['phf_font']['current']
								}
							return paramdef_generator('string', font_spec_table, 'characters', nil)
						end
	},
	-- Header/Footer Font Size
	phf_fontsize	=
	{
		tag		=	'psf:ParameterDef',
		attribute	=
			{
				key		=	'name',
				value	=	'ns0000:PageHeaderFooterFontSize'
			},
		children	=	function(tbl)
							return paramdef_generator('integer', tbl['phf_fontsize'], 'point', 1)
						end
	},
	-- Header/Footer Text String
	phf_text_userdefined	=
	{
		tag		=	'psf:ParameterDef',
		attribute	=
			{
				key		=	'name',
				value	=	'ns0000:PageHeaderFooterCustomText'
			},
		children	=	function(tbl)
							return paramdef_generator('string', tbl['phf_text_userdefined'], 'characters', nil)
						end
	},

	-- 03 Image Watermark
	piw_print	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:PageImageWatermark'
		},
		children	=	function(tbl)
			local result_tbl = {}

			result_tbl[ #result_tbl + 1 ] =
			{
				tag			= 	'psf:Property',
				attribute	=
				{
					key		=	'name',
					value	=	'psf:SelectionType'
				},
				children =
				{
					tag			= 	'psf:Value',
					attribute	=
					{
						key		=	'xsi:type',
						value	=	'xsd:QName'
					},
					text		=	'psk:PickOne'
				}
			}

			for i, v in pairs(tbl['piw_print']) do
				if('off' == v['name']) then
					result_tbl[ #result_tbl + 1 ] =
					{
						tag		=	'psf:Option',
						attribute	=
						{
							key		=	'name',
							value	=	'ns0000:Off'
						}
					}
				elseif('on' == v['name'] and nil ~= tbl['piw_namelist'])then
					for index, value in pairs(tbl['piw_namelist']) do
						if( value['name'] )then
							result_tbl[ #result_tbl + 1 ] =
							{
								tag		=	'psf:Option',
								attribute	=
								{
									key		=	'name',
									value	=	'ns0000:On'
								},
								children	=
								{
									tag			= 	'psf:ScoredProperty',
									attribute	=
									{
										key		=	'name',
										value	=	'ns0000:ImageWatermarkItem'
									},
									children =
									{
										tag			= 	'psf:Value',
										attribute	=
										{
											key		=	'xsi:type',
											value	=	'xsd:string'
										},
										text		=	value['name']
									}
								}
							}
						end
					end
				end
			end

			return result_tbl
		end
	},
	-- 04 OverlayWatermark
	pow_print	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:PageOverlayWatermark'
		},
		children	=	function(tbl)
			local result_tbl = {}

			result_tbl[ #result_tbl + 1 ] =
			{
				tag			= 	'psf:Property',
				attribute	=
				{
					key		=	'name',
					value	=	'psf:SelectionType'
				},
				children =
				{
					tag			= 	'psf:Value',
					attribute	=
					{
						key		=	'xsi:type',
						value	=	'xsd:QName'
					},
					text		=	'psk:PickOne'
				}
			}

			for i, v in pairs(tbl['pow_print']) do
				if('off' == v['name']) then
					result_tbl[ #result_tbl + 1 ] =
					{
						tag		=	'psf:Option',
						attribute	=
						{
							key		=	'name',
							value	=	'ns0000:Off'
						}
					}
				elseif('on' == v['name'] and nil ~= tbl['pow_namelist'])then
					for index, value in pairs(tbl['pow_namelist']) do
						if( value['name'] )then
							result_tbl[ #result_tbl + 1 ] =
							{
								tag		=	'psf:Option',
								attribute	=
								{
									key		=	'name',
									value	=	'ns0000:On'
								},
								children	=
								{
									tag			= 	'psf:ScoredProperty',
									attribute	=
									{
										key		=	'name',
										value	=	'ns0000:OverlayWatermarkItem'
									},
									children =
									{
										tag			= 	'psf:Value',
										attribute	=
										{
											key		=	'xsi:type',
											value	=	'xsd:string'
										},
										text		=	value['name']
									}
								}
							}
						end
					end
				end
			end

			return result_tbl
		end
	},
-- [07 Security]
	-- 01 Copy Control Print (Printer)
	cc_print	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:JobCopyControlPrint'
		},
		children	=	function(tbl)
			local element_table	= OrderedHash(
				{ 'off',					'ns0000:Off'},
				{ 'preventioncopypattern',	'ns0000:CopyPreventionDataSecurity'},
				{ 'copyguard',				'ns0000:CopyGuard'}
			)
			local copyguard_table
			
			if(nil ~= tbl['cc_type'])then
				copyguard_table = tbl['cc_type']
			end
			table.insert(copyguard_table,{ name = 'off', condition = {}})

			return standard_option_generator(copyguard_table, element_table)
		end
	},
	-- 01-01 Prevention Copy Pattern (Printer)
	cc_preventioncopypattern_text	=
	{
		{
			tag			= 	'psf:Feature',
			attribute	=
			{
				key		=	'name',
				value	=	'ns0000:JobCopyPrevention'
			},
			children	=	function(tbl)
				local result_tbl = {}
				
				local preventiontext_table = {}
				local preventiontext_scored_table = {}
				
				-- CopyPreventionText
				if(nil ~= tbl['cc_preventioncopypattern_text'])then
					table.insert(preventiontext_scored_table, {
								tag			= 	'psf:ScoredProperty',
								attribute	=
								{
									key		=	'name',
									value	=	'ns0000:CopyPreventionTextString'
								},
								children =
								{
									tag			= 	'psf:ParameterRef',
									attribute	=
										{
											key		=	'name',
											value	=	'ns0000:JobCopyPreventionTextString'
										}
								}
							}
						)
				end

				-- FontSize
				if(nil ~= tbl['cc_preventioncopypattern_size'])then
					table.insert(preventiontext_scored_table, {
								tag			= 	'psf:ScoredProperty',
								attribute	=
								{
									key		=	'name',
									value	=	'ns0000:CopyPreventionFontSize'
								},
								children =
								{
									tag			= 	'psf:ParameterRef',
									attribute	=
										{
											key		=	'name',
											value	=	'ns0000:JobCopyPreventionFontSize'
										}
								}
							}
						)
				end
				
				table.insert(preventiontext_table, {
						tag			= 	'psf:Property',
						attribute	=
						{
							key		=	'name',
							value	=	'psf:SelectionType'
						},
						children = {
							tag			= 	'psf:Value',
							attribute	=
							{
								key		=	'xsi:type',
								value	=	'xsd:QName'
							},
							text		=	'psk:PickOne'
						}
					}
				)
				table.insert(preventiontext_table, {
						tag			=	'psf:Option',
						children	=	preventiontext_scored_table
					}
				)
				
				-- Font
				if(nil ~= tbl['cc_preventioncopypattern_font'])then
					local font_element_table	= OrderedHash(
						{ 'albertus-medium',						'ns0000:Albertus_Medium'},
						{ 'albertus-extra-bold',					'ns0000:Albertus_Extra_Bold'},
						{ 'antiqueolive',							'ns0000:AntiqueOlive'},
						{ 'albertus',								'ns0000:Albertus'},
						{ 'albertus-italic',						'ns0000:Albertus_Italic'},
						{ 'albertus-light',							'ns0000:Albertus_Light'},
						{ 'antiqueolive-bold',						'ns0000:AntiqueOlive_Bold'},
						{ 'antiqueolive-compact',					'ns0000:AntiqueOlive_Compact'},
						{ 'antiqueolive-italic',					'ns0000:AntiqueOlive_Italic'},
						{ 'antiqueolive-roman',						'ns0000:AntiqueOlive_Roman'},
						{ 'apple-chancery',							'ns0000:Apple_Chancery'},
						{ 'arial',									'ns0000:Arial'},
						{ 'arial-bold',								'ns0000:Arial_Bold'},
						{ 'arial-bold-italic',						'ns0000:Arial_Bold_Italic'},
						{ 'arial-italic',							'ns0000:Arial_Italic'},
						{ 'bodoni',									'ns0000:Bodoni'},
						{ 'bodoni-bold',							'ns0000:Bodoni_Bold'},
						{ 'bodoni-bold-italic',						'ns0000:Bodoni_Bold_Italic'},
						{ 'bodoni-italic',							'ns0000:Bodoni_Italic'},
						{ 'bodoni-poster',							'ns0000:Bodoni_Poster'},
						{ 'bodoni-poster-compressed',				'ns0000:Bodoni_Poster_Compressed'},
						{ 'carta',									'ns0000:Carta'},
						{ 'cg-omega',								'ns0000:CG_Omega'},
						{ 'cg-omega-bold',							'ns0000:CG_Omega_Bold'},
						{ 'cg-omega-bold-italic',					'ns0000:CG_Omega_Bold_Italic'},
						{ 'cg-omega-italic',						'ns0000:CG_Omega_Italic'},
						{ 'cg-times',								'ns0000:CG_Times'},
						{ 'cg-times-bold',							'ns0000:CG_Times_Bold'},
						{ 'cg-times-bold-italic',					'ns0000:CG_Times_Bold_Italic'},
						{ 'cg-times-italic',						'ns0000:CG_Times_Italic'},
						{ 'chicago',								'ns0000:Chicago'},
						{ 'clarendon',								'ns0000:Clarendon'},
						{ 'clarendon-bold',							'ns0000:Clarendon_Bold'},
						{ 'clarendon-light',						'ns0000:Clarendon_Light'},
						{ 'clarendon-condensed',					'ns0000:Clarendon_Condensed'},
						{ 'cooperblack',							'ns0000:CooperBlack'},
						{ 'cooperblack-italic',						'ns0000:CooperBlack_Italic'},
						{ 'copperplate-gothic-32bc',				'ns0000:Copperplate_Gothic_33BC'},
						{ 'copperplate-gothic-33bc',				'ns0000:Copperplate_Gothic_32BC'},
						{ 'coronet',								'ns0000:Coronet'},
						{ 'courier',								'ns0000:Courier'},
						{ 'courier-bold',							'ns0000:Courier_Bold'},
						{ 'courier-bold-oblique',					'ns0000:Courier_Bold_Oblique'},
						{ 'courier-bold-italic',					'ns0000:Courier_Bold_Italic'},
						{ 'courier-italic',							'ns0000:Courier_Italic'},
						{ 'courier-oblique',						'ns0000:Courier_Oblique'},
						{ 'courierps',								'ns0000:CourierPS'},
						{ 'courierps-bold',							'ns0000:CourierPS_Bold'},
						{ 'courierps-bold-oblique',					'ns0000:CourierPS_Bold_Oblique'},
						{ 'courierps-oblique',						'ns0000:CourierPS_Oblique'},
						{ 'eurostile',								'ns0000:Eurostile'},
						{ 'eurostile-bold',							'ns0000:Eurostile_Bold'},
						{ 'eurostile-bold-extended-two',			'ns0000:Eurostile_Bold_Extended_Two'},
						{ 'eurostile-extended-two',					'ns0000:Eurostile_Extended_Two'},
						{ 'garamond-antiqua',						'ns0000:Garamond_Antiqua'},
						{ 'garamond-halbfett',						'ns0000:Garamond_Halbfett'},
						{ 'garamond-kursiv',						'ns0000:Garamond_Kursiv'},
						{ 'garamond-kursiv-halbfett',				'ns0000:Garamond_Kursiv_Halbfett'},
						{ 'geneva',									'ns0000:Geneva'},
						{ 'gillsans',								'ns0000:GillSans'},
						{ 'gillsans-bold',							'ns0000:GillSans_Bold'},
						{ 'gillsans-bold-italic',					'ns0000:GillSans_Bold_Italic'},
						{ 'gillsans-condensed',						'ns0000:GillSans_Condensed'},
						{ 'gillsans-condensed-bold',				'ns0000:GillSans_Condensed_Bold'},
						{ 'gillsans-extra-bold',					'ns0000:GillSans_Extra_Bold'},
						{ 'gillsans-italic',						'ns0000:GillSans_Italic'},
						{ 'gillsans-light',							'ns0000:GillSans_Light'},
						{ 'gillsans-light-italic',					'ns0000:GillSans_Light_Italic'},
						{ 'goudy-bold',								'ns0000:Goudy_Bold'},
						{ 'goudy-bold-italic',						'ns0000:Goudy_Bold_Italic'},
						{ 'goudy-extrabold',						'ns0000:Goudy_ExtraBold'},
						{ 'goudy-oldstyle',							'ns0000:Goudy_Oldstyle'},
						{ 'goudy-oldstyle-italic',					'ns0000:Goudy_Oldstyle_Italic'},
						{ 'helvetica',								'ns0000:Helvetica'},
						{ 'helvetica-bold',							'ns0000:Helvetica_Bold'},
						{ 'helvetica-bold-oblique',					'ns0000:Helvetica_Bold_Oblique'},
						{ 'helvetica-condensed',					'ns0000:Helvetica_Condensed'},
						{ 'helvetica-condensed-bold',				'ns0000:Helvetica_Condensed_Bold'},
						{ 'helvetica-condensed-bold-oblique',		'ns0000:Helvetica_Condensed_Bold_Oblique'},
						{ 'helvetica-condensed-oblique',			'ns0000:Helvetica_Condensed_Oblique'},
						{ 'helvetica-narrow',						'ns0000:Helvetica_Narrow'},
						{ 'helvetica-narrow-bold',					'ns0000:Helvetica_Narrow_Bold'},
						{ 'helvetica-narrow-bold-oblique',			'ns0000:Helvetica_Narrow_Bold_Oblique'},
						{ 'helvetica-narrow-oblique',				'ns0000:Helvetica_Narrow_Oblique'},
						{ 'helvetica-oblique',						'ns0000:Helvetica_Oblique'},
						{ 'hoeflertext',							'ns0000:HoeflerText'},
						{ 'hoeflertext-black',						'ns0000:HoeflerText_Black'},
						{ 'hoeflertext-black-italic',				'ns0000:HoeflerText_Black_Italic'},
						{ 'hoeflertext-italic',						'ns0000:HoeflerText_Italic'},
						{ 'hoeflertext-ornaments',					'ns0000:HoeflerText_Ornaments'},
						{ 'itc-avantgarde-gothic-book',				'ns0000:ITC_AvantGarde_Gothic_Book'},
						{ 'itc-avantgarde-gothic-book-oblique',		'ns0000:ITC_AvantGarde_Gothic_Book_Oblique'},
						{ 'itc-avantgarde-gothic-demi',				'ns0000:ITC_AvantGarde_Gothic_Demi'},
						{ 'itc-avantgarde-gothic-demi-oblique',		'ns0000:ITC_AvantGarde_Gothic_Demi_Oblique'},
						{ 'itc-bookman-demi',						'ns0000:ITC_Bookman_Demi'},
						{ 'itc-bookman-demi-italic',				'ns0000:ITC_Bookman_Demi_Italic'},
						{ 'itc-bookman-light',						'ns0000:ITC_Bookman_Light'},
						{ 'itc-bookman-light-italic',				'ns0000:ITC_Bookman_Light_Italic'},
						{ 'itc-lubalin-graph-book',					'ns0000:ITC_Lubalin_Graph_Book'},
						{ 'itc-lubalin-graph-book-oblique',			'ns0000:ITC_Lubalin_Graph_Book_Oblique'},
						{ 'itc-lubalin-graph-demi',					'ns0000:ITC_Lubalin_Graph_Demi'},
						{ 'itc-lubalin-graph-demi-oblique',			'ns0000:ITC_Lubalin_Graph_Demi_Oblique'},
						{ 'itc-mona-lisa-recut',					'ns0000:ITC_Mona_Lisa_Recut'},
						{ 'itc-symbol',								'ns0000:ITC_Symbol'},
						{ 'itc-zapfchancery-mediumitalic',			'ns0000:ITC_ZapfChancery_MediumItalic'},
						{ 'itc-zapfdingbats',						'ns0000:ITC_ZapfDingbats'},
						{ 'joanna',									'ns0000:Joanna'},
						{ 'joanna-bold',							'ns0000:Joanna_Bold'},
						{ 'joanna-bold-italic',						'ns0000:Joanna_Bold_Italic'},
						{ 'joanna-italic',							'ns0000:Joanna_Italic'},
						{ 'lettergothic',							'ns0000:LetterGothic'},
						{ 'lettergothic-bold',						'ns0000:LetterGothic_Bold'},
						{ 'lettergothic-bold-slanted',				'ns0000:LetterGothic_Bold_Slanted'},
						{ 'lettergothic-slanted',					'ns0000:LetterGothic_Slanted'},
						{ 'lettergothic-italic',					'ns0000:LetterGothic_Italic'},
						{ 'marigold',								'ns0000:Marigold'},
						{ 'monaco',									'ns0000:Monaco'},
						{ 'new-century-schoolbook-bold',			'ns0000:New_Century_Schoolbook_Bold'},
						{ 'new-century-schoolbook-bold-italic',		'ns0000:New_Century_Schoolbook_Bold_Italic'},
						{ 'new-century-schoolbook-italic',			'ns0000:New_Century_Schoolbook_Italic'},
						{ 'new-century-schoolbook-roman',			'ns0000:New_Century_Schoolbook_Roman'},
						{ 'newyork',								'ns0000:NewYork'},
						{ 'optima',									'ns0000:Optima'},
						{ 'optima-bold',							'ns0000:Optima_Bold'},
						{ 'optima-bold-italic',						'ns0000:Optima_Bold_Italic'},
						{ 'optima-italic',							'ns0000:Optima_Italic'},
						{ 'oxford',									'ns0000:Oxford'},
						{ 'palatino-bold',							'ns0000:Palatino_Bold'},
						{ 'palatino-bold-italic',					'ns0000:Palatino_Bold_Italic'},
						{ 'palatino-italic',						'ns0000:Palatino_Italic'},
						{ 'palatino-roman',							'ns0000:Palatino_Roman'},
						{ 'stempel-garamond-bold',					'ns0000:Stempel_Garamond_Bold'},
						{ 'stempel-garamond-bold-italic',			'ns0000:Stempel_Garamond_Bold_Italic'},
						{ 'stempel-garamond-italic',				'ns0000:Stempel_Garamond_Italic'},
						{ 'stempel-garamond-roman',					'ns0000:Stempel_Garamond_Roman'},
						{ 'symbol',									'ns0000:Symbol'},
						{ 'symbolps',								'ns0000:SymbolPS'},
						{ 'tekton',									'ns0000:Tekton'},
						{ 'times-bold',								'ns0000:Times_Bold'},
						{ 'times-bold-italic',						'ns0000:Times_Bold_Italic'},
						{ 'times-italic',							'ns0000:Times_Italic'},
						{ 'times-new-roman',						'ns0000:Times_New_Roman'},
						{ 'times-new-roman-bold',					'ns0000:Times_New_Roman_Bold'},
						{ 'times-new-roman-bold-italic',			'ns0000:Times_New_Roman_Bold_Italic'},
						{ 'times-new-roman-italic',					'ns0000:Times_New_Roman_Italic'},
						{ 'times-roman',							'ns0000:Times_Roman'},
						{ 'univers',								'ns0000:Univers'},
						{ 'univers-bold',							'ns0000:Univers_Bold'},
						{ 'univers-bold-oblique',					'ns0000:Univers_Bold_Oblique'},
						{ 'univers-bold-condensed',					'ns0000:Univers_Bold_Condensed'},
						{ 'univers-bold-condensed-italic',			'ns0000:Univers_Bold_Condensed_Italic'},
						{ 'univers-bold-italic',					'ns0000:Univers_Bold_Italic'},
						{ 'univers-medium',							'ns0000:Univers_Medium'},
						{ 'univers-medium-condensed',				'ns0000:Univers_Medium_Condensed'},
						{ 'univers-medium-condensed-italic',		'ns0000:Univers_Medium_Condensed_Italic'},
						{ 'univers-medium-italic',					'ns0000:Univers_Medium_Italic'},
						{ 'univers-light',							'ns0000:Univers_Oblique'},
						{ 'univers-light-oblique',					'ns0000:Univers_Light'},
						{ 'univers-oblique',						'ns0000:Univers_Light_Oblique'},
						{ 'universcondensed',						'ns0000:UniversCondensed'},
						{ 'universcondensed-bold',					'ns0000:UniversCondensed_Bold'},
						{ 'universcondensed-bold-oblique',			'ns0000:UniversCondensed_Bold_Oblique'},
						{ 'universcondensed-oblique',				'ns0000:UniversCondensed_Oblique'},
						{ 'universesextended',						'ns0000:UniversesExtended'},
						{ 'universesextended-bold',					'ns0000:UniversesExtended_Bold'},
						{ 'universesextended-bold-oblique',			'ns0000:UniversesExtended_Bold_Oblique'},
						{ 'universesextended-oblique',				'ns0000:UniversesExtended_Oblique'},
						{ 'wingdings',								'ns0000:Wingdings'},
						{ 'heiseikakugo-w5-unijis-utf8-h',			'ns0000:HeiseiKakuGo_W5'},
						{ 'heiseimin-w3-unijis-utf8-h',				'ns0000:HeiseiMin_W3'},
						{ 'ryumin-light-unijis-utf8-h',				'ns0000:Ryumin_Light'},
						{ 'gothicbbb-medium-unijis-utf8-h',			'ns0000:GothicBBB_Medium'},
						{ 'hgheiseikakugo-w5-unijis-utf8-h',		'ns0000:HGHeiseiKakuGo_W5'},
						{ 'hgheiseimin-w3-unijis-utf8-h',			'ns0000:HGHeiseiMin_W3'},
						{ 'nimbussans-bold',						'ns0000:NimbusSans_Bold'},
						{ 'nimbusmonops-bold',						'ns0000:NimbusMonoPS_Bold'},
						{ 'nimbusroman-bold',						'ns0000:NimbusRoman_Bold'}
					)

					table.insert(preventiontext_table, {
								tag			= 	'psf:Feature',
								attribute	=
								{
									key		=	'name',
									value	=	'ns0000:CopyPreventionTextFont'
								},
								children	=	standard_option_generator(tbl['cc_preventioncopypattern_font'], font_element_table)
							}
						)
				end
				
				if(nil ~= tbl['cc_preventioncopypattern_font_irips'])then
					local font_element_table	= OrderedHash(
						{ 'nimbussans-bold',						'ns0000:NimbusSans_Bold'},
						{ 'nimbusmonops-bold',						'ns0000:NimbusMonoPS_Bold'},
						{ 'nimbusroman-bold',						'ns0000:NimbusRoman_Bold'},
						{ 'hgheiseimin-w3-unijis-utf8-h',			'ns0000:HGHeiseiMin_W3'},
						{ 'hgheiseikakugo-w5-unijis-utf8-h',		'ns0000:HGHeiseiKakuGo_W5'}
					)

					table.insert(preventiontext_table, {
								tag			= 	'psf:Feature',
								attribute	=
								{
									key		=	'name',
									value	=	'ns0000:CopyPreventionTextFontIrips'
								},
								children	=	standard_option_generator(tbl['cc_preventioncopypattern_font_irips'], font_element_table)
							}
						)
				end
				
				-- TextPosition
				if(nil ~= tbl['cc_preventioncopypattern_position'])then
					local position_element_table	= OrderedHash(
						{ 'center',			{value = 'ns0000:Center', scoredproperty = {}}},
						{ 'topleft',		{value = 'ns0000:TopLeft', scoredproperty = {}}},
						{ 'centertop',		{value = 'ns0000:TopCenter', scoredproperty = {}}},
						{ 'topright',		{value = 'ns0000:TopRight', scoredproperty = {}}},
						{ 'bottomleft',		{value = 'ns0000:BottomLeft', scoredproperty = {}}},
						{ 'centerbottom',	{value = 'ns0000:BottomCenter', scoredproperty = {}}},
						{ 'bottomright',	{value = 'ns0000:BottomRight', scoredproperty = {}}}
					)
					--Set ScoredProperty Info--
					---TextAngle--
					if(nil ~= tbl['cc_preventioncopypattern_angle'])then
						table.insert(position_element_table['center']['scoredproperty'],{name = 'ns0000:CopyPreventionTextAngle', def = 'ns0000:JobCopyPreventionTextAngle'})
						table.insert(position_element_table['topleft']['scoredproperty'],{name = 'ns0000:CopyPreventionTextAngle', def = 'ns0000:JobCopyPreventionTextAngle'})
						table.insert(position_element_table['centertop']['scoredproperty'],{name = 'ns0000:CopyPreventionTextAngle', def = 'ns0000:JobCopyPreventionTextAngle'})
						table.insert(position_element_table['topright']['scoredproperty'],{name = 'ns0000:CopyPreventionTextAngle', def = 'ns0000:JobCopyPreventionTextAngle'})
						table.insert(position_element_table['bottomleft']['scoredproperty'],{name = 'ns0000:CopyPreventionTextAngle', def = 'ns0000:JobCopyPreventionTextAngle'})
						table.insert(position_element_table['centerbottom']['scoredproperty'],{name = 'ns0000:CopyPreventionTextAngle', def = 'ns0000:JobCopyPreventionTextAngle'})
						table.insert(position_element_table['bottomright']['scoredproperty'],{name = 'ns0000:CopyPreventionTextAngle', def = 'ns0000:JobCopyPreventionTextAngle'})
					end

					table.insert(preventiontext_table, {
								tag			= 	'psf:Feature',
								attribute	=
								{
									key		=	'name',
									value	=	'ns0000:CopyPreventionTextPosition'
								},
								children	=	standard_option_generator(tbl['cc_preventioncopypattern_position'], position_element_table)
							}
						)
				end

				-- TextType
				if(nil ~= tbl['cc_preventioncopypattern_texttype'])then
					local texttype_element_table	= OrderedHash(
						{ 'userdefined',								'ns0000:UserDefined'},
						{ 'loginname',									'ns0000:WindowsLoginName'},
						{ 'printjobname',								'ns0000:PrintJobName'},
						{ 'printjobtime',								'ns0000:PrintJobTime'},
						{ 'loginnameandprintjobname',					'ns0000:WindowsLoginName_PrintJobName'},
						{ 'loginnameandprintjobtime',					'ns0000:WindowsLoginName_PrintJobTime'},
						{ 'printjobnameandprintjobtime',				'ns0000:PrintJobName_PrintJobTime'},
						{ 'loginnameandprintjobnameandprintjobtime',	'ns0000:WindowsLoginName_PrintJobName_PrintJobTime'}
					)

					table.insert(preventiontext_table, {
								tag			= 	'psf:Feature',
								attribute	=
								{
									key		=	'name',
									value	=	'ns0000:CopyPreventionTextType'
								},
								children	=	standard_option_generator(tbl['cc_preventioncopypattern_texttype'], texttype_element_table)
							}
						)
				end
				
				-- RepeatText
				if(nil ~= tbl['cc_preventioncopypattern_repeat'])then
					local repeat_element_table	= OrderedHash(
						{ 'off',							{value = 'ns0000:Off', scoredproperty = {}}},
						{ 'repeat',							{value = 'ns0000:Repeat', scoredproperty = {}}},
						{ 'repeatandrotatecarriagereturn',	{value = 'ns0000:RepeatAndRotate', scoredproperty = {}}}
					)
					--Set ScoredProperty Info--
					---LineSpace--
					if(nil ~= tbl['cc_preventioncopypattern_linespace'])then
						table.insert(repeat_element_table['repeat']['scoredproperty'],{name = 'ns0000:CopyPreventionLineSpace', def = 'ns0000:JobCopyPreventionLineSpace'})
						table.insert(repeat_element_table['repeatandrotatecarriagereturn']['scoredproperty'],{name = 'ns0000:CopyPreventionLineSpace', def = 'ns0000:JobCopyPreventionLineSpace'})
					end

					table.insert(preventiontext_table, {
								tag			= 	'psf:Feature',
								attribute	=
								{
									key		=	'name',
									value	=	'ns0000:CopyPreventionRepeatedText'
								},
								children	=	standard_option_generator(tbl['cc_preventioncopypattern_repeat'], repeat_element_table)
							}
						)
				end

				table.insert(result_tbl, {
						tag			= 	'psf:Feature',
						attribute	=
						{
							key		=	'name',
							value	=	'ns0000:CopyPreventionText'
						},
						children	=	preventiontext_table
					}
				)
				
				-- MaskType
				if(nil ~= tbl['cc_preventioncopypattern_masktype'])then
					local masktype_element_table	= OrderedHash(
						{ 'none',					'ns0000:None'},
						{ 'wavecrest',				'ns0000:WaveCrest'},
						{ 'mesh',					'ns0000:Mesh'},
						{ 'lattice1',				'ns0000:Lattice1'},
						{ 'lattice2',				'ns0000:Lattice2'},
						{ 'interlockingcircles',	'ns0000:InterlockingCircles'},
						{ 'shokkoh',				'ns0000:Shokkoh'},
						{ 'matsukawabishi',			'ns0000:Matsukawabishi'},
						{ 'scale',					'ns0000:Scale'},
						{ 'higaki',					'ns0000:Higaki'},
						{ 'hexagonal',				'ns0000:Hexagonal'}
					)

					table.insert(result_tbl, {
								tag			= 	'psf:Feature',
								attribute	=
								{
									key		=	'name',
									value	=	'ns0000:CopyPreventionMaskType'
								},
								children	=	standard_option_generator(tbl['cc_preventioncopypattern_masktype'], masktype_element_table)
							}
						)
				end
				-- PatternEffects
				if(nil ~= tbl['cc_preventioncopypattern_patterneffects'])then
					local patterneffects_element_table	= OrderedHash(
						{ 'normal',								'ns0000:Normal'},
						{ 'reversepatterns',					'ns0000:ReversePatterns'},
						{ 'reversepatternsandbackgroundonly',	'ns0000:ReverseAndBackgroundOnly'},
						{ 'textonly',							'ns0000:TextOnly'}
					)

					table.insert(result_tbl, {
								tag			= 	'psf:Feature',
								attribute	=
								{
									key		=	'name',
									value	=	'ns0000:CopyPreventionPatternEffects'
								},
								children	=	standard_option_generator(tbl['cc_preventioncopypattern_patterneffects'], patterneffects_element_table)
							}
						)
				end

				-- Color
				if(nil ~= tbl['cc_preventioncopypattern_color'])then
					local color_element_table	= OrderedHash(
						{ 'black',		'ns0000:Black'},
						{ 'cyan',		'ns0000:Cyan'},
						{ 'magenta',	'ns0000:Magenta'}
					)

					table.insert(result_tbl, {
								tag			= 	'psf:Feature',
								attribute	=
								{
									key		=	'name',
									value	=	'ns0000:CopyPreventionColor'
								},
								children	=	standard_option_generator(tbl['cc_preventioncopypattern_color'], color_element_table)
							}
						)
				end
				
				-- Density
				if(nil ~= tbl['cc_preventioncopypattern_density'])then
					local density_element_table	= OrderedHash(
						{ 'verylight',	'ns0000:VeryLight'},
						{ 'light',		'ns0000:Light'},
						{ 'middle',		'ns0000:Middle'},
						{ 'dark',		'ns0000:Dark'},
						{ 'verydark',	'ns0000:VeryDark'}
					)

					table.insert(result_tbl, {
								tag			= 	'psf:Feature',
								attribute	=
								{
									key		=	'name',
									value	=	'ns0000:CopyPreventionDensity'
								},
								children	=	standard_option_generator(tbl['cc_preventioncopypattern_density'], density_element_table)
							}
						)
				end

				return result_tbl
			end
		},
		-- CopyPreventionText
		{
			tag		=	'psf:ParameterDef',
			attribute	=
				{
					key		=	'name',
					value	=	'ns0000:JobCopyPreventionTextString'
				},
			children	=	function(tbl)
								return paramdef_generator('string', tbl['cc_preventioncopypattern_text'], 'characters', nil)
							end
		}
	},

	-- CopyPreventionTextFontSize
	cc_preventioncopypattern_size	=
	{
		tag		=	'psf:ParameterDef',
		attribute	=
			{
				key		=	'name',
				value	=	'ns0000:JobCopyPreventionFontSize'
			},
		children	=	function(tbl)
							return paramdef_generator('integer', tbl['cc_preventioncopypattern_size'], 'point', 1)
						end
	},
	-- CopyPreventionTextAngle
	cc_preventioncopypattern_angle	=
	{
		tag		=	'psf:ParameterDef',
		attribute	=
			{
				key		=	'name',
				value	=	'ns0000:JobCopyPreventionTextAngle'
			},
		children	=	function(tbl)
							return paramdef_generator('integer', tbl['cc_preventioncopypattern_angle'], 'degree', 1)
						end
	},
	-- CopyPreventionTextLineSpace
	cc_preventioncopypattern_linespace	=
	{
		tag		=	'psf:ParameterDef',
		attribute	=
			{
				key		=	'name',
				value	=	'ns0000:JobCopyPreventionLineSpace'
			},
		children	=	function(tbl)
							return paramdef_generator('integer', tbl['cc_preventioncopypattern_linespace'], 'point', 1)
						end
	},
	
	-- 01-02 Copy Guard (Printer)
	cc_copyguard_text	=
	{
		{
			tag			= 	'psf:Feature',
			attribute	=
			{
				key		=	'name',
				value	=	'ns0000:JobCopyGuard'
			},
			children	=	function(tbl)
				local result_tbl = {}
				
				local copyguardtext_table = {}
				local copyguardtext_scored_table = {}
				
				-- CopyGuardText
				if(nil ~= tbl['cc_copyguard_text'])then
					table.insert(copyguardtext_scored_table, {
								tag			= 	'psf:ScoredProperty',
								attribute	=
								{
									key		=	'name',
									value	=	'ns0000:CopyGuardTextString'
								},
								children =
								{
									tag			= 	'psf:ParameterRef',
									attribute	=
										{
											key		=	'name',
											value	=	'ns0000:JobCopyGuardTextString'
										}
								}
							}
						)
				end
	
				-- FontSize
				if(nil ~= tbl['cc_copyguard_size'])then
					table.insert(copyguardtext_scored_table, {
								tag			= 	'psf:ScoredProperty',
								attribute	=
								{
									key		=	'name',
									value	=	'ns0000:CopyGuardFontSize'
								},
								children =
								{
									tag			= 	'psf:ParameterRef',
									attribute	=
										{
											key		=	'name',
											value	=	'ns0000:JobCopyGuardFontSize'
										}
								}
							}
						)
				end
				
				table.insert(copyguardtext_table, {
						tag			= 	'psf:Property',
						attribute	=
						{
							key		=	'name',
							value	=	'psf:SelectionType'
						},
						children = {
							tag			= 	'psf:Value',
							attribute	=
							{
								key		=	'xsi:type',
								value	=	'xsd:QName'
							},
							text		=	'psk:PickOne'
						}
					}
				)
				table.insert(copyguardtext_table, {
							tag			=	'psf:Option',
							children	=	copyguardtext_scored_table
					}
				)
				-- TextPosition
				if(nil ~= tbl['cc_copyguard_position'])then
					local position_element_table	= OrderedHash(
						{ 'center',			{value = 'ns0000:Center', scoredproperty = {}}},
						{ 'topleft',		{value = 'ns0000:TopLeft', scoredproperty = {}}},
						{ 'centertop',		{value = 'ns0000:TopCenter', scoredproperty = {}}},
						{ 'topright',		{value = 'ns0000:TopRight', scoredproperty = {}}},
						{ 'bottomleft',		{value = 'ns0000:BottomLeft', scoredproperty = {}}},
						{ 'centerbottom',	{value = 'ns0000:BottomCenter', scoredproperty = {}}},
						{ 'bottomright',	{value = 'ns0000:BottomRight', scoredproperty = {}}}
					)
					--Set ScoredProperty Info--
					---TextAngle--
					if(nil ~= tbl['cc_copyguard_angle'])then
						table.insert(position_element_table['center']['scoredproperty'],{name = 'ns0000:CopyGuardTextAngle', def = 'ns0000:JobCopyGuardTextAngle'})
						table.insert(position_element_table['topleft']['scoredproperty'],{name = 'ns0000:CopyGuardTextAngle', def = 'ns0000:JobCopyGuardTextAngle'})
						table.insert(position_element_table['centertop']['scoredproperty'],{name = 'ns0000:CopyGuardTextAngle', def = 'ns0000:JobCopyGuardTextAngle'})
						table.insert(position_element_table['topright']['scoredproperty'],{name = 'ns0000:CopyGuardTextAngle', def = 'ns0000:JobCopyGuardTextAngle'})
						table.insert(position_element_table['bottomleft']['scoredproperty'],{name = 'ns0000:CopyGuardTextAngle', def = 'ns0000:JobCopyGuardTextAngle'})
						table.insert(position_element_table['centerbottom']['scoredproperty'],{name = 'ns0000:CopyGuardTextAngle', def = 'ns0000:JobCopyGuardTextAngle'})
						table.insert(position_element_table['bottomright']['scoredproperty'],{name = 'ns0000:CopyGuardTextAngle', def = 'ns0000:JobCopyGuardTextAngle'})
					end

					table.insert(copyguardtext_table, {
								tag			= 	'psf:Feature',
								attribute	=
								{
									key		=	'name',
									value	=	'ns0000:CopyGuardTextPosition'
								},
								children	=	standard_option_generator(tbl['cc_copyguard_position'], position_element_table)
							}
						)
				end
				-- Font
				if(nil ~= tbl['cc_copyguard_font'])then
					local font_element_table	= OrderedHash(
						{ 'albertus-medium',						'ns0000:Albertus_Medium'},
						{ 'albertus-extra-bold',					'ns0000:Albertus_Extra_Bold'},
						{ 'antiqueolive',							'ns0000:AntiqueOlive'},
						{ 'albertus',								'ns0000:Albertus'},
						{ 'albertus-italic',						'ns0000:Albertus_Italic'},
						{ 'albertus-light',							'ns0000:Albertus_Light'},
						{ 'antiqueolive-bold',						'ns0000:AntiqueOlive_Bold'},
						{ 'antiqueolive-compact',					'ns0000:AntiqueOlive_Compact'},
						{ 'antiqueolive-italic',					'ns0000:AntiqueOlive_Italic'},
						{ 'antiqueolive-roman',						'ns0000:AntiqueOlive_Roman'},
						{ 'apple-chancery',							'ns0000:Apple_Chancery'},
						{ 'arial',									'ns0000:Arial'},
						{ 'arial-bold',								'ns0000:Arial_Bold'},
						{ 'arial-bold-italic',						'ns0000:Arial_Bold_Italic'},
						{ 'arial-italic',							'ns0000:Arial_Italic'},
						{ 'bodoni',									'ns0000:Bodoni'},
						{ 'bodoni-bold',							'ns0000:Bodoni_Bold'},
						{ 'bodoni-bold-italic',						'ns0000:Bodoni_Bold_Italic'},
						{ 'bodoni-italic',							'ns0000:Bodoni_Italic'},
						{ 'bodoni-poster',							'ns0000:Bodoni_Poster'},
						{ 'bodoni-poster-compressed',				'ns0000:Bodoni_Poster_Compressed'},
						{ 'carta',									'ns0000:Carta'},
						{ 'cg-omega',								'ns0000:CG_Omega'},
						{ 'cg-omega-bold',							'ns0000:CG_Omega_Bold'},
						{ 'cg-omega-bold-italic',					'ns0000:CG_Omega_Bold_Italic'},
						{ 'cg-omega-italic',						'ns0000:CG_Omega_Italic'},
						{ 'cg-times',								'ns0000:CG_Times'},
						{ 'cg-times-bold',							'ns0000:CG_Times_Bold'},
						{ 'cg-times-bold-italic',					'ns0000:CG_Times_Bold_Italic'},
						{ 'cg-times-italic',						'ns0000:CG_Times_Italic'},
						{ 'chicago',								'ns0000:Chicago'},
						{ 'clarendon',								'ns0000:Clarendon'},
						{ 'clarendon-bold',							'ns0000:Clarendon_Bold'},
						{ 'clarendon-light',						'ns0000:Clarendon_Light'},
						{ 'clarendon-condensed',					'ns0000:Clarendon_Condensed'},
						{ 'cooperblack',							'ns0000:CooperBlack'},
						{ 'cooperblack-italic',						'ns0000:CooperBlack_Italic'},
						{ 'copperplate-gothic-32bc',				'ns0000:Copperplate_Gothic_33BC'},
						{ 'copperplate-gothic-33bc',				'ns0000:Copperplate_Gothic_32BC'},
						{ 'coronet',								'ns0000:Coronet'},
						{ 'courier',								'ns0000:Courier'},
						{ 'courier-bold',							'ns0000:Courier_Bold'},
						{ 'courier-bold-oblique',					'ns0000:Courier_Bold_Oblique'},
						{ 'courier-bold-italic',					'ns0000:Courier_Bold_Italic'},
						{ 'courier-italic',							'ns0000:Courier_Italic'},
						{ 'courier-oblique',						'ns0000:Courier_Oblique'},
						{ 'courierps',								'ns0000:CourierPS'},
						{ 'courierps-bold',							'ns0000:CourierPS_Bold'},
						{ 'courierps-bold-oblique',					'ns0000:CourierPS_Bold_Oblique'},
						{ 'courierps-oblique',						'ns0000:CourierPS_Oblique'},
						{ 'eurostile',								'ns0000:Eurostile'},
						{ 'eurostile-bold',							'ns0000:Eurostile_Bold'},
						{ 'eurostile-bold-extended-two',			'ns0000:Eurostile_Bold_Extended_Two'},
						{ 'eurostile-extended-two',					'ns0000:Eurostile_Extended_Two'},
						{ 'garamond-antiqua',						'ns0000:Garamond_Antiqua'},
						{ 'garamond-halbfett',						'ns0000:Garamond_Halbfett'},
						{ 'garamond-kursiv',						'ns0000:Garamond_Kursiv'},
						{ 'garamond-kursiv-halbfett',				'ns0000:Garamond_Kursiv_Halbfett'},
						{ 'geneva',									'ns0000:Geneva'},
						{ 'gillsans',								'ns0000:GillSans'},
						{ 'gillsans-bold',							'ns0000:GillSans_Bold'},
						{ 'gillsans-bold-italic',					'ns0000:GillSans_Bold_Italic'},
						{ 'gillsans-condensed',						'ns0000:GillSans_Condensed'},
						{ 'gillsans-condensed-bold',				'ns0000:GillSans_Condensed_Bold'},
						{ 'gillsans-extra-bold',					'ns0000:GillSans_Extra_Bold'},
						{ 'gillsans-italic',						'ns0000:GillSans_Italic'},
						{ 'gillsans-light',							'ns0000:GillSans_Light'},
						{ 'gillsans-light-italic',					'ns0000:GillSans_Light_Italic'},
						{ 'goudy-bold',								'ns0000:Goudy_Bold'},
						{ 'goudy-bold-italic',						'ns0000:Goudy_Bold_Italic'},
						{ 'goudy-extrabold',						'ns0000:Goudy_ExtraBold'},
						{ 'goudy-oldstyle',							'ns0000:Goudy_Oldstyle'},
						{ 'goudy-oldstyle-italic',					'ns0000:Goudy_Oldstyle_Italic'},
						{ 'helvetica',								'ns0000:Helvetica'},
						{ 'helvetica-bold',							'ns0000:Helvetica_Bold'},
						{ 'helvetica-bold-oblique',					'ns0000:Helvetica_Bold_Oblique'},
						{ 'helvetica-condensed',					'ns0000:Helvetica_Condensed'},
						{ 'helvetica-condensed-bold',				'ns0000:Helvetica_Condensed_Bold'},
						{ 'helvetica-condensed-bold-oblique',		'ns0000:Helvetica_Condensed_Bold_Oblique'},
						{ 'helvetica-condensed-oblique',			'ns0000:Helvetica_Condensed_Oblique'},
						{ 'helvetica-narrow',						'ns0000:Helvetica_Narrow'},
						{ 'helvetica-narrow-bold',					'ns0000:Helvetica_Narrow_Bold'},
						{ 'helvetica-narrow-bold-oblique',			'ns0000:Helvetica_Narrow_Bold_Oblique'},
						{ 'helvetica-narrow-oblique',				'ns0000:Helvetica_Narrow_Oblique'},
						{ 'helvetica-oblique',						'ns0000:Helvetica_Oblique'},
						{ 'hoeflertext',							'ns0000:HoeflerText'},
						{ 'hoeflertext-black',						'ns0000:HoeflerText_Black'},
						{ 'hoeflertext-black-italic',				'ns0000:HoeflerText_Black_Italic'},
						{ 'hoeflertext-italic',						'ns0000:HoeflerText_Italic'},
						{ 'hoeflertext-ornaments',					'ns0000:HoeflerText_Ornaments'},
						{ 'itc-avantgarde-gothic-book',				'ns0000:ITC_AvantGarde_Gothic_Book'},
						{ 'itc-avantgarde-gothic-book-oblique',		'ns0000:ITC_AvantGarde_Gothic_Book_Oblique'},
						{ 'itc-avantgarde-gothic-demi',				'ns0000:ITC_AvantGarde_Gothic_Demi'},
						{ 'itc-avantgarde-gothic-demi-oblique',		'ns0000:ITC_AvantGarde_Gothic_Demi_Oblique'},
						{ 'itc-bookman-demi',						'ns0000:ITC_Bookman_Demi'},
						{ 'itc-bookman-demi-italic',				'ns0000:ITC_Bookman_Demi_Italic'},
						{ 'itc-bookman-light',						'ns0000:ITC_Bookman_Light'},
						{ 'itc-bookman-light-italic',				'ns0000:ITC_Bookman_Light_Italic'},
						{ 'itc-lubalin-graph-book',					'ns0000:ITC_Lubalin_Graph_Book'},
						{ 'itc-lubalin-graph-book-oblique',			'ns0000:ITC_Lubalin_Graph_Book_Oblique'},
						{ 'itc-lubalin-graph-demi',					'ns0000:ITC_Lubalin_Graph_Demi'},
						{ 'itc-lubalin-graph-demi-oblique',			'ns0000:ITC_Lubalin_Graph_Demi_Oblique'},
						{ 'itc-mona-lisa-recut',					'ns0000:ITC_Mona_Lisa_Recut'},
						{ 'itc-symbol',								'ns0000:ITC_Symbol'},
						{ 'itc-zapfchancery-mediumitalic',			'ns0000:ITC_ZapfChancery_MediumItalic'},
						{ 'itc-zapfdingbats',						'ns0000:ITC_ZapfDingbats'},
						{ 'joanna',									'ns0000:Joanna'},
						{ 'joanna-bold',							'ns0000:Joanna_Bold'},
						{ 'joanna-bold-italic',						'ns0000:Joanna_Bold_Italic'},
						{ 'joanna-italic',							'ns0000:Joanna_Italic'},
						{ 'lettergothic',							'ns0000:LetterGothic'},
						{ 'lettergothic-bold',						'ns0000:LetterGothic_Bold'},
						{ 'lettergothic-bold-slanted',				'ns0000:LetterGothic_Bold_Slanted'},
						{ 'lettergothic-slanted',					'ns0000:LetterGothic_Slanted'},
						{ 'lettergothic-italic',					'ns0000:LetterGothic_Italic'},
						{ 'marigold',								'ns0000:Marigold'},
						{ 'monaco',									'ns0000:Monaco'},
						{ 'new-century-schoolbook-bold',			'ns0000:New_Century_Schoolbook_Bold'},
						{ 'new-century-schoolbook-bold-italic',		'ns0000:New_Century_Schoolbook_Bold_Italic'},
						{ 'new-century-schoolbook-italic',			'ns0000:New_Century_Schoolbook_Italic'},
						{ 'new-century-schoolbook-roman',			'ns0000:New_Century_Schoolbook_Roman'},
						{ 'newyork',								'ns0000:NewYork'},
						{ 'optima',									'ns0000:Optima'},
						{ 'optima-bold',							'ns0000:Optima_Bold'},
						{ 'optima-bold-italic',						'ns0000:Optima_Bold_Italic'},
						{ 'optima-italic',							'ns0000:Optima_Italic'},
						{ 'oxford',									'ns0000:Oxford'},
						{ 'palatino-bold',							'ns0000:Palatino_Bold'},
						{ 'palatino-bold-italic',					'ns0000:Palatino_Bold_Italic'},
						{ 'palatino-italic',						'ns0000:Palatino_Italic'},
						{ 'palatino-roman',							'ns0000:Palatino_Roman'},
						{ 'stempel-garamond-bold',					'ns0000:Stempel_Garamond_Bold'},
						{ 'stempel-garamond-bold-italic',			'ns0000:Stempel_Garamond_Bold_Italic'},
						{ 'stempel-garamond-italic',				'ns0000:Stempel_Garamond_Italic'},
						{ 'stempel-garamond-roman',					'ns0000:Stempel_Garamond_Roman'},
						{ 'symbol',									'ns0000:Symbol'},
						{ 'symbolps',								'ns0000:SymbolPS'},
						{ 'tekton',									'ns0000:Tekton'},
						{ 'times-bold',								'ns0000:Times_Bold'},
						{ 'times-bold-italic',						'ns0000:Times_Bold_Italic'},
						{ 'times-italic',							'ns0000:Times_Italic'},
						{ 'times-new-roman',						'ns0000:Times_New_Roman'},
						{ 'times-new-roman-bold',					'ns0000:Times_New_Roman_Bold'},
						{ 'times-new-roman-bold-italic',			'ns0000:Times_New_Roman_Bold_Italic'},
						{ 'times-new-roman-italic',					'ns0000:Times_New_Roman_Italic'},
						{ 'times-roman',							'ns0000:Times_Roman'},
						{ 'univers',								'ns0000:Univers'},
						{ 'univers-bold',							'ns0000:Univers_Bold'},
						{ 'univers-bold-oblique',					'ns0000:Univers_Bold_Oblique'},
						{ 'univers-bold-condensed',					'ns0000:Univers_Bold_Condensed'},
						{ 'univers-bold-condensed-italic',			'ns0000:Univers_Bold_Condensed_Italic'},
						{ 'univers-bold-italic',					'ns0000:Univers_Bold_Italic'},
						{ 'univers-medium',							'ns0000:Univers_Medium'},
						{ 'univers-medium-condensed',				'ns0000:Univers_Medium_Condensed'},
						{ 'univers-medium-condensed-italic',		'ns0000:Univers_Medium_Condensed_Italic'},
						{ 'univers-medium-italic',					'ns0000:Univers_Medium_Italic'},
						{ 'univers-light',							'ns0000:Univers_Oblique'},
						{ 'univers-light-oblique',					'ns0000:Univers_Light'},
						{ 'univers-oblique',						'ns0000:Univers_Light_Oblique'},
						{ 'universcondensed',						'ns0000:UniversCondensed'},
						{ 'universcondensed-bold',					'ns0000:UniversCondensed_Bold'},
						{ 'universcondensed-bold-oblique',			'ns0000:UniversCondensed_Bold_Oblique'},
						{ 'universcondensed-oblique',				'ns0000:UniversCondensed_Oblique'},
						{ 'universesextended',						'ns0000:UniversesExtended'},
						{ 'universesextended-bold',					'ns0000:UniversesExtended_Bold'},
						{ 'universesextended-bold-oblique',			'ns0000:UniversesExtended_Bold_Oblique'},
						{ 'universesextended-oblique',				'ns0000:UniversesExtended_Oblique'},
						{ 'wingdings',								'ns0000:Wingdings'},
						{ 'heiseikakugo-w5-unijis-utf8-h',			'ns0000:HeiseiKakuGo_W5'},
						{ 'heiseimin-w3-unijis-utf8-h',				'ns0000:HeiseiMin_W3'},
						{ 'ryumin-light-unijis-utf8-h',				'ns0000:Ryumin_Light'},
						{ 'gothicbbb-medium-unijis-utf8-h',			'ns0000:GothicBBB_Medium'},
						{ 'hgheiseikakugo-w5-unijis-utf8-h',		'ns0000:HGHeiseiKakuGo_W5'},
						{ 'hgheiseimin-w3-unijis-utf8-h',			'ns0000:HGHeiseiMin_W3'},
						{ 'nimbussans-bold',						'ns0000:NimbusSans_Bold'},
						{ 'nimbusmonops-bold',						'ns0000:NimbusMonoPS_Bold'},
						{ 'nimbusroman-bold',						'ns0000:NimbusRoman_Bold'}
					)

					table.insert(copyguardtext_table, {
								tag			= 	'psf:Feature',
								attribute	=
								{
									key		=	'name',
									value	=	'ns0000:CopyGuardTextFont'
								},
								children	=	standard_option_generator(tbl['cc_copyguard_font'], font_element_table)
							}
						)
				end
				
				if(nil ~= tbl['cc_copyguard_font_irips'])then
					local font_element_table	= OrderedHash(
						{ 'nimbussans-bold',						'ns0000:NimbusSans_Bold'},
						{ 'nimbusmonops-bold',						'ns0000:NimbusMonoPS_Bold'},
						{ 'nimbusroman-bold',						'ns0000:NimbusRoman_Bold'},
						{ 'hgheiseimin-w3-unijis-utf8-h',			'ns0000:HGHeiseiMin_W3'},
						{ 'hgheiseikakugo-w5-unijis-utf8-h',		'ns0000:HGHeiseiKakuGo_W5'}
					)

					table.insert(copyguardtext_table, {
								tag			= 	'psf:Feature',
								attribute	=
								{
									key		=	'name',
									value	=	'ns0000:CopyGuardTextFontIrips'
								},
								children	=	standard_option_generator(tbl['cc_copyguard_font_irips'], font_element_table)
							}
						)
				end
				
				-- TextType
				if(nil ~= tbl['cc_copyguard_texttype'])then
					local texttype_element_table	= OrderedHash(
						{ 'userdefined',								'ns0000:UserDefined'},
						{ 'loginname',									'ns0000:WindowsLoginName'},
						{ 'printjobname',								'ns0000:PrintJobName'},
						{ 'printjobtime',								'ns0000:PrintJobTime'},
						{ 'loginnameandprintjobname',					'ns0000:WindowsLoginName_PrintJobName'},
						{ 'loginnameandprintjobtime',					'ns0000:WindowsLoginName_PrintJobTime'},
						{ 'printjobnameandprintjobtime',				'ns0000:PrintJobName_PrintJobTime'},
						{ 'loginnameandprintjobnameandprintjobtime',	'ns0000:WindowsLoginName_PrintJobName_PrintJobTime'}
					)

					table.insert(copyguardtext_table, {
								tag			= 	'psf:Feature',
								attribute	=
								{
									key		=	'name',
									value	=	'ns0000:CopyGuardTextType'
								},
								children	=	standard_option_generator(tbl['cc_copyguard_texttype'], texttype_element_table)
							}
						)
				end
				-- RepeatText
				if(nil ~= tbl['cc_copyguard_repeat'])then
					local repeat_element_table	= OrderedHash(
						{ 'off',							{value = 'ns0000:Off', scoredproperty = {}}},
						{ 'repeat',							{value = 'ns0000:Repeat', scoredproperty = {}}},
						{ 'repeatandrotatecarriagereturn',	{value = 'ns0000:RepeatAndRotate', scoredproperty = {}}}
					)
					--Set ScoredProperty Info--
					---LineSpace--
					if(nil ~= tbl['cc_copyguard_linespace'])then
						table.insert(repeat_element_table['repeat']['scoredproperty'],{name = 'ns0000:CopyGuardLineSpace', def = 'ns0000:JobCopyGuardLineSpace'})
						table.insert(repeat_element_table['repeatandrotatecarriagereturn']['scoredproperty'],{name = 'ns0000:CopyGuardLineSpace', def = 'ns0000:JobCopyGuardLineSpace'})
					end

					table.insert(copyguardtext_table, {
								tag			= 	'psf:Feature',
								attribute	=
								{
									key		=	'name',
									value	=	'ns0000:CopyGuardRepeatedText'
								},
								children	=	standard_option_generator(tbl['cc_copyguard_repeat'], repeat_element_table)
							}
						)
				end
				
				table.insert(result_tbl, {
						tag			= 	'psf:Feature',
						attribute	=
						{
							key		=	'name',
							value	=	'ns0000:CopyGuardText'
						},
						children	=	copyguardtext_table
					}
				)

				-- PatternEffects
				if(nil ~= tbl['cc_copyguard_patterneffects'])then
					local patterneffects_element_table	= OrderedHash(
						{ 'normal',			'ns0000:Normal'},
						{ 'backgroundonly',	'ns0000:BackgroundOnly'}
					)

					table.insert(result_tbl, {
								tag			= 	'psf:Feature',
								attribute	=
								{
									key		=	'name',
									value	=	'ns0000:CopyGuardPatternEffects'
								},
								children	=	standard_option_generator(tbl['cc_copyguard_patterneffects'], patterneffects_element_table)
							}
						)
				end
				
				return result_tbl
			end
		},
		-- CopyGuardText
		{
			tag		=	'psf:ParameterDef',
			attribute	=
				{
					key		=	'name',
					value	=	'ns0000:JobCopyGuardTextString'
				},
			children	=	function(tbl)
								return paramdef_generator('string', tbl['cc_copyguard_text'], 'characters', nil)
							end
		}
	},
	-- CopyGuardTextFont

	-- CopyGuardTextFontSize
	cc_copyguard_size	=
	{
		tag		=	'psf:ParameterDef',
		attribute	=
			{
				key		=	'name',
				value	=	'ns0000:JobCopyGuardFontSize'
			},
		children	=	function(tbl)
							return paramdef_generator('integer', tbl['cc_copyguard_size'], 'point', 1)
						end
	},
	-- CopyGuardTextAngle
	cc_copyguard_angle	=
	{
		tag		=	'psf:ParameterDef',
		attribute	=
			{
				key		=	'name',
				value	=	'ns0000:JobCopyGuardTextAngle'
			},
		children	=	function(tbl)
							return paramdef_generator('integer', tbl['cc_copyguard_angle'], 'degree', 1)
						end
	},
	-- CopyGuardTextLineSpace
	cc_copyguard_linespace	=
	{
		tag		=	'psf:ParameterDef',
		attribute	=
			{
				key		=	'name',
				value	=	'ns0000:JobCopyGuardLineSpace'
			},
		children	=	function(tbl)
							return paramdef_generator('integer', tbl['cc_copyguard_linespace'], 'point', 1)
						end
	},
	-- 02 Copy Control Print (Driver)
	pdcc_print	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:PageCopyControlPrint'
		},
		children	=	function(tbl)
			local element_table	= OrderedHash(
				{ 'off',					'ns0000:Off'},
				{ 'preventioncopypattern',	'ns0000:CopyPreventionDataSecurity'},
				{ 'copyguard',				'ns0000:CopyGuard'}
			)
			local copyguard_table
			
			if(nil ~= tbl['pdcc_type'])then
				copyguard_table = tbl['pdcc_type']
			end
			table.insert(copyguard_table,{ name = 'off', condition = {}})

			return standard_option_generator(copyguard_table, element_table)
		end
	},
	-- 02-01 Prevention Copy Pattern (Driver)
	pdcc_preventioncopypattern_text	=
	{
		{
			tag			= 	'psf:Feature',
			attribute	=
			{
				key		=	'name',
				value	=	'ns0000:PageCopyPrevention'
			},
			children	=	function(tbl)
				local result_tbl = {}
				
				local preventiontext_table = {}
				local preventiontext_scored_table = {}
				-- CopyPreventionText
				if(nil ~= tbl['pdcc_preventioncopypattern_text'])then
					table.insert(preventiontext_scored_table, {
								tag			= 	'psf:ScoredProperty',
								attribute	=
								{
									key		=	'name',
									value	=	'ns0000:CopyPreventionTextString'
								},
								children =
								{
									tag			= 	'psf:ParameterRef',
									attribute	=
										{
											key		=	'name',
											value	=	'ns0000:PageCopyPreventionTextString'
										}
								}
							}
						)
				end
				-- TextType
				if(nil ~= tbl['pdcc_preventioncopypattern_texttype'])then
					local texttype_element_table	= OrderedHash(
						{ 'userdefined',								'ns0000:UserDefined'},
						{ 'loginname',									'ns0000:WindowsLoginName'},
						{ 'printjobname',								'ns0000:PrintJobName'},
						{ 'printjobtime',								'ns0000:PrintJobTime'},
						{ 'loginnameandprintjobname',					'ns0000:WindowsLoginName_PrintJobName'},
						{ 'loginnameandprintjobtime',					'ns0000:WindowsLoginName_PrintJobTime'},
						{ 'printjobnameandprintjobtime',				'ns0000:PrintJobName_PrintJobTime'},
						{ 'loginnameandprintjobnameandprintjobtime',	'ns0000:WindowsLoginName_PrintJobName_PrintJobTime'}
					)

					table.insert(preventiontext_table, {
								tag			= 	'psf:Feature',
								attribute	=
								{
									key		=	'name',
									value	=	'ns0000:CopyPreventionTextType'
								},
								children	=	standard_option_generator(tbl['pdcc_preventioncopypattern_texttype'], texttype_element_table)
							}
						)
				end
				-- Font
				if(nil ~= tbl['pdcc_preventioncopypattern_font'])then
					table.insert(preventiontext_scored_table, {
								tag			= 	'psf:ScoredProperty',
								attribute	=
								{
									key		=	'name',
									value	=	'ns0000:CopyPreventionFont'
								},
								children =
								{
									tag			= 	'psf:ParameterRef',
									attribute	=
										{
											key		=	'name',
											value	=	'ns0000:PageCopyPreventionFont'
										}
								}
							}
						)
				end
				-- FontSize
				if(nil ~= tbl['pdcc_preventioncopypattern_size'])then
					table.insert(preventiontext_scored_table, {
								tag			= 	'psf:ScoredProperty',
								attribute	=
								{
									key		=	'name',
									value	=	'ns0000:CopyPreventionFontSize'
								},
								children =
								{
									tag			= 	'psf:ParameterRef',
									attribute	=
										{
											key		=	'name',
											value	=	'ns0000:PageCopyPreventionFontSize'
										}
								}
							}
						)
				end
				
				table.insert(preventiontext_table, {
						tag			= 	'psf:Property',
						attribute	=
						{
							key		=	'name',
							value	=	'psf:SelectionType'
						},
						children = {
							tag			= 	'psf:Value',
							attribute	=
							{
								key		=	'xsi:type',
								value	=	'xsd:QName'
							},
							text		=	'psk:PickOne'
						}
					}
				)
				table.insert(preventiontext_table, {
						tag			=	'psf:Option',
						children	=	preventiontext_scored_table
					}
				)
				-- TextPosition
				if(nil ~= tbl['pdcc_preventioncopypattern_position'])then
					local position_element_table	= OrderedHash(
						{ 'center',			{value = 'ns0000:Center', scoredproperty = {}}},
						{ 'topleft',		{value = 'ns0000:TopLeft', scoredproperty = {}}},
						{ 'centertop',		{value = 'ns0000:TopCenter', scoredproperty = {}}},
						{ 'topright',		{value = 'ns0000:TopRight', scoredproperty = {}}},
						{ 'bottomleft',		{value = 'ns0000:BottomLeft', scoredproperty = {}}},
						{ 'centerbottom',	{value = 'ns0000:BottomCenter', scoredproperty = {}}},
						{ 'bottomright',	{value = 'ns0000:BottomRight', scoredproperty = {}}}
					)
					--Set ScoredProperty Info--
					---TextAngle--
					if(nil ~= tbl['pdcc_preventioncopypattern_angle'])then
						table.insert(position_element_table['center']['scoredproperty'],{name = 'ns0000:CopyPreventionTextAngle', def = 'ns0000:PageCopyPreventionTextAngle'})
						table.insert(position_element_table['topleft']['scoredproperty'],{name = 'ns0000:CopyPreventionTextAngle', def = 'ns0000:PageCopyPreventionTextAngle'})
						table.insert(position_element_table['centertop']['scoredproperty'],{name = 'ns0000:CopyPreventionTextAngle', def = 'ns0000:PageCopyPreventionTextAngle'})
						table.insert(position_element_table['topright']['scoredproperty'],{name = 'ns0000:CopyPreventionTextAngle', def = 'ns0000:PageCopyPreventionTextAngle'})
						table.insert(position_element_table['bottomleft']['scoredproperty'],{name = 'ns0000:CopyPreventionTextAngle', def = 'ns0000:PageCopyPreventionTextAngle'})
						table.insert(position_element_table['centerbottom']['scoredproperty'],{name = 'ns0000:CopyPreventionTextAngle', def = 'ns0000:PageCopyPreventionTextAngle'})
						table.insert(position_element_table['bottomright']['scoredproperty'],{name = 'ns0000:CopyPreventionTextAngle', def = 'ns0000:PageCopyPreventionTextAngle'})
					end

					table.insert(preventiontext_table, {
								tag			= 	'psf:Feature',
								attribute	=
								{
									key		=	'name',
									value	=	'ns0000:CopyPreventionTextPosition'
								},
								children	=	standard_option_generator(tbl['pdcc_preventioncopypattern_position'], position_element_table)
							}
						)
				end
				
				-- FontStyle
				if(nil ~= tbl['pdcc_preventioncopypattern_fontstyle'])then
					local fontstyle_element_table	= OrderedHash(
						{ 'regular',	'ns0000:Normal'},
						{ 'bold',		'ns0000:Bold'},
						{ 'italic',		'ns0000:Italic'},
						{ 'bolditalic',	'ns0000:BoldItalic'}
					)

					table.insert(preventiontext_table, {
								tag			= 	'psf:Feature',
								attribute	=
								{
									key		=	'name',
									value	=	'ns0000:CopyPreventionTextFontStyle'
								},
								children	=	standard_option_generator(tbl['pdcc_preventioncopypattern_fontstyle'], fontstyle_element_table)
							}
						)
				end
				
				-- RepeatText
				if(nil ~= tbl['pdcc_preventioncopypattern_repeat'])then
					local repeat_element_table	= OrderedHash(
						{ 'off',							{value = 'ns0000:Off', scoredproperty = {}}},
						{ 'repeat',							{value = 'ns0000:Repeat', scoredproperty = {}}},
						{ 'repeatandrotatecarriagereturn',	{value = 'ns0000:RepeatAndRotate', scoredproperty = {}}}
					)
					--Set ScoredProperty Info--
					---LineSpace--
					if(nil ~= tbl['pdcc_preventioncopypattern_linespace'])then
						table.insert(repeat_element_table['repeat']['scoredproperty'],{name = 'ns0000:CopyPreventionLineSpace', def = 'ns0000:PageCopyPreventionLineSpace'})
						table.insert(repeat_element_table['repeatandrotatecarriagereturn']['scoredproperty'],{name = 'ns0000:CopyPreventionLineSpace', def = 'ns0000:PageCopyPreventionLineSpace'})
					end

					table.insert(preventiontext_table, {
								tag			= 	'psf:Feature',
								attribute	=
								{
									key		=	'name',
									value	=	'ns0000:CopyPreventionRepeatedText'
								},
								children	=	standard_option_generator(tbl['pdcc_preventioncopypattern_repeat'], repeat_element_table)
							}
						)
				end
				
				-- OutlineText
				if(nil ~= tbl['pdcc_preventioncopypattern_outline'])then
					local outline_element_table	= OrderedHash(
						{ 'off',	'ns0000:Off'},
						{ 'on',		'ns0000:On'}
					)

					table.insert(preventiontext_table, {
								tag			= 	'psf:Feature',
								attribute	=
								{
									key		=	'name',
									value	=	'ns0000:CopyPreventionTextOutline'
								},
								children	=	standard_option_generator(tbl['pdcc_preventioncopypattern_outline'], outline_element_table)
							}
						)
				end
				
				-- TextOnly
				if(nil ~= tbl['pdcc_preventioncopypattern_textonly'])then
					local textonly_element_table	= OrderedHash(
						{ 'off',	'ns0000:Off'},
						{ 'on',		'ns0000:On'}
					)

					table.insert(preventiontext_table, {
								tag			= 	'psf:Feature',
								attribute	=
								{
									key		=	'name',
									value	=	'ns0000:CopyPreventionTextOnly'
								},
								children	=	standard_option_generator(tbl['pdcc_preventioncopypattern_textonly'], textonly_element_table)
							}
						)
				end
				
				table.insert(result_tbl, {
						tag			= 	'psf:Feature',
						attribute	=
						{
							key		=	'name',
							value	=	'ns0000:CopyPreventionText'
						},
						children	=	preventiontext_table
					}
				)
				
				-- MaskType
				if(nil ~= tbl['pdcc_preventioncopypattern_masktype'])then
					local masktype_element_table	= OrderedHash(
						{ 'none',					'ns0000:None'},
						{ 'wavecrest',				'ns0000:WaveCrest'},
						{ 'mesh',					'ns0000:Mesh'},
						{ 'lattice1',				'ns0000:Lattice1'},
						{ 'lattice2',				'ns0000:Lattice2'},
						{ 'interlockingcircles',	'ns0000:InterlockingCircles'},
						{ 'shokkoh',				'ns0000:Shokkoh'},
						{ 'matsukawabishi',			'ns0000:Matsukawabishi'},
						{ 'scale',					'ns0000:Scale'},
						{ 'higaki',					'ns0000:Higaki'},
						{ 'hexagonal',				'ns0000:Hexagonal'}
					)
					
					local masktype_table = standard_option_generator(tbl['pdcc_preventioncopypattern_masktype'], masktype_element_table)
					
					-- ReversePatterns
					if(nil ~= tbl['pdcc_preventioncopypattern_reversepatterns'])then
						local reversepattern_element_table	= OrderedHash(
							{ 'off',	'ns0000:Off'},
							{ 'on',		'ns0000:On'}
						)

						table.insert(masktype_table, {
									tag			= 	'psf:Feature',
									attribute	=
									{
										key		=	'name',
										value	=	'ns0000:CopyPreventionReversePatterns'
									},
									children	=	standard_option_generator(tbl['pdcc_preventioncopypattern_reversepatterns'], reversepattern_element_table)
								}
							)
					end
					
					table.insert(result_tbl, {
								tag			= 	'psf:Feature',
								attribute	=
								{
									key		=	'name',
									value	=	'ns0000:CopyPreventionMaskType'
								},
								children	=	masktype_table
							}
						)
				end
				
				-- Clip
				if(nil ~= tbl['pdcc_preventioncopypattern_clip'])then
					local clip_element_table	= OrderedHash(
						{ 'off',	{value = 'ns0000:Off', scoredproperty = {}}},
						{ 'on',		{value = 'ns0000:On', scoredproperty = {}}}
					)
					
					--Set ScoredProperty Info--
					---Left--
					if(nil ~= tbl['pdcc_preventioncopypattern_left'])then
						table.insert(clip_element_table['on']['scoredproperty'],{name = 'ns0000:CopyPreventionClippingLeft', def = 'ns0000:PageCopyPreventionClippingLeft'})
					end
					---Right--
					if(nil ~= tbl['pdcc_preventioncopypattern_right'])then
						table.insert(clip_element_table['on']['scoredproperty'],{name = 'ns0000:CopyPreventionClippingRight', def = 'ns0000:PageCopyPreventionClippingRight'})
					end
					---Top--
					if(nil ~= tbl['pdcc_preventioncopypattern_top'])then
						table.insert(clip_element_table['on']['scoredproperty'],{name = 'ns0000:CopyPreventionClippingTop', def = 'ns0000:PageCopyPreventionClippingTop'})
					end
					---Bottom--
					if(nil ~= tbl['pdcc_preventioncopypattern_bottom'])then
						table.insert(clip_element_table['on']['scoredproperty'],{name = 'ns0000:CopyPreventionClippingBottom', def = 'ns0000:PageCopyPreventionClippingBottom'})
					end

					table.insert(result_tbl, {
								tag			= 	'psf:Feature',
								attribute	=
								{
									key		=	'name',
									value	=	'ns0000:CopyPreventionClipping'
								},
								children	=	standard_option_generator(tbl['pdcc_preventioncopypattern_clip'], clip_element_table)
							}
						)
				end
				
				-- Color
				if(nil ~= tbl['pdcc_preventioncopypattern_color'])then
					local color_element_table	= OrderedHash(
						{ 'black',		'ns0000:Black'},
						{ 'cyan',		'ns0000:Cyan'},
						{ 'magenta',	'ns0000:Magenta'}
					)

					table.insert(result_tbl, {
								tag			= 	'psf:Feature',
								attribute	=
								{
									key		=	'name',
									value	=	'ns0000:CopyPreventionColor'
								},
								children	=	standard_option_generator(tbl['pdcc_preventioncopypattern_color'], color_element_table)
							}
						)
				end
				
				-- Density
				if(nil ~= tbl['pdcc_preventioncopypattern_density'])then
					local density_element_table	= OrderedHash(
						{ 'verylight',		'ns0000:VeryLight'},
						{ 'light',			'ns0000:Light'},
						{ 'middle',			'ns0000:Middle'},
						{ 'dark',			'ns0000:Dark'},
						{ 'verydark',		'ns0000:VeryDark'}
					)

					table.insert(result_tbl, {
								tag			= 	'psf:Feature',
								attribute	=
								{
									key		=	'name',
									value	=	'ns0000:CopyPreventionDensity'
								},
								children	=	standard_option_generator(tbl['pdcc_preventioncopypattern_density'], density_element_table)
							}
						)
				end

				return result_tbl
			end
		},
		-- CopyPreventionText
		{
			tag		=	'psf:ParameterDef',
			attribute	=
				{
					key		=	'name',
					value	=	'ns0000:PageCopyPreventionTextString'
				},
			children	=	function(tbl)
								return paramdef_generator('string', tbl['pdcc_preventioncopypattern_text'], 'characters', nil)
							end
		}
	},
	-- CopyPreventionTextFont
	pdcc_preventioncopypattern_font	=
	{
		tag		=	'psf:ParameterDef',
		attribute	=
			{
				key		=	'name',
				value	=	'ns0000:PageCopyPreventionFont'
			},
		children	=	function(tbl)
							local font_spec_table	=
								{
									max = '256',
									min = '1',
									default = tbl['pdcc_preventioncopypattern_font']['default'],
									current = tbl['pdcc_preventioncopypattern_font']['current']
								}
							return paramdef_generator('string', font_spec_table, 'characters', nil)
						end
	},
	-- CopyPreventionTextFontSize
	pdcc_preventioncopypattern_size	=
	{
		tag		=	'psf:ParameterDef',
		attribute	=
			{
				key		=	'name',
				value	=	'ns0000:PageCopyPreventionFontSize'
			},
		children	=	function(tbl)
							return paramdef_generator('integer', tbl['pdcc_preventioncopypattern_size'], 'point', 1)
						end
	},
	-- CopyPreventionTextAngle
	pdcc_preventioncopypattern_angle	=
	{
		tag		=	'psf:ParameterDef',
		attribute	=
			{
				key		=	'name',
				value	=	'ns0000:PageCopyPreventionTextAngle'
			},
		children	=	function(tbl)
							return paramdef_generator('integer', tbl['pdcc_preventioncopypattern_angle'], 'degree', 1)
						end
	},
	-- CopyPreventionTextLineSpace
	pdcc_preventioncopypattern_linespace	=
	{
		tag		=	'psf:ParameterDef',
		attribute	=
			{
				key		=	'name',
				value	=	'ns0000:PageCopyPreventionLineSpace'
			},
		children	=	function(tbl)
							return paramdef_generator('integer', tbl['pdcc_preventioncopypattern_linespace'], 'point', 1)
						end
	},
	
	-- CopyPreventionClipLeft
	pdcc_preventioncopypattern_left	=
	{
		tag		=	'psf:ParameterDef',
		attribute	=
			{
				key		=	'name',
				value	=	'ns0000:PageCopyPreventionClippingLeft'
			},
		children	=	function(tbl)
							--default
							local default_val = tonumber(tbl['pdcc_preventioncopypattern_left']['default'])
							if(nil ~= default_val)then
								tbl['pdcc_preventioncopypattern_left']['default'] = tostring(default_val * 1000)
							else
								tbl['pdcc_preventioncopypattern_left']['default'] = '0'
							end
							
							--max
							local max_val = tonumber(tbl['pdcc_preventioncopypattern_left']['max'])
							if(nil ~= max_val)then
								tbl['pdcc_preventioncopypattern_left']['max'] = tostring(max_val * 1000)
							else
								tbl['pdcc_preventioncopypattern_left']['max'] = '0'
							end
							
							--min
							local min_val = tonumber(tbl['pdcc_preventioncopypattern_left']['min'])
							if(nil ~= min_val)then
								tbl['pdcc_preventioncopypattern_left']['min'] = tostring(min_val * 1000)
							else
								tbl['pdcc_preventioncopypattern_left']['min'] = '0'
							end
							
							return paramdef_generator('integer', tbl['pdcc_preventioncopypattern_left'], 'microns', 1000)
						end
	},
	-- CopyPreventionClipRight
	pdcc_preventioncopypattern_right	=
	{
		tag		=	'psf:ParameterDef',
		attribute	=
			{
				key		=	'name',
				value	=	'ns0000:PageCopyPreventionClippingRight'
			},
		children	=	function(tbl)
							--default
							local default_val = tonumber(tbl['pdcc_preventioncopypattern_right']['default'])
							if(nil ~= default_val)then
								tbl['pdcc_preventioncopypattern_right']['default'] = tostring(default_val * 1000)
							else
								tbl['pdcc_preventioncopypattern_right']['default'] = '0'
							end
							
							--max
							local max_val = tonumber(tbl['pdcc_preventioncopypattern_right']['max'])
							if(nil ~= max_val)then
								tbl['pdcc_preventioncopypattern_right']['max'] = tostring(max_val * 1000)
							else
								tbl['pdcc_preventioncopypattern_right']['max'] = '0'
							end
							
							--min
							local min_val = tonumber(tbl['pdcc_preventioncopypattern_right']['min'])
							if(nil ~= min_val)then
								tbl['pdcc_preventioncopypattern_right']['min'] = tostring(min_val * 1000)
							else
								tbl['pdcc_preventioncopypattern_right']['min'] = '0'
							end
							
							return paramdef_generator('integer', tbl['pdcc_preventioncopypattern_right'], 'microns', 1000)
						end
	},
	-- CopyPreventionClipTop
	pdcc_preventioncopypattern_top	=
	{
		tag		=	'psf:ParameterDef',
		attribute	=
			{
				key		=	'name',
				value	=	'ns0000:PageCopyPreventionClippingTop'
			},
		children	=	function(tbl)
							--default
							local default_val = tonumber(tbl['pdcc_preventioncopypattern_top']['default'])
							if(nil ~= default_val)then
								tbl['pdcc_preventioncopypattern_top']['default'] = tostring(default_val * 1000)
							else
								tbl['pdcc_preventioncopypattern_top']['default'] = '0'
							end
							
							--max
							local max_val = tonumber(tbl['pdcc_preventioncopypattern_top']['max'])
							if(nil ~= max_val)then
								tbl['pdcc_preventioncopypattern_top']['max'] = tostring(max_val * 1000)
							else
								tbl['pdcc_preventioncopypattern_top']['max'] = '0'
							end
							
							--min
							local min_val = tonumber(tbl['pdcc_preventioncopypattern_top']['min'])
							if(nil ~= min_val)then
								tbl['pdcc_preventioncopypattern_top']['min'] = tostring(min_val * 1000)
							else
								tbl['pdcc_preventioncopypattern_top']['min'] = '0'
							end
							
							return paramdef_generator('integer', tbl['pdcc_preventioncopypattern_top'], 'microns', 1000)
						end
	},
	-- CopyPreventionClipBottom
	pdcc_preventioncopypattern_bottom	=
	{
		tag		=	'psf:ParameterDef',
		attribute	=
			{
				key		=	'name',
				value	=	'ns0000:PageCopyPreventionClippingBottom'
			},
		children	=	function(tbl)
							--default
							local default_val = tonumber(tbl['pdcc_preventioncopypattern_bottom']['default'])
							if(nil ~= default_val)then
								tbl['pdcc_preventioncopypattern_bottom']['default'] = tostring(default_val * 1000)
							else
								tbl['pdcc_preventioncopypattern_bottom']['default'] = '0'
							end
							
							--max
							local max_val = tonumber(tbl['pdcc_preventioncopypattern_bottom']['max'])
							if(nil ~= max_val)then
								tbl['pdcc_preventioncopypattern_bottom']['max'] = tostring(max_val * 1000)
							else
								tbl['pdcc_preventioncopypattern_bottom']['max'] = '0'
							end
							
							--min
							local min_val = tonumber(tbl['pdcc_preventioncopypattern_bottom']['min'])
							if(nil ~= min_val)then
								tbl['pdcc_preventioncopypattern_bottom']['min'] = tostring(min_val * 1000)
							else
								tbl['pdcc_preventioncopypattern_bottom']['min'] = '0'
							end
							
							return paramdef_generator('integer', tbl['pdcc_preventioncopypattern_bottom'], 'microns', 1000)
						end
	},
	
	-- 02-02 Copy Guard (Driver)
	pdcc_copyguard_text	=
	{
		{
			tag			= 	'psf:Feature',
			attribute	=
			{
				key		=	'name',
				value	=	'ns0000:PageCopyGuard'
			},
			children	=	function(tbl)
				local result_tbl = {}
				
				local copyguardtext_table = {}
				local copyguardtext_scored_table = {}
				
				-- CopyGuardText
				if(nil ~= tbl['pdcc_copyguard_text'])then
					table.insert(copyguardtext_scored_table, {
								tag			= 	'psf:ScoredProperty',
								attribute	=
								{
									key		=	'name',
									value	=	'ns0000:CopyGuardTextString'
								},
								children =
								{
									tag			= 	'psf:ParameterRef',
									attribute	=
										{
											key		=	'name',
											value	=	'ns0000:PageCopyGuardTextString'
										}
								}
							}
						)
				end
				-- Font
				if(nil ~= tbl['pdcc_copyguard_font'])then
					table.insert(copyguardtext_scored_table, {
								tag			= 	'psf:ScoredProperty',
								attribute	=
								{
									key		=	'name',
									value	=	'ns0000:CopyGuardFont'
								},
								children =
								{
									tag			= 	'psf:ParameterRef',
									attribute	=
										{
											key		=	'name',
											value	=	'ns0000:PageCopyGuardFont'
										}
								}
							}
						)
				end
				-- FontSize
				if(nil ~= tbl['pdcc_copyguard_size'])then
					table.insert(copyguardtext_scored_table, {
								tag			= 	'psf:ScoredProperty',
								attribute	=
								{
									key		=	'name',
									value	=	'ns0000:CopyGuardFontSize'
								},
								children =
								{
									tag			= 	'psf:ParameterRef',
									attribute	=
										{
											key		=	'name',
											value	=	'ns0000:PageCopyGuardFontSize'
										}
								}
							}
						)
				end
				
				table.insert(copyguardtext_table, {
						tag			= 	'psf:Property',
						attribute	=
						{
							key		=	'name',
							value	=	'psf:SelectionType'
						},
						children = {
							tag			= 	'psf:Value',
							attribute	=
							{
								key		=	'xsi:type',
								value	=	'xsd:QName'
							},
							text		=	'psk:PickOne'
						}
					}
				)
				table.insert(copyguardtext_table, {
							tag			=	'psf:Option',
							children	=	copyguardtext_scored_table
					}
				)
				-- TextPosition
				if(nil ~= tbl['pdcc_copyguard_position'])then
					local position_element_table	= OrderedHash(
						{ 'center',			{value = 'ns0000:Center', scoredproperty = {}}},
						{ 'topleft',		{value = 'ns0000:TopLeft', scoredproperty = {}}},
						{ 'centertop',		{value = 'ns0000:TopCenter', scoredproperty = {}}},
						{ 'topright',		{value = 'ns0000:TopRight', scoredproperty = {}}},
						{ 'bottomleft',		{value = 'ns0000:BottomLeft', scoredproperty = {}}},
						{ 'centerbottom',	{value = 'ns0000:BottomCenter', scoredproperty = {}}},
						{ 'bottomright',	{value = 'ns0000:BottomRight', scoredproperty = {}}}
					)
					--Set ScoredProperty Info--
					---TextAngle--
					if(nil ~= tbl['pdcc_copyguard_angle'])then
						table.insert(position_element_table['center']['scoredproperty'],{name = 'ns0000:CopyGuardTextAngle', def = 'ns0000:PageCopyGuardTextAngle'})
						table.insert(position_element_table['topleft']['scoredproperty'],{name = 'ns0000:CopyGuardTextAngle', def = 'ns0000:PageCopyGuardTextAngle'})
						table.insert(position_element_table['centertop']['scoredproperty'],{name = 'ns0000:CopyGuardTextAngle', def = 'ns0000:PageCopyGuardTextAngle'})
						table.insert(position_element_table['topright']['scoredproperty'],{name = 'ns0000:CopyGuardTextAngle', def = 'ns0000:PageCopyGuardTextAngle'})
						table.insert(position_element_table['bottomleft']['scoredproperty'],{name = 'ns0000:CopyGuardTextAngle', def = 'ns0000:PageCopyGuardTextAngle'})
						table.insert(position_element_table['centerbottom']['scoredproperty'],{name = 'ns0000:CopyGuardTextAngle', def = 'ns0000:PageCopyGuardTextAngle'})
						table.insert(position_element_table['bottomright']['scoredproperty'],{name = 'ns0000:CopyGuardTextAngle', def = 'ns0000:PageCopyGuardTextAngle'})
					end

					table.insert(copyguardtext_table, {
								tag			= 	'psf:Feature',
								attribute	=
								{
									key		=	'name',
									value	=	'ns0000:CopyGuardTextPosition'
								},
								children	=	standard_option_generator(tbl['pdcc_copyguard_position'], position_element_table)
							}
						)
				end
				
				-- FontStyle
				if(nil ~= tbl['pdcc_copyguard_fontstyle'])then
					local fontstyle_element_table	= OrderedHash(
						{ 'regular',		'ns0000:Normal'},
						{ 'bold',			'ns0000:Bold'},
						{ 'italic',			'ns0000:Italic'},
						{ 'bolditalic',		'ns0000:BoldItalic'}
					)

					table.insert(copyguardtext_table, {
								tag			= 	'psf:Feature',
								attribute	=
								{
									key		=	'name',
									value	=	'ns0000:CopyGuardTextFontStyle'
								},
								children	=	standard_option_generator(tbl['pdcc_copyguard_fontstyle'], fontstyle_element_table)
							}
						)
				end
				
				-- TextType
				if(nil ~= tbl['pdcc_copyguard_texttype'])then
					local texttype_element_table	= OrderedHash(
						{ 'userdefined',								'ns0000:UserDefined'},
						{ 'loginname',									'ns0000:WindowsLoginName'},
						{ 'printjobname',								'ns0000:PrintJobName'},
						{ 'printjobtime',								'ns0000:PrintJobTime'},
						{ 'loginnameandprintjobname',					'ns0000:WindowsLoginName_PrintJobName'},
						{ 'loginnameandprintjobtime',					'ns0000:WindowsLoginName_PrintJobTime'},
						{ 'printjobnameandprintjobtime',				'ns0000:PrintJobName_PrintJobTime'},
						{ 'loginnameandprintjobnameandprintjobtime',	'ns0000:WindowsLoginName_PrintJobName_PrintJobTime'}
					)

					table.insert(copyguardtext_table, {
								tag			= 	'psf:Feature',
								attribute	=
								{
									key		=	'name',
									value	=	'ns0000:CopyGuardTextType'
								},
								children	=	standard_option_generator(tbl['pdcc_copyguard_texttype'], texttype_element_table)
							}
						)
				end
				
				-- RepeatText
				if(nil ~= tbl['pdcc_copyguard_repeat'])then
					local repeat_element_table	= OrderedHash(
						{ 'off',							{value = 'ns0000:Off', scoredproperty = {}}},
						{ 'repeat',							{value = 'ns0000:Repeat', scoredproperty = {}}},
						{ 'repeatandrotatecarriagereturn',	{value = 'ns0000:RepeatAndRotate', scoredproperty = {}}}
					)
					--Set ScoredProperty Info--
					---LineSpace--
					if(nil ~= tbl['pdcc_copyguard_linespace'])then
						table.insert(repeat_element_table['repeat']['scoredproperty'],{name = 'ns0000:CopyGuardLineSpace', def = 'ns0000:PageCopyGuardLineSpace'})
						table.insert(repeat_element_table['repeatandrotatecarriagereturn']['scoredproperty'],{name = 'ns0000:CopyGuardLineSpace', def = 'ns0000:PageCopyGuardLineSpace'})
					end

					table.insert(copyguardtext_table, {
								tag			= 	'psf:Feature',
								attribute	=
								{
									key		=	'name',
									value	=	'ns0000:CopyGuardRepeatedText'
								},
								children	=	standard_option_generator(tbl['pdcc_copyguard_repeat'], repeat_element_table)
							}
						)
				end
				
				-- OutlineText
				if(nil ~= tbl['pdcc_copyguard_outline'])then
					local outline_element_table	= OrderedHash(
						{ 'off',	'ns0000:Off'},
						{ 'on',		'ns0000:On'}
					)

					table.insert(copyguardtext_table, {
								tag			= 	'psf:Feature',
								attribute	=
								{
									key		=	'name',
									value	=	'ns0000:CopyGuardTextOutline'
								},
								children	=	standard_option_generator(tbl['pdcc_copyguard_outline'], outline_element_table)
							}
						)
				end
				
				table.insert(result_tbl, {
						tag			= 	'psf:Feature',
						attribute	=
						{
							key		=	'name',
							value	=	'ns0000:CopyGuardText'
						},
						children	=	copyguardtext_table
					}
				)
				
				return result_tbl
			end
		},
		-- CopyGuardText
		{
			tag		=	'psf:ParameterDef',
			attribute	=
				{
					key		=	'name',
					value	=	'ns0000:PageCopyGuardTextString'
				},
			children	=	function(tbl)
								return paramdef_generator('string', tbl['pdcc_copyguard_text'], 'characters', nil)
							end
		}
	},
	-- CopyGuardTextFont
	pdcc_copyguard_font	=
	{
		tag		=	'psf:ParameterDef',
		attribute	=
			{
				key		=	'name',
				value	=	'ns0000:PageCopyGuardFont'
			},
		children	=	function(tbl)
							local font_spec_table	=
								{
									max = '256',
									min = '1',
									default = tbl['pdcc_copyguard_font']['default'],
									current = tbl['pdcc_copyguard_font']['current']
								}
							return paramdef_generator('string', font_spec_table, 'characters', nil)
						end
	},
	-- CopyGuardTextFontSize
	pdcc_copyguard_size	=
	{
		tag		=	'psf:ParameterDef',
		attribute	=
			{
				key		=	'name',
				value	=	'ns0000:PageCopyGuardFontSize'
			},
		children	=	function(tbl)
							return paramdef_generator('integer', tbl['pdcc_copyguard_size'], 'point', 1)
						end
	},
	-- CopyGuardTextAngle
	pdcc_copyguard_angle	=
	{
		tag		=	'psf:ParameterDef',
		attribute	=
			{
				key		=	'name',
				value	=	'ns0000:PageCopyGuardTextAngle'
			},
		children	=	function(tbl)
							return paramdef_generator('integer', tbl['pdcc_copyguard_angle'], 'degree', 1)
						end
	},
	-- CopyGuardTextLineSpace
	pdcc_copyguard_linespace	=
	{
		tag		=	'psf:ParameterDef',
		attribute	=
			{
				key		=	'name',
				value	=	'ns0000:PageCopyGuardLineSpace'
			},
		children	=	function(tbl)
							return paramdef_generator('integer', tbl['pdcc_copyguard_linespace'], 'point', 1)
						end
	},
	-- 04 SecretNumbering
	secretnumbering	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:JobSecretNumbering'
		},
		children	=	function(tbl)
			local result_tbl = {}
			
			local element_table	= OrderedHash(
				{ 'off',	{value = 'ns0000:Off', scoredproperty = {}}},
				{ 'on',		{value = 'ns0000:On', scoredproperty = {}}}
			)
			--Set ScoredProperty Info--

			---StartNumber--
			if(nil ~= tbl['secretnumbering_number'])then
				table.insert(element_table['on']['scoredproperty'],{name = 'ns0000:StartNumber', def = 'ns0000:JobSecretNumberingStartNumber'})
			end

			local result_tbl = standard_option_generator(tbl['secretnumbering'], element_table)

			-- secretnumbering_size
			if(nil ~= tbl['secretnumbering_size'])then
				local secretnumbering_size_element_table	= OrderedHash(
					{ 'small',	'ns0000:Small' },
					{ 'middle',	'ns0000:Middle' },
					{ 'large',	'ns0000:Large' }
				)
				table.insert(result_tbl, {
							tag			= 	'psf:Feature',
							attribute	=
							{
								key		=	'name',
								value	=	'ns0000:SecretNumberingSize'
							},
							children	=	standard_option_generator(tbl['secretnumbering_size'], secretnumbering_size_element_table)
						}
					)
			end
			-- secretnumbering_density
			if(nil ~= tbl['secretnumbering_density'])then
				local secretnumbering_density_element_table	= OrderedHash(
					{ 'light',		'ns0000:Light' },
					{ 'middle',		'ns0000:Middle' },
					{ 'littledark',	'ns0000:LittleDark' },
					{ 'dark',		'ns0000:Dark' }
				)
				table.insert(result_tbl, {
							tag			= 	'psf:Feature',
							attribute	=
							{
								key		=	'name',
								value	=	'ns0000:SecretNumberingDensity'
							},
							children	=	standard_option_generator(tbl['secretnumbering_density'], secretnumbering_density_element_table)
						}
					)
			end

			-- secretnumbering_color
			if(nil ~= tbl['secretnumbering_color'])then
				local secretnumbering_color_element_table	= OrderedHash(
					{ 'black',		'ns0000:Black' },
					{ 'red',		'ns0000:Red' },
					{ 'green',		'ns0000:Green' },
					{ 'blue',		'ns0000:Blue' },
					{ 'cyan',		'ns0000:Cyan' },
					{ 'magenta',	'ns0000:Magenta' },
					{ 'yellow',		'ns0000:Yellow' }
				)
				table.insert(result_tbl, {
							tag			= 	'psf:Feature',
							attribute	=
							{
								key		=	'name',
								value	=	'ns0000:SecretNumberingColor'
							},
							children	=	standard_option_generator(tbl['secretnumbering_color'], secretnumbering_color_element_table)
						}
					)
			end

			return result_tbl
		end
	},
	-- SecretNumbering StartNumber
	secretnumbering_number	=
	{
		tag		=	'psf:ParameterDef',
		attribute	=
			{
				key		=	'name',
				value	=	'ns0000:JobSecretNumberingStartNumber'
			},
		children	=	function(tbl)
							return paramdef_generator('integer', tbl['secretnumbering_number'], 'reg', 1)
						end
	},
-- [08 PrintQuality]
	-- 01 Color/Black and White
	colorbw	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'psk:PageOutputColor'
		},
		children	=	function(tbl)
			return standard_option_generator(
				tbl['colorbw'],
				OrderedHash(
					{ 'color',			'psk:Color' },
					{ 'blackandwhite',	'psk:Monochrome' }
				) )
		end
	},
	-- 02 Print Quality Preset
	printqualitypreset	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:JobPrintQualityPreset'
		},
		children	=	function(tbl)
			return standard_option_generator(
				tbl['printqualitypreset'],
				OrderedHash(
					{ 'genericdocument',	'ns0000:GenericDocument'},
					{ 'photographic',		'ns0000:Photographic'},
					{ 'dtp',				'ns0000:DTP'},
					{ 'pop',				'ns0000:POP'},
					{ 'cad',				'ns0000:CAD'},
					{ 'automatic',			'ns0000:Auto'},
					{ 'manual',				'ns0000:Manual'}
				) )
		end
	},
	-- 02-01 Print Quality Adjustment
	printqualityadjustment	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:JobPrintQualityAdjustment'
		},
		children	=	function(tbl)
			return standard_option_generator(
				tbl['printqualityadjustment'],
				OrderedHash(
					{ 'standard',			'ns0000:Standard'},
					{ 'advanced',			'ns0000:Advanced'}
				) )
		end
	},
	-- 03 Color Profile
	colorprofile_setting	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:JobColorProfileSelectObject'
		},
		children	=	function(tbl)
			return standard_option_generator(
				tbl['colorprofile_setting'],
				OrderedHash(
					{ 'setallobject',		'ns0000:SetAllObject'},
					{ 'seteachobject',		'ns0000:SetEachObject'}
				) )
		end
	},
	-- 03-01 Color Profile :: ALL
	colorprofile	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:PageColorProfile'
		},
		children	=	function(tbl)
			return standard_option_generator(
				tbl['colorprofile'],
				OrderedHash(
					{ 'automatic',				'ns0000:Auto'},
					{ 'automaticbright',		'ns0000:AutomaticBright'},
					{ 'automaticdeep',			'ns0000:AutomaticDeep'},
					{ 'text',					'ns0000:Text'},
					{ 'textbright',				'ns0000:TextBright'},
					{ 'textdeep',				'ns0000:TextDeep'},
					{ 'photographic',			'ns0000:Photographic'},
					{ 'photographicbright',		'ns0000:PhotographicBright'},
					{ 'photographicdeep',		'ns0000:PhotographicDeep'},
					{ 'presentation',			'ns0000:Presentation'},
					{ 'presentationbright',		'ns0000:PresentationBright'},
					{ 'presentationdeep',		'ns0000:PresentationDeep'},
					{ 'cad',					'ns0000:AutoCADDocument'},
					{ 'solidcolor',				'ns0000:SolidColor'},
					{ 'pop',					'ns0000:POP'},
					{ 'usersetting',			'ns0000:PSUser'},
					{ 'clpsimulation7',			'ns0000:CLPSimulation7'},
					{ 'clpsimulation4',			'ns0000:CLPSimulation4'},
					{ 'clpsimulation1',			'ns0000:CLPSimulation1'},
					{ 'clpsimulation6',			'ns0000:CLPSimulation6'},
					{ 'clpsimulation3',			'ns0000:CLPSimulation3'},
					{ 'clpsimulation5',			'ns0000:CLPSimulation5'},
					{ 'clpsimulation9',			'ns0000:CLPSimulation9'},
					{ 'clpsimulation2',			'ns0000:CLPSimulation2'},
					{ 'clpsimulation8',			'ns0000:CLPSimulation8'},
					{ 'custom1photograph',		'ns0000:Custom1Photograph'},
					{ 'custom1business',		'ns0000:Custom1Business'},
					{ 'custom1colorimetric',	'ns0000:Custom1Colorimetric'},
					{ 'custom2photograph',		'ns0000:Custom2Photograph'},
					{ 'custom2business',		'ns0000:Custom2Business'},
					{ 'custom2colorimetric',	'ns0000:Custom2Colorimetric'},
					{ 'off',					'ns0000:None'},
					{ 'manual',					'ns0000:Manual'},
					{ 'useicm',					'ns0000:UseICM'},
					{ 'automaticgray',			'ns0000:AutomaticGray'},
					{ 'coloruniversaldesign',	'ns0000:ColorUniversalDesign'}
				) )
		end
	},
	pcp_colorprofile	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:PageColorProfile'
		},
		children	=	function(tbl)
			local element_table = OrderedHash(
				{ 'automatic',				'ns0000:Auto'},
				{ 'automaticbright',		'ns0000:AutomaticBright'},
				{ 'automaticdeep',			'ns0000:AutomaticDeep'},
				{ 'text',					'ns0000:Text'},
				{ 'textbright',				'ns0000:TextBright'},
				{ 'textdeep',				'ns0000:TextDeep'},
				{ 'photographic',			'ns0000:Photographic'},
				{ 'photographicbright',		'ns0000:PhotographicBright'},
				{ 'photographicdeep',		'ns0000:PhotographicDeep'},
				{ 'presentation',			'ns0000:Presentation'},
				{ 'presentationbright',		'ns0000:PresentationBright'},
				{ 'presentationdeep',		'ns0000:PresentationDeep'},
				{ 'cad',					'ns0000:AutoCADDocument'},
				{ 'solidcolor',				'ns0000:SolidColor'},
				{ 'pop',					'ns0000:POP'},
				{ 'usersetting',			'ns0000:PSUser'},
				{ 'clpsimulation7',			'ns0000:CLPSimulation7'},
				{ 'clpsimulation4',			'ns0000:CLPSimulation4'},
				{ 'clpsimulation1',			'ns0000:CLPSimulation1'},
				{ 'clpsimulation6',			'ns0000:CLPSimulation6'},
				{ 'clpsimulation3',			'ns0000:CLPSimulation3'},
				{ 'clpsimulation5',			'ns0000:CLPSimulation5'},
				{ 'clpsimulation9',			'ns0000:CLPSimulation9'},
				{ 'clpsimulation2',			'ns0000:CLPSimulation2'},
				{ 'custom1photograph',		'ns0000:Custom1Photograph'},
				{ 'custom1business',		'ns0000:Custom1Business'},
				{ 'custom1colorimetric',	'ns0000:Custom1Colorimetric'},
				{ 'custom2photograph',		'ns0000:Custom2Photograph'},
				{ 'custom2business',		'ns0000:Custom2Business'},
				{ 'custom2colorimetric',	'ns0000:Custom2Colorimetric'},
				{ 'off',					'ns0000:None'},
				{ 'manual',					'ns0000:Manual'},
				{ 'useicm',					'ns0000:UseICM'},
				{ 'automaticgray',			'ns0000:AutomaticGray'},
				{ 'coloruniversaldesign',	'ns0000:ColorUniversalDesign'}
			)

			local result_tbl = standard_option_generator(tbl.pcp_colorprofile,element_table)

			-- AddProfile
			for i,v in pairs( tbl.pcp_colorprofile ) do
				if v.name and not element_table[ v.name ] then
					result_tbl[ #result_tbl + 1 ]	=
					{
						tag			=	'psf:Option',
						attribute	=
						{
							key		=	'name',
							value	=	'ns0000:AddProfile'
						},
						children	=
						{
							tag			= 	'psf:ScoredProperty',
							attribute	=
							{
								key		=	'name',
								value	=	'ns0000:ProfileName'
							},
							children	= 
							{
								tag 		=	'psf:Value',
								attribute	=
								{
									key 	=	'xsi:type',
									value	=	'xsd:string'
								},
								text		=	v.name
							}
						}
					}
				end
			end

			return result_tbl
		end
	},
	-- 03-02 Color Profile :: Object
	text_colorprofile	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:PageColorProfileForText'
		},
		children	=	function(tbl)
			return standard_option_generator(
				tbl['text_colorprofile'],
				OrderedHash(
					{ 'off',				'ns0000:None'},
					{ 'text',				'ns0000:Text'},
					{ 'textbright',			'ns0000:TextBright'},
					{ 'textdeep',			'ns0000:TextDeep'},
					{ 'presentation',		'ns0000:Presentation'},
					{ 'presentationbright',	'ns0000:PresentationBright'},
					{ 'presentationdeep',	'ns0000:PresentationDeep'},
					{ 'photographic',		'ns0000:Photographic'},
					{ 'photographicbright',	'ns0000:PhotographicBright'},
					{ 'photographicdeep',	'ns0000:PhotographicDeep'},
					{ 'pop',				'ns0000:POP'},
					{ 'clpsimulation7',		'ns0000:CLPSimulation7'},
					{ 'clpsimulation4',		'ns0000:CLPSimulation4'},
					{ 'clpsimulation1',		'ns0000:CLPSimulation1'},
					{ 'clpsimulation6',		'ns0000:CLPSimulation6'},
					{ 'clpsimulation3',		'ns0000:CLPSimulation3'},
					{ 'clpsimulation5',		'ns0000:CLPSimulation5'},
					{ 'clpsimulation9',		'ns0000:CLPSimulation9'},
					{ 'clpsimulation2',		'ns0000:CLPSimulation2'},
					{ 'clpsimulation8',		'ns0000:CLPSimulation8'},
					{ 'chinared',			'ns0000:CustomRed'}
				) )
		end
	},
	pcp_text_colorprofile	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:PageColorProfileForText'
		},
		children	=	function(tbl)
			local element_table = OrderedHash(
				{ 'off',				'ns0000:None'},
				{ 'text',				'ns0000:Text'},
				{ 'textbright',			'ns0000:TextBright'},
				{ 'textdeep',			'ns0000:TextDeep'},
				{ 'presentation',		'ns0000:Presentation'},
				{ 'presentationbright',	'ns0000:PresentationBright'},
				{ 'presentationdeep',	'ns0000:PresentationDeep'},
				{ 'photographic',		'ns0000:Photographic'},
				{ 'photographicbright',	'ns0000:PhotographicBright'},
				{ 'photographicdeep',	'ns0000:PhotographicDeep'},
				{ 'pop',				'ns0000:POP'},
				{ 'clpsimulation7',		'ns0000:CLPSimulation7'},
				{ 'clpsimulation4',		'ns0000:CLPSimulation4'},
				{ 'clpsimulation1',		'ns0000:CLPSimulation1'},
				{ 'clpsimulation6',		'ns0000:CLPSimulation6'},
				{ 'clpsimulation3',		'ns0000:CLPSimulation3'},
				{ 'clpsimulation5',		'ns0000:CLPSimulation5'},
				{ 'clpsimulation9',		'ns0000:CLPSimulation9'},
				{ 'clpsimulation2',		'ns0000:CLPSimulation2'},
				{ 'chinared',			'ns0000:CustomRed'}
			)

			local result_tbl = standard_option_generator(tbl['pcp_text_colorprofile'],element_table)

			-- AddProfile
			for i,v in pairs( tbl.pcp_text_colorprofile ) do
				if v.name and not element_table[ v.name ] then
					result_tbl[ #result_tbl + 1 ]	=
					{
						tag			=	'psf:Option',
						attribute	=
						{
							key		=	'name',
							value	=	'ns0000:AddProfile'
						},
						children	=
						{
							tag			= 	'psf:ScoredProperty',
							attribute	=
							{
								key		=	'name',
								value	=	'ns0000:ProfileName'
							},
							children	= 
							{
								tag 		=	'psf:Value',
								attribute	=
								{
									key 	=	'xsi:type',
									value	=	'xsd:string'
								},
								text		=	v.name
							}
						}
					}
				end
			end

			return result_tbl
		end
	},
	photo_colorprofile	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:PageColorProfileForPhoto'
		},
		children	=	function(tbl)
			return standard_option_generator(
				tbl['photo_colorprofile'],
				OrderedHash(
					{ 'off',				'ns0000:None'},
					{ 'text',				'ns0000:Text'},
					{ 'textbright',			'ns0000:TextBright'},
					{ 'textdeep',			'ns0000:TextDeep'},
					{ 'presentation',		'ns0000:Presentation'},
					{ 'presentationbright',	'ns0000:PresentationBright'},
					{ 'presentationdeep',	'ns0000:PresentationDeep'},
					{ 'photographic',		'ns0000:Photographic'},
					{ 'photographicbright',	'ns0000:PhotographicBright'},
					{ 'photographicdeep',	'ns0000:PhotographicDeep'},
					{ 'pop',				'ns0000:POP'},
					{ 'clpsimulation7',		'ns0000:CLPSimulation7'},
					{ 'clpsimulation4',		'ns0000:CLPSimulation4'},
					{ 'clpsimulation1',		'ns0000:CLPSimulation1'},
					{ 'clpsimulation6',		'ns0000:CLPSimulation6'},
					{ 'clpsimulation3',		'ns0000:CLPSimulation3'},
					{ 'clpsimulation5',		'ns0000:CLPSimulation5'},
					{ 'clpsimulation9',		'ns0000:CLPSimulation9'},
					{ 'clpsimulation2',		'ns0000:CLPSimulation2'},
					{ 'clpsimulation8',		'ns0000:CLPSimulation8'},
					{ 'chinared',			'ns0000:CustomRed'}
				) )
		end
	},
	pcp_photo_colorprofile	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:PageColorProfileForPhoto'
		},
		children	=	function(tbl)
			local element_table = OrderedHash(
				{ 'off',				'ns0000:None'},
				{ 'text',				'ns0000:Text'},
				{ 'textbright',			'ns0000:TextBright'},
				{ 'textdeep',			'ns0000:TextDeep'},
				{ 'presentation',		'ns0000:Presentation'},
				{ 'presentationbright',	'ns0000:PresentationBright'},
				{ 'presentationdeep',	'ns0000:PresentationDeep'},
				{ 'photographic',		'ns0000:Photographic'},
				{ 'photographicbright',	'ns0000:PhotographicBright'},
				{ 'photographicdeep',	'ns0000:PhotographicDeep'},
				{ 'pop',				'ns0000:POP'},
				{ 'clpsimulation7',		'ns0000:CLPSimulation7'},
				{ 'clpsimulation4',		'ns0000:CLPSimulation4'},
				{ 'clpsimulation1',		'ns0000:CLPSimulation1'},
				{ 'clpsimulation6',		'ns0000:CLPSimulation6'},
				{ 'clpsimulation3',		'ns0000:CLPSimulation3'},
				{ 'clpsimulation5',		'ns0000:CLPSimulation5'},
				{ 'clpsimulation9',		'ns0000:CLPSimulation9'},
				{ 'clpsimulation2',		'ns0000:CLPSimulation2'},
				{ 'chinared',			'ns0000:CustomRed'}
			)

			local result_tbl = standard_option_generator(tbl['pcp_photo_colorprofile'],element_table)

			-- AddProfile
			for i,v in pairs( tbl.pcp_photo_colorprofile ) do
				if v.name and not element_table[ v.name ] then
					result_tbl[ #result_tbl + 1 ]	=
					{
						tag			=	'psf:Option',
						attribute	=
						{
							key		=	'name',
							value	=	'ns0000:AddProfile'
						},
						children	=
						{
							tag			= 	'psf:ScoredProperty',
							attribute	=
							{
								key		=	'name',
								value	=	'ns0000:ProfileName'
							},
							children	= 
							{
								tag 		=	'psf:Value',
								attribute	=
								{
									key 	=	'xsi:type',
									value	=	'xsd:string'
								},
								text		=	v.name
							}
						}
					}
				end
			end

			return result_tbl
		end
	},
	graphics_colorprofile	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:PageColorProfileForGraphics'
		},
		children	=	function(tbl)
			return standard_option_generator(
				tbl['graphics_colorprofile'],
				OrderedHash(
					{ 'off',				'ns0000:None'},
					{ 'text',				'ns0000:Text'},
					{ 'textbright',			'ns0000:TextBright'},
					{ 'textdeep',			'ns0000:TextDeep'},
					{ 'presentation',		'ns0000:Presentation'},
					{ 'presentationbright',	'ns0000:PresentationBright'},
					{ 'presentationdeep',	'ns0000:PresentationDeep'},
					{ 'photographic',		'ns0000:Photographic'},
					{ 'photographicbright',	'ns0000:PhotographicBright'},
					{ 'photographicdeep',	'ns0000:PhotographicDeep'},
					{ 'pop',				'ns0000:POP'},
					{ 'clpsimulation7',		'ns0000:CLPSimulation7'},
					{ 'clpsimulation4',		'ns0000:CLPSimulation4'},
					{ 'clpsimulation1',		'ns0000:CLPSimulation1'},
					{ 'clpsimulation6',		'ns0000:CLPSimulation6'},
					{ 'clpsimulation3',		'ns0000:CLPSimulation3'},
					{ 'clpsimulation5',		'ns0000:CLPSimulation5'},
					{ 'clpsimulation9',		'ns0000:CLPSimulation9'},
					{ 'clpsimulation2',		'ns0000:CLPSimulation2'},
					{ 'clpsimulation8',		'ns0000:CLPSimulation8'},
					{ 'chinared',			'ns0000:CustomRed'}
				) )
		end
	},
	pcp_graphics_colorprofile	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:PageColorProfileForGraphics'
		},
		children	=	function(tbl)
			local element_table = OrderedHash(
				{ 'off',				'ns0000:None'},
				{ 'text',				'ns0000:Text'},
				{ 'textbright',			'ns0000:TextBright'},
				{ 'textdeep',			'ns0000:TextDeep'},
				{ 'presentation',		'ns0000:Presentation'},
				{ 'presentationbright',	'ns0000:PresentationBright'},
				{ 'presentationdeep',	'ns0000:PresentationDeep'},
				{ 'photographic',		'ns0000:Photographic'},
				{ 'photographicbright',	'ns0000:PhotographicBright'},
				{ 'photographicdeep',	'ns0000:PhotographicDeep'},
				{ 'pop',				'ns0000:POP'},
				{ 'clpsimulation7',		'ns0000:CLPSimulation7'},
				{ 'clpsimulation4',		'ns0000:CLPSimulation4'},
				{ 'clpsimulation1',		'ns0000:CLPSimulation1'},
				{ 'clpsimulation6',		'ns0000:CLPSimulation6'},
				{ 'clpsimulation3',		'ns0000:CLPSimulation3'},
				{ 'clpsimulation5',		'ns0000:CLPSimulation5'},
				{ 'clpsimulation9',		'ns0000:CLPSimulation9'},
				{ 'clpsimulation2',		'ns0000:CLPSimulation2'},
				{ 'chinared',			'ns0000:CustomRed'}
			)

			local result_tbl = standard_option_generator(tbl['pcp_graphics_colorprofile'],element_table)

			-- AddProfile
			for i,v in pairs( tbl.pcp_graphics_colorprofile ) do
				if v.name and not element_table[ v.name ] then
					result_tbl[ #result_tbl + 1 ]	=
					{
						tag			=	'psf:Option',
						attribute	=
						{
							key		=	'name',
							value	=	'ns0000:AddProfile'
						},
						children	=
						{
							tag			= 	'psf:ScoredProperty',
							attribute	=
							{
								key		=	'name',
								value	=	'ns0000:ProfileName'
							},
							children	= 
							{
								tag 		=	'psf:Value',
								attribute	=
								{
									key 	=	'xsi:type',
									value	=	'xsd:string'
								},
								text		=	v.name
							}
						}
					}
				end
			end

			return result_tbl
		end
	},
	-- 03-03 ICM Intent
	icmintent 	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'psk:PageICMRenderingIntent'
		},
		children	=	function(tbl)
			return standard_option_generator(
				tbl['icmintent'],
				OrderedHash(
					{ 'saturation',				'psk:BusinessGraphics'},
					{ 'perceptual',				'psk:Photographs'},
					{ 'relativecolormetric',	'psk:RelativeColorimetric'},
					{ 'absolutecolormetric',	'psk:AbsoluteColorimetric'}
				) )
		end
	},
	pcp_icmintent 	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'psk:PageICMRenderingIntent'
		},
		children	=	function(tbl)
			return standard_option_generator(
				tbl['pcp_icmintent'],
				OrderedHash(
					{ 'saturation',				'psk:BusinessGraphics'},
					{ 'perceptual',				'psk:Photographs'},
					{ 'relativecolormetric',	'psk:RelativeColorimetric'},
					{ 'absolutecolormetric',	'psk:AbsoluteColorimetric'}
				) )
		end
	},
	-- 03-04 CMYK Simulation Profile
	cmyksimulationprofile 	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:DocumentRcmyksimulation'
		},
		children	=	function(tbl)
			return standard_option_generator(
				tbl['cmyksimulationprofile'],
				OrderedHash(
					{ 'off',			'ns0000:Off'},
					{ 'usoffsetprint',	'ns0000:USOffsetPrint'},
					{ 'euroscale',		'ns0000:Euroscale'},
					{ 'japancolor',		'ns0000:JapanColor'},
					{ 'jmpa',			'ns0000:JPMA'},
					{ 'pantone',		'ns0000:PANTONE'},
					{ 'trans01',		'ns0000:Trans01'},
					{ 'custom1',		'ns0000:Custom1'},
					{ 'custom2',		'ns0000:Custom2'},
					{ 'custom3',		'ns0000:Custom3'}
				) )
		end
	},
	-- 03-05 Color Profile Distribution(PS)
	colorprofile_distribution 	=
	{
		{
			tag			= 	'psf:Feature',
			attribute	=
			{
				key		=	'name',
				value	=	'ns0000:PageColorProfileDistribution'
			},
			children	=	function(tbl)
				return standard_option_generator(
					tbl['colorprofile_distribution'],
					OrderedHash(
						{ 'host',				'ns0000:Host'},
						{ 'printer',			'ns0000:Printer'},
						{ 'printercalibration',	'ns0000:PrinterCalibration'}
					) )
			end
		},
		{
			tag			= 	'psf:Feature',
			attribute	=
			{
				key		=	'name',
				value	=	'psk:PageColorManagement'
			},
			children	=	function(tbl)
				local element_table	= OrderedHash(
					{ 'off',				'psk:None'},
					{ 'host',				'psk:System'},
					{ 'printer',			'psk:Driver'},
					{ 'printercalibration',	'psk:Device'}
				)
				local item_table = tbl['colorprofile_distribution']
				table.insert(item_table, {name = 'off', condition = {}})
				return standard_option_generator(item_table, element_table)
			end
		},
	},
	-- 03-05 Color Profile Distribution(RPCS)
	pcp_distribution 	=
	{
		{
			tag			= 	'psf:Feature',
			attribute	=
			{
				key		=	'name',
				value	=	'psk:PageColorManagement'
			},
			children	=	function(tbl)

				local colorprofile_table = nil
				if( nil ~= tbl['colorprofile'] ) then
					colorprofile_table = tbl['colorprofile']
				else
					colorprofile_table = tbl['pcp_colorprofile']
				end
				if( colorprofile_table ) then

					local has_useicm		= false
					local has_except_useicm	= false

					for key , value in pairs( colorprofile_table ) do
						if( value.name == 'useicm' ) then
							has_useicm = true
						else
							has_except_useicm = true
						end
					end

					local item_table = tbl['pcp_distribution']

					--Useicm + Except useicm
					if( has_useicm and has_except_useicm ) then
						local element_table	= OrderedHash(
							{ 'pcpdistributionnone',	'psk:None'},
							{ 'host',					'psk:System'},
							{ 'printer',				'psk:Driver'},
							{ 'printercalibration',		'psk:Device'},
							{ 'printerdriver',			'psk:Driver'}
						)

						table.insert( item_table, { name = 'pcpdistributionnone', condition = {} } )
						return standard_option_generator(item_table, element_table)
					--Only useicm
					elseif( has_useicm ) then
						local element_table	= OrderedHash(
							{ 'host',				'psk:System'},
							{ 'printer',			'psk:Driver'},
							{ 'printercalibration',	'psk:Device'},
							{ 'printerdriver',		'psk:Driver'}
						)
						return standard_option_generator(item_table, element_table)
					--Only except useicm
					elseif( has_except_useicm ) then
						local element_table	= OrderedHash(
							{ 'pcpdistributionnone',	'psk:None'}
						)
						table.insert( item_table, { name = 'pcpdistributionnone', condition = {} } )
						return standard_option_generator(item_table, element_table)
						
					--nothing
					else

					end
				end
			end
		},
		{
			tag			= 	'psf:Feature',
			attribute	=
			{
				key		=	'name',
				value	=	'ns0000:PageColorProfileDistribution'
			},
			children	=	function(tbl)
				return standard_option_generator(
					tbl['pcp_distribution'],
					OrderedHash(
						{ 'host',				'ns0000:Host'},
						{ 'printer',			'ns0000:Printer'},
						{ 'printercalibration',	'ns0000:PrinterCalibration'},
						{ 'printerdriver',		'ns0000:PrinterDriver'}
					) )
			end
		}
	},
	-- 04 Gray Reproduction
	grayreproduction	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:JobGrayReproduction'
		},
		children	=	function(tbl)
			return standard_option_generator(
				tbl['grayreproduction'],
				OrderedHash(
					{ 'kblack',						'ns0000:BlackForAllObjects'},
					{ 'cmyk',						'ns0000:CMYK'},
					{ 'kblackfortext',				'ns0000:BlackForTextOnly'},
					{ 'kblackperdot',				'ns0000:BlackForGrayPixelsOnly'},
					{ 'kblackperpage',				'ns0000:BlackForGrayPageOnly'},
					{ 'kblackonlypureblack',		'ns0000:BlackOnlyPureBlack'},
					{ 'kblackonlypureblackfortext',	'ns0000:BlackOnlyPureBlackForText'},
					{ 'kblackbyprofile',			'ns0000:BlackByProfile'},
					{ 'kblackfortextandgraphic',	'ns0000:BlackForTextAndGraphic'},
					{ 'kblackforallobject',			'ns0000:BlackAndUCRForAllObjects'},
					{ 'auto',						'ns0000:Auto'}
				) )
		end
	},
	-- 05 Dithering
	dithering_setting	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:JobDitheringSelectObject'
		},
		children	=	function(tbl)
			return standard_option_generator(
				tbl['dithering_setting'],
				OrderedHash(
					{ 'setallobject',		'ns0000:SetAllObject'},
					{ 'seteachobject',		'ns0000:SetEachObject'}
				) )
		end
	},
	dithering	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:JobDither'
		},
		children	=	function(tbl)
			return standard_option_generator(
				tbl['dithering'],
				OrderedHash(
					{ 'automatic',								'ns0000:Auto'},
					{ 'photographic',							'ns0000:Photographic'},
					{ 'text',									'ns0000:Text'},
					{ 'textline',								'ns0000:Line'},
					{ 'cad',									'ns0000:CAD'},
					{ 'usersetting',							'ns0000:User'},
					{ 'multilevel',								'ns0000:Multi-Level'},
					{ 'automaticuseerrordiffusionimage',		'ns0000:Auto_ErrorDiffusionImageOnly'},
					{ 'photographicuseerrordiffusionimage',		'ns0000:Photographic_ErrorDiffusionImageOnly'},
					{ 'textuseerrordiffusionimage',				'ns0000:Text_ErrorDiffusionImageOnly'},
					{ 'textlineuseerrordiffusionimage',			'ns0000:Line_ErrorDiffusionImageOnly'},
					{ 'automaticuseerrordiffusionall',			'ns0000:Auto_ErrorDiffusionAllObject'},
					{ 'photographicuseerrordiffusionall',		'ns0000:Photographic_ErrorDiffusionAllObject'},
					{ 'textuseerrordiffusionall',				'ns0000:Text_ErrorDiffusionAllObject'},
					{ 'textlineuseerrordiffusionall',			'ns0000:Line_ErrorDiffusionAllObject'},
					{ 'manual',									'ns0000:Manual'}
				) )
		end
	},
	text_dithering	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:JobDitherForText'
		},
		children	=	function(tbl)
			return standard_option_generator(
				tbl['text_dithering'],
				OrderedHash(
					{ 'text',				'ns0000:Text'},
					{ 'photographic',		'ns0000:Photographic'},
					{ 'textline',			'ns0000:Line'},
					{ 'multilevel',			'ns0000:Multi-Level'}
				) )
		end
	},
	photo_dithering	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:JobDitherForPhoto'
		},
		children	=	function(tbl)
			return standard_option_generator(
				tbl['photo_dithering'],
				OrderedHash(
					{ 'text',				'ns0000:Text'},
					{ 'photographic',		'ns0000:Photographic'},
					{ 'textline',			'ns0000:Line'}
				) )
		end
	},
	graphics_dithering	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:JobDitherForGraphics'
		},
		children	=	function(tbl)
			return standard_option_generator(
				tbl['graphics_dithering'],
				OrderedHash(
					{ 'text',				'ns0000:Text'},
					{ 'photographic',		'ns0000:Photographic'},
					{ 'textline',			'ns0000:Line'},
					{ 'multilevel',			'ns0000:Multi-Level'}
				) )
		end
	},
	-- Enlarge Dither Brush Pattern
	dithering_enlargebrushpattern 	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:JobEnlargeDitherBrushPattern'
		},
		children	=	function(tbl)
			return standard_option_generator(
				tbl['dithering_enlargebrushpattern'],
				OrderedHash(
					{ 'off',	'ns0000:Off'},
					{ 'on',		'ns0000:On'}
				) )
		end
	},
	-- Shift Dither Brush Position
	dithering_shiftbrushpattern 	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:JobShiftDitherBrushPosition'
		},
		children	=	function(tbl)
			return standard_option_generator(
				tbl['dithering_shiftbrushpattern'],
				OrderedHash(
					{ 'off',	'ns0000:Off'},
					{ 'on',		'ns0000:On'}
				) )
		end
	},
	-- 06 Balance Adjustment
	balanceadjustment	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:JobBalanceAdjustment'
		},
		children	=	function(tbl)
			local element_table	=OrderedHash(
				{ 'setallobject',	{value = 'ns0000:SetAllObject', scoredproperty = {}}},
				{ 'seteachobject',	{value = 'ns0000:SetEachObject', scoredproperty = {}}}
			)
			--Set ScoredProperty Info--
			---AllBrightness--
			if(nil ~= tbl['all_brightness'])then
				table.insert(element_table['setallobject']['scoredproperty'],{name = 'ns0000:BrightnessAll', def = 'ns0000:JobPrintQualityAllBrightness'})
			end
			---AllContrast--
			if(nil ~= tbl['all_contrast'])then
				table.insert(element_table['setallobject']['scoredproperty'],{name = 'ns0000:ContrastAll', def = 'ns0000:JobPrintQualityAllContrast'})
			end
			---AllSaturation--
			if(nil ~= tbl['all_saturation'])then
				table.insert(element_table['setallobject']['scoredproperty'],{name = 'ns0000:SaturationAll', def = 'ns0000:JobPrintQualityAllSaturation'})
			end
			
			---AllColorBalanceRed--
			if(nil ~= tbl['all_colorbalance_red'])then
				table.insert(element_table['setallobject']['scoredproperty'],{name = 'ns0000:ColorBalanceRAll', def = 'ns0000:JobPrintQualityAllColorBalanceR'})
			end
			---AllColorBalanceGreen--
			if(nil ~= tbl['all_colorbalance_green'])then
				table.insert(element_table['setallobject']['scoredproperty'],{name = 'ns0000:ColorBalanceGAll', def = 'ns0000:JobPrintQualityAllColorBalanceG'})
			end
			---AllColorBalanceBlue--
			if(nil ~= tbl['all_colorbalance_blue'])then
				table.insert(element_table['setallobject']['scoredproperty'],{name = 'ns0000:ColorBalanceBAll', def = 'ns0000:JobPrintQualityAllColorBalanceB'})
			end
			
			---TextBrightness--
			if(nil ~= tbl['text_brightness'])then
				table.insert(element_table['seteachobject']['scoredproperty'],{name = 'ns0000:BrightnessText', def = 'ns0000:JobPrintQualityTextBrightness'})
			end
			---TextContrast--
			if(nil ~= tbl['text_contrast'])then
				table.insert(element_table['seteachobject']['scoredproperty'],{name = 'ns0000:ContrastText', def = 'ns0000:JobPrintQualityTextContrast'})
			end
			---TextSaturation--
			if(nil ~= tbl['text_saturation'])then
				table.insert(element_table['seteachobject']['scoredproperty'],{name = 'ns0000:SaturationText', def = 'ns0000:JobPrintQualityTextSaturation'})
			end
			
			---TextColorBalanceRed--
			if(nil ~= tbl['text_colorbalance_red'])then
				table.insert(element_table['seteachobject']['scoredproperty'],{name = 'ns0000:ColorBalanceRText', def = 'ns0000:JobPrintQualityTextColorBalanceR'})
			end
			---TextColorBalanceGreen--
			if(nil ~= tbl['text_colorbalance_green'])then
				table.insert(element_table['seteachobject']['scoredproperty'],{name = 'ns0000:ColorBalanceGText', def = 'ns0000:JobPrintQualityTextColorBalanceG'})
			end
			---TextColorBalanceBlue--
			if(nil ~= tbl['text_colorbalance_blue'])then
				table.insert(element_table['seteachobject']['scoredproperty'],{name = 'ns0000:ColorBalanceBText', def = 'ns0000:JobPrintQualityTextColorBalanceB'})
			end

			---PhotoBrightness--
			if(nil ~= tbl['photo_brightness'])then
				table.insert(element_table['seteachobject']['scoredproperty'],{name = 'ns0000:BrightnessPhoto', def = 'ns0000:JobPrintQualityPhotoBrightness'})
			end
			---PhotoContrast--
			if(nil ~= tbl['photo_contrast'])then
				table.insert(element_table['seteachobject']['scoredproperty'],{name = 'ns0000:ContrastPhoto', def = 'ns0000:JobPrintQualityPhotoContrast'})
			end
			---PhotoSaturation--
			if(nil ~= tbl['photo_saturation'])then
				table.insert(element_table['seteachobject']['scoredproperty'],{name = 'ns0000:SaturationPhoto', def = 'ns0000:JobPrintQualityPhotoSaturation'})
			end
			
			---PhotoColorBalanceRed--
			if(nil ~= tbl['photo_colorbalance_red'])then
				table.insert(element_table['seteachobject']['scoredproperty'],{name = 'ns0000:ColorBalanceRPhoto', def = 'ns0000:JobPrintQualityPhotoColorBalanceR'})
			end
			---PhotoColorBalanceGreen--
			if(nil ~= tbl['photo_colorbalance_green'])then
				table.insert(element_table['seteachobject']['scoredproperty'],{name = 'ns0000:ColorBalanceGPhoto', def = 'ns0000:JobPrintQualityPhotoColorBalanceG'})
			end
			---PhotoColorBalanceBlue--
			if(nil ~= tbl['photo_colorbalance_blue'])then
				table.insert(element_table['seteachobject']['scoredproperty'],{name = 'ns0000:ColorBalanceBPhoto', def = 'ns0000:JobPrintQualityPhotoColorBalanceB'})
			end
			
			
			---GraphicsBrightness--
			if(nil ~= tbl['graphics_brightness'])then
				table.insert(element_table['seteachobject']['scoredproperty'],{name = 'ns0000:BrightnessGraphics', def = 'ns0000:JobPrintQualityGraphicsBrightness'})
			end
			---GraphicsContrast--
			if(nil ~= tbl['graphics_contrast'])then
				table.insert(element_table['seteachobject']['scoredproperty'],{name = 'ns0000:ContrastGraphics', def = 'ns0000:JobPrintQualityGraphicsContrast'})
			end
			---GraphicsSaturation--
			if(nil ~= tbl['graphics_saturation'])then
				table.insert(element_table['seteachobject']['scoredproperty'],{name = 'ns0000:SaturationGraphics', def = 'ns0000:JobPrintQualityGraphicsSaturation'})
			end
			
			---GraphicsColorBalanceRed--
			if(nil ~= tbl['graphics_colorbalance_red'])then
				table.insert(element_table['seteachobject']['scoredproperty'],{name = 'ns0000:ColorBalanceRGraphics', def = 'ns0000:JobPrintQualityGraphicsColorBalanceR'})
			end
			---GraphicsColorBalanceGreen--
			if(nil ~= tbl['graphics_colorbalance_green'])then
				table.insert(element_table['seteachobject']['scoredproperty'],{name = 'ns0000:ColorBalanceGGraphics', def = 'ns0000:JobPrintQualityGraphicsColorBalanceG'})
			end
			---GraphicsColorBalanceBlue--
			if(nil ~= tbl['graphics_colorbalance_blue'])then
				table.insert(element_table['seteachobject']['scoredproperty'],{name = 'ns0000:ColorBalanceBGraphics', def = 'ns0000:JobPrintQualityGraphicsColorBalanceB'})
			end

			return standard_option_generator(tbl['balanceadjustment'], element_table)
		end
	},
	-- AllBrightness
	all_brightness	=
	{
		tag		=	'psf:ParameterDef',
		attribute	=
			{
				key		=	'name',
				value	=	'ns0000:JobPrintQualityAllBrightness'
			},
		children	=	function(tbl)
							
							--default
							local default_val = tonumber(tbl['all_brightness']['default'])
							if(nil ~= default_val)then
								tbl['all_brightness']['default'] = tostring(default_val)
							else
								tbl['all_brightness']['default'] = '0'
							end
							
							--max
							local max_val = tonumber(tbl['all_brightness']['max'])
							if(nil ~= max_val)then
								tbl['all_brightness']['max'] = tostring(max_val)
							else
								tbl['all_brightness']['max'] = '0'
							end
							
							--min
							local min_val = tonumber(tbl['all_brightness']['min'])
							if(nil ~= min_val)then
								tbl['all_brightness']['min'] = tostring(min_val)
							else
								tbl['all_brightness']['min'] = '0'
							end
							
							return paramdef_generator('integer', tbl['all_brightness'], 'colorbalance', 1)
						end
	},
	---AllContrast
	all_contrast	=
	{
		tag		=	'psf:ParameterDef',
		attribute	=
			{
				key		=	'name',
				value	=	'ns0000:JobPrintQualityAllContrast'
			},
		children	=	function(tbl)
							
							--default
							local default_val = tonumber(tbl['all_contrast']['default'])
							if(nil ~= default_val)then
								tbl['all_contrast']['default'] = tostring(default_val)
							else
								tbl['all_contrast']['default'] = '0'
							end
							
							--max
							local max_val = tonumber(tbl['all_contrast']['max'])
							if(nil ~= max_val)then
								tbl['all_contrast']['max'] = tostring(max_val)
							else
								tbl['all_contrast']['max'] = '0'
							end
							
							--min
							local min_val = tonumber(tbl['all_contrast']['min'])
							if(nil ~= min_val)then
								tbl['all_contrast']['min'] = tostring(min_val)
							else
								tbl['all_contrast']['min'] = '0'
							end
							
							return paramdef_generator('integer', tbl['all_contrast'], 'colorbalance', 1)
						end
	},
	-- AllSaturation
	all_saturation	=
	{
		tag		=	'psf:ParameterDef',
		attribute	=
			{
				key		=	'name',
				value	=	'ns0000:JobPrintQualityAllSaturation'
			},
		children	=	function(tbl)
							
							--default
							local default_val = tonumber(tbl['all_saturation']['default'])
							if(nil ~= default_val)then
								tbl['all_saturation']['default'] = tostring(default_val)
							else
								tbl['all_saturation']['default'] = '0'
							end
							
							--max
							local max_val = tonumber(tbl['all_saturation']['max'])
							if(nil ~= max_val)then
								tbl['all_saturation']['max'] = tostring(max_val)
							else
								tbl['all_saturation']['max'] = '0'
							end
							
							--min
							local min_val = tonumber(tbl['all_saturation']['min'])
							if(nil ~= min_val)then
								tbl['all_saturation']['min'] = tostring(min_val)
							else
								tbl['all_saturation']['min'] = '0'
							end
							
							return paramdef_generator('integer', tbl['all_saturation'], 'colorbalance', 1)
						end
	},
	-- AllColorBalanceRed
	all_colorbalance_red	=
	{
		tag		=	'psf:ParameterDef',
		attribute	=
			{
				key		=	'name',
				value	=	'ns0000:JobPrintQualityAllColorBalanceR'
			},
		children	=	function(tbl)
							
							--default
							local default_val = tonumber(tbl['all_colorbalance_red']['default'])
							if(nil ~= default_val)then
								tbl['all_colorbalance_red']['default'] = tostring(default_val)
							else
								tbl['all_colorbalance_red']['default'] = '0'
							end
							
							--max
							local max_val = tonumber(tbl['all_colorbalance_red']['max'])
							if(nil ~= max_val)then
								tbl['all_colorbalance_red']['max'] = tostring(max_val)
							else
								tbl['all_colorbalance_red']['max'] = '0'
							end
							
							--min
							local min_val = tonumber(tbl['all_colorbalance_red']['min'])
							if(nil ~= min_val)then
								tbl['all_colorbalance_red']['min'] = tostring(min_val)
							else
								tbl['all_colorbalance_red']['min'] = '0'
							end
							
							return paramdef_generator('integer', tbl['all_colorbalance_red'], 'colorbalance', 1)
						end
	},
	-- AllColorBalanceGreen
	all_colorbalance_green	=
	{
		tag		=	'psf:ParameterDef',
		attribute	=
			{
				key		=	'name',
				value	=	'ns0000:JobPrintQualityAllColorBalanceG'
			},
		children	=	function(tbl)
							
							--default
							local default_val = tonumber(tbl['all_colorbalance_green']['default'])
							if(nil ~= default_val)then
								tbl['all_colorbalance_green']['default'] = tostring(default_val)
							else
								tbl['all_colorbalance_green']['default'] = '0'
							end
							
							--max
							local max_val = tonumber(tbl['all_colorbalance_green']['max'])
							if(nil ~= max_val)then
								tbl['all_colorbalance_green']['max'] = tostring(max_val)
							else
								tbl['all_colorbalance_green']['max'] = '0'
							end
							
							--min
							local min_val = tonumber(tbl['all_colorbalance_green']['min'])
							if(nil ~= min_val)then
								tbl['all_colorbalance_green']['min'] = tostring(min_val)
							else
								tbl['all_colorbalance_green']['min'] = '0'
							end
							
							return paramdef_generator('integer', tbl['all_colorbalance_green'], 'colorbalance', 1)
						end
	},
	-- AllColorBalanceBlue
	all_colorbalance_blue	=
	{
		tag		=	'psf:ParameterDef',
		attribute	=
			{
				key		=	'name',
				value	=	'ns0000:JobPrintQualityAllColorBalanceB'
			},
		children	=	function(tbl)
							
							--default
							local default_val = tonumber(tbl['all_colorbalance_blue']['default'])
							if(nil ~= default_val)then
								tbl['all_colorbalance_blue']['default'] = tostring(default_val)
							else
								tbl['all_colorbalance_blue']['default'] = '0'
							end
							
							--max
							local max_val = tonumber(tbl['all_colorbalance_blue']['max'])
							if(nil ~= max_val)then
								tbl['all_colorbalance_blue']['max'] = tostring(max_val)
							else
								tbl['all_colorbalance_blue']['max'] = '0'
							end
							
							--min
							local min_val = tonumber(tbl['all_colorbalance_blue']['min'])
							if(nil ~= min_val)then
								tbl['all_colorbalance_blue']['min'] = tostring(min_val)
							else
								tbl['all_colorbalance_blue']['min'] = '0'
							end
							
							return paramdef_generator('integer', tbl['all_colorbalance_blue'], 'colorbalance', 1)
						end
	},
	-- TextBrightness
	text_brightness	=
	{
		tag		=	'psf:ParameterDef',
		attribute	=
			{
				key		=	'name',
				value	=	'ns0000:JobPrintQualityTextBrightness'
			},
		children	=	function(tbl)
							
							--default
							local default_val = tonumber(tbl['text_brightness']['default'])
							if(nil ~= default_val)then
								tbl['text_brightness']['default'] = tostring(default_val)
							else
								tbl['text_brightness']['default'] = '0'
							end
							
							--max
							local max_val = tonumber(tbl['text_brightness']['max'])
							if(nil ~= max_val)then
								tbl['text_brightness']['max'] = tostring(max_val)
							else
								tbl['text_brightness']['max'] = '0'
							end
							
							--min
							local min_val = tonumber(tbl['text_brightness']['min'])
							if(nil ~= min_val)then
								tbl['text_brightness']['min'] = tostring(min_val)
							else
								tbl['text_brightness']['min'] = '0'
							end
							
							return paramdef_generator('integer', tbl['text_brightness'], 'colorbalance', 1)
						end
	},
	---TextContrast
	text_contrast	=
	{
		tag		=	'psf:ParameterDef',
		attribute	=
			{
				key		=	'name',
				value	=	'ns0000:JobPrintQualityTextContrast'
			},
		children	=	function(tbl)
							
							--default
							local default_val = tonumber(tbl['text_contrast']['default'])
							if(nil ~= default_val)then
								tbl['text_contrast']['default'] = tostring(default_val)
							else
								tbl['text_contrast']['default'] = '0'
							end
							
							--max
							local max_val = tonumber(tbl['text_contrast']['max'])
							if(nil ~= max_val)then
								tbl['text_contrast']['max'] = tostring(max_val)
							else
								tbl['text_contrast']['max'] = '0'
							end
							
							--min
							local min_val = tonumber(tbl['text_contrast']['min'])
							if(nil ~= min_val)then
								tbl['text_contrast']['min'] = tostring(min_val)
							else
								tbl['text_contrast']['min'] = '0'
							end
							
							return paramdef_generator('integer', tbl['text_contrast'], 'colorbalance', 1)
						end
	},
	-- TextSaturation
	text_saturation	=
	{
		tag		=	'psf:ParameterDef',
		attribute	=
			{
				key		=	'name',
				value	=	'ns0000:JobPrintQualityTextSaturation'
			},
		children	=	function(tbl)
							
							--default
							local default_val = tonumber(tbl['text_saturation']['default'])
							if(nil ~= default_val)then
								tbl['text_saturation']['default'] = tostring(default_val)
							else
								tbl['text_saturation']['default'] = '0'
							end
							
							--max
							local max_val = tonumber(tbl['text_saturation']['max'])
							if(nil ~= max_val)then
								tbl['text_saturation']['max'] = tostring(max_val)
							else
								tbl['text_saturation']['max'] = '0'
							end
							
							--min
							local min_val = tonumber(tbl['text_saturation']['min'])
							if(nil ~= min_val)then
								tbl['text_saturation']['min'] = tostring(min_val)
							else
								tbl['text_saturation']['min'] = '0'
							end
							
							return paramdef_generator('integer', tbl['text_saturation'], 'colorbalance', 1)
						end
	},
	-- TextColorBalanceRed
	text_colorbalance_red	=
	{
		tag		=	'psf:ParameterDef',
		attribute	=
			{
				key		=	'name',
				value	=	'ns0000:JobPrintQualityTextColorBalanceR'
			},
		children	=	function(tbl)
							
							--default
							local default_val = tonumber(tbl['text_colorbalance_red']['default'])
							if(nil ~= default_val)then
								tbl['text_colorbalance_red']['default'] = tostring(default_val)
							else
								tbl['text_colorbalance_red']['default'] = '0'
							end
							
							--max
							local max_val = tonumber(tbl['text_colorbalance_red']['max'])
							if(nil ~= max_val)then
								tbl['text_colorbalance_red']['max'] = tostring(max_val)
							else
								tbl['text_colorbalance_red']['max'] = '0'
							end
							
							--min
							local min_val = tonumber(tbl['text_colorbalance_red']['min'])
							if(nil ~= min_val)then
								tbl['text_colorbalance_red']['min'] = tostring(min_val)
							else
								tbl['text_colorbalance_red']['min'] = '0'
							end
							
							return paramdef_generator('integer', tbl['text_colorbalance_red'], 'colorbalance', 1)
						end
	},
	-- TextColorBalanceGreen
	text_colorbalance_green	=
	{
		tag		=	'psf:ParameterDef',
		attribute	=
			{
				key		=	'name',
				value	=	'ns0000:JobPrintQualityTextColorBalanceG'
			},
		children	=	function(tbl)
							
							--default
							local default_val = tonumber(tbl['text_colorbalance_green']['default'])
							if(nil ~= default_val)then
								tbl['text_colorbalance_green']['default'] = tostring(default_val)
							else
								tbl['text_colorbalance_green']['default'] = '0'
							end
							
							--max
							local max_val = tonumber(tbl['text_colorbalance_green']['max'])
							if(nil ~= max_val)then
								tbl['text_colorbalance_green']['max'] = tostring(max_val)
							else
								tbl['text_colorbalance_green']['max'] = '0'
							end
							
							--min
							local min_val = tonumber(tbl['text_colorbalance_green']['min'])
							if(nil ~= min_val)then
								tbl['text_colorbalance_green']['min'] = tostring(min_val)
							else
								tbl['text_colorbalance_green']['min'] = '0'
							end
							
							return paramdef_generator('integer', tbl['text_colorbalance_green'], 'colorbalance', 1)
						end
	},
	-- TextColorBalanceBlue
	text_colorbalance_blue	=
	{
		tag		=	'psf:ParameterDef',
		attribute	=
			{
				key		=	'name',
				value	=	'ns0000:JobPrintQualityTextColorBalanceB'
			},
		children	=	function(tbl)
							
							--default
							local default_val = tonumber(tbl['text_colorbalance_blue']['default'])
							if(nil ~= default_val)then
								tbl['text_colorbalance_blue']['default'] = tostring(default_val)
							else
								tbl['text_colorbalance_blue']['default'] = '0'
							end
							
							--max
							local max_val = tonumber(tbl['text_colorbalance_blue']['max'])
							if(nil ~= max_val)then
								tbl['text_colorbalance_blue']['max'] = tostring(max_val)
							else
								tbl['text_colorbalance_blue']['max'] = '0'
							end
							
							--min
							local min_val = tonumber(tbl['text_colorbalance_blue']['min'])
							if(nil ~= min_val)then
								tbl['text_colorbalance_blue']['min'] = tostring(min_val)
							else
								tbl['text_colorbalance_blue']['min'] = '0'
							end
							
							return paramdef_generator('integer', tbl['text_colorbalance_blue'], 'colorbalance', 1)
						end
	},
	-- PhotoBrightness
	photo_brightness	=
	{
		tag		=	'psf:ParameterDef',
		attribute	=
			{
				key		=	'name',
				value	=	'ns0000:JobPrintQualityPhotoBrightness'
			},
		children	=	function(tbl)
							
							--default
							local default_val = tonumber(tbl['photo_brightness']['default'])
							if(nil ~= default_val)then
								tbl['photo_brightness']['default'] = tostring(default_val)
							else
								tbl['photo_brightness']['default'] = '0'
							end
							
							--max
							local max_val = tonumber(tbl['photo_brightness']['max'])
							if(nil ~= max_val)then
								tbl['photo_brightness']['max'] = tostring(max_val)
							else
								tbl['photo_brightness']['max'] = '0'
							end
							
							--min
							local min_val = tonumber(tbl['photo_brightness']['min'])
							if(nil ~= min_val)then
								tbl['photo_brightness']['min'] = tostring(min_val)
							else
								tbl['photo_brightness']['min'] = '0'
							end
							
							return paramdef_generator('integer', tbl['photo_brightness'], 'colorbalance', 1)
						end
	},
	---PhotoContrast
	photo_contrast	=
	{
		tag		=	'psf:ParameterDef',
		attribute	=
			{
				key		=	'name',
				value	=	'ns0000:JobPrintQualityPhotoContrast'
			},
		children	=	function(tbl)
							
							--default
							local default_val = tonumber(tbl['photo_contrast']['default'])
							if(nil ~= default_val)then
								tbl['photo_contrast']['default'] = tostring(default_val)
							else
								tbl['photo_contrast']['default'] = '0'
							end
							
							--max
							local max_val = tonumber(tbl['photo_contrast']['max'])
							if(nil ~= max_val)then
								tbl['photo_contrast']['max'] = tostring(max_val)
							else
								tbl['photo_contrast']['max'] = '0'
							end
							
							--min
							local min_val = tonumber(tbl['photo_contrast']['min'])
							if(nil ~= min_val)then
								tbl['photo_contrast']['min'] = tostring(min_val)
							else
								tbl['photo_contrast']['min'] = '0'
							end
							
							return paramdef_generator('integer', tbl['photo_contrast'], 'colorbalance', 1)
						end
	},
	-- PhotoSaturation
	photo_saturation	=
	{
		tag		=	'psf:ParameterDef',
		attribute	=
			{
				key		=	'name',
				value	=	'ns0000:JobPrintQualityPhotoSaturation'
			},
		children	=	function(tbl)
							
							--default
							local default_val = tonumber(tbl['photo_saturation']['default'])
							if(nil ~= default_val)then
								tbl['photo_saturation']['default'] = tostring(default_val)
							else
								tbl['photo_saturation']['default'] = '0'
							end
							
							--max
							local max_val = tonumber(tbl['photo_saturation']['max'])
							if(nil ~= max_val)then
								tbl['photo_saturation']['max'] = tostring(max_val)
							else
								tbl['photo_saturation']['max'] = '0'
							end
							
							--min
							local min_val = tonumber(tbl['photo_saturation']['min'])
							if(nil ~= min_val)then
								tbl['photo_saturation']['min'] = tostring(min_val)
							else
								tbl['photo_saturation']['min'] = '0'
							end
							
							return paramdef_generator('integer', tbl['photo_saturation'], 'colorbalance', 1)
						end
	},
	-- PhotoColorBalanceRed
	photo_colorbalance_red	=
	{
		tag		=	'psf:ParameterDef',
		attribute	=
			{
				key		=	'name',
				value	=	'ns0000:JobPrintQualityPhotoColorBalanceR'
			},
		children	=	function(tbl)
							
							--default
							local default_val = tonumber(tbl['photo_colorbalance_red']['default'])
							if(nil ~= default_val)then
								tbl['photo_colorbalance_red']['default'] = tostring(default_val)
							else
								tbl['photo_colorbalance_red']['default'] = '0'
							end
							
							--max
							local max_val = tonumber(tbl['photo_colorbalance_red']['max'])
							if(nil ~= max_val)then
								tbl['photo_colorbalance_red']['max'] = tostring(max_val)
							else
								tbl['photo_colorbalance_red']['max'] = '0'
							end
							
							--min
							local min_val = tonumber(tbl['photo_colorbalance_red']['min'])
							if(nil ~= min_val)then
								tbl['photo_colorbalance_red']['min'] = tostring(min_val)
							else
								tbl['photo_colorbalance_red']['min'] = '0'
							end
							
							return paramdef_generator('integer', tbl['photo_colorbalance_red'], 'colorbalance', 1)
						end
	},
	-- PhotoColorBalanceGreen
	photo_colorbalance_green	=
	{
		tag		=	'psf:ParameterDef',
		attribute	=
			{
				key		=	'name',
				value	=	'ns0000:JobPrintQualityPhotoColorBalanceG'
			},
		children	=	function(tbl)
							
							--default
							local default_val = tonumber(tbl['photo_colorbalance_green']['default'])
							if(nil ~= default_val)then
								tbl['photo_colorbalance_green']['default'] = tostring(default_val)
							else
								tbl['photo_colorbalance_green']['default'] = '0'
							end
							
							--max
							local max_val = tonumber(tbl['photo_colorbalance_green']['max'])
							if(nil ~= max_val)then
								tbl['photo_colorbalance_green']['max'] = tostring(max_val)
							else
								tbl['photo_colorbalance_green']['max'] = '0'
							end
							
							--min
							local min_val = tonumber(tbl['photo_colorbalance_green']['min'])
							if(nil ~= min_val)then
								tbl['photo_colorbalance_green']['min'] = tostring(min_val)
							else
								tbl['photo_colorbalance_green']['min'] = '0'
							end
							
							return paramdef_generator('integer', tbl['photo_colorbalance_green'], 'colorbalance', 1)
						end
	},
	-- PhotoColorBalanceBlue
	photo_colorbalance_blue	=
	{
		tag		=	'psf:ParameterDef',
		attribute	=
			{
				key		=	'name',
				value	=	'ns0000:JobPrintQualityPhotoColorBalanceB'
			},
		children	=	function(tbl)
							
							--default
							local default_val = tonumber(tbl['photo_colorbalance_blue']['default'])
							if(nil ~= default_val)then
								tbl['photo_colorbalance_blue']['default'] = tostring(default_val)
							else
								tbl['photo_colorbalance_blue']['default'] = '0'
							end
							
							--max
							local max_val = tonumber(tbl['photo_colorbalance_blue']['max'])
							if(nil ~= max_val)then
								tbl['photo_colorbalance_blue']['max'] = tostring(max_val)
							else
								tbl['photo_colorbalance_blue']['max'] = '0'
							end
							
							--min
							local min_val = tonumber(tbl['photo_colorbalance_blue']['min'])
							if(nil ~= min_val)then
								tbl['photo_colorbalance_blue']['min'] = tostring(min_val)
							else
								tbl['photo_colorbalance_blue']['min'] = '0'
							end
							
							return paramdef_generator('integer', tbl['photo_colorbalance_blue'], 'colorbalance', 1)
						end
	},
	-- GraphicsBrightness
	graphics_brightness	=
	{
		tag		=	'psf:ParameterDef',
		attribute	=
			{
				key		=	'name',
				value	=	'ns0000:JobPrintQualityGraphicsBrightness'
			},
		children	=	function(tbl)
							
							--default
							local default_val = tonumber(tbl['graphics_brightness']['default'])
							if(nil ~= default_val)then
								tbl['graphics_brightness']['default'] = tostring(default_val)
							else
								tbl['graphics_brightness']['default'] = '0'
							end
							
							--max
							local max_val = tonumber(tbl['graphics_brightness']['max'])
							if(nil ~= max_val)then
								tbl['graphics_brightness']['max'] = tostring(max_val)
							else
								tbl['graphics_brightness']['max'] = '0'
							end
							
							--min
							local min_val = tonumber(tbl['graphics_brightness']['min'])
							if(nil ~= min_val)then
								tbl['graphics_brightness']['min'] = tostring(min_val)
							else
								tbl['graphics_brightness']['min'] = '0'
							end
							
							return paramdef_generator('integer', tbl['graphics_brightness'], 'colorbalance', 1)
						end
	},
	---GraphicsContrast
	graphics_contrast	=
	{
		tag		=	'psf:ParameterDef',
		attribute	=
			{
				key		=	'name',
				value	=	'ns0000:JobPrintQualityGraphicsContrast'
			},
		children	=	function(tbl)
							
							--default
							local default_val = tonumber(tbl['graphics_contrast']['default'])
							if(nil ~= default_val)then
								tbl['graphics_contrast']['default'] = tostring(default_val)
							else
								tbl['graphics_contrast']['default'] = '0'
							end
							
							--max
							local max_val = tonumber(tbl['graphics_contrast']['max'])
							if(nil ~= max_val)then
								tbl['graphics_contrast']['max'] = tostring(max_val)
							else
								tbl['graphics_contrast']['max'] = '0'
							end
							
							--min
							local min_val = tonumber(tbl['graphics_contrast']['min'])
							if(nil ~= min_val)then
								tbl['graphics_contrast']['min'] = tostring(min_val)
							else
								tbl['graphics_contrast']['min'] = '0'
							end
							
							return paramdef_generator('integer', tbl['graphics_contrast'], 'colorbalance', 1)
						end
	},
	-- GraphicsSaturation
	graphics_saturation	=
	{
		tag		=	'psf:ParameterDef',
		attribute	=
			{
				key		=	'name',
				value	=	'ns0000:JobPrintQualityGraphicsSaturation'
			},
		children	=	function(tbl)
							
							--default
							local default_val = tonumber(tbl['graphics_saturation']['default'])
							if(nil ~= default_val)then
								tbl['graphics_saturation']['default'] = tostring(default_val)
							else
								tbl['graphics_saturation']['default'] = '0'
							end
							
							--max
							local max_val = tonumber(tbl['graphics_saturation']['max'])
							if(nil ~= max_val)then
								tbl['graphics_saturation']['max'] = tostring(max_val)
							else
								tbl['graphics_saturation']['max'] = '0'
							end
							
							--min
							local min_val = tonumber(tbl['graphics_saturation']['min'])
							if(nil ~= min_val)then
								tbl['graphics_saturation']['min'] = tostring(min_val)
							else
								tbl['graphics_saturation']['min'] = '0'
							end
							
							return paramdef_generator('integer', tbl['graphics_saturation'], 'colorbalance', 1)
						end
	},
	-- GraphicsColorBalanceRed
	graphics_colorbalance_red	=
	{
		tag		=	'psf:ParameterDef',
		attribute	=
			{
				key		=	'name',
				value	=	'ns0000:JobPrintQualityGraphicsColorBalanceR'
			},
		children	=	function(tbl)
							
							--default
							local default_val = tonumber(tbl['graphics_colorbalance_red']['default'])
							if(nil ~= default_val)then
								tbl['graphics_colorbalance_red']['default'] = tostring(default_val)
							else
								tbl['graphics_colorbalance_red']['default'] = '0'
							end
							
							--max
							local max_val = tonumber(tbl['graphics_colorbalance_red']['max'])
							if(nil ~= max_val)then
								tbl['graphics_colorbalance_red']['max'] = tostring(max_val)
							else
								tbl['graphics_colorbalance_red']['max'] = '0'
							end
							
							--min
							local min_val = tonumber(tbl['graphics_colorbalance_red']['min'])
							if(nil ~= min_val)then
								tbl['graphics_colorbalance_red']['min'] = tostring(min_val)
							else
								tbl['graphics_colorbalance_red']['min'] = '0'
							end
							
							return paramdef_generator('integer', tbl['graphics_colorbalance_red'], 'colorbalance', 1)
						end
	},
	-- GraphicsColorBalanceGreen
	graphics_colorbalance_green	=
	{
		tag		=	'psf:ParameterDef',
		attribute	=
			{
				key		=	'name',
				value	=	'ns0000:JobPrintQualityGraphicsColorBalanceG'
			},
		children	=	function(tbl)
							
							--default
							local default_val = tonumber(tbl['graphics_colorbalance_green']['default'])
							if(nil ~= default_val)then
								tbl['graphics_colorbalance_green']['default'] = tostring(default_val)
							else
								tbl['graphics_colorbalance_green']['default'] = '0'
							end
							
							--max
							local max_val = tonumber(tbl['graphics_colorbalance_green']['max'])
							if(nil ~= max_val)then
								tbl['graphics_colorbalance_green']['max'] = tostring(max_val)
							else
								tbl['graphics_colorbalance_green']['max'] = '0'
							end
							
							--min
							local min_val = tonumber(tbl['graphics_colorbalance_green']['min'])
							if(nil ~= min_val)then
								tbl['graphics_colorbalance_green']['min'] = tostring(min_val)
							else
								tbl['graphics_colorbalance_green']['min'] = '0'
							end
							
							return paramdef_generator('integer', tbl['graphics_colorbalance_green'], 'colorbalance', 1)
						end
	},
	-- GraphicsColorBalanceBlue
	graphics_colorbalance_blue	=
	{
		tag		=	'psf:ParameterDef',
		attribute	=
			{
				key		=	'name',
				value	=	'ns0000:JobPrintQualityGraphicsColorBalanceB'
			},
		children	=	function(tbl)
							
							--default
							local default_val = tonumber(tbl['graphics_colorbalance_blue']['default'])
							if(nil ~= default_val)then
								tbl['graphics_colorbalance_blue']['default'] = tostring(default_val)
							else
								tbl['graphics_colorbalance_blue']['default'] = '0'
							end
							
							--max
							local max_val = tonumber(tbl['graphics_colorbalance_blue']['max'])
							if(nil ~= max_val)then
								tbl['graphics_colorbalance_blue']['max'] = tostring(max_val)
							else
								tbl['graphics_colorbalance_blue']['max'] = '0'
							end
							
							--min
							local min_val = tonumber(tbl['graphics_colorbalance_blue']['min'])
							if(nil ~= min_val)then
								tbl['graphics_colorbalance_blue']['min'] = tostring(min_val)
							else
								tbl['graphics_colorbalance_blue']['min'] = '0'
							end
							
							return paramdef_generator('integer', tbl['graphics_colorbalance_blue'], 'colorbalance', 1)
						end
	},
	-- 07 Resolution
	resolution	=
	{
		tag		=	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'psk:PageResolution'
		},
		children	=	function(tbl)
			local result_tbl = {}

			result_tbl[ #result_tbl + 1 ] =
			{
				tag			= 	'psf:Property',
				attribute	=
					{
						key		=	'name',
						value	=	'psf:SelectionType'
					},
				children = {
					tag			= 	'psf:Value',
					attribute	=
						{
							key		=	'xsi:type',
							value	=	'xsd:QName'
						},
					text		=	'psk:PickOne'
				}
			}

			-- Wording
			local func_name = get_display_wording( 'resolution' )
			if(func_name and '' ~= func_name)then
				result_tbl[ #result_tbl + 1 ] = 
				{
					tag			= 	'psf:Property',
					attribute	=
					{
						key		=	'name',
						value	=	'psk:DisplayName'
					},
					children = {
						tag			= 	'psf:Value',
						attribute	=
						{
							key		=	'xsi:type',
							value	=	'xsd:string'
						},
						text		=	func_name
					}
				}
			end

			local element_table	= OrderedHash(
				{ '200',					{ value = 'ns0000:_200dpi',						scoredproperty = '200',		wording = 'resolution_200'	}},
				{ '300',					{ value = 'ns0000:_300dpi',						scoredproperty = '300',		wording = 'resolution_300'	}},
				{ '300p1m1',					{ value = 'ns0000:_300dpi_P1_M1',					scoredproperty = '300',		wording = 'resolution_300p1m1'	}},
				{ '400',					{ value = 'ns0000:_400dpi',						scoredproperty = '400',		wording = 'resolution_400'	}},
				{ '300to600',				{ value = 'ns0000:_300to600dpi',				scoredproperty = '300',		wording = 'resolution_300to600'	}},
				{ '300x300to38400x600',		{ value = 'ns0000:_300x300to38400x600',			scoredproperty = '300',		wording = 'resolution_300x300to38400x600'	}},
				{ '600',					{ value = 'ns0000:_600dpi',						scoredproperty = '600',		wording = 'resolution_600'	}},
				{ '1200x600',				{ value = 'ns0000:_1200x600',					scoredproperty = '600',		wording = 'resolution_1200x600'	}},
				{ '600x1200',				{ value = 'ns0000:_600x1200',					scoredproperty = '600',		wording = 'resolution_600x1200'	}},
				{ '2400x600',				{ value = 'ns0000:_2400x600',					scoredproperty = '600',		wording = 'resolution_2400x600'	}},
				{ '9600x600',				{ value = 'ns0000:_9600x600',					scoredproperty = '600',		wording = 'resolution_9600x600'	}},
				{ '38400x600',				{ value = 'ns0000:_38400x600',					scoredproperty = '600',		wording = 'resolution_38400x600'	}},
				{ '1200',					{ value = 'ns0000:_1200dpi',					scoredproperty = '1200',	wording = 'resolution_1200'	}},
				{ '4800x1200',				{ value = 'ns0000:_4800x1200',					scoredproperty = '1200',	wording = 'resolution_4800x1200'	}},
				{ '150x150to300x150p1m1',	{ value = 'ns0000:_150x150to300x150dpi_P1_M1',	scoredproperty = '150',		wording = 'resolution_150x150to300x150p1m1'	}},
				{ '300x300to600x300p1m2',	{ value = 'ns0000:_300x300to600x300dpi_P1_M2',	scoredproperty = '300',		wording = 'resolution_300x300to600x300p1m2'	}},
				{ '300x300to600x300p1m5',	{ value = 'ns0000:_300x300to600x300dpi_P1_M5',	scoredproperty = '300',		wording = 'resolution_300x300to600x300p1m5'	}},
				{ '600p1m2',				{ value = 'ns0000:_600dpi_P1_M2',				scoredproperty = '600',		wording = 'resolution_600p1m2'	}},
				{ '600p1m5',				{ value = 'ns0000:_600dpi_P1_M5',				scoredproperty = '600',		wording = 'resolution_600p1m5'	}},
				{ '600p2m2',				{ value = 'ns0000:_600dpi_P2_M2',				scoredproperty = '600',		wording = 'resolution_600p2m2'	}},
				{ '600pf1b2m2',				{ value = 'ns0000:_600dpi_PF1B2_M2',			scoredproperty = '600',		wording = 'resolution_600pf1b2m2'	}},
				{ '600p2m2fy',				{ value = 'ns0000:_600dpi_P2_M2_FY',			scoredproperty = '600',		wording = 'resolution_600p2m2fy'	}},
				{ '1200p2m4',				{ value = 'ns0000:_1200dpi_P2_M4',				scoredproperty = '1200',	wording = 'resolution_1200p2m4'	}},
				{ '1200p4m4',					{ value = 'ns0000:_1200dpi_P4_M4',					scoredproperty = '1200',	wording = 'resolution_1200p4m4'	}},
				{ '1200x1200to4800x1200p16m4',	{ value = 'ns0000:_1200x1200to4800x1200dpi_P16_M4',	scoredproperty = '1200',	wording = 'resolution_1200x1200to4800x1200p16m4'}}
			)

			-- PickOne Item
			for i, k, v in OrderedHash_each( element_table ) do
				if( i and k and v and v.value and v.scoredproperty ) then
					local isSupported = false
					for index, value in pairs( tbl.resolution ) do
						if( k == value[ 'name' ] ) then
							isSupported = true
							break
						end
					end

					if( isSupported ) then
						result_tbl[ #result_tbl + 1 ] =
						{
							tag		=	'psf:Option',
							attribute	=
							{
								key		=	'name',
								value	=	v.value
							},
							children	=
							{
								{
									tag			= 	'psf:ScoredProperty',
									attribute	=
									{
										key		=	'name',
										value	=	'psk:ResolutionX'
									},
									children	= 
									{
										tag 		=	'psf:Value',
										attribute	=
										{
											key 	=	'xsi:type',
											value	=	'xsd:integer'
										},
										text		=	v.scoredproperty
									}
								},
								{
									tag			= 	'psf:ScoredProperty',
									attribute	=
									{
										key		=	'name',
										value	=	'psk:ResolutionY'
									},
									children	= 
									{
										tag 		=	'psf:Value',
										attribute	=
										{
											key 	=	'xsi:type',
											value	=	'xsd:integer'
										},
										text		=	v.scoredproperty
									}
								}
							}
						}

						-- DisplayName 
						local item_name = v.wording and get_display_wording(v.wording) or ''
						if(item_name and '' ~= item_name) then
							result_tbl[ #result_tbl ][ 'children' ][ #(result_tbl[ #result_tbl ][ 'children' ]) + 1 ] =
							{
								tag			= 	'psf:Property',
								attribute	=
								{
									key		=	'name',
									value	=	'psk:DisplayName'
								},
								children = {
									tag			= 	'psf:Value',
									attribute	=
									{
										key		=	'xsi:type',
										value	=	'xsd:string'
									},
									text		=	item_name
								}
							}
						end
					end
				end
			end

			return result_tbl
		end
	},
	-- 08 Two Toner Print
	twotonerprint	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:JobTwoTonerPrint'
		},
		children	=	function(tbl)
			return standard_option_generator(
				tbl['twotonerprint'],
				OrderedHash(
					{ 'off',				'ns0000:Off'},
					{ 'blackandcyan',		'ns0000:BlackAndCyan'},
					{ 'blackandmagenta',	'ns0000:BlackAndMagenta'},
					{ 'blackandyellow',		'ns0000:BlackAndYellow'},
					{ 'blackandred',		'ns0000:BlackAndRed'},
					{ 'redandblack',		'ns0000:RedAndBlack'}
				) )
		end
	},
	-- 10 Color Universal Design
	coloruniversaldesign	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:JobColorUniversalDesign'
		},
		children	=	function(tbl)
			return standard_option_generator(
				tbl['coloruniversaldesign'],
				OrderedHash(
					{ 'off',	'ns0000:Off'},
					{ 'on',		'ns0000:On'}
				) )
		end
	},
	-- 11 Image Processing
	imageprocessing	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'psk:PagePhotoPrintingIntent'
		},
		children	=	function(tbl)
			return standard_option_generator(
				tbl['imageprocessing'],
				OrderedHash(
					{ 'standard',			'psk:PhotoDraft'},
					{ 'highquality',		'psk:PhotoStandard'},
					{ 'best',				'psk:PhotoBest'},
					{ 'normal',				'ns0000:PhotoNormal'},
					{ 'lowcompression',		'ns0000:PhotoHighQuality'},
					{ 'highcompression',	'ns0000:PhotoSpeed'},
					{ 'highestcompression',	'ns0000:PhotoHighSpeed'}
				) )
		end
	},
	-- 12 Vector/Raster
	graphicsmode	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:JobGraphicsMode'
		},
		children	=	function(tbl)
			return standard_option_generator(
				tbl['graphicsmode'],
				OrderedHash(
					{ 'vector',		'ns0000:Vector'},
					{ 'raster',		'ns0000:Raster'}
				) )
		end
	},
	-- 13 Black Over Print
	blackoverprint	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:JobBlackOverPrint'
		},
		children	=	function(tbl)
			return standard_option_generator(
				tbl['blackoverprint'],
				OrderedHash(
					{ 'off',	'ns0000:Off'},
					{ 'on',		'ns0000:On'}
				) )
		end
	},
	-- 14 Edge Smoothing
	edgesmoothing	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:JobSmoothing'
		},
		children	=	function(tbl)
			return standard_option_generator(
				tbl['edgesmoothing'],
				OrderedHash(
					{ 'off',	'ns0000:Off'},
					{ 'on',		'ns0000:On'}
				) )
		end
	},
	-- 15 Print All Text as Black
	printalltextasblack	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:JobPrintTextAsBlack'
		},
		children	=	function(tbl)
			return standard_option_generator(
				tbl['printalltextasblack'],
				OrderedHash(
					{ 'off',	'ns0000:Off'},
					{ 'on',		'ns0000:On'}
				) )
		end
	},
	-- 16 Fuser Setting
	fusersetting	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:JobFuserSetting'
		},
		children	=	function(tbl)
			return standard_option_generator(
				tbl['fusersetting'],
				OrderedHash(
					{ 'standard',		'ns0000:Standard'},
					{ 'highquality',	'ns0000:HighQuality'}
				) )
		end
	},
	-- 18-01 Toner Saving
	tonersaving	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:JobPrintDensity'
		},
		children	=	function(tbl)
			return standard_option_generator(
				tbl['tonersaving'],
				OrderedHash(
					{ 'off',		'ns0000:Off'},
					{ 'on',			'ns0000:On'},
					{ 'tonersave1',	'ns0000:Mode1'},
					{ 'tonersave2',	'ns0000:Mode2'},
					{ 'levelcolor',	'ns0000:LevelColor'}
				) )
		end
	},
	-- 18-02 Enhanced Toner Saving
	enhancedtonersaving	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:JobPrintDensity'
		},
		children	=	function(tbl)
			local element_table	= OrderedHash(
				{ 'off',			'ns0000:Off'},
				{ 'mediumlight',	'ns0000:MediumLight'},
				{ 'light',			'ns0000:Light'},
				{ 'manual',			{ value = 'ns0000:Custom',	scoredproperty = {}}}
			)

			local isPhase4 = ( tbl['text_tonersavingnum'] and tbl['photo_tonersavingnum'] and tbl['graphics_tonersavingnum'] )

			-- Phase4
			if(isPhase4)then
				table.insert(element_table['manual']['scoredproperty'],{name = 'ns0000:TextTonerSaveValue', def = 'ns0000:JobTextTonerSaveValueDef'})
				table.insert(element_table['manual']['scoredproperty'],{name = 'ns0000:PhotoTonerSaveValue', def = 'ns0000:JobPhotoTonerSaveValueDef'})
				table.insert(element_table['manual']['scoredproperty'],{name = 'ns0000:GraphicsTonerSaveValue', def = 'ns0000:JobGraphicsTonerSaveValueDef'})
			end

			local result_table = standard_option_generator(tbl['enhancedtonersaving'], element_table)

			-- Phase3
			-- TextTonerSave
			if( ( not isPhase4 ) and tbl['text_tonersaving'] )then
				local text_element_table	= OrderedHash(
					{ 'off',			'ns0000:Off'},
					{ 'mediumlight',	'ns0000:MediumLight'},
					{ 'light',			'ns0000:Light'}
				)
				result_table[ #result_table + 1 ] =
				{
					tag			= 	'psf:Feature',
					attribute	=
					{
						key		=	'name',
						value	=	'ns0000:CustomTextDensity'
					},
					children	=	standard_option_generator(tbl['text_tonersaving'], text_element_table)
				}
			end

			-- PhotoTonerSave
			if( ( not isPhase4 ) and tbl['photo_tonersaving'] )then
				local photo_element_table	= OrderedHash(
					{ 'off',			'ns0000:Off'},
					{ 'mediumlight',	'ns0000:MediumLight'},
					{ 'light',			'ns0000:Light'}
				)
				result_table[ #result_table + 1 ] =
				{
					tag			= 	'psf:Feature',
					attribute	=
					{
						key		=	'name',
						value	=	'ns0000:CustomPhotoDensity'
					},
					children	=	standard_option_generator(tbl['photo_tonersaving'], photo_element_table)
				}
			end

			-- GraphicsTonerSave
			if( ( not isPhase4 ) and tbl['graphics_tonersaving'] )then
				local graphics_element_table	= OrderedHash(
					{ 'off',			'ns0000:Off'},
					{ 'mediumlight',	'ns0000:MediumLight'},
					{ 'light',			'ns0000:Light'}
				)
				result_table[ #result_table + 1 ] =
				{
					tag			= 	'psf:Feature',
					attribute	=
					{
						key		=	'name',
						value	=	'ns0000:CustomGraphicsDensity'
					},
					children	=	standard_option_generator(tbl['graphics_tonersaving'], graphics_element_table)
				}
			end

			return result_table
		end
	},
	-- 18-02-1 Manual Toner Saving
	text_tonersavingnum	=
	{
		tag		=	'psf:ParameterDef',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:JobTextTonerSaveValueDef'
		},
		children	=	function(tbl)
			--default
			local default_val = tonumber(tbl['text_tonersavingnum']['default'])
			if(nil ~= default_val)then
				tbl['text_tonersavingnum']['default'] = tostring(default_val)
			else
				tbl['text_tonersavingnum']['default'] = '0'
			end

			--max
			local max_val = tonumber(tbl['text_tonersavingnum']['max'])
			if(nil ~= max_val)then
				tbl['text_tonersavingnum']['max'] = tostring(max_val)
			else
				tbl['text_tonersavingnum']['max'] = '0'
			end

			--min
			local min_val = tonumber(tbl['text_tonersavingnum']['min'])
			if(nil ~= min_val)then
				tbl['text_tonersavingnum']['min'] = tostring(min_val)
			else
				tbl['text_tonersavingnum']['min'] = '0'
			end

			return paramdef_generator('integer', tbl['text_tonersavingnum'], 'percent', 1)
		end
	},
	photo_tonersavingnum	=
	{
		tag		=	'psf:ParameterDef',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:JobPhotoTonerSaveValueDef'
		},
		children	=	function(tbl)
			--default
			local default_val = tonumber(tbl['photo_tonersavingnum']['default'])
			if(nil ~= default_val)then
				tbl['photo_tonersavingnum']['default'] = tostring(default_val)
			else
				tbl['photo_tonersavingnum']['default'] = '0'
			end

			--max
			local max_val = tonumber(tbl['photo_tonersavingnum']['max'])
			if(nil ~= max_val)then
				tbl['photo_tonersavingnum']['max'] = tostring(max_val)
			else
				tbl['photo_tonersavingnum']['max'] = '0'
			end

			--min
			local min_val = tonumber(tbl['photo_tonersavingnum']['min'])
			if(nil ~= min_val)then
				tbl['photo_tonersavingnum']['min'] = tostring(min_val)
			else
				tbl['photo_tonersavingnum']['min'] = '0'
			end

			return paramdef_generator('integer', tbl['photo_tonersavingnum'], 'percent', 1)
		end
	},
	graphics_tonersavingnum	=
	{
		tag		=	'psf:ParameterDef',
		attribute	=
			{
				key		=	'name',
				value	=	'ns0000:JobGraphicsTonerSaveValueDef'
			},
		children	=	function(tbl)
			--default
			local default_val = tonumber(tbl['graphics_tonersavingnum']['default'])
			if(nil ~= default_val)then
				tbl['graphics_tonersavingnum']['default'] = tostring(default_val)
			else
				tbl['graphics_tonersavingnum']['default'] = '0'
			end
			
			--max
			local max_val = tonumber(tbl['graphics_tonersavingnum']['max'])
			if(nil ~= max_val)then
				tbl['graphics_tonersavingnum']['max'] = tostring(max_val)
			else
				tbl['graphics_tonersavingnum']['max'] = '0'
			end
			
			--min
			local min_val = tonumber(tbl['graphics_tonersavingnum']['min'])
			if(nil ~= min_val)then
				tbl['graphics_tonersavingnum']['min'] = tostring(min_val)
			else
				tbl['graphics_tonersavingnum']['min'] = '0'
			end
			
			return paramdef_generator('integer', tbl['graphics_tonersavingnum'], 'percent', 1)
		end
	},
	-- 18-03 Economy Color Mode
	economycolormode 	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:JobEconomyColorMode'
		},
		children	=	function(tbl)
			return standard_option_generator(
				tbl['economycolormode'],
				OrderedHash(
					{ 'off',	'ns0000:Off'},
					{ 'on',		'ns0000:On'}
				) )
		end
	},
	-- 19 Separate into CMYK 
	separateintocmyk_cyan 	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:PagePlaneSelectC'
		},
		children	=	function(tbl)
			return standard_option_generator(
				tbl['separateintocmyk_cyan'],
				OrderedHash(
					{ 'on',		'ns0000:On'},
					{ 'off',	'ns0000:Off'}
				) )
		end
	},
	separateintocmyk_magenta	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:PagePlaneSelectM'
		},
		children	=	function(tbl)
			return standard_option_generator(
				tbl['separateintocmyk_magenta'],
				OrderedHash(
					{ 'on',		'ns0000:On'},
					{ 'off',	'ns0000:Off'}
				) )
		end
	},
	separateintocmyk_yellow 	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:PagePlaneSelectY'
		},
		children	=	function(tbl)
			return standard_option_generator(
				tbl['separateintocmyk_yellow'],
				OrderedHash(
					{ 'on',		'ns0000:On'},
					{ 'off',	'ns0000:Off'}
				) )
		end
	},
	separateintocmyk_black 	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:PagePlaneSelectK'
		},
		children	=	function(tbl)
			return standard_option_generator(
				tbl['separateintocmyk_black'],
				OrderedHash(
					{ 'on',		'ns0000:On'},
					{ 'off',	'ns0000:Off'}
				) )
		end
	},
	-- 20 Adjust Photographs
	adjustphotographs	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:JobAdjustPhotographs'
		},
		children	=	function(tbl)
			return standard_option_generator(
				tbl['adjustphotographs'],
				OrderedHash(
					{ 'off',	'ns0000:Off'},
					{ 'on',		'ns0000:On'}
				) )
		end
	},
	-- 21 Bold Text
	boldtext	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:JobBoldText'
		},
		children	=	function(tbl)
			return standard_option_generator(
				tbl['boldtext'],
				OrderedHash(
					{ 'off',	'ns0000:Off'},
					{ 'on',		'ns0000:On'}
				) )
		end
	},
	-- 24 RGB revision
	rgbrevision	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:PageRGBRevision'
		},
		children	=	function(tbl)
			return standard_option_generator(
				tbl['rgbrevision'],
				OrderedHash(
					{ 'off',		'ns0000:Off'},
					{ 'fine',		'ns0000:Fine'},
					{ 'superfine',	'ns0000:SuperFine'}
				) )
		end
	},
	-- 25 Image Smoothing
	imagesmoothing	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:JobImageSmoothing'
		},
		children	=	function(tbl)
			return standard_option_generator(
				tbl['imagesmoothing'],
				OrderedHash(
					{ 'off',		'ns0000:Off'},
					{ 'on',			'ns0000:On'},
					{ 'auto',		'ns0000:Auto'},
					{ '90ppi',		'ns0000:LessThan90ppi'},
					{ '150ppi',		'ns0000:LessThan150ppi'},
					{ '200ppi',		'ns0000:LessThan200ppi'},
					{ '300ppi',		'ns0000:LessThan300ppi'}
				) )
		end
	},
	-- 31 Print density for particular media types
	printdensityforparticularmediatypes	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:JobPlainPrintDensity'
		},
		children	=	function(tbl)
			return standard_option_generator(
				tbl['printdensityforparticularmediatypes'],
				OrderedHash(
					{ 'optimize',		'ns0000:Optimize'},
					{ 'standard',		'ns0000:Standard'},
					{ 'light',			'ns0000:Light'},
					{ 'dark',			'ns0000:Dark'}
				) )
		end
	},
	-- 32 Uni-directional printing
	unidirectionalprinting	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:JobUniDirectionalPrintingOnly'
		},
		children	=	function(tbl)
			return standard_option_generator(
				tbl['unidirectionalprinting'],
				OrderedHash(
					{ 'on',		'ns0000:On'},
					{ 'off',	'ns0000:Off'}
				) )
		end
	},
	-- 33 Quality Balance Preset
	qualitybalancepreset	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:JobQualityBalancePreset'
		},
		children	=	function(tbl)
			return standard_option_generator(
				tbl['qualitybalancepreset'],
				OrderedHash(
					{ 'highspeedlinedraw',	'ns0000:HighSpeedLineDraw'},
					{ 'highspeed',			'ns0000:HighSpeed'},
					{ 'speedpriority',		'ns0000:SpeedPriority'},
					{ 'qualitypriority',	'ns0000:QualityPriority'},
					{ 'highquality',		'ns0000:HighQuality'},
					{ 'manual',				'ns0000:Manual'}
				) )
		end
	},
	-- 34 Smooth low resolution images
	smoothlowresolutionimages	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:JobSmoothLowResolutionImages'
		},
		children	=	function(tbl)
			return standard_option_generator(
				tbl['smoothlowresolutionimages'],
				OrderedHash(
					{ 'on',		'ns0000:On'},
					{ 'off',	'ns0000:Off'}
				) )
		end
	},
	-- 37 Printer Head Position Setting
	printerheadposition	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:JobPrinterHeadPosition'
		},
		children	=	function(tbl)
			return standard_option_generator(
				tbl['printerheadposition'],
				OrderedHash(
					{ 'auto',		'ns0000:Auto'},
					{ 'envelope',	'ns0000:Envelope'}
				) )
		end
	},
	
	-- 40 Print Darker For 2 Sided Print 
	printdarkerfor2sidedprint	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:JobPrintDarkerFor2SidedPrint'
		},
		children	=	function(tbl)
			return standard_option_generator(
				tbl['printdarkerfor2sidedprint'],
				OrderedHash(
					{ 'off',	'ns0000:Off'},
					{ 'on',		'ns0000:On'}
				) )
		end
	},
	
	-- 41 Print black using 4 colors (CMYK)
	printblackusing4colors	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:JobPrintBlackUsing4Colors'
		},
		children	=	function(tbl)
			return standard_option_generator(
				tbl['printblackusing4colors'],
				OrderedHash(
					{ 'off',	'ns0000:Off'},
					{ 'on',		'ns0000:On'}
				) )
		end
	},
	-- 42 Font Thickness Adjustment
	fontthicknessadjustment	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:JobFontThicknessAdjustment'
		},
		children	=	function(tbl)
			return standard_option_generator(
				tbl['fontthicknessadjustment'],
				OrderedHash(
					{ 'off',	'ns0000:Off'},
					{ 'on',		'ns0000:On'}
				) )
		end
	},
	-- 43 Envelope Selector Position
	envelopeselectorposition	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:JobEnvelopeSelectorPosition'
		},
		children	=	function(tbl)
			return standard_option_generator(
				tbl['envelopeselectorposition'],
				OrderedHash(
					{ 'normal',			'ns0000:Normal'},
					{ 'envelope',			'ns0000:Envelope'},
					{ 'printersetting',	'ns0000:PrinterSetting'}
				) )
		end
	},
	
	-- 45 Adjust Line Width
	adjustlinewidth	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:JobAdjustLineWidth'
		},
		children	=	function(tbl)
			return standard_option_generator(
				tbl['adjustlinewidth'],
				OrderedHash(
					{ 'morethin',	'ns0000:MoreThin'},
					{ 'thin',		'ns0000:Thin'},
					{ 'off',		'ns0000:Off'},
					{ 'thick',		'ns0000:Thick'},
					{ 'morethick',	'ns0000:MoreThick'}
				) )
		end
	},
	
-- [10 Driver Behavior]
	-- 02 Change Data Processing 
	changedataprocessing	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:JobRPCSChangeDataProcessing'
		},
		children	=	function(tbl)
			return standard_option_generator(
				tbl['changedataprocessing'],
				OrderedHash(
					{ 'off',	'ns0000:Off'},
					{ 'on',		'ns0000:On'}
				) )
		end
	},
	-- 03 EMF Spool
	emfspool	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:JobSpoolFormat'
		},
		children	=	function(tbl)
			return standard_option_generator(
				tbl['emfspool'],
				OrderedHash(
					{ 'off',	'ns0000:Raw'},
					{ 'on',		'ns0000:EMF'}
				) )
		end
	},
	-- 04 Job Load Distribution
	jobloaddistribution	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:JobRPCSLoadDistribution'
		},
		children	=	function(tbl)
			return standard_option_generator(
				tbl['jobloaddistribution'],
				OrderedHash(
					{ 'shiftloadtodriver',		'ns0000:ShiftToDriver'},
					{ 'shiftloadtoprinter',		'ns0000:ShiftToPrinter'}
				) )
		end
	},
	-- 06 Restore Emulation
	restoreemulation	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:JobEmulationMode'
		},
		children	=	function(tbl)
			return standard_option_generator(
				tbl['restoreemulation'],
				OrderedHash(
					{ 'off',	'ns0000:None'},
					{ 'on',		'ns0000:LastUsed'}
				) )
		end
	},
	-- 07 Application Zoom
	applicationzoom	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:JobApplicationZoom'
		},
		children	=	function(tbl)
			return standard_option_generator(
				tbl['applicationzoom'],
				OrderedHash(
					{ 'off',	'ns0000:Off'},
					{ 'on',		'ns0000:On'}
				) )
		end
	},
	-- 08 Preview
	preview	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:JobPreview'
		},
		children	=	function(tbl)
			return standard_option_generator(
				tbl['preview'],
				OrderedHash(
					{ 'off',	'ns0000:Off'},
					{ 'on',		'ns0000:On'}
				) )
		end
	},
	
	trackid	=
	{
		tag		=	'psf:ParameterDef',
		attribute	=
			{
				key		=	'name',
				value	=	'ns0000:JobTrackID'
			},
		children	=	function(tbl)
							return paramdef_generator('string', tbl['trackid'], 'characters', nil)
						end
	},
	
	-- 16 Fix dmColor
	fixdmcolor	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:JobFixdmColor'
		},
		children	=	function(tbl)
			return standard_option_generator(
				tbl['fixdmcolor'],
				OrderedHash(
					{ 'off',	'ns0000:Off'},
					{ 'on',		'ns0000:On'}
				) )
		end
	},
	
	-- Output Compressed Data Except PJL
	outputcompresseddataexceptpjl	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:JobOutputCompressedDataExceptPJL'
		},
		children	=	function(tbl)
			return standard_option_generator(
				tbl['outputcompresseddataexceptpjl'],
				OrderedHash(
					{ 'off',	'ns0000:Off'},
					{ 'on',		'ns0000:On'}
				) )
		end
	},
	
-- [24 Master Print Option]
	-- 01 Printer-side Operation
	printersideoperation	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:JobPrinterSideOperation'
		},
		children	=	function(tbl)
			return standard_option_generator(
				tbl['printersideoperation'],
				OrderedHash(
					{ 'off',	'ns0000:Off'},
					{ 'on',		'ns0000:On'}
				) )
		end
	},
	-- 02 Cycle Setting
	cyclesetting	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:JobCycleSetting'
		},
		children	=	function(tbl)
			return standard_option_generator(
				tbl['cyclesetting'],
				OrderedHash(
					{ 'autocycle',			'ns0000:AutoCycle'},
					{ 'masterprintcycle',	'ns0000:MasterPrintCycle'},
					{ 'mastercycle',		'ns0000:MasterCycle'},
					{ 'originalpagecycle',	'ns0000:OriginalPageCycle'}
				) )
		end
	},
	-- 03 Output Options
	outputoptionsetting	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:JobOutputOptionSetting'
		},
		children	=	function(tbl)
			return standard_option_generator(
				tbl['outputoptionsetting'],
				OrderedHash(
					{ 'off',			'ns0000:Off'},
					{ 'jobseparator',	'ns0000:JobSeparator'},
					{ 'sort',			'ns0000:Sort'}
				) )
		end
	},
	-- 04 Skip Feed
	skipfeed	=
	{
		{
			tag		= 	'psf:Feature',
			attribute	=
			{
				key		=	'name',
				value	=	'ns0000:JobSkipFeed'
			},
			children	=	function(tbl)
				return standard_option_generator(
					tbl['skipfeed'],
					OrderedHash(
					{ 'off',	{value = 'ns0000:Off', scoredproperty = {}}},
					{ 'on',		{value = 'ns0000:On', scoredproperty = {{name = 'ns0000:FeedNumber', def = 'ns0000:JobSkipFeedNumber'}}}}
					) )
			end
		},
		{
			tag			=	'psf:ParameterDef',
			attribute	=
			{
				key		=	'name',
				value	=	'ns0000:JobSkipFeedNumber'
			},
			children	=	function(tbl)
				return paramdef_generator('integer', tbl['skipfeednumber'], 'reg',1)
			end
		}
	},
	-- 05 Economy Mode
	economymode	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:JobEconomyMode'
		},
		children	=	function(tbl)
			return standard_option_generator(
				tbl['economymode'],
				OrderedHash(
					{ 'off',		'ns0000:Off'},
					{ 'level1',		'ns0000:Level1'},
					{ 'level2',		'ns0000:Level2'}
				) )
		end
	},
	-- 06 Speed Select
	speedselect	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:JobSpeedSelect'
		},
		children	=	function(tbl)
			return standard_option_generator(
				tbl['speedselect'],
				OrderedHash(
					{ 'printspeed1',	'ns0000:PrintSpeed1'},
					{ 'printspeed2',	'ns0000:PrintSpeed2'},
					{ 'printspeed3',	'ns0000:PrintSpeed3'},
					{ 'printspeed4',	'ns0000:PrintSpeed4'},
					{ 'printspeed5',	'ns0000:PrintSpeed5'},
					{ 'printspeed6',	'ns0000:PrintSpeed6'}
				) )
		end
	},
	-- 07 Reset Printer Panel
	resetprinterpanel	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:JobResetPrinterPanel'
		},
		children	=	function(tbl)
			return standard_option_generator(
				tbl['resetprinterpanel'],
				OrderedHash(
					{ 'off',	'ns0000:Off'},
					{ 'on',		'ns0000:On'}
				) )
		end
	},
	-- 08 Enable Short Master
	enableshortmaster	=
	{
		tag			= 	'psf:Feature',
		attribute	=
		{
			key		=	'name',
			value	=	'ns0000:JobEnableShortMaster'
		},
		children	=	function(tbl)
			return standard_option_generator(
				tbl['enableshortmaster'],
				OrderedHash(
					{ 'off',	'ns0000:Off'},
					{ 'on',		'ns0000:On'}
				) )
		end
	},

}
